
#include "PacketReceiver.h"

#include <QByteArray>
#include <QDebug>
#include <cstring>

#include "mlink/mlink_frame_header.h"
#include "mlink/frame_hw.h"

using namespace mlink;

PacketReceiver::PacketReceiver()
{
  reset();
}

void PacketReceiver::reset()
{
  save_frame_seq_110 = 0;
  linkStats.clear();
}

void PacketReceiver::printLinkStats() const
{
  linkStats.print();
}

size_t PacketReceiver::handle_rx_pkt()
{
#ifdef Q_OS_WIN
    qDebug() << "handle_rx_pkt: not implemented";
    return 0;
#else
  size_t packets_received = 0;
  int rawbuf[ML_RX_BUFFER_WORDS];
//  memset(rawbuf, 0, sizeof(rawbuf)); // not required
  int timeout = 10; // ms
//  bool eos = false;
  while (true)
//  while (!eos)
  {
    int rawbytes = sizeof(rawbuf);
        struct frameset_t frameset;
        int ret = ml_recv_data_frame_multi(rawbuf, rawbytes, &frameset, timeout);
        if (ret) {
//	  if (ret != -EAGAIN)
//		printf("ml_recv_data_frame_multi returned %s, break\n", ml_strerror(-ret));
          break;
        }
        packets_received += frameset.count;
        for (size_t fn = 0; fn<frameset.count; fn++) {
          const struct framedesc_t *fdesc = &frameset.fdesc[fn];
          const size_t datawords = fdesc->len - (ML_FRAME_HEADER_WORDS+ML_FRAME_TRAILER_WORDS);
          const int *frame_data = &fdesc->rawdata[ML_FRAME_HEADER_WORDS];
          const frame_hdr_t *fh = (const frame_hdr_t *) fdesc->rawdata;
          const int type = fh->type;

          if (type == 0x0110) {
              bytesRead(QByteArray(static_cast<const char *>(static_cast<const void *>(frame_data)), datawords*4));
          }
//	  if (type == 0x0110)
          {
                quint16 frame_seq_110 = fh->seq;
                if ((save_frame_seq_110 > 0) && ((quint16)(save_frame_seq_110+1) != frame_seq_110)) {
                  qWarning() << QString("frame sequence number bad: %1 changed to %2")
                          .arg(save_frame_seq_110).arg(frame_seq_110);
                }
                save_frame_seq_110 = frame_seq_110;
          }

          linkStats.countPacket(type, datawords);
          if (type == 0x0110) {
                for (size_t i = 0; i<datawords; i++) {
                  if ((frame_data[i]>>28) >= 8)
                        qDebug() << QString("data[%1]: %2").arg(i, 4, 16).arg(frame_data[i], 8, 16);
                  if ((frame_data[i]>>28) == 0xD) {
                        qDebug() <<"End of spill, break";
//			eos = true;
                  }
                }
          }
        }
  }
  return packets_received;
#endif
}
