//
//    Copyright 2002-2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DAQAPP_H
#define DAQAPP_H

#include <QObject>
#include <QTime>

#include "daqcommon/daq_types.h"
#include "hal/base_hwif.h"

class Storage;
class PacketReceiver;
class SpillManager;
class DaqAppPrivate;
class QByteArray;
class StartupConfig;
class hw_backend;
class dvs_t;
class Run_State_Machine;

class DaqApp : public QObject
{
    Q_OBJECT
public:
    explicit DaqApp(int argc, char **argv, QObject *parent = 0);
    virtual ~DaqApp();
    void dma_data_handler(const dma_buf_t *buf, const size_t start_off, const size_t end_off);
    bool wait_for_data_receive_complete();

public Q_SLOTS:
    void dataHandler(const QByteArray &data);
    void periodicTask();
    void cleanup();

private Q_SLOTS:
//    void performRestart();
    void task_interspill_test();
    void task_initialize_database();
    void task_init();
    void task_reset();
    void task_preparerun();
    void task_run();
    void task_pause();

private:
    void handle_raw_words(const dma_buf_t *buf, size_t vs, size_t ve, bool endspill);
    int decode_data_at_end_of_burst ();
    bool get_hardware_events ();
    void clearstats ();
    void task_end_of_burst_readout();
    void task_start_of_burst();
    void task_end_of_burst();
    void initialize_database();

    StartupConfig * const startupConfig;
    DaqAppPrivate * const d;
    //    Run_State_Machine_Impl * const runcontrol;
    hwev_queue_t hweq;
    PacketReceiver *const packetReceiver;
    Run_State_Machine * const runcontrol;
    hw_backend *hwb;
    dvs_t * const dvs;
    double runTimeSeconds;
    quint64 bsize;

public:
    Storage * const storage;
    SpillManager * const spillman;
};

#endif // DAQAPP_H
