#ifndef RUNCTRL_H
#define RUNCTRL_H

#include <QObject>
//#include <QTimer>

#include "daqcommon/daq_types.h"
#include "daqcommon/IpcShmStatus.h"

struct shm_ctl_t;

class RunControl: public QObject
{
    Q_OBJECT
public:
    RunControl(shm_ctl_t *control);
    void checkcontrol();

    shm_ctl_t * const shm_ctl;

Q_SIGNALS:
    void controlChanged(int magic);

private:
    void set_default_ctl();
    bool control_changed();
    void handle_reset_magic();
    void handle_ctl_change();

    unsigned int prev_ctl_reset_magic;

};

class Run_State_Machine : public QObject, public IpcShmStatus
{
    Q_OBJECT

public:
    Run_State_Machine(shm_status_t *status, shm_ctl_t *control);
    virtual ~Run_State_Machine();
    daq_state_t get_state() const {return ds;}
    void set_automode(bool state) {automode = state;}
    void poll();


public Q_SLOTS:
    void go_off();
    void go_init();
    void go_reset();
    void go_run();
    void go_preparerun();
    void go_pause();
    void go_error();

private Q_SLOTS:
    void onControlChange(int magic);

Q_SIGNALS:
    //  void task_interspill_test();
    //  void task_start_of_burst();
    //  void task_end_of_burst();
    void task_initialize_database();
    void task_init();
    void task_reset();
    void task_preparerun();
    void task_run();
    void task_pause();

private:
    static bool isTransitionAllowed(daq_state_t from, daq_state_t to);
    void changestate (daq_state_t st);
    void autoChangeState();

    daq_state_t ds;
    bool automode;
//    QTimer errorRecoveryTimer;
    RunControl * const rc;
};

#endif /* RUNCTRL_H */
