
#include <QString>
#include <QSettings>
#include <QCoreApplication>

#include "daqcfg.h"

DaqCfg::DaqCfg()
    : data(new daqcfg_t)
{
}

DaqCfg::~DaqCfg()
{
  delete data;
}

void DaqCfg::load ()
{
    QSettings settings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), "vmegui", parent());
    settings.beginGroup("DaqCfg");
    data->dec_en = settings.value("dec_en", true).toBool();
    data->currentHistName = settings.value("current_hist").toString();
    data->hist_en = settings.value("hist_en", true).toBool();
    data->hist_downscale = settings.value("hist_downscale", 1).toInt();
    data->hist_clear_by_burst = settings.value("hist_clear_by_burst", false).toBool();
    data->tdcCommonStopChannel = settings.value("tdc_common_stop_channel").toInt();
    settings.endGroup();
    Q_EMIT changed();
}

void DaqCfg::save () const
{
    QSettings settings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), "vmegui", parent());
    settings.beginGroup("DaqCfg");
    settings.setValue("dec_en", data->dec_en);
    settings.setValue("hist_en", data->hist_en);
    settings.setValue("current_hist", data->currentHistName);
    settings.setValue("hist_downscale", data->hist_downscale);
    settings.setValue("hist_clear_by_burst", data->hist_clear_by_burst);
    settings.setValue("tdc_common_stop_channel", data->tdcCommonStopChannel);
    settings.endGroup();
}

bool DaqCfg::get_dec_en() const
{
    return data->dec_en;
}

void DaqCfg::set_dec_en(bool state)
{
    if (state == data->dec_en)
        return;
    data->dec_en = state;
    Q_EMIT changed();
}

bool DaqCfg::get_hist_en() const
{
    return data->hist_en;
}

void DaqCfg::set_hist_en(bool state)
{
    if (state == data->hist_en)
        return;
    data->hist_en = state;
    Q_EMIT changed();
}

int DaqCfg::get_hist_downscale() const
{
    return data->hist_downscale;
}

void DaqCfg::set_hist_downscale(int value)
{
    if (value == data->hist_downscale)
        return;
    data->hist_downscale = value;
    Q_EMIT changed();
}

bool DaqCfg::get_hist_clear_by_burst() const
{
    return data->hist_clear_by_burst;
}

void DaqCfg::set_hist_clear_by_burst(bool state)
{
    if (state == data->hist_clear_by_burst)
        return;
    data->hist_clear_by_burst = state;
    Q_EMIT changed();
}

QString DaqCfg::getCurrentHist() const
{
    return data->currentHistName;
}

void DaqCfg::setCurrentHist(const QString &s)
{
    if (s == data->currentHistName)
        return;
    data->currentHistName = s;
    Q_EMIT changed();
}

int DaqCfg::getTdcCommonStopChannel() const
{
    return data->tdcCommonStopChannel;
}

void DaqCfg::setTdcCommonStopChannel(int value)
{
    if (value == data->tdcCommonStopChannel)
        return;
    data->tdcCommonStopChannel = value;
    Q_EMIT changed();
}
