//
//    Copyright 2003-2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "daqgui.h"

#include <QDebug>
#include <QMainWindow>
#include <QDesktopServices>
#include <QUrl>
#include <QAction>
#include <QApplication>
#include <QTimer>
#include <QMessageBox>
#include <q3progressdialog.h>
#include <QFileDialog>
#include <QWidget>
#include <q3listbox.h>
#include <QRegExp>
#include <QMap>
#include <q3ptrlist.h>
#include <QTabWidget>
#include <QCheckBox>
#include <QComboBox>
#include <q3groupbox.h>
#include <QSpinBox>
#include <QPushButton>
#include <QRadioButton>
#include <QTextBrowser>
#include <QLineEdit>
#include <QDateTime>
#include <QVector>
//#include <QSqlDatabase>
#include <QSettings>

#include <qwt/qwt_plot_marker.h>

#include <cstddef>
#include <cstring>
#include <cerrno>
#include <csignal>
#include <vector>
#include <iostream>
#include <sstream>
#include <cmath>

#include "oemstrings.h"
#include "main.h"
#include "daqcommon/daqexc.h"
#include "daqcommon/daq.h"
#include "daqcommon/daq_types.h"
#include "daqcommon/daq_ipc.h"
#include "decoder/dvs.h"
#include "daqcommon/sysutil.h"
#include "daqcommon/dsplib.h"
#include "startdaq.h"
#include "daqcfg.h"
#include "modules/trig_types.h"
#include "hal/hptdc_settings.h"
#include "qutil.h"
#include "comgui/dataplot.h"
#include "daqcommon/th1.h"
#include "histdialog.h"
#include "histplot.h"
#include "histmgr.h"
#include "hal/base_mod.h"
#include "hal/hw_frontend_shm.h"
#include "sysmon.h"
#include "daqcommon/rawdata.h"
#include "decoder/RawDumper.h"
#include "decoder/decoder.h"
#include "decoder/decoder_modlist_inc.h"
#include "msc_waveformview.h"
#include "tqdc_waveformview.h"
#include "mytable.h"
#include "tdcctrlgui.h"
#include "triggersview.h"
#include "hptdc/hptdc.h"
#include "daqcommon/RunStateTransitionRules.h"
#include "shmslave.h"
#include "daqcommon/QtMessageTypes.h"
#include "daqcommon/QtMessageHandler.h"
#include "daqcommon/QtMessageUdpReceiver.h"
#include "ui_daqguibase.h"

#define MODULE_TDC
#define MODULE_TQDC
#define MODULE_TRIG
#define MODULE_BRIC

const int timerUpdateDisplayPeriodMs = 200;
const int timerHandleEventsPeriodMs = 10;
const int timerReplotSpillPeriodMs = 150;
const int timerAutosavePeriodMs = 5000;

namespace {

OEMStrings oemstr;

double tv_dt(const struct timeval t1, const struct timeval t2)
{
    return (((t2.tv_sec)-(t1.tv_sec)) + 1e-6*((t2.tv_usec)-(t1.tv_usec)));
}

//---------------------------------------------------------------------------
int save_nburst, save_nev;
daq_state_t save_ds;

enum runstate_t { RS_OFF = 1, RS_READY = 2, RS_RUN = 3 };
runstate_t runstate = RS_OFF;

int disable_cfg_slots = 0;
int logview_level_warn = 1;
int logview_level_info = 1;
int logview_level_debug = 0;

// Spill Roll Picture
std::vector<double> spr_px;
std::vector<std::vector<double> > spr_py;
size_t spr_len() { return spr_px.size(); }
size_t spr_num() { return spr_py.size(); }
QwtPlotMarker *spr_m1 = 0;

std::map<std::string, quint64> ev_stats, ev_stats_save;
std::map<int, std::map<std::string, quint64> > mod_b_stats, mod_b_stats_save;

SystemMonitor sysmon;

// select which tqdc to setup
int tqdc_select = SET_ALL_CH;
burst_state_t save_bs = BS_OFF;
double decoder_progress = 0;

class CfgSlotDisabler {
public:
    CfgSlotDisabler() {disable_cfg_slots++;}
    ~CfgSlotDisabler() {disable_cfg_slots--;}
};


enum tqdc_tbl_col {
    tqdc_tbl_col_en = 0,
    tqdc_tbl_col_mch = 1,
    tqdc_tbl_col_inv = 2,
    tqdc_tbl_col_thr = 3,
    tqdc_tbl_col_hitst = 4,
    tqdc_tbl_col_hitsq = 5,
    tqdc_tbl_col_baseline = 6,
    tqdc_tbl_col_sumqraw = 7,
    tqdc_tbl_col_sumq = 8,
    tqdc_tbl_col_cnt = 9,
    tqdc_tbl_ncols = 10,
};

bool mtdc64_found(const hw_manager &h)
{
    return h.mf_present(MF_TDC) && h.mt_present(MODULE_ID_MTDC64);
}
}

// ---------- Histogram Management ---------------------------------------------
//HistDialogPtr DaqGui::findhistbyname (const QString &name)
//{
//    Q_FOREACH(HistDialogPtr p, hists)
//        if (p && (p->getHistName () == name))
//            return p;
//    return HistDialogPtr();
//}

void DaqGui::donehists ()
{
    hmgr->delete_hists();
}

void DaqGui::rescale_tdc_hists()
{
    // TODO: hmgr->rescale_hists();
}

void DaqGui::rescale_tqdc_hists()
{
    // TODO: hmgr->rescale_hists();
}

namespace {
QString TdcErrorStatsText(const th1Ptr h_tdc_errors)
{
    QString txt("<p><b>TDC Event Statistics</b></p>");
    if (!h_tdc_errors) {
        txt += "no data available";
        return txt;
    }
    txt += "<table>";
    if (h_tdc_errors) {
        const double totalEvents = h_tdc_errors->get_bin(31); // total events
        if (totalEvents>0) {
            QVector<double> sum(3, 0);
            for (unsigned int g=0; g<4; g++)
                for (int i=0; i<sum.size(); i++)
                    sum[i] += h_tdc_errors->get_bin((g<<3)+i);

            QMap<QString, double> m;
            m["Total events"] = totalEvents;
            m["Hits lost (module readout overflow)"] = h_tdc_errors->get_bin(16);
            m["Hits lost (module readout error)"] = h_tdc_errors->get_bin(17);
            m["Hits lost (TDC readout overflow)"] = sum[0];
            m["Hits lost (TDC L1 buffer overflow)"] = sum[1];
            m["Hits lost (hit error detected)"] = sum[2];
            m["Hits rejected (event size limit)"] = h_tdc_errors->get_bin(12);
            m["Event lost (trigger FIFO ovefrlow)"] = h_tdc_errors->get_bin(13);
            m["Internal chip error"] = h_tdc_errors->get_bin(14);

            Q_FOREACH(QString name, m.keys()) {
                qint64 value = static_cast<qint64>(m[name]);
                if (value <= 0) continue;
                txt += "<tr><td>"+name+"</td><td>"+QString::number(value)+"</td><td>"+QString::number(100.*value/totalEvents)+"%</td></tr>";
            }
        }
    }
    txt += "</table>";
    return txt;
}
}

// ---------- Histogram Management - END ---------------------------------------

//******************************************************************************
void DaqGui::release_hw ()
{
    release_hw_preaction();
    hwf->close();
}

// called from initialize_gui(), pbInit_clicked()
void DaqGui::open_hw ()
{
    hwf->open();
    open_hw_postaction ();
}

// called from release_hw()
void DaqGui::release_hw_preaction ()
{
    donehists ();
    clear_tbl_tdc();
    clear_tbl_tqdc();
    clear_tbl_hw();
}

// called from open_hw()
void DaqGui::open_hw_postaction ()
{
    ui->tdcCtrlGui1->setHwf(hwf);
    dataFormatChanged();
    set_tbl_hw();
    set_tbl_tdc();
    set_tbl_tqdc();
    init_plots ();
    set_control_ranges ();
    restore_all ();
    init_hists ();
    init_hist_controls();
    runstate = RS_OFF;
    save_nburst = -1;
    save_ds = ST_OFF;
    burstreset ();
}

void DaqGui::set_control_ranges ()
{
    Q_ASSERT(ui->tabMain);

    ui->tabMain->setTabEnabled(ui->TabTDC, hwf->mf_present(MF_TDC));
    ui->tabMain->setTabEnabled(ui->TabTQDC, hwf->mf_present(MF_TQDC));
    ui->tabMain->setTabEnabled(ui->TabTRIG, hwf->mf_present(MF_TRIG));
    ui->tabMain->setTabEnabled(ui->TabDCH, hwf->mf_present(MF_DCH));
    ui->tabMain->setTabEnabled(ui->TabMSC, hwf->mf_present(MF_MSC));
    if (hwf->mf_present(MF_TDC)) {
        ui->tabMain->insertTab(1, ui->TabTDC, tr("TDC"));
    } else {
        ui->tabMain->removePage(ui->TabTDC);
    }
    if (hwf->mf_present(MF_TQDC)) {
        ui->tabMain->insertTab(1, ui->TabTQDC, tr("TQDC"));
    } else {
        ui->tabMain->removePage(ui->TabTQDC);
    }
    if (hwf->mf_present(MF_TRIG)) {
        ui->tabMain->insertTab(1, ui->TabTRIG, tr("TRIG"));
    } else {
        ui->tabMain->removePage(ui->TabTRIG);
    }
    if (hwf->mf_present(MF_DCH)) {
        ui->tabMain->insertTab(1, ui->TabDCH, tr("DCH"));
    } else {
        ui->tabMain->removePage(ui->TabDCH);
    }
    if (hwf->mf_present(MF_MSC)) {
        ui->tabMain->insertTab(1, ui->TabMSC, tr("MSC"));
    } else {
        ui->tabMain->removePage(ui->TabMSC);
    }
    //  set_control_ranges_tdc ();
    set_control_ranges_tqdc ();
#ifdef MODULE_TRIG
    if (hwf->mf_present(MF_TRIG)) {
        //    sbtsi->setMaxValue (TRIG_SHAPER_MAXVAL);
        //    sbtso->setMaxValue (TRIG_SHAPER_MAXVAL);
    }
#endif
}

void DaqGui::savecfg ()
{
    writeSettings();
    cfg->save();
    timerAutosave->start();
}

void DaqGui::tdcResolutionChanged()
{
    dataFormatChanged();
// TODO: disable unused TDC channels in 25 ps mode
//    if (hwf->mf_present(MF_TDC)) {
//        int tdc_res_index = hwf->get_int("tdc_res", MF_TDC);
//        bool vh = tdc_res_vh(tdc_res_index);
//    }
}

void DaqGui::tdcDataFormatChanged()
{
    dataFormatChanged();
}

void DaqGui::dataFormatChanged()
{
    decoder->dataFormatChanged();
    hmgr->dataFormatChanged();
}

//QString DaqGui::getDbFileName() const
//{
//    QSettings settings;
//    return settings.value("DatabaseFileName",
//                                        QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation)
//                          + QDir::separator() + "vmedaq.sqlite"
//                                        ).toString();
//}

//bool DaqGui::dbOpen(const QString &fileName)
//{
//    db->addDatabase("QSQLITE");
//    db->setDatabaseName(fileName);
//    bool ok = db->open();
//    if (!ok) {
//        qWarning() << "Failed to open database" << fileName;
//        return false;
//    } else {
//        qDebug() << "Opened database" << fileName;
//    }
//    qDebug() << db->tables();
//    if (!db->tables().isEmpty()) {
//        QSettings settings;
//        settings.setValue("DatabaseFileName", fileName);
//    }
//    return true;
//}

DaqGui::DaqGui (QWidget* parent)
    : QMainWindow(parent),
      ui(new Ui::daqguibase),
      shm(new ShmSlaveImpl),
      logview_sptr(0),
//      db(new QSqlDatabase),
      hwf(new hw_frontend_shm()),
      decoder(new base_decoder(*hwf)),
      hmgr(new histmgr(*hwf)),
      eventStatsSum(new EventStatsSum(*hwf)),
      triggersView(new TriggersView(this)),
      cfg(new DaqCfg),
      logReceiver(new QtMessageUdpReceiver(this)),
      timerUpdateDisplay(new QTimer(this)),
      timerHandleEvents(new QTimer(this)),
      timerReplotSpill(new QTimer(this)),
      timerAutosave(new QTimer(this))
{
    ui->setupUi(this);
    cfg->load();
//    dbOpen(getDbFileName());
    connect(ui->tdcCtrlGui1, SIGNAL(resolutionChanged()), SLOT(tdcResolutionChanged()));
    connect(ui->tdcCtrlGui1, SIGNAL(dataFormatChanged()), SLOT(tdcDataFormatChanged()));

    qRegisterMetaType<HistList>();
    connect(hmgr, SIGNAL(histListChanged(HistList)), SLOT(updateHistList(HistList)));
    connect(hmgr, SIGNAL(histAvailable(th1)), SLOT(displayHist(th1)));
    connect(hmgr, SIGNAL(histAvailable(th1)), ui->histWidget, SLOT(setData(th1)));

    connect(this, SIGNAL(requestHist(QString)), hmgr, SLOT(requestHist(QString)));

    init_tbl_hw();
    init_tbl_tdc();
    init_tbl_tqdc();
    init_tbl_trig();
    initialize_gui();
    readSettings();
    connect(GlobalQtMessageHandler::Instance(), SIGNAL(qtMessage(QtMessage)), this, SLOT(qtMessage(QtMessage)));
}

DaqGui::~DaqGui ()
{
    delete cfg;
    delete triggersView;
    hwf->close();
    delete eventStatsSum;
    delete hmgr;
    delete decoder;
    delete hwf;
    delete shm;
    delete ui;
}

void DaqGui::init_tab_tqdc()
{
    ui->cb_tqdc_res->clear();
    ui->cb_tqdc_res->insertItem ("25 ps", 0);   // -1
    ui->cb_tqdc_res->insertItem ("100 ps", 1);  // 0
    ui->cb_tqdc_res->insertItem ("200 ps", 2);  // 1
    ui->cb_tqdc_res->insertItem ("400 ps", 3);  // 2
    ui->cb_tqdc_res->insertItem ("800 ps", 4);  // 3
    ui->cb_tqdc_res->insertItem ("1600 ps", 5); // 4
}

void DaqGui::initialize_gui()
{
    //              set_runparam_defaults();
    ui->actionSave_Calibrations->setEnabled(hmgr->isCalibrationEnabled());
    if (shm->ctl_writeable()) {
        ui->pbInit->setEnabled(true);
        ui->pbReset->setEnabled(true);
        ui->pbPrep->setEnabled(true);
        ui->pbRun->setEnabled(true);
        ui->pbPause->setEnabled(true);
    }
// temporarily disable incomplete functions
    ui->pb_show_this->setEnabled(false);
    ui->pb_hide_all->setEnabled(false);
    ui->groupBox_setRanges->setEnabled(false);
    ui->spinBox_ySens->setEnabled(false);

    ui->pb_startdaq->setEnabled(shm->runparam_writeable());
    ui->actionInitialize_Settings->setEnabled(shm->ctl_writeable() && shm->runparam_writeable());
#if !defined(Q_OS_UNIX)
    ui->actionKill_core->setEnabled(false);
    ui->actionInterrupt_core->setEnabled(false);
#endif
    init_logview();
    qDebug() << "Started Control Panel";

    // TDC setup in TQDC-16
    //    ui->frameTqdcTdcSettings->hide();
    ui->frameTqdcTdcWin->hide();

    ui->frameTqdcAdcSetSrch->hide();
    ui->frameTqdcAdcSetLat->hide();
    ui->checkBoxTqdcExpert->hide();

    init_tab_tqdc();
    QStringList sl;
    sl.push_back("1");
    sl.push_back("2");
    sl.push_back("5");
    sl.push_back("10");
    sl.push_back("20");
    sl.push_back("50");
    sl.push_back("100");
    sl.push_back("200");
    sl.push_back("500");
    sl.push_back("1000");
    ui->cb_hist_downscale->clear();
    ui->cb_hist_downscale->insertStringList(sl);
    //              if (new_hw()) exit(1);

    //              send_command(DAQCTL_CMD_INIT);
    open_hw ();

    connect (timerUpdateDisplay, SIGNAL (timeout ()), SLOT (updatedisplay ()));
    timerUpdateDisplay->setSingleShot(true);
    timerUpdateDisplay->setInterval(timerUpdateDisplayPeriodMs);
    timerUpdateDisplay->start();

    connect (timerHandleEvents, SIGNAL (timeout ()), SLOT (handle_events ()));
    timerHandleEvents->setSingleShot(true);
    timerHandleEvents->setInterval(timerHandleEventsPeriodMs);
    timerHandleEvents->start();

    connect (timerReplotSpill, SIGNAL (timeout ()), SLOT (replot_spr ()));
    timerReplotSpill->setSingleShot(true);
    timerReplotSpill->setInterval(timerReplotSpillPeriodMs);
    timerReplotSpill->start();

    connect (timerAutosave, SIGNAL (timeout ()), SLOT (savecfg()));
    timerAutosave->setSingleShot(true);
    timerAutosave->setInterval(timerAutosavePeriodMs);
    timerAutosave->start();
}

// called from open_hw_postaction()
void DaqGui::init_plots ()
{
    double spr_time = DAQ_SPR_TIME;
    spr_px.resize(DAQ_SPR_LEN);
    spr_py.resize(DAQ_SPR_NUM, std::vector<double>(spr_px.size(), 0.));
    for (int i = 0; i < DAQ_SPR_LEN; i++) {
        spr_px[i] = (spr_time * 1. * i / DAQ_SPR_LEN);
    }
}

void DaqGui::cleardisplay ()
{
    update_current_tab ();
    update_runstate ();
}

void DaqGui::endburst ()
{
    count_module_burst_stats();
    triggersView->end_of_burst();
    hmgr->endburst(*cfg);
}

void DaqGui::burstreset ()
{
    save_nev = 0;
    hmgr->burstreset(*cfg);
    decoder->reset();
    triggersView->burstreset();
    ui->mscWaveformView->clear();

    if (shm->runparamStore()) {
        int code;
        double bytes_total=1, bytes_avail=0;

        code = get_disk_space(shm->getRunparamFilename(), &bytes_total, &bytes_avail);
        if (code==0 && bytes_total!=0) {
            double p_free = bytes_avail/bytes_total;
            QString str = QString("Low disk space: %1 MB free,  %2% used")
                    .arg(bytes_avail/1024./1024., 0, 'f', 1).arg((1.-p_free)*100., 0, 'f', 1);

            if (p_free < 0.01) {
                qCritical() << str;
            } else {
                if (p_free < 0.05) {
                    qWarning() << str;
                }
            }
        }
    }
}

void DaqGui::display_daq_state ()
{
    const daq_state_t ds = shm->getDaqState();
    ui->pbInit->unsetPalette ();
    ui->pbReset->unsetPalette ();
    ui->pbPrep->unsetPalette ();
    ui->pbRun->unsetPalette ();
    ui->pbPause->unsetPalette ();
    ui->tlds->unsetPalette ();
    if (!shm->controlBusy()) {
        switch (ds) {
        case ST_OFF:
            ui->tlds->setText ("OFF");
            ui->tlds->setPaletteBackgroundColor (Qt::gray);
            ui->pbInit->setPaletteBackgroundColor (Qt::green);
            break;
        case ST_INIT:
            ui->tlds->setText ("INIT");
            ui->tlds->setPaletteBackgroundColor (Qt::white);
            ui->pbReset->setPaletteBackgroundColor (Qt::green);
            break;
        case ST_RESET:
            ui->tlds->setText ("RESET");
            ui->tlds->setPaletteBackgroundColor (Qt::cyan);
            ui->pbPrep->setPaletteBackgroundColor (Qt::green);
            break;
        case ST_PREPARERUN:
            ui->tlds->setText ("PREPRUN");
            ui->tlds->setPaletteBackgroundColor (Qt::yellow);
            ui->pbRun->setPaletteBackgroundColor (Qt::green);
            break;
        case ST_RUN:
            ui->tlds->setText ("RUN");
            ui->tlds->setPaletteBackgroundColor (Qt::green);
            ui->pbPause->setPaletteBackgroundColor (Qt::green);
            break;
        case ST_PAUSE:
            ui->tlds->setText ("PAUSE");
            ui->tlds->setPaletteBackgroundColor (Qt::yellow);
            ui->pbRun->setPaletteBackgroundColor (Qt::green);
            break;
        case ST_FAIL:
            ui->tlds->setText ("FAILURE");
            ui->tlds->setPaletteBackgroundColor (Qt::red);
            ui->pbInit->setPaletteBackgroundColor (Qt::green);
            break;
        default:
            ui->tlds->setText (QString("%1").arg(ds));
            ui->tlds->setPaletteBackgroundColor (Qt::red);
        }
        ui->pbInit->setEnabled (RunStateTransitionRules::isAllowed(ds, ST_INIT));
        ui->pbReset->setEnabled (RunStateTransitionRules::isAllowed(ds, ST_RESET));
        ui->pbPrep->setEnabled (RunStateTransitionRules::isAllowed(ds, ST_PREPARERUN));
        ui->pbRun->setEnabled (RunStateTransitionRules::isAllowed(ds, ST_RUN));
        ui->pbPause->setEnabled (RunStateTransitionRules::isAllowed(ds, ST_PAUSE));
    } else {
        ui->pbInit->setEnabled (false);
        ui->pbReset->setEnabled (false);
        ui->pbPrep->setEnabled (false);
        ui->pbRun->setEnabled (false);
        ui->pbPause->setEnabled (false);
        ui->tlds->setPaletteForegroundColor (Qt::white);
        ui->tlds->setPaletteBackgroundColor (Qt::blue);
        ctl_magic_t currentControlState = shm->getCurrentControlState();
        switch (currentControlState) {
        case DAQCTL_CMD_INIT:
            ui->tlds->setText (">INIT");
            break;
        case DAQCTL_CMD_RESET:
            ui->tlds->setText (">RESET");
            break;
        case DAQCTL_CMD_PREPARERUN:
            ui->tlds->setText (">PREPRUN");
            break;
        case DAQCTL_CMD_RUN:
            ui->tlds->setText (">RUN");
            break;
        case DAQCTL_CMD_PAUSE:
            ui->tlds->setText (">PAUSE");
            break;
        default:
            ui->tlds->setText (QString(">%1").arg(currentControlState));
            ui->tlds->setPaletteBackgroundColor (Qt::red);
        }
    }
}

namespace {
void update_decoder_speed_info(int total, int complete)
{
    double progress=0;
    if (total>0)
        progress=1.*complete/total;
    decoder_progress = progress;
}
}

void DaqGui::handle_events ()
{
    try {
        bool decoder_enabled = cfg->get_dec_en();
        const daq_state_t ds = shm->getDaqState();
        const burst_state_t bs = shm->getBurstState();
        //        const int spill_count = shm->getSpillCount();

        if (save_bs != bs) {
            if (!save_bs && bs) {
                qDebug() << "Spill started";
            }
            if (save_bs && !bs) {
                qDebug() << "Spill finished";
                endburst();
            }
            save_bs = bs;
        }
        int nev=0;

        if (shm->shm_evlist) {
            nev = shm->shm_evlist->nev;
            //            log_debug(5, "handle_events: ds:%s, bs:%s, nev:%d, save_nev:%d, spill_count:%d, save_nburst:%d",
            //                      daq_state_str(ds), burst_state_str(bs), nev, save_nev, spill_count, save_nburst);
            update_decoder_speed_info(nev, save_nev);
        }
        if (save_ds != ds) {
            //    display_daq_state();
            //            log_debug(1, "changed state from %d to %d", save_ds, ds);
            if ((save_ds != ST_RUN) && (save_ds != ST_PAUSE)) {
                //                log_debug(1, "state not RUN or PAUSE, reset");
                burstreset ();
                cleardisplay ();
                save_ds = ds;
                goto end;
            }
            save_ds = ds;
        }
        //        log_debug(5, "ds=%s, bs=%d", daq_state_str (ds), bs);
        if ((ds == ST_RUN)) {
            int spill_count = shm->getSpillCount();
            //            log_debug(5, "handle_events: nburst=%d, save_nburst:%d", spill_count, save_nburst);
            if (save_nburst != spill_count) {
                //                log_debug(2, "spill_count changed from %d to %d", save_nburst, spill_count);
                burstreset ();
                save_nburst = spill_count;
            }
            if (shm->shm_evlist) {
                //                log_debug(5, "handle_events: nev:%d, save_nev:%d", nev, save_nev);
                if (nev != save_nev) {
                    if (decoder_enabled) {
                        int code = getevents (save_nev, nev);
                        if (code > 0) {
                            save_nev = code;
                        } else { // error while decoding. skip.
                            if (code == -2) {// burst expected break
                            }
                            save_nev = nev;
                        }
                    }
                }
            } else {
                qWarning() << "handle_events: shm_evlist is NULL";
            }
        }
    }
    catch (exc & e) {
        qWarning() << e.what();
    }
    end:
    timerHandleEvents->start();
}

void DaqGui::update_resourcemon()
{
    int cpu_user = sysmon.get("cpu_user").toInt();
    int cpu_nice = sysmon.get("cpu_nice").toInt();
    int cpu_sys = sysmon.get("cpu_sys").toInt();
    int cpu_idle = sysmon.get("cpu_idle").toInt();
    //  int cpu_iowait = sysmon.get("cpu_iowait").toInt();
    //  int cpu_irq = sysmon.get("cpu_irq").toInt();
    //  int cpu_softirq = sysmon.get("cpu_softirq").toInt();

    int busy = cpu_nice+cpu_user+cpu_sys;
    int all = busy+cpu_idle;
    double value = 0;
    int total = 100;
    if (all>0) {
        value = total * 1. * busy / all;
    }
    ui->progressBar_cpu_load->setAlignment(Qt::AlignHCenter);
    ui->progressBar_cpu_load->setMaximum(total);
    ui->progressBar_cpu_load->setValue(static_cast<int>(value));

    {
        double bytes_total=1, bytes_avail=0;
        QString fn = shm->getRunparamFilename();
        bool ok = get_disk_space(fn, &bytes_total, &bytes_avail);
        if (ok && bytes_total>0)
            value = total * (bytes_total-bytes_avail)/bytes_total;
        else
            value=0;

        bool enable = shm->runparamStore();
        ui->textLabel_disk_ev->setEnabled(enable);
        ui->progressBar_disk_ev->setEnabled(enable);
        ui->progressBar_disk_ev->setAlignment(Qt::AlignHCenter);
        ui->progressBar_disk_ev->setMaximum(total);
        ui->progressBar_disk_ev->setValue(static_cast<int>(value));
    }
    ui->textLabel_disk_ev->setText("Disk");

    ui->progressBarDecoderProgress->setAlignment(Qt::AlignHCenter);
    ui->progressBarDecoderProgress->setMaximum(100);
    ui->progressBarDecoderProgress->setValue(static_cast<int>(100.*decoder_progress));
}

void DaqGui::updatedisplay ()
{
    try {
        display_daq_state ();
        restore_runparam ();
        update_current_tab ();
        update_runstate ();
        update_resourcemon();
        triggersView->update();
    }
    catch (exc & e) {
        qWarning() << e.what();
    }
    timerUpdateDisplay->start();
}

void DaqGui::update_current_tab ()
{
    QWidget *w = ui->tabMain->currentPage ();
//    if (w == ui->TabMSC)
        replot_msc ();
    if (w == ui->TabTDC)
        replot_tdc ();
    if (w == ui->TabTQDC)
        replot_tqdc ();
    if (w == ui->TabTRIG)
        display_refresh_trig_counts ();
    if (w == ui->TabDCH)
    { }
    if (w == ui->TabRun)
        replot_runstats ();
    if (w == ui->TabModules)
        display_refresh_tbl_hw();
    if (w == ui->TabHistograms)
        requestCurrentHist();
    if (w == ui->TabRawData)
        update_TabRawData ();
    if (w == ui->TabStatistics)
        update_TabStatistics ();
}

namespace {
void dump_evlist_statistics(const shm_evlist_t &el)
{
    int n_put = 0, n_got = 0, n_done = 0;
    int nev = el.nev;
    for (int i=0; i<nev; i++) {
        if (el.ev_flags[i] & 0x1)
            n_put++;
        if (el.ev_flags[i] & 0x2)
            n_got++;
        if (el.ev_flags[i] & 0x4)
            n_done++;
    }

    qDebug() << "*** Event List ***";
    qDebug() << "  nev:  " << nev;
    qDebug() << "  put:  " << n_put;
    qDebug() << "  got:  " << n_got;
    qDebug() << "  done: " << n_done;
}
}

// return last processed event number: [evfirst,evlast)
int DaqGui::getevents (int evfirst, int evlast)
{
    int ne=0;
    Q_ASSERT(shm->shm_evlist);
    //    if (debug_level_ge(2));
    //    dump_evlist_statistics(*shm->shm_evlist);
    if (evfirst > evlast)
        return 0;
    Q_ASSERT(evfirst <= evlast);
    //    qDebug() << QString("Got events %1 - %2").arg(evfirst).arg(evlast);
    //    log_debug(2, "First event address: 0x%08X-0x%08X", shm->shm_evlist->start[evfirst], shm->shm_evlist->end[evfirst]);
    //    log_debug(2, "Last  event address: 0x%08X-0x%08X", shm->shm_evlist->start[evlast - 1], shm->shm_evlist->end[evlast - 1]);
    struct timeval tv_start;
    gettimeofday(&tv_start, NULL);
    for (ne = evfirst; ne < evlast; ne++) {
        // FIXME
        struct timeval tv;
        gettimeofday(&tv, NULL);
        double elapsed = tv_dt(tv_start, tv);
        if (elapsed > 0.02) {// timeout
            //            qDebug() << QString("getevents: interrupting decode, timeout (%1% done)").arg(100.*ne/evlast);
            break;
        }
        int start = shm->shm_evlist->start[ne];
        int end = shm->shm_evlist->end[ne];
        unsigned int ev_flags = shm->shm_evlist->ev_flags[ne] >> 8;
        if (end < start) {
            qDebug() << QString("wrapped event %1 skipped").arg(ne);
            ne++;
            continue;
        }

        unsigned int *pstart = 0;
        unsigned int *pend = 0;
        pstart = (unsigned int *) &shm->shm_pio_rawdata->data[start];
        pend = (unsigned int *) &shm->shm_pio_rawdata->data[end];
        int evgood = 1;
        // warning! try-catch in a loop. Must not be run more than once - used 'break' later.
        try {
            decoder->decode_buf (pstart, end-start+1, ev_flags);
        }
        catch (exc_decoder &e) {
            qWarning() << e.what();
            evgood = 0;
        }
        if (evgood) {
            process_decoded_event ();
        } else {
            qDebug() << QString("event decode failed. Events %1 - %2").arg(evfirst).arg(evlast);
            //            log_debug(1, "event address: 0x%08lX-0x%08lX", (unsigned long) pstart, (unsigned long) pend);
            //            log_debug(1, "event data: %08lX .. %08lX", *pstart, *(pend - 1));
            break; // this is important. break loop and exit if exception caught.
        }
    }
    return ne;
}

void DaqGui::replot_tqdc_waveform ()
{
    ui->tqdcwave->add_event(decoder->get_event());
    ui->tqdcwave->update ();
}

void DaqGui::replot_tdc ()
{
    if (!hwf->mf_present(MF_TDC))
        return;

    display_refresh_tbl_tdc();
}

void DaqGui::replot_msc ()
{
    if (!hwf->mf_present(MF_MSC))
        return;

    ui->mscWaveformView->update();
}

void DaqGui::replot_spr ()
{
    const daq_spr_t &spr = shm->getSpr();
    double spr_time = spr.time;
    size_t spr_end = spr.end;
    spr_end++;
    if (spr_end > spr_len())
        spr_end = spr_len();
    for (unsigned int i = 0; i < spr_len(); i++) {
        spr_px[i] = (spr_time * 1. * i / spr_len());
    }
    for (unsigned int c = 0; c < DataPlot_NCH; c++) {
        if (c >= spr_num()) {
            ui->dp_spill->clearpoints (c);
        } else {
            double timebin = spr_time / spr_len();
            for (unsigned int i = 0; i < spr_len(); i++) {
                spr_py[c][i] = spr.v[c][i] / timebin;
            }
            ui->dp_spill->copypoints (c, &spr_px[0], &spr_py[c][0], spr_end);
        }
    }
    if (!spr_m1) {
        spr_m1 = new QwtPlotMarker();
        spr_m1->setLineStyle(QwtPlotMarker::VLine);
        spr_m1->setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
        spr_m1->setLinePen(QPen(Qt::green, 0, Qt::DashDotLine));
        spr_m1->attach(ui->dp_spill);
    }
    struct timeval tv_now;
    gettimeofday(&tv_now, NULL);
    spr_m1->setXValue(shm->getBurstTime());

    {
        int binmin = 0, binmax = spr_len(), i;
        for (unsigned i = 0; i < spr_len(); i++)
            if (spr_py[0][i] > 0) {
                binmin = i;
                break;
            }
        binmax = binmin + 1;
        for (i = spr_len() - 1; i > binmin; i--)
            if (spr_py[0][i] > 0) {
                binmax = i;
                break;
            }
        double amin, amax;
        amin = spr_px[binmin];
        amax = spr_px[binmax];
        ui->dp_spill->setAxisScale (QwtPlot::xBottom, amin, amax, 0);
    }
    ui->dp_spill->replot ();
    timerReplotSpill->start();
}

void DaqGui::replot_runstats ()
{
    //    // Spill History - Event Size
    //    const double spses_max = DAQ_SPSES_NBINS * DAQ_SPSES_BINWID;

    //    if (!shm->shm_status)
    //        return;

    //    std::vector<double> spses_px(DAQ_SPSES_NBINS, 0.);
    //    for (size_t i = 0; i < spses_px.size(); i++) {
    //        spses_px[i] = (spses_max * 1. * i / spses_px.size());
    //    }

    //    std::vector<double> spses_py(spses_px.size(), 0.);
    //    double binwid = 1. * spses_max / spses_px.size();
    //    for (size_t i = 0; i < spses_py.size(); i++) {
    //        spses_py[i] = shm->shm_status->spses[i] / binwid;
    //    }
    //    ui->dp_spillhist->setpoints (spses_px, spses_py);

    //    {
    //        int binmin = 0, binmax = spses_py.size();
    //        for (size_t i = 0; i < spses_py.size(); i++)
    //            if (spses_py[i] > 0) {
    //            binmin = i;
    //            break;
    //        }
    //        binmax = binmin + 1;
    //        for (int i = static_cast<int>(spses_py.size()) - 1; i > binmin; i--)
    //            if (spses_py[i] > 0) {
    //            binmax = i;
    //            break;
    //        }
    //        double amin, amax;
    //        amin = spses_px[binmin];
    //        amax = spses_px[binmax];

    //        ui->dp_spillhist->setAxisScale (QwtPlot::xBottom, amin, amax, 0);
    //        ui->dp_spillhist->replot ();
    //        ui->dp_spillhist->setAxisScale (QwtPlot::xBottom, amin, amax, 0);
    //    }
}

void DaqGui::restore_all ()
{
    restore_runparam ();
    restore_main_cfg ();
    restore_tqdc_cfg ();
    setup_tqdc_select();
    restore_trig_cfg ();
    restore_dch_cfg ();
    restore_msc_cfg ();
}

void DaqGui::restore_main_cfg ()
{
    ui->groupBoxDecoder->setChecked (cfg->get_dec_en());
    updateHistogramControls();

    int min_index = 0;
    for (int i=0;i < ui->cb_hist_downscale->count();i++) {
        if (ui->cb_hist_downscale->text(i).toInt() <= cfg->get_hist_downscale())
            min_index = i;
        ui->cb_hist_downscale->setCurrentItem (min_index);
    }
    ui->checkBox_clearByBurst->setChecked(cfg->get_hist_clear_by_burst());
}

int DaqGui::send_command (ctl_magic_t cmd)
{
    shm->sendControlCommand(cmd);

    int n_timeout = 20000;
    Q3ProgressDialog pg(tr("Waiting for command to complete..."), tr("Cancel"), n_timeout, this, "progress", true);
    int n_elapsed = 0;
    while (true) {
        ctl_magic_t state = shm->getCurrentControlState();
        if (state == DAQCTL_CMD_ACK) {
            pg.cancel();
            return 0; // ok
        }
        display_daq_state ();
        if (n_elapsed > 1000)
            pg.setProgress(n_elapsed);
        if (pg.wasCanceled()) return -1;
        qApp->processEvents(QEventLoop::AllEvents, 10);
        if (n_elapsed > n_timeout) {
            pg.cancel();
            bool restart_daemon = QMessageBox::warning (this, "Timeout",
                                                        QString("DAQ daemon is not responding. Kill?"),
                                                        QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes;
            if (restart_daemon)
                on_actionKill_core_triggered();
            return -1;
        }
        n_elapsed += 10;
    }
    return 0;
}

void DaqGui::on_pbInit_clicked ()
{
    if (!mayTerminateRun()) return;
    switch (QMessageBox::warning (this,
                                  "Initialize",
                                  "Do you want to delete any unsaved data, rescan hardware and load default setup?",
                                  "&Yes", "&No", 0, 1)) {
    case 0:
        break;
    default:
        return;
    }

    reset_logview();

    shm->setRunparam(runparam_t());
    restore_runparam ();
    release_hw ();
    send_command (DAQCTL_CMD_INIT);
    clearstats ();
    open_hw ();
}

void DaqGui::on_pbReset_clicked ()
{
    if (!mayTerminateRun()) return;
    send_command (DAQCTL_CMD_RESET);
    clearstats ();
}

void DaqGui::on_pbPrep_clicked ()
{
    if (!mayTerminateRun()) return;
    send_command (DAQCTL_CMD_PREPARERUN);
}

void DaqGui::on_pbRun_clicked ()
{
    send_command (DAQCTL_CMD_RUN);
}

void DaqGui::on_pbPause_clicked ()
{
    send_command (DAQCTL_CMD_PAUSE);
}

void DaqGui::process_decoded_event ()
{
    const event_t &ev = decoder->get_event();

    if (cfg->get_hist_en())
        hmgr->fill(ev, *cfg);

    triggersView->addEvent(ev);
    ui->mscWaveformView->add_event(ev);
    count_module_event_stats(ev);
}

//! Perform full initialization for statistical data. Clear any temporary state.
void DaqGui::clearstats ()
{
    ui->tqdcwave->clear();
    ui->mscWaveformView->clear();
    triggersView->clear();
    hmgr->reset();
    ev_stats.clear();
    ev_stats_save.clear();
    mod_b_stats.clear();
    mod_b_stats_save.clear();
    eventStatsSum->clear();
    ui->textBrowserRawData->setText(QString());
    clear_tbl_tqdc();
    update_current_tab ();
}

bool DaqGui::mayTerminateRun()
{
    bool storageActive = shm->runparamStore() && (shm->getStorageBytes() > 0);
    if (storageActive) {
        QMessageBox::StandardButton code = QMessageBox::question(this,
                              tr("Stop"),
                              tr("Stop current run?"),
                              QMessageBox::Yes|QMessageBox::No,
                              QMessageBox::No);
        if (code == QMessageBox::Yes) {
            runparam_t rp = shm->getRunparam();
            rp.store = false;
            shm->setRunparam(rp);
            restore_runparam();
            return true;
        } else {
            return false;
        }
/*
        QMessageBox::StandardButton code = QMessageBox::question(this,
                              tr("New Data Storage"),
                              tr("Assign new data file?"),
                              QMessageBox::Yes|QMessageBox::No|QMessageBox::Cancel,
                              QMessageBox::Cancel);
        if (code == QMessageBox::Cancel) return false;
        if (code == QMessageBox::Yes) {
            return setDataStorage();
        }
*/
    }
    return true;
}

bool DaqGui::setDataStorage()
{
    StartDAQ dlg(this);
    dlg.setdata(shm->getRunparam());
    if (dlg.exec() == QDialog::Accepted) {
        shm->setRunparam(dlg.getdata());
        restore_runparam();
        return true;
    }
    return false;
}

void DaqGui::on_pb_startdaq_clicked()
{
    if (!shm->runparam_writeable()) return;
    if (setDataStorage()) send_command (DAQCTL_CMD_PREPARERUN);
}

void DaqGui::set_runparam_defaults ()
{
    shm->setRunparam(runparam_t());
}

void DaqGui::restore_runparam ()
{
    const bool store = shm->runparamStore();
    double nbw = shm->getStorageBytes() * 1. / 1024 / 1024;
    if (store) {
        QString comment;
        if (!shm->getRunparamComment().isEmpty())
            comment = tr("Comment: %1").arg(shm->getRunparamComment());
        ui->tl_rundesc->setText(comment);
        QString tmp = tr("File: %1 MB %2 %3").arg(nbw, 0, 'f', 1).arg(shm->getRunparamFilename()).arg(shm->runparamCyclic() ? " (RECYCLE)" : " (STORAGE)");
        ui->tl_file->setText(tmp);
    } else {
        ui->tl_file->clear();
        ui->tl_rundesc->clear();
    }

    //    ui->tl_time->setEnabled(store);
    //    ui->tl_rundesc->setEnabled(store);
    //    ui->tl_diskspace->setEnabled(store);

    if (store) {
        //    gettimeofday(&shm->runparam()->tstart, NULL); ???
        QDateTime startTime = shm->getRunStartTime();
        if (!startTime.isNull())
            ui->tl_time->setText(tr("Time: ") +startTime.toString("dd MMM yyyy hh:mm:ss"));
        else
            ui->tl_time->clear();
    } else {
        ui->tl_time->clear();
    }

    if (store) {
        double bytes_avail = 0, bytes_total=1, p_free=0;
        bool ok = get_disk_space(shm->getRunparamFilename(), &bytes_total, &bytes_avail);
        if (ok && bytes_total>0) {
            p_free = bytes_avail/bytes_total;
        }
        QString tmp = QString("Disk: %1 MB free, %2% used").arg(bytes_avail/1024./1024., 0, 'f', 1).arg((1.-p_free)*100., 0, 'f', 1);
        if (p_free < 0.01) {
            ui->tl_diskspace->setPaletteForegroundColor(Qt::red);
        } else {
            if (p_free < 0.05) {
                ui->tl_diskspace->setPaletteForegroundColor(Qt::red);
            } else {
                ui->tl_diskspace->unsetPalette();
            }
        }
        ui->tl_diskspace->setText (tmp);
    } else {
        ui->tl_diskspace->unsetPalette();
        ui->tl_diskspace->clear();
    }

    if ((save_ds == ST_PREPARERUN) || (save_ds == ST_PAUSE)) {
        ui->pb_startdaq->setEnabled (shm->runparam_writeable());
    } else {
        ui->pb_startdaq->setEnabled (false);
    }
}

namespace {
QString timeIntervalStr (double seconds)
{
    QTime qtime;
    qtime = qtime.addMSecs(seconds*1e3);
    if (qtime < QTime(0,1)) // less than 1 minute
        return qtime.toString("s.zzz");
    if (qtime < QTime(1,0)) // less than 1 hour
        return qtime.toString("h:mm:ss");
    return qtime.toString("h:mm:ss");
}
}

void DaqGui::update_runstate ()
{
    const runcounts_t &rc = shm->getRunCounts();
    const runcounts_t &bc = shm->getBurstCounts();

    //    double btime = 0;
    //    btime = shm->getLastBurstDuration();
    //    ui->tl_rsn->setText(QString("%1\n%2\n%3\n%4\n%5\n%6\n%7").arg(shm->getSpillCount()).arg(btime).arg(twait).arg(rc.burst).arg(rc.event).arg(rc.xoff).arg(rc.byte / 1024 / 1024));
    //    ui->tl_rsh->setText ("Burst\nBtime\nWtime\nBursts\nEvents\nXOFFs\nMBytes");

    QString text;
    text.append ("<b>Run Statistics</b><br>");
    text.append (QString("Spill count: %1<br>").arg(shm->getSpillCount()));

    QString wallTimeRunStr = timeIntervalStr(rc.walltime);
    QString daqTimeRunStr = timeIntervalStr(rc.daqtime);
    QString wallTimeBurstStr = timeIntervalStr(bc.walltime);
    QString daqTimeBurstStr = timeIntervalStr(bc.daqtime);

    text.append ("<table><tr><td>Name</td><td>Spill counts</td><td>Run counts</td></tr>");
    text.append (QString("<tr><td>WallTime</td><td>%1</td><td>%2</td></tr>").arg(wallTimeBurstStr).arg(wallTimeRunStr));
    text.append (QString("<tr><td>DaqTime</td><td>%1</td><td>%2</td></tr>").arg(daqTimeBurstStr).arg(daqTimeRunStr));
    text.append (QString("<tr><td>Event</td><td>%1</td><td>%2</td></tr>").arg(bc.event).arg(rc.event));
    text.append (QString("<tr><td>XOFF</td><td>%1</td><td>%2</td></tr>").arg(bc.xoff).arg(rc.xoff));
    text.append (QString("<tr><td>MByte</td><td>%1</td><td>%2</td></tr>")
                 .arg(bc.byte / 1024. / 1024., 0, 'g', 3)
                 .arg(rc.byte / 1024. / 1024., 0, 'g', 3));
    text.append ("</table>");
    ui->tbrs->setText (text);
}

/*******************************************************************************/
/* Main window slots */
/*******************************************************************************/

void DaqGui::on_tabMain_currentChanged (QWidget *)
{
    update_current_tab ();
}

void DaqGui::on_groupBoxDecoder_toggled (bool newstate)
{
    if (disable_cfg_slots)
        return;
    cfg->set_dec_en(newstate);
    updateHistogramControls();
}

void DaqGui::updateHistogramControls()
{
    bool enabled = cfg->get_dec_en();
    bool checked = cfg->get_hist_en();
    ui->groupBoxHistograms->setEnabled(enabled);
    ui->groupBoxHistograms->setChecked(checked);
    //    ui->cb_hist_downscale->setEnabled(enabled);
    //    ui->checkBox_clearByBurst->setEnabled(enabled);
    ui->groupBoxRawDataStorage->setEnabled(enabled);
}

void DaqGui::replot_tbl_hw ()
{
    display_refresh_tbl_hw();
}

void DaqGui::display_refresh_tbl_hw ()
{
    set_tbl_hw();
}

namespace {
enum tbl_hw_col {
    tbl_hw_col_type = 0,
    tbl_hw_col_fam = 1,
    //  tbl_hw_col_base = 2,
    tbl_hw_col_fw = 2,
    tbl_hw_col_serial = 3,
    tbl_hw_col_ga = 4,
    tbl_hw_col_f = 5,
    tbl_hw_col_temp = 6,
    tbl_hw_col_ch = 7,
    tbl_hw_col_dev = 8,
    tbl_hw_col_cblt = 9,
    tbl_hw_col_stat_total = 10,
    tbl_hw_col_stat_warn = 11,
    tbl_hw_col_stat_over = 12,
    tbl_hw_col_stat_err = 13,
    tbl_hw_col_stat_fatal = 14,
    tbl_hw_col_key = 15,
    tbl_hw_ncol = 16,
};
}

void DaqGui::init_tbl_hw()
{
    Q3Table *t = ui->tbl_hw;

    t->setNumCols (tbl_hw_ncol);
    t->setNumRows (0);
    Q3Header *hh = t->horizontalHeader ();

    hh->setLabel (tbl_hw_col_type, "Name");
    hh->setLabel (tbl_hw_col_fam, "Class");
    hh->setLabel (tbl_hw_col_ga, "Slot");
    //  hh->setLabel (tbl_hw_col_base, "AM:Base");
    hh->setLabel (tbl_hw_col_fw, "Firmware");
    hh->setLabel (tbl_hw_col_serial, "S/N");
    hh->setLabel (tbl_hw_col_f, "Status");
    hh->setLabel (tbl_hw_col_temp, "t \u00b0C");
    hh->setLabel (tbl_hw_col_ch, "# ch");
    hh->setLabel (tbl_hw_col_dev, "# dev");
    hh->setLabel (tbl_hw_col_cblt, "Read");
    hh->setLabel (tbl_hw_col_key, "DDB key");

    hh->setLabel (tbl_hw_col_stat_total, "OK Spills");
    hh->setLabel (tbl_hw_col_stat_over, "Overflows");
    hh->setLabel (tbl_hw_col_stat_warn, "Warnings");
    hh->setLabel (tbl_hw_col_stat_err, "Errors");
    hh->setLabel (tbl_hw_col_stat_fatal, "Fails");
    //  t->hideColumn(tbl_hw_col_dev);
    t->hideColumn(tbl_hw_col_dev);
    t->hideColumn(tbl_hw_col_key);
    for (int n=0; n<t->numCols(); n++)
        if (n!=tbl_hw_col_cblt)
            t->setColumnReadOnly(n, true);
}

void DaqGui::count_module_event_stats(const event_t &ev)
{
    ev_stats["total"]++;
    if (ev.tdc.error & 0x10000) ev_stats["over"]++;
    if (ev.tqdc.tdc_error & 0x10000) ev_stats["over"]++;
    eventStatsSum->add(ev.eventStats);
}

void DaqGui::count_module_burst_stats()
{
    mod_b_stats_save = mod_b_stats;
    const modmap_t mods = hwf->get_modmap();
    Q_FOREACH(int key, mods.keys()) {
        const base_mod *mod = mods[key];
        if (!mod) continue;
        const mod_stats_t &s = mod->get_stats();
        mod_b_stats[key]["total"] = s.total;
        mod_b_stats[key]["fail"] = s.fatal;
        mod_b_stats[key]["err"] = s.err;
        mod_b_stats[key]["warn"] = s.warn;
        mod_b_stats[key]["over"] = s.over;
    }
}

void DaqGui::set_tbl_hw()
{
    Q3Table *t = ui->tbl_hw;
    int nmod_old = t->numRows();
    int nmod = 0;
    int create = 0;

    // find all modules
    const modmap_t mods = hwf->get_modmap();
    nmod = mods.size();

    if (nmod_old != nmod) {
        qDebug() << QString("set_tbl_hw: nmod changed %1 to %2").arg(nmod_old).arg(nmod);
        clear_tbl_hw();
        create=1;
        t->setNumRows (nmod);
    }

    if (create) {
        // set labels
        Q3Header *vh = t->verticalHeader ();
        for (int i = 0; i < nmod; i++) {
            vh->setLabel (i, QString::number(i));
            // insert checkboxes
            t->setItem (i, tbl_hw_col_cblt, new Q3CheckTableItem (t, ""));
            // custom text items
            t->setItem (i, tbl_hw_col_stat_total, new QMyTableItem (t));
            t->setItem (i, tbl_hw_col_stat_over, new QMyTableItem (t));
            t->setItem (i, tbl_hw_col_stat_warn, new QMyTableItem (t));
            t->setItem (i, tbl_hw_col_stat_err, new QMyTableItem (t));
            t->setItem (i, tbl_hw_col_stat_fatal, new QMyTableItem (t));
            t->setItem (i, tbl_hw_col_f, new QMyTableItem (t));
        }
    }

    Q_FOREACH(int key, mods.keys()) {
        const base_mod *mod = mods[key];

        //    for (int c = 1; c < tbl_hw_ncol; c++) t->clearCell (id, c);
        if (!mod) continue;
        t->setText (key, tbl_hw_col_type, mod->get_name ());
        t->setText (key, tbl_hw_col_fam, mod->get_fam_name ());
        double temp = mod->get_temperature ();
        if (temp > 0)
            t->setText (key, tbl_hw_col_temp, QString::number(temp, 'f', 1));
        else
            t->setText (key, tbl_hw_col_temp, "");
        int slot = mod->get_slot();
        int nch = mod->get_nch();
        t->setText (key, tbl_hw_col_ga, QString::number(slot));
        t->setText (key, tbl_hw_col_fw, mod->getFirmwareStr());
        t->setText (key, tbl_hw_col_serial, QString("%1").arg(mod->get_serial(), 6, 16));
        //    t->setText (key, tbl_hw_col_base, s.sprintf("%08X", mod->get_baseaddr()));
        t->setText (key, tbl_hw_col_ch, QString::number(nch));
        t->setText (key, tbl_hw_col_key, QString("%1").arg(mod->get_ddb_key(), 8, 16));
        t->setText (key, tbl_hw_col_dev, QString::number(mod->get_ndev()));
        t->setText (key, tbl_hw_col_stat_fatal, QString::number(mod_b_stats[key]["fatal"]));
        t->setText (key, tbl_hw_col_stat_err, QString::number(mod_b_stats[key]["err"]));
        t->setText (key, tbl_hw_col_stat_warn, QString::number(mod_b_stats[key]["warn"]));
        t->setText (key, tbl_hw_col_stat_over, QString::number(mod_b_stats[key]["over"]));
        t->setText (key, tbl_hw_col_stat_total, QString::number(mod_b_stats[key]["total"]));
        Q3CheckTableItem *cb;
        cb = dynamic_cast<Q3CheckTableItem *>(t->item(key, tbl_hw_col_cblt));
        if (cb) {
            bool cb_state = cb->isChecked();
            bool new_state = mod->get_cblt_enable();
            if (cb_state != new_state) {
                //    	disable_cfg_slots++;
                cb->setChecked (new_state);
                //    	disable_cfg_slots--;
            }
            if (mtdc64_found(*hwf)) {
                cb->setEnabled(false);
            } else {
                cb->setEnabled(true);
            }
        }
        // flags are special
        QStringList fs;
        const base_mod_flags f = mod->get_flags();

        //        if (f & STDMOD_FLAG_DISABLE) fs += "Disable";
        if (f & STDMOD_FLAG_UNINITIALIZED) fs += "NO_SETUP";
        if (f & STDMOD_FLAG_TESTOK) fs += "TEST_OK";
        if (f & STDMOD_FLAG_OVER) fs += "OVF";
        if (f & STDMOD_FLAG_FAIL) fs += "FAIL";
        else if (f & STDMOD_FLAG_ERR) fs += "ERR";
        else if (f & STDMOD_FLAG_WARN) fs += "WRN";

        bool f_ok = (f&(STDMOD_FLAG_UNINITIALIZED|STDMOD_FLAG_OVER|STDMOD_FLAG_FAIL|STDMOD_FLAG_ERR|STDMOD_FLAG_WARN)) == 0;
        if (f_ok) {
            if (f & STDMOD_FLAG_CBLT) fs += "Readout Ok";
        }

        t->setText (key, tbl_hw_col_f, fs.join(" "));

        // set event statistics items with color
        std::map<std::string, bool> bm;
        if (mod_b_stats[key]["fatal"] > mod_b_stats_save[key]["fatal"]) bm["fatal"] = true;
        if (mod_b_stats[key]["err"] > mod_b_stats_save[key]["err"]) bm["err"] = true;
        if (mod_b_stats[key]["warn"] > mod_b_stats_save[key]["warn"]) bm["warn"] = true;
        if (mod_b_stats[key]["over"] > mod_b_stats_save[key]["over"]) bm["over"] = true;
        if (mod_b_stats[key]["total"] > mod_b_stats_save[key]["total"]) bm["total"] = true;

        QColor clr;
        clr = Qt::white;
        //	if (bm["total"]) clr = Qt::green;
        {
            QMyTableItem *ti = dynamic_cast<QMyTableItem*>(ui->tbl_hw->item(key, tbl_hw_col_stat_total));
            if (ti)
                ti->setBackgroundColor(clr);
        }
        clr = Qt::white;
        if (bm["over"]) clr = Qt::yellow;
        {
            QMyTableItem *ti = dynamic_cast<QMyTableItem*>(ui->tbl_hw->item(key, tbl_hw_col_stat_over));
            if (ti)
                ti->setBackgroundColor(clr);
        }
        clr = Qt::white;
        if (bm["warn"]) clr = Qt::yellow;
        {
            QMyTableItem *ti = dynamic_cast<QMyTableItem*>(ui->tbl_hw->item(key, tbl_hw_col_stat_warn));
            if (ti)
                ti->setBackgroundColor(clr);
        }
        clr = Qt::white;
        if (bm["err"]) clr = Qt::red;
        {
            QMyTableItem *ti = dynamic_cast<QMyTableItem*>(ui->tbl_hw->item(key, tbl_hw_col_stat_err));
            if (ti)
                ti->setBackgroundColor(clr);
        }
        clr = Qt::white;
        if (bm["fatal"]) clr = Qt::red;
        {
            QMyTableItem *ti = dynamic_cast<QMyTableItem*>(ui->tbl_hw->item(key, tbl_hw_col_stat_fatal));
            if (ti)
                ti->setBackgroundColor(clr);
        }

        // Flags with color
        clr = Qt::white;
        if (f & STDMOD_FLAG_FAIL) clr = Qt::magenta;
        else if (f & STDMOD_FLAG_ERR) clr = Qt::red;
        else if (f & (STDMOD_FLAG_WARN|STDMOD_FLAG_OVER)) clr = Qt::yellow;

        Q3TableItem *item = ui->tbl_hw->item(key, tbl_hw_col_f);
        QMyTableItem *ti = dynamic_cast<QMyTableItem*> (item);
        if (ti)
            ti->setBackgroundColor(clr);
    }

    for (int n=0; n<t->numCols(); n++)
        t->adjustColumn(n);
}

void DaqGui::clear_tbl_hw()
{
    Q3Table *t = ui->tbl_hw;
    int nr = t->numRows();
    int nc = t->numCols();
    for (int r=0;r<nr;r++)
        for (int c=0;c<nc;c++)
            t->clearCellWidget(r, c);
}

void DaqGui::on_tbl_hw_valueChanged( int r, int c)
{
    //  if (disable_cfg_slots) return;
    modmap_t mods = hwf->get_modmap();

    Q3Table *t = ui->tbl_hw;
    if (c == tbl_hw_col_cblt) {
        Q3CheckTableItem *cb = NULL;
        cb = dynamic_cast<Q3CheckTableItem *>(t->item (r, c));
        if (cb) {
            bool new_cblt_state = cb->isChecked ();
            ddb_key_t ddbkey = t->text (r, tbl_hw_col_key).toULongLong(NULL, 16);
            // find module by ddb key
            Q_FOREACH(base_mod *mod, mods) {
                Q_ASSERT(mod);
                if (mod && mod->get_ddb_key() == ddbkey) {
                    mod->set_cblt_enable(new_cblt_state);
                    break;
                }
            }
        }
    }
}

/*******************************************************************************/
/* TDC tab slots */
/*******************************************************************************/
void DaqGui::init_tbl_tdc()
{
    Q3Table *t = ui->tbl_tdc;
    if (!t) return;
    t->setNumCols(3);
    Q3Header *hh = t->horizontalHeader ();

    hh->setLabel(0, "En");
    hh->setLabel(1, "Raw Count");
    hh->setLabel(2, "TDC Hits");
    t->adjustColumn (0);
    t->adjustColumn (1);

    t->hideColumn(1); // FIXME
    t->setColumnReadOnly(1, true);
    t->setColumnReadOnly(2, true);
}

void DaqGui::set_tbl_tdc()
{
    Q3Table *t = ui->tbl_tdc;
    if (!t) return;
    int nr = hwf->get_nch_byfam (MF_TDC);
    if (nr <= 0) return;
    CfgSlotDisabler csd;
    Q3Header *vh = t->verticalHeader ();
    t->setNumRows(nr);
    for (int r=0; r<nr ;r++) {
        vh->setLabel (r, QString::number(r));
        Q3CheckTableItem *cb = dynamic_cast<Q3CheckTableItem *>(t->item (r, 0));
        if (!cb) {
            t->setItem (r, 0, new Q3CheckTableItem (t, ""));
        }
    }
    t->adjustColumn (0);
    ui->spinBoxCommonStop->setMaxValue(nr);
    ui->spinBoxCommonStop->setValue(cfg->getTdcCommonStopChannel());
}

void DaqGui::clear_tbl_tdc()
{
    Q3Table *t = ui->tbl_tdc;
    int nr = t->numRows();
    int nc = t->numCols();
    for (int r=0;r<nr;r++)
        for (int c=0;c<nc;c++)
            t->clearCellWidget(r, c);
}

void DaqGui::display_refresh_tbl_tdc ()
{
    mfam_t mf = MF_TDC;
    Q3Table *t = ui->tbl_tdc;

    if (!t)
        return;

    int nr = hwf->get_nch_byfam (mf);
    if (nr <= 0) return;
    CfgSlotDisabler csd;

    const th1Ptr h_tdc_ch = hmgr->getHist("tdc_ch");
    const th1Ptr h_tdc_input_ch = hmgr->getHist("tdc_input_ch");
    for (int i = 0; i < nr; i++) {
        Q3CheckTableItem *cb = dynamic_cast<Q3CheckTableItem *>(t->item (i, 0));
        if (cb)
            cb->setChecked (hwf->get_int("en", mf, i));

        if (h_tdc_input_ch) {
            unsigned int n = (unsigned int)(h_tdc_input_ch->get_bin(i));
            t->setText(i, 1, QString::number(n));
        }
        if (h_tdc_ch) {
            unsigned int n = (unsigned int)(h_tdc_ch->get_bin(i));
            t->setText(i, 2, QString::number(n));
        }
    }

    // TDC Statistics Display
    QString txt = TdcErrorStatsText(hmgr->getHist("tdc_errors"));
    ui->textBrowserTdcStats->setText(txt);
    //  spinBoxCommonStop->setEnabled(checkBoxCommonStop->isChecked());
//    ui->spinBoxCommonStop->setMaxValue(nr);
//    ui->spinBoxCommonStop->setValue(cfg->getTdcCommonStopChannel());
}

void DaqGui::on_tbl_tdc_valueChanged( int r, int c)
{
    Q_ASSERT(cfg);
    if (disable_cfg_slots)
        return;

    Q3Table *t = ui->tbl_tdc;
    if (c == 0) {
        Q3CheckTableItem *cb = NULL;
        cb = dynamic_cast<Q3CheckTableItem *>(t->item (r, c));
        if (cb) {
            hwf->set_int(cb->isChecked (), "en", MF_TDC, r);
        }
    }
}

void DaqGui::on_spinBoxCommonStop_valueChanged(int value)
{
    hmgr->tdc_common_stop_channel = value;
    cfg->setTdcCommonStopChannel(value);
}

/*******************************************************************************/
/* TQDC tab slots */
/*******************************************************************************/
void DaqGui::set_control_ranges_tqdc () {
    mfam_t mf = MF_TQDC;
    if (!hwf->mf_present(mf)) return;
    CfgSlotDisabler csd;
    ui->sb_tqdc_mw->setMinValue (hptdc_settings::get_min_win());
    ui->sb_tqdc_mw->setMaxValue (hptdc_settings::get_max_win());
    ui->sb_tqdc_lat->setMinValue (hptdc_settings::get_min_lat());
    ui->sb_tqdc_lat->setMaxValue (hptdc_settings::get_max_lat());
    ui->sb_tqdc_adcwin->setMinValue(hwf->get_int("min_adc_match", mf));
    ui->sb_tqdc_adcwin->setMaxValue(hwf->get_int("max_adc_match", mf));
    ui->sb_tqdc_adcsrch->setMinValue(hwf->get_int("min_adc_search", mf));
    ui->sb_tqdc_adcsrch->setMaxValue(hwf->get_int("max_adc_search", mf));
    ui->sb_tqdc_adclat->setMinValue(hwf->get_int("min_trlat", mf));
    ui->sb_tqdc_adclat->setMaxValue(hwf->get_int("max_trlat", mf));
    ui->sb_tqdc_thr->setMinValue(hwf->get_int("min_thr", mf));
    ui->sb_tqdc_thr->setMaxValue(hwf->get_int("max_thr", mf));
    ui->sb_tqdc_shape->setMinValue(hwf->get_int("min_tr_shape", mf));
    ui->sb_tqdc_shape->setMaxValue(hwf->get_int("max_tr_shape", mf));
    ui->sb_tqdc_alt1->setMaxValue(15);
    ui->sb_tqdc_alt2->setMaxValue(15);
}

void DaqGui::restore_tqdc_cfg ()
{
    mfam_t mf = MF_TQDC;
    if (!hwf->mf_present(mf))
        return;
    CfgSlotDisabler csd;
    //  sb_tqdc_ch->setValue (cfg->tqdc.cur_ch);
    ui->sb_tqdc_mw->setValue (hwf->get_int("win", mf));
    ui->sb_tqdc_lat->setValue (hwf->get_int("lat", mf));
    //  chg_tqdc_ch (sb_tqdc_ch->value ()); // force signals
    on_sb_tqdc_mw_valueChanged (ui->sb_tqdc_mw->value ());
    on_sb_tqdc_lat_valueChanged (ui->sb_tqdc_lat->value ());
    int rcadj[5];
    hptdc_unpack_rcadj(hwf->get_int("rcadj", mf), rcadj);
    ui->sb_tqdc_rc0->setValue (rcadj[0]);
    ui->sb_tqdc_rc1->setValue (rcadj[1]);
    ui->sb_tqdc_rc2->setValue (rcadj[2]);
    ui->sb_tqdc_rc3->setValue (rcadj[3]);
    ui->cb_tqdc_twoedge->setChecked (hwf->get_int("twoedge", mf));
//    ui->cb_tqdc_lead->setChecked (hwf->get_int("edge", mf) & 1);
//    ui->cb_tqdc_trail->setChecked (hwf->get_int("edge", mf) & 2);
    //  cb_tqdc_lead_toggled ();
    //  cb_tqdc_trail_toggled ();
//    ui->cb_tqdc_en_hdr->setChecked (hwf->get_int("hdr", mf));
//    ui->cb_tqdc_en_trl->setChecked (hwf->get_int("trl", mf));
//    on_cb_tqdc_en_hdr_toggled (ui->cb_tqdc_en_hdr->isChecked());
//    on_cb_tqdc_en_trl_toggled (ui->cb_tqdc_en_trl->isChecked());
    ui->cb_tqdc_res->setCurrentItem (hwf->get_int("res", mf) + 1);
    //  pb_show_tqdc_rctime->setEnabled (tdc_res_vh (cb_tqdc_res->currentItem()-1));

    // tqdc:adc part
    ui->sb_tqdc_adcwin->setValue (hwf->get_int("adc_match", mf));
    ui->sb_tqdc_adcsrch->setValue (hwf->get_int("adc_search", mf));
    ui->sb_tqdc_adclat->setValue (hwf->get_int("trlat", mf));
    ui->sb_tqdc_thr->setValue (hwf->get_int("thr", mf, tqdc_select));

    ui->sb_tqdc_match->setMinValue(hwf->get_int("tqdc_matchwin_min", mf));
    ui->sb_tqdc_match->setMaxValue(hwf->get_int("tqdc_matchwin_max", mf));
    ui->sb_tqdc_match->setValue(hwf->get_int("tqdc_matchwin", mf));
    on_sb_tqdc_match_valueChanged(ui->sb_tqdc_match->value());
    ui->sb_tqdc_latency->setMinValue(hwf->get_int("tqdc_latency_min", mf));
    ui->sb_tqdc_latency->setMaxValue(hwf->get_int("tqdc_latency_max", mf));
    ui->sb_tqdc_latency->setValue(hwf->get_int("tqdc_latency", mf));
    on_sb_tqdc_latency_valueChanged(ui->sb_tqdc_latency->value());

    // tqdc:trigger part
    ui->cb_tqdc_match8->setChecked(hwf->get_int("tr_match8", mf));
    ui->sb_tqdc_maj->setValue(hwf->get_int("tr_maj", mf));
    ui->sb_tqdc_min->setValue(hwf->get_int("tr_min", mf));
    ui->cb_tqdc_en1->setChecked( hwf->get_int("tr_en1", mf) );
    ui->cb_tqdc_en2->setChecked( hwf->get_int("tr_en2", mf) );
    int tr_shape = hwf->get_int("tr_shape", mf);
    ui->sb_tqdc_shape->setValue(tr_shape);
    update_tqdc_tr_shape();
    ui->sb_tqdc_alt1->setValue( hwf->get_int("tr_alt1", mf) );
    ui->sb_tqdc_alt2->setValue( hwf->get_int("tr_alt2", mf) );

    int tqdc_mode = hwf->get_int("adc_mode", mf);
//    if (tqdc_mode < 1 || tqdc_mode > 3) {
//        tqdc_mode = 2;
//    }
    ui->rb_tqdc_baseline->setChecked(tqdc_mode == 1);
    ui->rb_tqdc_waveform->setChecked(tqdc_mode == 2);
    ui->rb_tqdc_integrator->setChecked(tqdc_mode == 3);

    ui->frame_tqdc_neg_pol->setEnabled(false);
    ui->frame_tqdc_neg_pol->hide();
    restore_tqdc_neg_buttons();

    display_refresh_tbl_tqdc();
}

void DaqGui::restore_tqdc_neg_buttons()
{
    const mfam_t mf = MF_TQDC;
    ui->checkBox_tqdc_neg_a->setTristate(!hwf->is_all_channels_same("neg_a", mf));
    ui->checkBox_tqdc_neg_a->setChecked(hwf->get_int("neg_a", mf));
    ui->checkBox_tqdc_neg_b->setTristate(!hwf->is_all_channels_same("neg_b", mf));
    ui->checkBox_tqdc_neg_b->setChecked(hwf->get_int("neg_b", mf));
}

void DaqGui::setup_tqdc_select()
{
    const modmap_t mods = hwf->get_modmap();
    QStringList sl;
    sl += "All TQDC modules";
    int n = 0;
    Q_FOREACH(const base_mod *mod, mods) {
        if (!mod || mod->get_fam() != MF_TQDC) continue;
        QString s;
        s = "TQDC #"+QString::number(n)+", slot "+QString::number(mod->get_slot());
        sl += s;
        n++;
    }
    ui->comboBox_tqdc_select->clear();
    ui->comboBox_tqdc_select->insertStringList(sl);
    ui->comboBox_tqdc_select->setCurrentItem(0);
    tqdc_select = SET_ALL_CH;
}

void DaqGui::on_sb_tqdc_mw_valueChanged( int value )
{
    mfam_t mf = MF_TQDC;
    if (!hwf->mf_present(mf))
        return;

    int mw = value;
    double mwus = mw * 0.025;
    ui->tl_tqdc_mw->setText(QString("%1 µs").arg(mwus, 0, 'g', 3));
    if (disable_cfg_slots)
        return;

    hwf->set_int(value, "win", mf);
    //  rescale_tqdc_hists ();
}

void DaqGui::on_sb_tqdc_lat_valueChanged( int value)
{
    mfam_t mf = MF_TQDC;
    if (!hwf->mf_present(mf))
        return;

    int lat = value;
    double latus = lat * 0.025;
    ui->tl_tqdc_lat->setText (QString("%1 µs").arg(latus, 0, 'g', 3));
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "lat", mf);
    //  rescale_tqdc_hists ();
}

void DaqGui::replot_tqdc ()
{
    display_refresh_tbl_tqdc();
    replot_tqdc_waveform();
}


void DaqGui::display_refresh_tbl_tqdc ()
{
    mfam_t mf = MF_TQDC;
    Q3Table *t = ui->tbl_tqdc;

    if (!hwf->mf_present(mf))
        return;

    // TDC Statistics Display
    QString txt = TdcErrorStatsText(hmgr->getHist("tqdc_errors"));
    ui->textBrowserTqdcErrors->setText(txt);

    double btime = shm->getLastBurstDuration();
    if (btime==0) btime=1;
    int nr = hwf->get_nch_byfam (mf);
    for (int i = 0; i < nr; i++) {
        Q3CheckTableItem *cb;

        cb = dynamic_cast<Q3CheckTableItem *>(t->item (i, tqdc_tbl_col_en));
        if (cb)
            cb->setChecked (hwf->get_int("en", mf, i));

        if (i%8==0) {
            cb = dynamic_cast<Q3CheckTableItem *>(t->item (i, tqdc_tbl_col_inv));
            if (cb) {
                bool neg = hwf->get_int("neg", mf, i);
                cb->setChecked (neg);
                cb->setText(neg?"-":"+");
                //      cb->setEnabled(i%8==0); // only first of 8 is active
            }
        }

        t->setText(i, tqdc_tbl_col_mch, QString("%1/%2").arg(i / 16).arg(i % 16));
        t->setText(i, tqdc_tbl_col_thr, QString("%1").arg(hwf->get_int("thr", mf, i)));
    }

    const th1Ptr ht = hmgr->getHist("tqdc_ch_t");
    const th1Ptr hq = hmgr->getHist("tqdc_ch_q");
    const th1Ptr hc = hmgr->getHist("tqdc_ch_rawhits");
    for (int i = 0; ht && hq && hc && i < nr; i++) {
        const int nhits_t = ht->get_bin(i);
        t->setText(i, tqdc_tbl_col_hitst, QString::number(nhits_t));

        const int nhits_q = hq->get_bin(i);
        t->setText(i, tqdc_tbl_col_hitsq, QString::number(nhits_q));

        double cnt = hc->get_bin(i) / btime / 1000.;
        if (cnt > 0) {
            t->setText(i, tqdc_tbl_col_cnt, QString::number(cnt, 'f', 3));
        } else {
            t->setText(i, tqdc_tbl_col_cnt, "");
        }

        const double baseline = hmgr->get_calibration_constant("tqdc_baseline", i);
        t->setText(i, tqdc_tbl_col_baseline, QString::number(baseline, 'f', 1));

        if (nhits_q>0) {
            double sumq_raw = hmgr->getbincontent("tqdc_ch_sumq_raw", i);
            t->setText(i, tqdc_tbl_col_sumqraw, QString::number(sumq_raw/nhits_q, 'f', 1));
            double sumq = hmgr->getbincontent("tqdc_ch_sumq", i);
            t->setText(i, tqdc_tbl_col_sumq, QString::number(sumq/nhits_q, 'f', 1));
        } else {
            t->setText(i, tqdc_tbl_col_sumqraw, "");
            t->setText(i, tqdc_tbl_col_sumq, "");
        }
    }
    for (int i=0;i<tqdc_tbl_ncols;i++) {
        t->adjustColumn (i);
    }
}

void DaqGui::init_tbl_tqdc()
{
    Q3Table *t = ui->tbl_tqdc;
    t->setNumRows(0);
    t->setNumCols(tqdc_tbl_ncols);
    Q3Header *hh = t->horizontalHeader ();

    hh->setLabel(tqdc_tbl_col_en, "En");
    hh->setLabel(tqdc_tbl_col_mch, "Ch");
    hh->setLabel(tqdc_tbl_col_inv, "+/-");
    hh->setLabel(tqdc_tbl_col_thr, "Thr");
    hh->setLabel(tqdc_tbl_col_hitst, "Hits T");
    hh->setLabel(tqdc_tbl_col_hitsq, "Hits Q");
    hh->setLabel(tqdc_tbl_col_baseline, "Baseline");
    hh->setLabel(tqdc_tbl_col_sumqraw, "SumQ RAW");
    hh->setLabel(tqdc_tbl_col_sumq, "SumQ    ");
    hh->setLabel(tqdc_tbl_col_cnt, "Rate, kHz");
    t->hideColumn(tqdc_tbl_col_sumqraw);
    for (int i=0;i<tqdc_tbl_ncols;i++) {
        if (i!=tqdc_tbl_col_en && i!=tqdc_tbl_col_inv && i!=tqdc_tbl_col_thr)
            t->setColumnReadOnly(i, true);
        t->adjustColumn (i);
    }
}

void DaqGui::set_tbl_tqdc()
{
    Q3Table *t = ui->tbl_tqdc;
    int nr = hwf->get_nch_byfam (MF_TQDC);
    Q3Header *vh = t->verticalHeader ();

    t->setNumRows(nr);
    for (int r=0;r<nr;r++) {
        vh->setLabel (r, QString::number(r));
        t->setItem (r, tqdc_tbl_col_en, new Q3CheckTableItem (t, ""));
        if (r%8==0) {
            Q3CheckTableItem *cti2 = new Q3CheckTableItem (t, "");
            //  	  cti2->setEnabled(false);
            t->setItem (r, tqdc_tbl_col_inv, cti2);
            cti2->setSpan(8, 1);
        }
    }
    t->adjustColumn (0);
}

void DaqGui::clear_tbl_tqdc()
{
    Q3Table *t = ui->tbl_tqdc;
    int nr = t->numRows();
    int nc = t->numCols();
    for (int r=0;r<nr;r++)
        for (int c=0;c<nc;c++)
            t->clearCellWidget(r, c);
}

void DaqGui::on_tbl_tqdc_valueChanged(int r, int c)
{
    if (disable_cfg_slots)
        return;

    Q3Table *t = ui->tbl_tqdc;
    if (c == tqdc_tbl_col_en) {
        Q3CheckTableItem *cb = NULL;
        cb = dynamic_cast<Q3CheckTableItem *>(t->item (r, c));
        if (cb) {
            bool state = cb->isChecked ();
            hwf->set_int(state, "en", MF_TQDC, r);
        }
    }
    if (c == tqdc_tbl_col_inv) {
        Q3CheckTableItem *cb = NULL;
        cb = dynamic_cast<Q3CheckTableItem *>(t->item (r, c));
        if (cb) {
            bool state = cb->isChecked ();
            bool is_a = (r%16) < 8;
            if (is_a)
                hwf->set_int(state, "neg_a", MF_TQDC, r);
            else
                hwf->set_int(state, "neg_b", MF_TQDC, r);
        }
        restore_tqdc_neg_buttons();
    }
    if (c == tqdc_tbl_col_thr) {
        bool ok = false;
        int v = t->text(r, c).toInt(&ok);
        if (hwf->mf_present(MF_TQDC) && !disable_cfg_slots) {
            hwf->set_int(v, "thr", MF_TQDC, r);
        }
    }
}

void DaqGui::on_cb_tqdc_twoedge_toggled (bool state)
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "twoedge", MF_TQDC);
}

void DaqGui::on_sb_tqdc_rc0_valueChanged(int value)
{
    off_sb_tqdc_rcx_valueChanged(0, value);
}

void DaqGui::on_sb_tqdc_rc1_valueChanged(int value)
{
    off_sb_tqdc_rcx_valueChanged(1, value);
}

void DaqGui::on_sb_tqdc_rc2_valueChanged(int value)
{
    off_sb_tqdc_rcx_valueChanged(2, value);
}

void DaqGui::on_sb_tqdc_rc3_valueChanged(int value)
{
    off_sb_tqdc_rcx_valueChanged(3, value);
}

void DaqGui::off_sb_tqdc_rcx_valueChanged(int n, int value)
{
    if (disable_cfg_slots)
        return;
    Q_ASSERT((n >= 0) && (n <= 3));
    if (n < 0 || n > 3)
        return;
    int rcadj[5];
    hptdc_unpack_rcadj(hwf->get_int("rcadj", MF_TQDC), rcadj);
    rcadj[n] = value;
    hwf->set_int(hptdc_pack_rcadj(rcadj), "rcadj", MF_TQDC);
}


// tqdc:adc part slots
void DaqGui::on_sb_tqdc_adcwin_valueChanged( int value )
{
    if (!hwf->mf_present(MF_TQDC))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "adc_match", MF_TQDC);
}

void DaqGui::on_sb_tqdc_adcsrch_valueChanged( int value)
{
    if (!hwf->mf_present(MF_TQDC))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "adc_search", MF_TQDC);
}

void DaqGui::on_sb_tqdc_adclat_valueChanged( int value )
{
    if (!hwf->mf_present(MF_TQDC))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "trlat", MF_TQDC);
}

void DaqGui::on_sb_tqdc_thr_valueChanged( int value )
{
    if (!hwf->mf_present(MF_TQDC))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "thr", MF_TQDC, tqdc_select);
}

void DaqGui::on_cb_tqdc_match8_toggled( bool state )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "tr_match8", MF_TQDC);
}

void DaqGui::on_sb_tqdc_maj_valueChanged( int value )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "tr_maj", MF_TQDC);
    ui->sb_tqdc_min->setMaxValue(value);
}

void DaqGui::on_sb_tqdc_min_valueChanged( int value )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "tr_min", MF_TQDC);
    ui->sb_tqdc_maj->setMinValue(value);
}

void DaqGui::on_cb_tqdc_en1_toggled( bool state )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "tr_en1", MF_TQDC);
}

void DaqGui::on_cb_tqdc_en2_toggled( bool state )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "tr_en2", MF_TQDC);
}

void DaqGui::on_sb_tqdc_shape_valueChanged( int value )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "tr_shape", MF_TQDC);
    update_tqdc_tr_shape();
}

void DaqGui::update_tqdc_tr_shape()
{
    double tr_shape = 12.5 * (1+hwf->get_int("tr_shape", MF_TQDC));
    ui->tl_tqdc_shape->setText(QString("%1 ns").arg(tr_shape));
}

void DaqGui::on_sb_tqdc_alt1_valueChanged( int value )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "tr_alt1", MF_TQDC);
}

void DaqGui::on_sb_tqdc_alt2_valueChanged( int value )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "tr_alt2", MF_TQDC);
}

void DaqGui::on_rb_tqdc_baseline_toggled(bool value)
{
    if (disable_cfg_slots)
        return;
    if (value)
        hwf->set_int(1, "adc_mode", MF_TQDC);
}

void DaqGui::on_rb_tqdc_waveform_toggled(bool value)
{
    if (disable_cfg_slots)
        return;
    if (value)
        hwf->set_int(2, "adc_mode", MF_TQDC);
}

void DaqGui::on_rb_tqdc_integrator_toggled(bool value)
{
    if (disable_cfg_slots)
        return;
    if (value)
        hwf->set_int(3, "adc_mode", MF_TQDC);
}

//void DaqGui::on_bg_tqdc_mode_clicked( int value )
//{
//    if (disable_cfg_slots)
//        return;
//    int tqdc_mode;
//    if (ui->rb_tqdc_baseline->isChecked())
//        tqdc_mode == 1;
//    if (ui->rb_tqdc_waveform->isChecked())
//        tqdc_mode == 2;
//    if (ui->rb_tqdc_integrator->isChecked())
//        tqdc_mode == 3;
//    hwf->set_int(tqdc_mode, "adc_mode", MF_TQDC);
//}

void DaqGui::on_cb_tqdc_res_activated (int item)
{
    if (disable_cfg_slots)
        return;
    int newres = item - 1;
    hwf->set_int (newres, "res", MF_TQDC);
    clearstats ();
    rescale_tdc_hists ();
}

void DaqGui::on_sb_tqdc_match_valueChanged( int value )
{
    ui->tl_tqdc_match->setText(QString("%1 ns").arg(value*50));
    mfam_t mf = MF_TQDC;
    if (!hwf->mf_present(mf))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "tqdc_matchwin", mf);
    ui->sb_tqdc_adcsrch->setValue (hwf->get_int("adc_search", mf));
    ui->sb_tqdc_mw->setValue (hwf->get_int("win", mf));
}

void DaqGui::on_sb_tqdc_latency_valueChanged( int value )
{
    ui->tl_tqdc_latency->setText(QString("%1 ns").arg(value*50));
    mfam_t mf = MF_TQDC;
    if (!hwf->mf_present(mf))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "tqdc_latency", mf);
    ui->sb_tqdc_adclat->setValue (hwf->get_int("trlat", mf));
    ui->sb_tqdc_lat->setValue (hwf->get_int("lat", mf));
}

void DaqGui::on_comboBox_tqdc_select_activated( int index )
{
    tqdc_select = -1 - index;
    restore_tqdc_cfg();
}


void DaqGui::on_checkBox_tqdc_neg_a_toggled( bool value )
{
    Q_UNUSED(value);
    /*
  mfam_t mf = MF_TQDC;
  if (!hwf->mf_present(mf))
    return;
  if (disable_cfg_slots)
    return;
  hwf->set_int(value, "neg_a", mf);
*/
}


void DaqGui::on_checkBox_tqdc_neg_b_toggled( bool value )
{
    Q_UNUSED(value);
    /*
  mfam_t mf = MF_TQDC;
  if (!hwf->mf_present(mf))
    return;
  if (disable_cfg_slots)
    return;
  hwf->set_int(value, "neg_b", mf);
*/
}

/*******************************************************************************/
/* TRIG tab slots */
/*******************************************************************************/
// rows 0-15 - trigger state counters
// row 16 - output trigger counter
namespace {
const unsigned int n_choices = 5;
const unsigned int TRIG_N_STATES = (1<<TRIG_LOGIC_WIRES);
const unsigned int tbl_trig_num_rows = (TRIG_N_STATES);
enum tbl_trig_row {
    tbl_trig_row_outtrig = 0,
    tbl_trig_row_states = 1,
};
enum tbl_trig_col {
    tbl_trig_col_text = 0,
    tbl_trig_col_state_count = 1,
    tbl_trig_col_check = 2,
    tbl_trig_col_check2 = 3,
    tbl_trig_col_check3 = 4,
    tbl_trig_col_check4 = 5,
    tbl_trig_num_cols = 6,
};

typedef unsigned int trig_preset_t;
typedef QMap<QString, trig_preset_t> trig_preset_map_t;
trig_preset_map_t trig_presets;

unsigned int trig_bit_index(int state_index)
{
    return 1<<state_index;
}

void init_trig_presets()
{
    enum {
        L_ST = 1, // ST
        L_Q1 = 2, // TQDC1
        L_Q2 = 4, // TQDC2
        L_NM = 8, // NIM
    };

    trig_presets["ST"]    = trig_preset_t(1<<L_ST);
    trig_presets["ST & NIM"]  = trig_preset_t(1<<(L_ST |L_NM));
    trig_presets["TQDC1"] = trig_preset_t(0x0004);
    trig_presets["TQDC2"] = trig_preset_t(0x0010);
    trig_presets["NIM"]   = trig_preset_t(0x0100);
    trig_presets["NIM & TQDC1"]   = trig_preset_t(1<<(L_NM|L_Q1));
    trig_presets["NIM & TQDC2"]   = trig_preset_t(1<<(L_NM|L_Q2));
    trig_presets["NIM & TQDC1 & TQDC2"]   = trig_preset_t(1<<(L_NM|L_Q1|L_Q2));
    trig_presets["TQDC1 & TQDC2"] = trig_preset_t(0x0040);
    trig_presets["ST & TQDC1"] = trig_preset_t(1<<(L_ST|L_Q1));
    trig_presets["ST & TQDC2"] = trig_preset_t(1<<(L_ST|L_Q2));
    trig_presets["ST & TQDC1 & TQDC2"] = trig_preset_t(1<<(L_ST|L_Q1|L_Q2));
    trig_presets["Any (ST + TQDC1 + TQDC2 + NIM)"]   = trig_preset_t(0xFFFE);
    trig_presets["All (ST & TQDC1 & TQDC2 & NIM)"]   = trig_preset_t(1<<15);
    trig_presets["Custom"]   = 0;
    trig_presets["Off"]   = 0;

    trig_presets["ST + NIM"]  = trig_presets["ST"] | trig_presets["NIM"] | trig_presets["ST & NIM"];
    trig_presets["NIM + TQDC1"] = trig_presets["NIM"] | trig_presets["TQDC1"] | trig_presets["NIM & TQDC1"];
    trig_presets["NIM + TQDC2"] = trig_presets["NIM"] | trig_presets["TQDC2"] | trig_presets["NIM & TQDC2"];
    trig_presets["TQDC1 + TQDC2"] = trig_presets["TQDC1"] | trig_presets["TQDC2"] | trig_presets["TQDC1 & TQDC2"];
    trig_presets["NIM + TQDC1 + TQDC2"] = trig_presets["NIM"] | trig_presets["TQDC1"] | trig_presets["TQDC2"]
            | trig_presets["NIM + TQDC1"] | trig_presets["NIM + TQDC2"] | trig_presets["TQDC1 + TQDC2"]
            | trig_presets["NIM & TQDC1"] | trig_presets["NIM & TQDC2"] | trig_presets["TQDC1 & TQDC2"]
            | trig_presets["NIM & TQDC1 & TQDC2"];
    trig_presets["ST + TQDC1"] = trig_presets["ST"] | trig_presets["TQDC1"] | trig_presets["ST & TQDC1"];
    trig_presets["ST + TQDC2"] = trig_presets["ST"] | trig_presets["TQDC2"] | trig_presets["ST & TQDC2"];
    trig_presets["ST + TQDC1 + TQDC2"] = trig_presets["ST"] | trig_presets["TQDC1"] | trig_presets["TQDC2"]
            | trig_presets["ST + TQDC1"] | trig_presets["ST + TQDC2"] | trig_presets["TQDC1 + TQDC2"]
            | trig_presets["ST & TQDC1"] | trig_presets["ST & TQDC2"] | trig_presets["TQDC1 & TQDC2"]
            | trig_presets["ST & TQDC1 & TQDC2"];

//    Q_FOREACH(const QString &title, trig_presets.keys()) {
//        unsigned int logic = trig_presets[title];
//        qDebug(title+": "+QString::number(logic, 16));
//    }
}
}

void DaqGui::init_tbl_trig()
{
    init_trig_presets();

    QStringList keys = trig_presets.keys();
    keys.sort();
    ui->comboBox_trig_preset->insertStringList(keys);

    Q3Table *t = ui->table_trig_states;
    t->setNumRows(tbl_trig_num_rows);
    t->setNumCols(tbl_trig_num_cols);
    Q3Header *vh = t->verticalHeader ();
    Q3Header *hh = t->horizontalHeader ();

    // enumerate rows
    for (unsigned int i = 0; i < tbl_trig_num_rows; i++) {
        vh->setLabel(i, QString::number(i));
    }

    t->setText(tbl_trig_row_outtrig, tbl_trig_col_text, "Total:");
    t->setText(0, tbl_trig_col_text, "Pulser");
    for (unsigned int i = 1; i < TRIG_N_STATES; i++) {
        QString s = QString("State %1%2%3%4").arg((i&8)?1:0).arg((i&4)?1:0).arg((i&2)?1:0).arg((i&1)?1:0);
        t->setText(tbl_trig_row_states+i-1, tbl_trig_col_text, s);
    }
    hh->setLabel(tbl_trig_col_text, "Title");
    hh->setLabel(tbl_trig_col_state_count, "Count");
    hh->setLabel(tbl_trig_col_check, "ST");
    hh->setLabel(tbl_trig_col_check2, "TQDC1");
    hh->setLabel(tbl_trig_col_check3, "TQDC2");
    hh->setLabel(tbl_trig_col_check4, "NIM");

    for (unsigned int i = 0; i<tbl_trig_num_cols; i++) {
        t->setColumnReadOnly (i, true);
        t->adjustColumn (i);
    }
}

void DaqGui::display_refresh_trig_table ()
{
    Q3Table *t = ui->table_trig_states;
    unsigned int logic = hwf->get_int("logic", MF_TRIG);
    bool ch_enable[TRIG_LOGIC_WIRES];

    for (unsigned int c=0;c<TRIG_LOGIC_WIRES;c++)
        ch_enable[c] = hwf->get_int("en", MF_TRIG, c);
    QString s;
    for (unsigned int i = 1; i < TRIG_N_STATES ; i++) {
        if (logic & (1<<i)) {
            unsigned int b[TRIG_LOGIC_WIRES];
            for (unsigned int c=0;c<TRIG_LOGIC_WIRES;c++)
                b[c] = i & (1<<c);
            s="";
            for (int c=TRIG_LOGIC_WIRES-1;c>=0;c--) {
                s="";
                if (b[c]) {
                    if (ch_enable[c])
                        s += QString("+");
                    else
                        s += QString("x");
                } else {
                    s += QString("-");
                }
                t->setText(tbl_trig_row_states+i-1, tbl_trig_col_check+c, s);
            }
        } else {
            for (int c=TRIG_LOGIC_WIRES-1;c>=0;c--) {
                t->setText(tbl_trig_row_states+i-1, tbl_trig_col_check+c, "");
            }
        }
    }
}

QString DaqGui::TrigEventStatsText(const QVector<double> &h_trig_incnt, const QVector<double> &h_burst_history, const QVector<double> &h_trig_bits) const
{
    QString txt("<p><b>TRIG Event Statistics</b></p>");
    if (h_trig_incnt.isEmpty() || h_burst_history.isEmpty() || h_trig_bits.isEmpty()) {
        txt += "no data available";
        return txt;
    }

    txt += "<table>";
    quint64 t = 0;
    for (int b=0; b<h_trig_incnt.count(); b++)
        t += (quint64)h_trig_incnt[b];
    unsigned int r = (unsigned int)(h_burst_history.count()); // first bin - number of events in current spill
    if (t>0) {
        txt += "<tr><td>Unblocked triggers</td><td>"+QString::number(t)+"</td><td> </td></tr>";
        txt += "<tr><td>Read out triggers</td><td>"+QString::number(r)+"</td><td>"+QString::number(100.*r/t)+"%</td></tr>";
    }
    txt += "</table>";

    if (r==0) return txt;

    std::map<int, QString> TriggerBitName;
    if (mtdc64_found(*hwf)) {
        TriggerBitName[0] = "Pulser";
        TriggerBitName[1] = "NIM";
        TriggerBitName[2] = "Any Hit";
        TriggerBitName[3] = "TLU";
        TriggerBitName[4] = "LVDS";
    } else {
        TriggerBitName[0] = "Pulser";
        TriggerBitName[1] = "NIM 1";
        TriggerBitName[2] = "NIM 2";
        TriggerBitName[3] = "NIM 3";
        TriggerBitName[4] = "NIM 4";
        TriggerBitName[5] = "NIM 5";
        TriggerBitName[6] = "NIM 6";
        TriggerBitName[7] = "NIM 7";
        TriggerBitName[8] = "LVDS1 ST";
        TriggerBitName[9] = "LVDS6 TQDC1";
        TriggerBitName[10] = "LVDS10 TQDC1";
        TriggerBitName[11] = "LVDS14 TQDC1";
        TriggerBitName[12] = "LVDS18 TQDC2";
        TriggerBitName[13] = "LVDS22 TQDC2";
        TriggerBitName[14] = "LVDS26 TQDC2";
        TriggerBitName[15] = "LVDS30 TQDC2";
    }
    TriggerBitName[19] = "Endspill";
    txt += "<p><b>Trigger Bits</b></p>";
    txt += "<table>";
    for (int b=0; b<h_trig_bits.count(); b++) {
        long n = h_trig_bits[b];
        if (n==0) continue;
        QString str = TriggerBitName[b];
        if (str.isEmpty())
            str = QString("%1").arg(b);
        else
            str = QString("%1 (%2)").arg(b).arg(str);
        txt += "<tr><td>"+str+"</td><td>"+QString::number(n)+"</td><td>"+QString::number(100.*n/r)+"%</td></tr>";
    }
    txt += "</table>";
    return txt;
}

void DaqGui::display_refresh_trig_counts ()
{
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;

    Q3Table *t = ui->table_trig_states;
    const th1Ptr h_trig_st = hmgr->getHist("trig_states");
    if (h_trig_st) {
        for (unsigned int i=0; i<TRIG_N_STATES; i++) {
            unsigned int n = (unsigned int)h_trig_st->get_bin(i);
            t->setText(i, tbl_trig_col_state_count, QString::number(n));
        }
    }

    // TRIG Statistics Display for MTDC-64
    const th1Ptr h_trig_incnt = hmgr->getHist("trig_incnt");
    const th1Ptr h_burst_history = hmgr->getHist("burst_history");
    const th1Ptr h_trig_bits = hmgr->getHist("trig_bits");

    QString txt;
    if (h_trig_incnt && h_burst_history && h_trig_bits) {
        txt = TrigEventStatsText(
                    QVector<double>::fromStdVector(h_trig_incnt->get_py()),
                    QVector<double>::fromStdVector(h_burst_history->get_py()),
                    QVector<double>::fromStdVector(h_trig_bits->get_py()));
    }
    ui->textBrowserTrigStats->setText(txt);
}

void DaqGui::display_refresh_trig_preset()
{
    unsigned int logic = hwf->get_int("logic", MF_TRIG);
    QString found_preset;
    Q_FOREACH(const QString &title, trig_presets.keys()) {
        if (trig_presets[title] == logic) {
            found_preset = title;
        }
    }
    if (found_preset.isEmpty())
        found_preset = "Custom";

    for (int i=0;i < ui->comboBox_trig_preset->count();i++) {
        if (ui->comboBox_trig_preset->text(i) == found_preset) {
            disable_cfg_slots++;
            ui->comboBox_trig_preset->setCurrentItem(i);
            disable_cfg_slots--;
            break;
        }
    }
}

void DaqGui::display_refresh_trig ()
{
    display_refresh_trig_counts();
    display_refresh_trig_preset();
    display_refresh_trig_table();
}

void DaqGui::restore_trig_cfg ()
{
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    CfgSlotDisabler csd;

    // Spill control
    ui->sb_trig_spill_delay->setMinValue(1/*hwf->get_int("min_spill_delay", mf)*/);
    ui->sb_trig_spill_delay->setMaxValue(hwf->get_int("max_spill_delay", mf));
    ui->sb_trig_spill_delay->setValue(hwf->get_int("spill_delay", mf));
    ui->sb_trig_spill_duration->setMinValue(1 /*hwf->get_int("min_spill_duration", mf)*/);
    ui->sb_trig_spill_duration->setMaxValue(hwf->get_int("max_spill_duration", mf));
    ui->sb_trig_spill_duration->setValue(hwf->get_int("spill_duration", mf));
    ui->sb_trig_spill_repeat->setMinValue(hwf->get_int("min_spill_repeat", mf));
    ui->sb_trig_spill_repeat->setMaxValue(hwf->get_int("max_spill_repeat", mf));
    ui->sb_trig_spill_repeat->setValue(hwf->get_int("spill_repeat", mf));
    int spill_select = hwf->get_int("spill_select", mf);
    //  gb_test_spill_select->setChecked(spill_select);
    ui->rb_trig_spill_timer->setChecked(spill_select == TRIG_SPILL_TIMER);
    ui->rb_trig_spill_ext->setChecked(spill_select == TRIG_SPILL_EXT);
    ui->rb_trig_spill_gate->setChecked(spill_select == TRIG_SPILL_GATE);
    ui->gb_trig_spill_generator->setEnabled(spill_select != TRIG_SPILL_GATE);

    // Delay timers
    ui->sb_trig_delay_read->setMinValue(hwf->get_int("min_read_delay", mf));
    ui->sb_trig_delay_read->setMaxValue(hwf->get_int("max_read_delay", mf));
    ui->sb_trig_delay_read->setValue(hwf->get_int("read_delay", mf));
    ui->sb_trig_delay_ttc->setMinValue(hwf->get_int("min_ttc_delay", mf));
    ui->sb_trig_delay_ttc->setMaxValue(hwf->get_int("max_ttc_delay", mf));
    ui->sb_trig_delay_ttc->setValue(hwf->get_int("ttc_delay", mf));

    // DCH clock phase adjustment
    int adj = hwf->get_int("dch_clock_adj", mf);
    ui->spinBox_DCH_window_adj->setValue(adj);

    // XOFF input wait timer
    ui->sb_trig_xoff_wait->setMinValue(hwf->get_int("min_xoff_wait", mf));
    ui->sb_trig_xoff_wait->setMaxValue(hwf->get_int("max_xoff_wait", mf));
    ui->sb_trig_xoff_wait->setValue(hwf->get_int("xoff_wait", mf));
    ui->cb_trig_en_xoff_in->setChecked(hwf->get_int("en_xoff_in", mf));
    ui->cb_trig_en_xoff_in->hide();
    ui->sb_trig_xoff_wait->setEnabled(ui->cb_trig_en_xoff_in->isChecked());
    ui->sb_trig_xoff_wait->hide();

    // Wait for datattaking to complete on spill end
    //  cb_trig_en_spill_end_wait->setChecked(hwf->get_int("en_spill_end_wait", mf));
    ui->cb_trig_en_spill_end_wait->hide();

    // Internal pulsers
    ui->cb_trig_en_noise->setChecked(hwf->get_int("noise_trigger", mf));
    ui->cb_trig_en_pulser->setChecked(hwf->get_int("pulser_trigger", mf));
    ui->doubleSpinBoxPulserFreq->setValue(hwf->get_int("pulser_freq", mf));

    ui->spinBox_trig_in_shape->setMinValue(0);
    ui->spinBox_trig_in_shape->setMaxValue(15);
    ui->spinBox_trig_in_shape->setValue(hwf->get_int("in_shape", mf));

    // alternative view

    ui->checkBoxTrigSetupPulser->setChecked(hwf->get_int("pulser_trigger", mf));
    ui->checkBoxTrigSetupLVDS->setChecked(hwf->get_int("en", mf, 0)); // ST/LVDS line
    ui->checkBoxTrigSetupHit->setChecked(hwf->get_int("en", mf, 1)); // TQDC1/HIT line
    ui->checkBoxTrigSetupTLU->setChecked(hwf->get_int("en", mf, 2)); // TQDC2/TLU line
    ui->checkBoxTrigSetupNim->setChecked(hwf->get_int("en", mf, 3)); // NIM/NIM line

    ui->checkBoxTrigNim1->setChecked(get_trig_in_en(1));
    ui->checkBoxTrigNim2->setChecked(get_trig_in_en(2));
    ui->checkBoxTrigNim3->setChecked(get_trig_in_en(3));
    ui->checkBoxTrigNim4->setChecked(get_trig_in_en(4));
    ui->checkBoxTrigNim5->setChecked(get_trig_in_en(5));
    ui->checkBoxTrigNim6->setChecked(get_trig_in_en(6));
    ui->checkBoxTrigNim7->setChecked(get_trig_in_en(7));
    ui->checkBoxTrigLvds1->setChecked(get_trig_in_en(8));
    ui->checkBoxTrigLvds6->setChecked(get_trig_in_en(13));
    ui->checkBoxTrigLvds10->setChecked(get_trig_in_en(17));
    ui->checkBoxTrigLvds14->setChecked(get_trig_in_en(21));
    ui->checkBoxTrigLvds18->setChecked(get_trig_in_en(25));
    ui->checkBoxTrigLvds22->setChecked(get_trig_in_en(29));
    ui->checkBoxTrigLvds26->setChecked(get_trig_in_en(33));
    ui->checkBoxTrigLvds30->setChecked(get_trig_in_en(37));

    ui->spinBox_trig_nim_min->setMinimum(1);
    ui->spinBox_trig_nim_min->setMaximum(7);
    ui->spinBox_trig_nim_max->setMinimum(1);
    ui->spinBox_trig_nim_max->setMaximum(7);
    ui->spinBox_trig_nim_min->setValue(hwf->get_int("nim_maj_min", mf));
    ui->spinBox_trig_nim_max->setValue(hwf->get_int("nim_maj_max", mf));

    ui->doubleSpinBoxPulserFreq->setValue(hwf->get_int("pulser_freq", mf));

    QStringList trigOutText;
    trigOutText << "Logic 0";
    trigOutText << "Logic 1";
    trigOutText << "Spill (gate)";
    trigOutText << "XOFF (dead time)";
    trigOutText << "Trigger";
    trigOutText << "Trigger & Spill";
    trigOutText << "Pulser (Hz)";
    trigOutText << "Pulser (Noise)";
    trigOutText << "BRIC strobe (10 MHz)";

    ui->comboBoxTrigOut1->clear();
    ui->comboBoxTrigOut1->insertItems(0, trigOutText);
    ui->comboBoxTrigOut1->setCurrentItem(hwf->get_int("out_sel_1", mf));
    ui->comboBoxTrigOut2->clear();
    ui->comboBoxTrigOut2->insertItems(0, trigOutText);
    ui->comboBoxTrigOut2->setCurrentItem(hwf->get_int("out_sel_2", mf));
    if (const base_mod *m = hwf->find_mod_by_fam_ch_const(MF_TRIG, 0, NULL)) {
        int fw_rev = m->get_fw_rev();
        if (fw_rev != 0 && fw_rev < 12449) {
            qWarning() << QString("TRIG module firmware too old: %1").arg(fw_rev);
            ui->groupBoxTrigOutputs->setEnabled(false);
        }
    }

    display_refresh_trig();

    show_trig_text();

    if (mtdc64_found(*hwf)) {
        ui->tl_trig_spill_rep_text->setText("Pause");
        ui->comboBox_trig_preset->hide();
        ui->frameTriggerLogic2->show();
        ui->table_trig_states->hide();
        ui->groupBoxEnableNim->hide();
        ui->gb_trig_spill_delay->hide();
        ui->gb_trig_delays->hide();
        ui->groupBoxTrigOutputs->hide();
    } else {
        ui->comboBox_trig_preset->show();
        ui->frameTriggerLogic2->hide();
        ui->table_trig_states->show();
    }
//    ui->gb_trig_spill_delay->setEnabled(false);
    ui->gb_trig_spill_delay->hide();
}


void DaqGui::show_trig_text ()
{
    if (!hwf->mf_present(MF_TRIG))
        return;
    QString s;
    s.sprintf("%d ns", static_cast<int>((hwf->get_int("in_shape", MF_TRIG, 3) + 1) * 12.5));
    ui->label_trig_in_shape->setText (s);

    s.sprintf("%.3f s", (hwf->get_int("spill_delay", MF_TRIG) * 1e-3));
    ui->tl_trig_spill_delay->setText (s);
    s.sprintf("%.3f s", (hwf->get_int("spill_duration", MF_TRIG) * 1e-3));
    ui->tl_trig_spill_duration->setText (s);
    s.sprintf("%.3f s", (hwf->get_int("spill_repeat", MF_TRIG) * 1e-3));
    ui->tl_trig_spill_rep->setText (s);

    s.sprintf("%.2f us", ((hwf->get_int("read_delay", MF_TRIG) + TRIG_READ_DELAY_OFFSET) * 1. * TRIG_READ_DELAY_STEP_US));
    ui->tl_trig_delay_read->setText (s);
    s.sprintf("%.3f us", (hwf->get_int("ttc_delay", MF_TRIG) * 1. * TRIG_TTC_DELAY_STEP_US));
    ui->tl_trig_delay_ttc->setText (s);
    s.sprintf("%.3f us", ((hwf->get_int("xoff_wait", MF_TRIG) + TRIG_XOFF_WAIT_OFFSET) * 1. * TRIG_XOFF_WAIT_STEP_US));
    ui->tl_trig_xoff_wait->setText (s);
}

void DaqGui::on_sb_trig_xoff_wait_valueChanged( int value )
{
    if (!hwf->mf_present(MF_TRIG))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "xoff_wait", MF_TRIG);
    show_trig_text ();
}

void DaqGui::on_cb_trig_en_xoff_in_toggled( bool state )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "en_xoff_in", MF_TRIG);
    ui->sb_trig_xoff_wait->setEnabled(state);
}

void DaqGui::on_cb_trig_en_spill_end_wait_toggled( bool state )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "en_spill_end_wait", MF_TRIG);
}

void DaqGui::on_sb_trig_spill_delay_valueChanged( int value )
{
    if (!hwf->mf_present(MF_TRIG))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "spill_delay", MF_TRIG);
    show_trig_text ();
}

void DaqGui::on_sb_trig_spill_duration_valueChanged( int value )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "spill_duration", MF_TRIG);
    show_trig_text ();
}


void DaqGui::on_sb_trig_spill_repeat_valueChanged( int value )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "spill_repeat", MF_TRIG);
    show_trig_text ();
}

void DaqGui::on_rb_trig_spill_ext_clicked()
{
    ui->gb_trig_spill_generator->setEnabled(true);
    if (disable_cfg_slots)
        return;
    hwf->set_int(TRIG_SPILL_EXT, "spill_select", MF_TRIG);
}

void DaqGui::on_rb_trig_spill_timer_clicked()
{
    ui->gb_trig_spill_generator->setEnabled(true);
    if (disable_cfg_slots)
        return;
    hwf->set_int(TRIG_SPILL_TIMER, "spill_select", MF_TRIG);
}

void DaqGui::on_rb_trig_spill_gate_clicked()
{
    ui->gb_trig_spill_generator->setEnabled(false);
    if (disable_cfg_slots)
        return;
    hwf->set_int(TRIG_SPILL_GATE, "spill_select", MF_TRIG);
}

void DaqGui::on_gb_test_spill_select_clicked( int value )
{
    Q_UNUSED(value);
    /*
  if (disable_cfg_slots)
    return;
  hwf->set_int(value, "spill_select", MF_TRIG);
*/
}

void DaqGui::on_sb_trig_delay_ttc_valueChanged( int value )
{
    if (!hwf->mf_present(MF_TRIG))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "ttc_delay", MF_TRIG);
    show_trig_text ();
}

void DaqGui::on_sb_trig_delay_read_valueChanged( int value )
{
    if (!hwf->mf_present(MF_TRIG))
        return;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "read_delay", MF_TRIG);
    show_trig_text ();
}

void DaqGui::sb_trig_delay_valueChanged( int n, int value)
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "delay", MF_TRIG, n);
    show_trig_text ();
}

void DaqGui::on_spinBox_trig_in_shape_valueChanged(int value)
{
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(value, "in_shape", mf);
    show_trig_text();
}

void DaqGui::cb_trig_en_toggled( int n, bool state)
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "en", MF_TRIG, n);
    display_refresh_trig_table();
}

void DaqGui::on_cb_trig_en_noise_toggled(bool checked)
{
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(checked, "noise_trigger", mf);
}

void DaqGui::on_cb_trig_en_pulser_toggled(bool checked)
{
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(checked, "pulser_trigger", mf);
}

void DaqGui::on_comboBox_trig_preset_activated( int )
{
    QString name = ui->comboBox_trig_preset->currentText();
    trig_preset_map_t::ConstIterator p = trig_presets.find(name);
    if (p != trig_presets.end()) {
        unsigned int logic = p.data();
        hwf->set_int(logic, "logic", MF_TRIG);
        display_refresh_trig_table();
    }
}

void DaqGui::on_spinBox_DCH_window_adj_valueChanged( int v )
{
    if (disable_cfg_slots)
        return;
    if (v < 0) v = 0;
    if (v > 3) v = 3;
    hwf->set_int(v, "dch_clock_adj", MF_TRIG, SET_ALL_CH);
}

void DaqGui::on_checkBoxTrigSetupPulser_toggled( bool state )
{
    if (disable_cfg_slots)
        return;
    hwf->set_int(state, "pulser_trigger", MF_TRIG);
}

void DaqGui::on_checkBoxTrigSetupLVDS_toggled( bool state )
{
    cb_trig_en_toggled(0, state);
}

void DaqGui::on_checkBoxTrigSetupHit_toggled( bool state )
{
    cb_trig_en_toggled(1, state);
}

void DaqGui::on_checkBoxTrigSetupTLU_toggled( bool state )
{
    cb_trig_en_toggled(2, state);
}

void DaqGui::on_checkBoxTrigSetupNim_toggled( bool state )
{
    cb_trig_en_toggled(3, state);
}

bool DaqGui::get_trig_in_en(int n) const
{
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return false;
    QMap<QString, QVariant> m = hwf->get_var("in_en", mf).toMap();
    return m.value(QString::number(n)).toBool();
}

void DaqGui::set_trig_in_en(int n, bool state)
{
    if (disable_cfg_slots)
        return;
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    QMap<QString, QVariant> m = hwf->get_var("in_en", mf).toMap();
    m[QString::number(n)] = state;
    hwf->set_var(m, "in_en", mf);
}

void DaqGui::on_checkBoxTrigNim1_toggled(bool checked)
{
    set_trig_in_en(1, checked);
}
void DaqGui::on_checkBoxTrigNim2_toggled(bool checked)
{
    set_trig_in_en(2, checked);
}
void DaqGui::on_checkBoxTrigNim3_toggled(bool checked)
{
    set_trig_in_en(3, checked);
}
void DaqGui::on_checkBoxTrigNim4_toggled(bool checked)
{
    set_trig_in_en(4, checked);
}
void DaqGui::on_checkBoxTrigNim5_toggled(bool checked)
{
    set_trig_in_en(5, checked);
}
void DaqGui::on_checkBoxTrigNim6_toggled(bool checked)
{
    set_trig_in_en(6, checked);
}
void DaqGui::on_checkBoxTrigNim7_toggled(bool checked)
{
    set_trig_in_en(7, checked);
}
void DaqGui::on_checkBoxTrigLvds1_toggled(bool checked)
{
    set_trig_in_en(8, checked);
}
void DaqGui::on_checkBoxTrigLvds6_toggled(bool checked)
{
    set_trig_in_en(13, checked);
}
void DaqGui::on_checkBoxTrigLvds10_toggled(bool checked)
{
    set_trig_in_en(17, checked);
}
void DaqGui::on_checkBoxTrigLvds14_toggled(bool checked)
{
    set_trig_in_en(21, checked);
}
void DaqGui::on_checkBoxTrigLvds18_toggled(bool checked)
{
    set_trig_in_en(25, checked);
}
void DaqGui::on_checkBoxTrigLvds22_toggled(bool checked)
{
    set_trig_in_en(29, checked);
}
void DaqGui::on_checkBoxTrigLvds26_toggled(bool checked)
{
    set_trig_in_en(33, checked);
}
void DaqGui::on_checkBoxTrigLvds30_toggled(bool checked)
{
    set_trig_in_en(37, checked);
}

void DaqGui::on_spinBox_trig_nim_min_valueChanged(int arg1)
{
    if (disable_cfg_slots)
        return;
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(arg1, "nim_maj_min", mf);
}

void DaqGui::on_spinBox_trig_nim_max_valueChanged(int arg1)
{
    if (disable_cfg_slots)
        return;
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(arg1, "nim_maj_max", mf);
}

void DaqGui::on_doubleSpinBoxPulserFreq_valueChanged(double arg1)
{
    if (disable_cfg_slots)
        return;
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(arg1, "pulser_freq", mf);
}

void DaqGui::on_comboBoxTrigOut1_currentIndexChanged(int index)
{
    if (disable_cfg_slots)
        return;
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(index, "out_sel_1", mf);
}

void DaqGui::on_comboBoxTrigOut2_currentIndexChanged(int index)
{
    if (disable_cfg_slots)
        return;
    mfam_t mf = MF_TRIG;
    if (!hwf->mf_present(mf))
        return;
    hwf->set_int(index, "out_sel_2", mf);
}

/*******************************************************************************/
/* DCH tab slots */
/*******************************************************************************/

namespace {
QMap<int, int> bric_get_branch_size_limit_map()
{
    QMap<int, int> m;
    for (int i=0; i<16; i++)
        m[i] = 31+32*i;
    return m;
}
}

void DaqGui::restore_dch_cfg ()
{
    mfam_t mf = MF_DCH;
    if (!hwf->mf_present(mf))
        return;
    CfgSlotDisabler csd;
    if (hwf->get_nch_byfam(mf) > 0) {
        int mask = hwf->get_int("win_enable", mf, GET_FIRST_CH);
        int v = -1;
        for (int i=0;i<16;i++)
            if (mask & (1<<i)) {
                v = i;
                break;
            }
        if (mask == 0xFFFF)
            v = -1;
        ui->spinBox_DCH_window->setValue(v);

        int branch_size_limit_index = hwf->get_int("branch_size_limit_index", mf, GET_FIRST_CH);
        QMap<int, int> m = bric_get_branch_size_limit_map();
        ui->comboBoxDchCardsLimit->clear();
        Q_FOREACH(int index, m.keys()) {
            int limit = m[index];
            ui->comboBoxDchCardsLimit->insertItem(QString::number(limit), index);
        }
        ui->comboBoxDchCardsLimit->setCurrentItem(branch_size_limit_index);
        ui->textLabelDchCardsLimit->setEnabled(false);
        ui->comboBoxDchCardsLimit->setEnabled(false);
    }
}

void DaqGui::on_comboBoxDchCardsLimit_activated( int value )
{
    hwf->set_int(value, "branch_size_limit_index", MF_DCH, SET_ALL_CH);
}

void DaqGui::on_spinBox_DCH_window_valueChanged( int v )
{
    if (disable_cfg_slots)
        return;
    int mask = 1<<v;
    if (v == -1) mask = 0xFFFF; // special value
    hwf->set_int(mask, "win_enable", MF_DCH, SET_ALL_CH);
}

/*******************************************************************************/
/* MSC tab slots */
/*******************************************************************************/

void DaqGui::restore_msc_cfg ()
{
    mfam_t mf = MF_MSC;
    if (!hwf->mf_present(mf))
        return;
    CfgSlotDisabler csd;
    if (hwf->get_nch_byfam(mf) <= 0) return;
    int min_thr = hwf->get_int("min_thr", mf);
    int max_thr = hwf->get_int("max_thr", mf);
    int thr = hwf->get_int("thr", mf);
    ui->spinBoxMscThr->setMinimum(min_thr);
    ui->spinBoxMscThr->setMaximum(max_thr);
    ui->spinBoxMscThr->setValue(thr);
}

void DaqGui::on_spinBoxMscThr_valueChanged(int value)
{
    mfam_t mf = MF_MSC;
    if (disable_cfg_slots)
        return;
    hwf->set_int(value, "thr", mf);
}

/*******************************************************************************/
/* Log Viewer tab slots */
/*******************************************************************************/


void DaqGui::init_logview()
{
    ui->teLog->setTextFormat(Qt::LogText);
    QPalette palette = ui->teLog->palette();
    palette.setColor(QPalette::Base, Qt::black);
    ui->teLog->setPalette(palette);
    ui->teLog->document()->setMaximumBlockCount(1000);
    //    ui->tabMain->removePage(ui->TabLog); // TODO: logview
    /*
  using namespace ipclog;
  extern struct shm_usrlog_t *shm_usrlog;

  if (shm_usrlog)
    logview_sptr = shm_usrlog->head;
  else
        logview_sptr = 0;
#if QT_VERSION >= 0x030300
  teLog->setMaxLogLines(LOGBUFFER_SIZE);
#endif
  reset_logview();
*/
}

void DaqGui::reset_logview()
{
    ui->teLog->clear();
}

void DaqGui::logview_add_line(const QtMessage &message)
{
    //    QString style_open, style_close;
    QColor color(Qt::gray);
    switch (message.type) {
    case QtFatalMsg:
        color = Qt::magenta;
        break;
    case QtCriticalMsg:
        color = Qt::red;
        break;
    case QtWarningMsg:
        color = Qt::yellow;
        break;
    case QtDebugMsg:
        break;
    }

    QString stimestamp = message.date.toString("hh:mm:ss.zzz");

    ui->teLog->setColor(color);
    ui->teLog->append(stimestamp + " " + message.text);
}

void DaqGui::on_cb_logview_debug_toggled( bool state )
{
    logview_level_debug = state;
}

void DaqGui::on_cb_logview_info_toggled( bool state )
{
    logview_level_info = state;
}

void DaqGui::on_cb_logview_warn_toggled( bool state )
{
    logview_level_warn = state;
}

void DaqGui::on_pb_logview_clear_clicked()
{
    reset_logview();
}

/*******************************************************************************/
/* Histogram tab slots */
/*******************************************************************************/

void DaqGui::init_hists ()
{
    ui->groupBox_setRanges->setChecked(false);
    ui->spinBox_ySens->setMinValue(1);
    ui->spinBox_ySens->setMaxValue(20);
    ui->spinBox_ySens->setValue(20);
    //  spinBox_ySens->hide();
    //  textLabel_ySens->hide();

    hmgr->create_hists();
/*
    const th1_list_t hl = hmgr->get_hist_list ();

    ui->cb_hist_1->clear();

    const QString &currentHistName = cfg->getCurrentHist();
    int index = 0;
    Q_FOREACH(const th1Ptr h1, hl) {
        Q_ASSERT(h1);
        hists.append(HistDialogPtr(new HistDialog(h1, this)));
        QString name;
        if (h1) name = h1->getname();
        QString itemText = name;
        ui->cb_hist_1->insertItem(index, itemText);
        if (name == currentHistName)
            ui->cb_hist_1->setCurrentIndex(index);
        index++;
    }
    ui->tl_hist_1->clear();
    */
}

void DaqGui::init_hist_controls ()
{
    /*
  HistDialog *histWidget;

  cb_hist_1->clear();
  for (histWidget = hists->first (); histWidget; histWidget = hists->next ()) {
        const th1Ptr h1 = histWidget->get_h();
        if (!h1) continue;
        const std::string &name = histWidget->getname();
    QString itemText = name;
        if (currentAccessLevel < h1->getAccessLevel())
          continue;
        cb_hist_1->insertItem(itemText);
  }
  cb_hist_1->setCurrentItem (0);
*/
}

void DaqGui::displayHist(const th1 &h1)
{
    QString histName = h1.getname();
    cfg->setCurrentHist(histName);

//    HistDialogPtr histWidget = findhistbyname (text);
//    if (!histWidget) return;
//    const QString histName = histWidget->getHistName();
//    if (!histName.isEmpty())
//        cfg->setCurrentHist(histName);
//    const th1Ptr h1 = histWidget->get_h();
//    if (!h1) return;

//    HistPlot *histPlot = histWidget->getplot();
//    if (!histPlot) return;
/*
    const hist_pt px(histPlot->getpointsx());
    const hist_pt py(histPlot->getpointsy());
    ui->hist_plot_1->setpoints(px, py);

    ui->spinBox_xMin->setMaxValue(px.size());
    ui->spinBox_xMax->setMaxValue(px.size());

    if (! ui->groupBox_setRanges->isChecked()) {
        ui->hist_plot_1->set_visible_bin_range(0, 0);
        ui->spinBox_xMin->setValue(ui->hist_plot_1->get_ar_imin());
        ui->spinBox_xMax->setValue(ui->hist_plot_1->get_ar_imax());
    } else {
        ui->hist_plot_1->set_visible_bin_range(ui->spinBox_xMin->value(), ui->spinBox_xMax->value());
        //  	  hist_plot_1->set_x_scale(h1->bincenter(spinBox_xMin->value()), h1->bincenter(spinBox_xMax->value()));
        //  	  hist_plot_1->set_y_scale(0, spinBox_ySens->value());
    }
    double maxv = ui->spinBox_ySens->maxValue();
    double minv = ui->spinBox_ySens->minValue();
    double x = (maxv - ui->spinBox_ySens->value() + 1) / (maxv-minv);
    double v = exp(7.*x)/exp(7.);
    ui->hist_plot_1->set_autorange_threshold(v/10); // 1..20, 0.025 optimal
    ui->hist_plot_1->autorange();

    QString xlabel = h1->getXLabel();
    QString ylabel = h1->getYLabel();
    QString hname = h1->getname();

    ui->hist_plot_1->setAxisTitle(QwtPlot::xBottom, xlabel);
    ui->hist_plot_1->setAxisTitle(QwtPlot::yLeft, ylabel);
    ui->hist_plot_1->setTitle(hname);

    ui->hist_plot_1->replot();

    */

//    QString title = QString("<b>") + h1.getname() + "</b><br><br>";
//    QString label;
//    label.append(title);
//    label.append(QString("Ent: %1<br>").arg(h1.get_nent()));
//    label.append(QString("Under: %1<br>").arg(h1.get_under()));
//    label.append(QString("Over: %1<br>").arg(h1.get_over()));
//    ui->tl_hist_1->setText(label);

    // print description
    QString desc;
    desc += h1.getDescription();
    ui->textBrowserHistDescription->setText(desc);
}

void DaqGui::on_cb_hist_1_activated(int index)
{
    Q_UNUSED(index);
    requestCurrentHist();
}

void DaqGui::requestCurrentHist()
{
    int index = ui->cb_hist_1->currentIndex();
    QString id = histIndexes[index];
    if (!id.isNull())
        emit requestHist(id);
}

void DaqGui::on_pb_show_this_clicked()
{
    HistDialog *histDialog = new HistDialog(this);
    connect(histDialog, SIGNAL(), SLOT());
//    QString text = ui->cb_hist_1->currentText();
//    HistDialogPtr histWidget = findhistbyname (text);
//    if (histWidget)
//        histWidget->show();
}

void DaqGui::on_pb_hide_all_clicked()
{
//    Q_FOREACH(HistDialogPtr histWidget, hists) {
//        histWidget->hide();
//    }
}

void DaqGui::on_cb_hist_downscale_activated( const QString & text)
{
    int new_value = text.toInt();
    if (new_value != cfg->get_hist_downscale()) {
        cfg->set_hist_downscale(new_value);
    }
}

void DaqGui::on_checkBox_clearByBurst_toggled( bool new_value )
{
    if (new_value != cfg->get_hist_clear_by_burst()) {
        cfg->set_hist_clear_by_burst(new_value);
    }
}

void DaqGui::on_spinBox_xMax_valueChanged( int )
{
    requestCurrentHist();
}

void DaqGui::on_spinBox_xMin_valueChanged( int )
{
    requestCurrentHist();
}

void DaqGui::on_groupBox_setRanges_toggled( bool state )
{
    //  if (!state) { // auto -> manual
    //	double xmin, xmax;
    //	hist_plot_1->get_x_scale(&xmin, &xmax);
    //  	spinBox_xMin->setValue(hist_plot_1->get_ar_imin());
    //  	spinBox_xMax->setValue(hist_plot_1->get_ar_imax());
    //  }
    ui->spinBox_ySens->setEnabled(!state);
    requestCurrentHist();
}

void DaqGui::on_spinBox_ySens_valueChanged( int value )
{
    Q_UNUSED(value);
    //  hist_plot_1->set_y_sens(value);
    requestCurrentHist();
}

bool DaqGui::kill_core(int signal)
{
#if defined(Q_OS_UNIX)
    pid_t pid = shm->getDaemonPid();
    if (pid==0) {
        QMessageBox::warning (this, "Error", "Core is not running");
        return false;
    }
    switch (QMessageBox::
            warning (this, "Confirm action",
                     "Do you really want to kill core process?",
                     QMessageBox::Yes, QMessageBox::No)) {
    case QMessageBox::Yes: {
        int ret = -1;
        ret = kill(pid, signal);
        if (ret != 0) {
            QMessageBox::warning (this, "Error", QString("Can't send signal to core process: %1").arg(strerror(errno)));
            return false;
        }
        return true; // successfully sent signal
    }
    break;
    }
#endif
    return false;
}

void DaqGui::update_TabStatistics()
{
    QString s = QString::fromStdString(eventStatsSum->getHtml());
    ui->textBrowserStatistics->setText(s);
}

void DaqGui::update_TabRawData()
{
    if (!shm->shm_pio_rawdata) {
        ui->textBrowserRawData->setText(QString());
        return;
    }
    QString s;
    size_t total_count = shm->shm_pio_rawdata->count;
    if (total_count <= 0) return;

    size_t dump_size = 1024; // default
    if (dump_size > total_count)
        dump_size = total_count;

    // defaults
    size_t start = ui->spinBoxRawStartOffset->value();
    if (start > total_count) start = total_count;
    size_t finish = start + dump_size;
    if (finish > total_count) finish = total_count;

    localWaitCursor lwc(this);

    //    for (size_t i=start; i<finish; i++) {
    //        s += QString::fromStdString(daq_dbuf_str (shm->shm_pio_rawdata->data, i))+"\n";
    //    }
    RawDumper dumper;
    s += dumper.dump_raw_str(shm->shm_pio_rawdata->data, start, finish, -1, false);
    ui->textBrowserRawData->setFont(QFont("Courier"));
    ui->textBrowserRawData->setText(s);
}

void DaqGui::on_spinBoxRawStartOffset_valueChanged( int )
{
    update_TabRawData();
}

void DaqGui::on_pb_clear_all_2_clicked()
{
    clearstats ();
}

void DaqGui::on_pb_clear_hist_1_clicked()
{
    clearstats ();
}

void DaqGui::qtMessage(const QtMessage &message)
{
    logview_add_line(message);
}

void DaqGui::on_actionExit_triggered()
{
    close();
}

void DaqGui::on_actionAbout_triggered()
{
    QString astr = QString("<p>")+oemstr.name()+QString("<br>")+
            oemstr.version() +", "+ oemstr.revision() + "<br><br>" +
            oemstr.copyright() +"<br><br>" +
            oemstr.address() +"<br><br>" +
            "<a href=\""+oemstr.www()+"\">"+oemstr.www()+"</a>"+
            "</p>";
    QMessageBox::about (this, "About", astr);
}

void DaqGui::on_actionVMEDAQ_Manual_triggered()
{
    QDesktopServices::openUrl(QUrl("http://afi.jinr.ru/VMEDAQ"));
}

void DaqGui::on_actionCheck_for_updates_triggered()
{
    QDesktopServices::openUrl(QUrl("http://afi.jinr.ru/Downloads"));
}

void DaqGui::on_actionAbout_Qt_triggered()
{
    qApp->aboutQt();
}

void DaqGui::on_action_Triggers_triggered()
{
    triggersView->show();
}

void DaqGui::on_actionInterrupt_core_triggered()
{
#if defined(Q_OS_UNIX)
    kill_core(SIGUSR1);
#endif
}

void DaqGui::on_actionKill_core_triggered()
{
#if defined(Q_OS_UNIX)
    bool st = kill_core(SIGTERM);
    if (!st) return; // failed
    int n_timeout = 5000;
    Q3ProgressDialog pg(tr("Waiting for core to terminate..."), tr("Cancel"), n_timeout, this, "progress", true);
    int n_elapsed = 0;
    while (true) {
        if (shm->getDaemonPid() == 0) {
            pg.cancel();
            return; // ok
        }
        pg.setProgress(n_elapsed);
        if (pg.wasCanceled()) return;
        qApp->processEvents(QEventLoop::AllEvents, 10);
        if (n_elapsed > n_timeout) {
            pg.cancel();
            if (QMessageBox::warning (this, "Kill core",
                                      QString("Core does not respond. Terminate it immediately?"),
                                      QMessageBox::Yes, QMessageBox::No)
                    != QMessageBox::Yes) return;
        }
        n_elapsed += 10;
    }
    kill_core(SIGKILL);
#endif
}

void DaqGui::on_actionInitialize_Settings_triggered()
{
    if (QMessageBox::warning (this, "Initialize Database",
                              QString("All settings will be set to defaults. Continue?"),
                              QMessageBox::Yes, QMessageBox::No)
            != QMessageBox::Yes) return;

    release_hw ();

    reset_logview();
    shm->setRunparam(runparam_t());
    restore_runparam();

    send_command (DAQCTL_CMD_INITIALIZE_DATABASE);
    send_command (DAQCTL_CMD_INIT);
    clearstats ();
    open_hw ();
}

void DaqGui::on_groupBoxHistograms_toggled(bool checked)
{
    if (disable_cfg_slots)
        return;
    cfg->set_hist_en(checked);
}

void DaqGui::on_actionSave_Calibrations_triggered()
{
    hmgr->saveCalibrations();
}

void DaqGui::readSettings()
{
    QSettings settings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), "vmegui", this);
    settings.beginGroup("MainWindow");
    restoreGeometry(settings.value("geometry").toByteArray());
    restoreState(settings.value("windowState").toByteArray());
    ui->tabMain->setCurrentIndex(settings.value("tabMain.index").toInt());
    settings.endGroup();
}

void DaqGui::writeSettings()
{
    QSettings settings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), "vmegui", this);
    settings.beginGroup("MainWindow");
    settings.setValue("geometry", saveGeometry());
    settings.setValue("windowState", saveState());
    settings.setValue("tabMain.index", ui->tabMain->currentIndex());
    settings.endGroup();
}

void DaqGui::updateHistList(const HistList &hl)
{
    histIndexes.clear();
    int index = 0;
    QString currentText = ui->cb_hist_1->currentText();
    int currentIndex = -1;
    QStringList cbItems;
    Q_FOREACH(HistListItem item, hl) {
        histIndexes[index] = item.id;
        cbItems << item.title;
        if (currentText == item.title)
            currentIndex = index;
        index++;
    }

    ui->tl_hist_1->clear();
    ui->cb_hist_1->clear();
    ui->cb_hist_1->insertStringList(cbItems);
    if (cbItems.contains(currentText))
        ui->cb_hist_1->setCurrentIndex(currentIndex);
}
