//
//    Copyright 2003-2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DAQGUI_H
#define DAQGUI_H

#include <QtCore/QtGlobal>
#include <QMainWindow>
//#include <QSharedPointer>
//#include <QList>
#include <QMap>

#include "daqcommon/daq_ipc.h"
#include "daqcommon/HistTypes.h"

namespace Ui {
    class daqguibase;
}

class event_t;
class TriggersView;
class DaqCfg;
//class HistDialog;
class base_decoder;
class hwf;
class histmgr;
class EventStatsSum;
class hw_frontend;
class th1;
class ShmSlaveImpl;
class QtMessageUdpReceiver;
class QtMessage;
class QtMessageHandler;
class QTimer;
//class QSqlDatabase;

//typedef QSharedPointer < HistDialog > HistDialogPtr;

class DaqGui : public QMainWindow
{
    Q_OBJECT
public:
    explicit DaqGui(QWidget* parent = 0);
    virtual ~DaqGui();
    void initialize_gui();

signals:
    void requestHist(const QString &);

private:
//    QString getDbFileName() const;
//    bool dbOpen(const QString &fileName);
    void release_hw();
    void open_hw();
    void release_hw_preaction();
    void open_hw_postaction();
    void set_control_ranges();
    void init_tab_tqdc();
    void init_plots();
    void cleardisplay();
    void endburst();
    void burstreset();
    void display_daq_state();
    void update_resourcemon();
    void update_current_tab();
    int getevents( int evfirst, int evlast );
    void replot_tqdc_waveform();
    void replot_tdc();
    void replot_msc();
    void restore_all();
    void restore_main_cfg();
    int send_command(ctl_magic_t cmd);
    void process_decoded_event();
    void clearstats();
    void set_runparam_defaults();
    void restore_runparam();
    void update_runstate();
    void replot_tbl_hw();
    void display_refresh_tbl_hw();
    void init_tbl_hw();
    void count_module_event_stats(const event_t &ev);
    void count_module_burst_stats();
    void set_tbl_hw();
    void clear_tbl_hw();
    void init_tbl_tdc();
    void set_tbl_tdc();
    void clear_tbl_tdc();
    void display_refresh_tbl_tdc();
    void set_control_ranges_tqdc();
    void restore_tqdc_cfg();
    void setup_tqdc_select();
    void replot_tqdc();
    void display_refresh_tbl_tqdc();
    void init_tbl_tqdc();
    void set_tbl_tqdc();
    void clear_tbl_tqdc();
    void init_tbl_trig();
    void display_refresh_trig_table();
    void display_refresh_trig_counts();
    void display_refresh_trig_preset();
    void display_refresh_trig();
    void restore_trig_cfg();
    void show_trig_text();
    bool get_trig_in_en(int n) const;
    void set_trig_in_en(int n, bool state);
    void cb_trig_en_toggled( int n, bool state );
    void cb_trig_logic_toggled( int x, int y, bool state );
    void restore_dch_cfg();
    void restore_msc_cfg();
    void init_logview();
    void reset_logview();
    void logview_add_line(const QtMessage &message);
    void init_hists();
    void init_hist_controls();
//    void update_hist();
    bool kill_core( int signal );
    void restore_tqdc_neg_buttons();
    void update_TabStatistics();
    void update_TabRawData();
    void sb_trig_delay_valueChanged( int n, int value );
    void replot_runstats();
    void rescale_tdc_hists();
    void rescale_tqdc_hists();
    void requestCurrentHist();
    bool mayTerminateRun();
    bool setDataStorage();

private Q_SLOTS:
    void on_pb_clear_hist_1_clicked();
    void on_cb_hist_1_activated(int index);
    void on_spinBoxCommonStop_valueChanged(int value);
    void savecfg();
    void tdcResolutionChanged();
    void tdcDataFormatChanged();
    void dataFormatChanged();
    void handle_events();
    void updatedisplay();
    void replot_spr();
    void on_pbInit_clicked();
    void on_pbReset_clicked();
    void on_pbPrep_clicked();
    void on_pbRun_clicked();
    void on_pbPause_clicked();
    void on_pb_startdaq_clicked();
    void on_tabMain_currentChanged( QWidget * );
    void on_tbl_hw_valueChanged( int r, int c );
    void on_tbl_tdc_valueChanged( int r, int c );
    void on_sb_tqdc_mw_valueChanged( int value );
    void on_sb_tqdc_lat_valueChanged( int value );
    void on_tbl_tqdc_valueChanged( int r, int c );
    void on_cb_tqdc_twoedge_toggled( bool state );
    void on_sb_tqdc_adcwin_valueChanged( int value );
    void on_sb_tqdc_adcsrch_valueChanged( int value );
    void on_sb_tqdc_adclat_valueChanged( int value );
    void on_sb_tqdc_thr_valueChanged( int value );
    void on_cb_tqdc_match8_toggled( bool state );
    void on_sb_tqdc_maj_valueChanged( int value );
    void on_sb_tqdc_min_valueChanged( int value );
    void on_cb_tqdc_en1_toggled( bool state );
    void on_cb_tqdc_en2_toggled( bool state );
    void on_sb_tqdc_shape_valueChanged( int value );
    void update_tqdc_tr_shape();
    void on_sb_tqdc_alt1_valueChanged( int value );
    void on_sb_tqdc_alt2_valueChanged( int value );
    void on_rb_tqdc_baseline_toggled(bool value);
    void on_rb_tqdc_waveform_toggled(bool value);
    void on_rb_tqdc_integrator_toggled(bool value);
//    void on_bg_tqdc_mode_clicked( int value );
    void on_cb_tqdc_res_activated( int item );
    void on_sb_tqdc_match_valueChanged( int value );
    void on_sb_tqdc_latency_valueChanged( int value );
    void on_comboBox_tqdc_select_activated( int index );
    void on_checkBox_tqdc_neg_a_toggled( bool value );
    void on_checkBox_tqdc_neg_b_toggled( bool value );
    void on_sb_trig_xoff_wait_valueChanged( int value );
    void on_cb_trig_en_xoff_in_toggled( bool state );
    void on_cb_trig_en_spill_end_wait_toggled( bool state );
    void on_sb_trig_spill_delay_valueChanged( int value );
    void on_sb_trig_spill_duration_valueChanged( int value );
    void on_sb_trig_spill_repeat_valueChanged( int value );
    void on_rb_trig_spill_ext_clicked();
    void on_rb_trig_spill_timer_clicked();
    void on_rb_trig_spill_gate_clicked();
    void on_gb_test_spill_select_clicked( int value );
    void on_sb_trig_delay_ttc_valueChanged( int value );
    void on_sb_trig_delay_read_valueChanged( int value );
    void on_comboBox_trig_preset_activated( int );
    void on_spinBox_DCH_window_adj_valueChanged( int v );
    void on_checkBoxTrigSetupPulser_toggled( bool state );
    void on_checkBoxTrigSetupHit_toggled( bool state );
    void on_checkBoxTrigSetupTLU_toggled( bool state );
    void on_checkBoxTrigSetupLVDS_toggled( bool state );
    void on_checkBoxTrigSetupNim_toggled( bool state );
    void on_comboBoxDchCardsLimit_activated( int value );
    void on_spinBox_DCH_window_valueChanged( int v );
    void on_cb_logview_debug_toggled( bool state );
    void on_cb_logview_info_toggled( bool state );
    void on_cb_logview_warn_toggled( bool state );
    void on_pb_logview_clear_clicked();
    void on_pb_show_this_clicked();
    void on_pb_hide_all_clicked();
    void on_cb_hist_downscale_activated( const QString &text );
    void on_checkBox_clearByBurst_toggled( bool new_value );
    void on_spinBox_xMax_valueChanged( int );
    void on_spinBox_xMin_valueChanged( int );
    void on_groupBox_setRanges_toggled( bool state );
    void on_spinBox_ySens_valueChanged( int value );
    void on_spinBoxRawStartOffset_valueChanged( int );
    void on_pb_clear_all_2_clicked();
    void qtMessage(const QtMessage &message);

    void on_actionExit_triggered();
    void on_actionAbout_triggered();
    void on_actionVMEDAQ_Manual_triggered();
    void on_actionCheck_for_updates_triggered();
    void on_actionAbout_Qt_triggered();
    void on_action_Triggers_triggered();
    void on_actionInterrupt_core_triggered();
    void on_actionKill_core_triggered();
    void on_actionInitialize_Settings_triggered();

    void on_groupBoxHistograms_toggled(bool );
    void on_groupBoxDecoder_toggled( bool newstate );
    void updateHistogramControls();

    void on_sb_tqdc_rc0_valueChanged(int value);
    void on_sb_tqdc_rc1_valueChanged(int value);
    void on_sb_tqdc_rc2_valueChanged(int value);
    void on_sb_tqdc_rc3_valueChanged(int value);
    void off_sb_tqdc_rcx_valueChanged(int n, int value);

    void on_actionSave_Calibrations_triggered();
    void displayHist(const th1 &);
    void updateHistList(const HistList &);

    void on_spinBox_trig_in_shape_valueChanged(int arg1);
    void on_spinBox_trig_nim_min_valueChanged(int arg1);
    void on_spinBox_trig_nim_max_valueChanged(int arg1);
    void on_checkBoxTrigNim1_toggled(bool checked);
    void on_checkBoxTrigNim2_toggled(bool checked);
    void on_checkBoxTrigNim3_toggled(bool checked);
    void on_checkBoxTrigNim4_toggled(bool checked);
    void on_checkBoxTrigNim5_toggled(bool checked);
    void on_checkBoxTrigNim6_toggled(bool checked);
    void on_checkBoxTrigNim7_toggled(bool checked);
    void on_checkBoxTrigLvds1_toggled(bool checked);
    void on_checkBoxTrigLvds6_toggled(bool checked);
    void on_checkBoxTrigLvds10_toggled(bool checked);
    void on_checkBoxTrigLvds14_toggled(bool checked);
    void on_checkBoxTrigLvds18_toggled(bool checked);
    void on_checkBoxTrigLvds22_toggled(bool checked);
    void on_checkBoxTrigLvds26_toggled(bool checked);
    void on_checkBoxTrigLvds30_toggled(bool checked);

    void on_doubleSpinBoxPulserFreq_valueChanged(double arg1);

    void on_cb_trig_en_noise_toggled(bool checked);

    void on_cb_trig_en_pulser_toggled(bool checked);

    void on_comboBoxTrigOut1_currentIndexChanged(int index);

    void on_comboBoxTrigOut2_currentIndexChanged(int index);

    void on_spinBoxMscThr_valueChanged(int value);

private:
    void donehists();
    QString TrigEventStatsText(const QVector<double> &h_trig_incnt, const QVector<double> &h_burst_history, const QVector<double> &h_trig_bits) const;
//    HistDialogPtr findhistbyname (const QString &name);
    // save/restore main window state
    void readSettings();
    void writeSettings();

private:
    Ui::daqguibase * const ui;
    ShmSlaveImpl * const shm;
    int logview_sptr;
//    QSqlDatabase * const db;
    hw_frontend * const hwf;
    base_decoder * const decoder;
    histmgr * const hmgr;
    EventStatsSum * const eventStatsSum;
    TriggersView * const triggersView;
    DaqCfg * const cfg;
//    QList<HistDialogPtr> hists;
    QtMessageUdpReceiver * const logReceiver;
    QTimer * const timerUpdateDisplay;
    QTimer * const timerHandleEvents;
    QTimer * const timerReplotSpill;
    QTimer * const timerAutosave;
    QMap<int, QString> histIndexes;
};

#endif // DAQGUI_H
