#ifndef HISTMGR_H
#define HISTMGR_H

#include "hal/mf_types.h"
#include "daqcommon/th1.h"
#include "hptdc/hptdc_types.h"
#include "daqcommon/HistTypes.h"

#include <QtCore/QtGlobal>
#include <QSharedPointer>
#include <QList>
#include <QMap>
#include <QString>
#include <vector>

//class th1;
typedef QSharedPointer<th1> th1Ptr;

typedef QMap<HistId, th1Ptr> th1_list_t;
typedef QMap<HistId, const th1Ptr> th1_list_ct;
typedef th1_list_t::iterator th1_list_it;
typedef th1_list_t::const_iterator th1_list_cit;
typedef QMap<int, double> hptdc_inl_corr_t;

class DaqCfg;
class event_t;
class hitsorter;
class hw_frontend;
class tqdc_calibration_set;
class HistDescr;
class CalibrationData;

class histmgr: public QObject
{
    Q_OBJECT

public:

  histmgr(const hw_frontend &p, QObject *parent = 0);
  ~histmgr();
  void create_hists();
  void delete_hists();
  void rescale_hists ();
  th1_list_t get_hist_list() const;
  bool get_hist_enable(const HistId &id) const;
  void burstreset(const DaqCfg &cfg);
  void endburst(const DaqCfg &cfg);
  void reset();
  void clear();
  void hold(bool state);
  const th1Ptr getHist (const HistId &id) const;
  const th1Ptr getHistByName (const QString &id) const;
//  void fill (const DaqCfg &cfg, const std::string &name, double value, size_t count=1);
  void fill(const event_t &ev, const DaqCfg &cfg);
  double getbincontent(const HistId &id, int bin) const;
  double get_calibration_constant(const std::string &id, int ch=0) const;
  void update_descriptions();
  void dataFormatChanged();
  bool isCalibrationEnabled() const;

  unsigned int tdc_common_stop_channel;

signals:
  void histAvailable(const th1 &);
  void histListChanged(const HistList &);

public slots:
  void read_corrections();
  void saveCalibrations();
  void requestHist(const QString &id);

protected:
  th1Ptr find (const HistId &id) { return hl[id];}
//  th1Ptr find_by_name (const std::string &name);
  void get_tdc_params(mfam_t mf, TdcResIndex *fres, TdcResIndex *fpres, int *fwin) const;
  void rescale_hist(const HistId &name, size_t new_nbins, double new_xmin, double new_xmax);
  QList<HistDescr> getHistDescrs() const;
  void fillHist(const HistId &id, double value, double count = 1);
  void fillHist(const HistId &id, const std::vector<double> &values, double count = 1);
  void fillHist(const HistId &id, const std::map<double, double> &values);

  void fillGeneral(const event_t &ev, int dsf);
  void fillBRIC(const event_t &ev, int dsf);
  void fillTRIG(const event_t &ev, int dsf);
  void fillTDC(const event_t &ev, int dsf);
  void fillTQDC(const event_t &ev, int dsf);


//  int write();
//  hptdc_inl_corr_t read_hptdc_inl_corr(const QString &filename);
  void setInlCorr(const QString &boardName, quint64 serialId, int channel, const QVector<double> &data);
  hptdc_inl_corr_t getInlCorr(const QString &boardName, quint64 serialId = 0, int channel = 0);
//  void saveTdcInlCorr(QString tag, const hptdc_inl_corr_t &data);

  const hw_frontend &hwf;
  hitsorter * const hs;
  th1_list_t hl;

  tqdc_calibration_set * const tqdc_cal;
  quint64 save_trig_ts;
  QMap<int, hptdc_inl_corr_t> tdc_inl_corrections;
  QMap<int, hptdc_inl_corr_t> tqdc_inl_corrections;
  CalibrationData * const calibrationData;

protected slots:
#ifdef DEBUG
  void fillDebugHist();
//  void sendDebugHist();
#endif
};

#endif /* HISTMGR_H */
