
#include "stdio.h"

#include "hitsorter.h"

// nc: overall number of channels
hitsorter::hitsorter(int nc)
    : maxc(nc),
    sqbuf(new hit_sq_buf_t[maxc]),
    tbuf(new hit_t_buf_t[maxc]),
    stqbuf(new hit_stq_buf_t[maxc])
{
}

hitsorter::~hitsorter()
{
    delete[] stqbuf;
    delete[] tbuf;
    delete[] sqbuf;
}

void hitsorter::clear()
{
    for (int c=0; c<maxc; c++) {
        stqbuf[c].n= 0;
        sqbuf[c].n = 0;
        tbuf[c].n  = 0;
        stqbuf[c].over = 0;
        sqbuf[c].over  = 0;
        tbuf[c].over   = 0;
    }
}

bool hitsorter::add_hit_t(int c, double t)
{
    if (c >= maxc) return false;
    int nh = tbuf[c].n;
    if (nh >= T_BUF_MAX) {tbuf[c].over++; return false;}
    tbuf[c].hit[nh].t = t;
    tbuf[c].n++;
    return true;
}

bool hitsorter::add_hit_q(int c, double s, double q)
{
    if (c >= maxc) return false;
    int nh = sqbuf[c].n;
    if (nh >= SQ_BUF_MAX) {sqbuf[c].over++; return false;}
    sqbuf[c].hit[nh].s = s;
    sqbuf[c].hit[nh].q = q;
    sqbuf[c].n++;
    return true;
}

bool hitsorter::add_hit_stq(int c, double t, double s, double q)
{
    if (c >= maxc) return false;
    int nh = stqbuf[c].n;
    if (nh >= STQ_BUF_MAX) {stqbuf[c].over++; return false;}
    stqbuf[c].hit[nh].t = t;
    stqbuf[c].hit[nh].s = s;
    stqbuf[c].hit[nh].q = q;
    stqbuf[c].n++;
    //  printf(" + add_hit_stq: ch %d, t=%f, s=%f, q=%f\n", c, t, s, q);
    return true;
}

void hitsorter::sort_hits()
{
    for (int c=0;c<maxc;c++) {
        if (tbuf[c].n <= 0) continue;
        if (sqbuf[c].n <= 0) continue;
        sort_ch(c);
    }
}

void hitsorter::sort_ch(int c)
{
    //  printf("sorter: ch %d: %d T, %d SQ\n", c, tbuf[c].n, sqbuf[c].n);
    for (int nt=0;nt<tbuf[c].n;nt++)
        for (int nsq=0;nsq<sqbuf[c].n;nsq++) {
        double t = tbuf[c].hit[nt].t;
        double s = sqbuf[c].hit[nsq].s;
        double q = sqbuf[c].hit[nsq].q;
        double dt = t-s;
        //	  double dt_min = 0;
        //	  double dt_max = 12;
        double dt_min = -50;
        double dt_max = 50;
        if (dt >= dt_min && dt <= dt_max) {
            //		printf(" + sorter: dt=%f, tt=%f, tq=%f, q=%f\n", dt, tt, tq, q);
            add_hit_stq(c, t, s, q);
        }
    }
}
