#ifndef HITSORTER_H
#define HITSORTER_H

typedef struct {
    float t; // TDC time, ns
    float s; // ADC timestamp, ns
    float q; // ADC charge
} hit_stq_t;
typedef struct {
    float t; // TDC time, ns
} hit_t_t;
typedef struct {
    float s; // ADC timestamp, ns
    float q; // ADC charge
} hit_sq_t;

const int SQ_BUF_MAX = 8;  /* max number of hits in buffer */
const int T_BUF_MAX = 8;   /* max number of hits in buffer */
const int STQ_BUF_MAX = 8; /* max number of hits in buffer */
typedef struct {
    unsigned short n;
    unsigned short over;
    hit_sq_t hit[SQ_BUF_MAX];
} hit_sq_buf_t;
typedef struct {
    unsigned short n;
    unsigned short over;
    hit_t_t hit[T_BUF_MAX];
} hit_t_buf_t;
typedef struct {
    unsigned short n;
    unsigned short over;
    hit_stq_t hit[STQ_BUF_MAX];
} hit_stq_buf_t;

class hitsorter
{
public:
    hitsorter(int nc);
    ~hitsorter();
    void clear();
    bool add_hit_t(int c, double t);
    bool add_hit_q(int c, double s, double q);
    void sort_hits();
    int get_nch() const {return maxc;}
    hit_stq_buf_t *get_stq_buf() {return stqbuf;}

private:
    void sort_ch(int i);
    bool add_hit_stq(int c, double t, double s, double q);

    const int maxc; // overall number of channels
    // input buffer
    hit_sq_buf_t * const sqbuf;
    hit_t_buf_t * const tbuf;
    // output buffer
    hit_stq_buf_t * const stqbuf;
};

#endif /* HITSORTER_H */
