
#include "msc_waveformview.h"

#include <QFile>
#include <QTextStream>
#include <QDebug>

#include "wfdata.h"
#include "foreach.h"
#include "modules/msc_types.h"
#include "decoder/event.h"
#include "decoder/event_types.h"
#include "daqcommon/th1.h"
#include "QDateTimeCompat.h"

const qint64 updatePeriodMs = 300;

MSC_WaveformView::MSC_WaveformView(QWidget *p)
    : WaveformView(p),
    msc_data(new msc_data_t)
{
}

void MSC_WaveformView::add_event(const event_t &ev)
{
    const msc_event_t &msc = ev.msc;
    const msc_data_t &data = msc.data;
    // add new counts
    Q_FOREACH(int ch, data.keys()) {
        QVector<int> src = data[ch];
        QVector<int> &dst = (*msc_data)[ch];
        for(int i=0; i<src.size(); i++)
        dst.append(src[i]);
    }
}

class MscDecodedData : public QMap<int, QVector<int> > {};
/*
MscDecodedData decodeRawData(const msc_data_t &msc_data)
{
//    QDateTime start = QDateTime::currentDateTime();
    MscDecodedData d;
//    int steps = msc_data.size();
    Q_FOREACH(qint64 ts, msc_data.keys()) {
        const msc_counts_t &cnt = msc_data[ts];
        Q_FOREACH(int data_ch, cnt.keys()) {
            d[data_ch].push_back(cnt[data_ch]);
        }
    }
//    QDateTime finish = QDateTime::currentDateTime();
//    qCritical() << QDateTimeMsescTo(start, finish);
    return d;
}
*/
MscDecodedData decodeRawData(const msc_data_t &msc_data)
{
    MscDecodedData d;
    Q_FOREACH(int ch, msc_data.keys()) {
        if (msc_data[ch].empty()) continue;
//        d[ch] = msc_data[ch];
        int n = msc_data[ch].size();
        d[ch].resize(n);
        d[ch][0] = msc_data[ch][0];
        for(int i=1; i<n; i++) {
//            int v = msc_data[ch][i];
            int v = msc_data[ch][i] - msc_data[ch][i-1];
            if (v < 0) v += 128;
            d[ch][i] = v;
        }

//        qDebug() << msc_data[ch];
    }
    return d;
}

WaveformDataAll MSC_WaveformView::prepareWaveform() const
{
    QDateTime start = QDateTime::currentDateTime();
    const MscDecodedData &d = decodeRawData(*msc_data);
    WaveformDataAll wda;

    Q_FOREACH(int ch, d.keys()) {
        const QVector<int> &v = d[ch];
        WaveformData wd;
        wd.c = ch;
        QString id = QString("%1").arg(ch);
        wd.name = id.toStdString();
        wd.p.resize(v.size());
        for(int i=0; i<v.size(); i++) {
            wd.p[i].x = i;
            wd.p[i].y = v[i];
        }
        wda.push_back(wd);
    }
//    QDateTime finish = QDateTime::currentDateTime();
//    qCritical() << QDateTimeMsescTo(start, finish);
    return wda;
}

void MSC_WaveformView::update()
{
    QDateTime current = QDateTime::currentDateTime();
    if (abs(QDateTimeMsescTo(updateTimestamp, current)) > updatePeriodMs) {
        updateTimestamp = current;
        const WaveformDataAll &wda = prepareWaveform();
        setWaveformData(wda);
        WaveformView::update();
    }
}

void MSC_WaveformView::clear()
{
    msc_data->clear();
    WaveformView::clear();
}
