//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef SHMSLAVE_H
#define SHMSLAVE_H

#include <QSharedMemory>

#include "daqcommon/daq_ipc.h"

class QDateTime;
struct runparam_t;
struct runcounts_t;
struct shm_status_t;
struct shm_evlist_t;
struct shm_pio_rawdata_t;

class ShmSlaveStatus
{
public:
    bool runparam_writeable() const { return _runparam_writeable; }
    bool status_writeable() const { return _status_writeable; }

    QString getRunparamFilename() const;
    void setRunparamFilename(const QString &fileName);
    QString getRunparamComment() const;
    void setRunparamComment(const QString &text) const;
    qint64 getStorageBytes() const;
    QDateTime getRunStartTime() const;
    runparam_t getRunparam() const;
    void setRunparam(const runparam_t &rp);
    bool runparamStore() const;
    bool runparamCyclic() const;
    double getLastBurstDuration() const;
    daq_state_t getDaqState() const;
    burst_state_t getBurstState() const;
    int getSpillCount() const;
    runcounts_t getRunCounts() const;
    runcounts_t getBurstCounts() const;
    daq_spr_t getSpr() const;
    double getBurstTime() const;

protected:
    ShmSlaveStatus();

private:
    shm_status_t *allocShmStatus();

    QSharedMemory shmStatus;
    bool _runparam_writeable;
    bool _status_writeable;
    shm_status_t * const shm_status;
};

class ShmSlaveControl
{
public:
    bool ctl_writeable() const { return _ctl_writeable; }

    ctl_magic_t getCurrentControlState() const;
    bool sendControlCommand(ctl_magic_t cmd);
    pid_t getDaemonPid() const;

protected:
    ShmSlaveControl();

private:
    shm_ctl_t *allocShmCtl();

    QSharedMemory shmCtl;
    bool _ctl_writeable;
    shm_ctl_t * const shm_ctl;
};

class ShmSlave : public ShmSlaveStatus, public ShmSlaveControl
{
public:
    ShmSlave();

protected:
    shm_evlist_t *attachEvlist();
    shm_pio_rawdata_t *attachRawdata();

    QSharedMemory shmEvlist;
    QSharedMemory shmRawdata;

public:
    shm_pio_rawdata_t * const shm_pio_rawdata;
    shm_evlist_t * const shm_evlist;

protected:
};

class ShmSlaveImpl : public ShmSlave
{
public:
    bool controlBusy() const;

};

#endif // SHMSLAVE_H
