
#include "startdaq.h"

#include <QFileDialog>
#include <QLineEdit>
#include <QCheckBox>
#include <QPushButton>
#include <QDateTime>

#include "daqcommon/daq_ipc.h"

#include "ui_startdaqbase.h"

StartDAQ::StartDAQ (QWidget* parent)
    : QDialog(parent),
    ui(new Ui::StartDAQBase),
    data(new runparam_t)
{
    ui->setupUi(this);
}

StartDAQ::~StartDAQ ()
{
    delete data;
    delete ui;
}

void StartDAQ::on_pb_Browse_clicked ()
{
    le_fn_accept();
    le_rd_accept();
    QString fileName = QFileDialog::getSaveFileName(
                this,
                tr("Save Data File As..."),
                QString::fromLocal8Bit(data->fn),
                "Data files (*.dat);;RAW  files (*.raw);;Any files (*)");
    if (!fileName.isEmpty()) {
        if (!fileName.contains("."))
            fileName += ".dat";
        qstrncpy(data->fn, fileName.toLocal8Bit(), sizeof(data->fn));
        ui->le_fn->setText (data->fn);
    }
}

void StartDAQ::le_fn_accept ()
{
    qstrncpy(data->fn, ui->le_fn->text (), sizeof(data->fn));
}

void StartDAQ::le_rd_accept ()
{
    qstrncpy(data->rundesc, ui->le_rd->text (), sizeof(data->fn));
}

void StartDAQ::accept()
{
    le_fn_accept();
    le_rd_accept();
    QDialog::accept();
}

void StartDAQ::setdata (const runparam_t &data_in)
{
    *data = data_in;
    data->store = 1;
    data->cyclic = 0;
    restore();
}

void StartDAQ::updateRawFileName()
{
    QString rawFileName = makeRawFileName();
    qstrncpy(data->fn, QDir::convertSeparators(rawFileName).toLocal8Bit(), sizeof(data->fn));
    data->fn[sizeof(data->fn)-1] = 0;
    updateFilenameDisplay();
}

QString StartDAQ::makeRawFileName()
{
    QString prefix = "/data";
    QString cyclic_filename = "vme.dat";
    if (data->cyclic) return prefix+"/"+cyclic_filename;

    QString timestr = QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm-ss");
    QString s = prefix + "/" + "vme_" + timestr + ".dat";
    return s;
}

runparam_t StartDAQ::getdata () const
{
    return *data;
}

void StartDAQ::restore ()
{
    ui->cb_stor->setChecked (data->store);
    updateEnabled();
    ui->cb_recycle->setChecked (data->cyclic);
    updateRawFileName();
    ui->le_rd->setText (data->rundesc);
    updateFilenameDisplay();
}

void StartDAQ::updateFilenameDisplay()
{
    ui->le_fn->setText (data->fn);
}

void StartDAQ::on_cb_stor_toggled(bool checked)
{
    data->store = checked;
    updateEnabled();
}

void StartDAQ::on_cb_recycle_toggled (bool state)
{
    data->cyclic = state;
    updateRawFileName();
}

void StartDAQ::updateEnabled()
{
    bool state = data->store;
    ui->le_rd->setEnabled(state);
    ui->le_fn->setEnabled(state);
    ui->cb_recycle->setEnabled(state);
    ui->pb_Browse->setEnabled(state);
}

