
#include "sysmon_impl.h"

#include <qfile.h>
#include <qstringlist.h>
#include <QTextStream>

#include <ctime>
#include <climits>
#include <iostream>

namespace {
    const int cache_time = 1; // seconds
}

SystemMonitorImpl::SystemMonitorImpl()
{
    last_updated = 0;
}

void SystemMonitorImpl::check_update() const
{
    time_t now = time(NULL);
    if (now-last_updated >= cache_time) read();
}

const QString SystemMonitorImpl::get(const QString & name) const
{
    check_update();
    return data[name];
}

const data_t SystemMonitorImpl::get_all() const
{
    check_update();
    return data;
}

void SystemMonitorImpl::read_meminfo() const
{
    QMap<QString,QString> memstats;

    QFile file( "/proc/meminfo" );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            QString name  = line.section(":",0,0).stripWhiteSpace().lower();
            QString rhs = line.section(":",1,1).stripWhiteSpace();
            QString value = rhs.section(" ",0,0);
            //            QString units = rhs.section(" ",1,1);
            if (!value.isEmpty())
                memstats[name]=value;
        }
        file.close();
    }

    data["memtotal"] = memstats["memtotal"];
    data["memfree"]  = memstats["memfree"];
    /*
  size_t memtotal = memstats["memtotal"].toULong();
  size_t memfree = memstats["memfree"].toULong();
  if (memtotal && memfree)
        data["MemoryUsage"] = QString::number((memtotal-memfree)/memtotal);
*/
}

void SystemMonitorImpl::read_stat() const
{
    sysstats_t sysstats;

    QFile file( "/proc/stat" );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            const QStringList all = QStringList::split(" ", line);
            const QString name = all[0].lower();
            sysstats[name]=all;
        }
        file.close();
    }

    QStringList descr;
    descr += "item";
    descr += "user";
    descr += "nice";
    descr += "sys";
    descr += "idle";
    descr += "iowait";
    descr += "irq";
    descr += "softirq";
    QString item = "cpu";
    sysstats_t current;
    for (int i=0; i<sysstats[item].count();i++) {
        bool ok;
        const QStringList last_values = last_sysstats[item];
        if (last_values.isEmpty()) break;
        if (last_values.size() <= i) continue;
        unsigned long last = last_values[i].toULong(&ok);
        if (!ok) continue;
        unsigned long curr = sysstats[item][i].toULong(&ok);
        if (!ok) continue;
        unsigned long value;
        if (last<=curr)
            value = curr-last;
        else
            value = ULONG_MAX-last-curr;
        const QString name = item+"_"+descr[i];
        //	double pcnt = 100.*value / HZ;
        data[name] = QString::number(value);
    }

    last_sysstats[item] = sysstats[item];
    /*
  char buff[60];
        FILE *meminfo;
        QString buffer;

        stat = fopen("/proc/stat", "r");
  if (stat) {
        long cur_user, cur_nice, cur_sys, cur_idle;

        fread(buff, 59, 1, stat);
        fclose(stat);

        buffer = buff;
        cur_user = (buffer.section(' ', 2, 2)).toLong();
        cur_nice = (buffer.section(' ', 3, 3)).toLong();
        cur_sys = (buffer.section(' ', 4, 4)).toLong();
        cur_idle = (buffer.section(' ', 5, 5)).toLong();

        user = cur_user - last_user;
        nice = cur_nice - last_nice;
        sys = cur_sys - last_sys;
        idle = cur_idle - last_idle;

        last_user = cur_user;
        last_nice = cur_nice;
        last_sys = cur_sys;
        last_idle = cur_idle;

        total = user + nice + sys + idle + 1;

        mem_total >>= 10;
        mem_free >>= 10;
        swap_total >>= 10;
        swap_free >>= 10;
  }

        left_popup->changeItem(left_popup->idAt(1), i18n("User\t: ") + QString::number(user * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(3), i18n("Sys\t: ") + QString::number(sys  * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(2), i18n("Nice\t: ") + QString::number(nice * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(4), i18n("Idle\t: ") + QString::number(idle * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(6), i18n("Ram\t: ") + QString::number(mem_total - mem_free)   + " / " + QString::number(mem_total)  + i18n(" MB"));
        left_popup->changeItem(left_popup->idAt(7), i18n("Swap\t: ") + QString::number(swap_total - swap_free) + " / " + QString::number(swap_total) + i18n(" MB"));
*/
}

void SystemMonitorImpl::read() const
{
    mutex.lock();
    read_meminfo();
    read_stat();
    last_updated = time(NULL);
    mutex.unlock();
}
