
#include "triggersview_waveformview.h"

#include <QFile>
#include <QTextStream>

#include "wfdata.h"
//#include "hptimer.h"
#include "foreach.h"
#include "modules/trig_types.h"
#include "decoder/event.h"
#include "daqcommon/th1.h"

namespace {
    // set bin size to 10000./32768. * N
    const size_t triggersViewBins = 1000;
    const double triggersViewRange = 10.;
    const double triggersViewStep = triggersViewRange / triggersViewBins;

    QString trigger_line_name(int n)
    {
        if (n==0) return "TEST";
        if (n >= 1 && n<= 7) return QString("NIM ")+QString::number(n);
        if (n == 8) return QString("LVDS ")+QString::number(n-8) + QString(" (ST)");
        if (n == 13 || n==17 || n==21) return QString("LVDS ")+QString::number(n-8) + QString(" (TQDC 1)");
        if (n == 25 || n==29 || n==33 || n==37) return QString("LVDS ")+QString::number(n-8) + QString(" (TQDC 2)");
        if (n >= 8 && n<= 39) return QString("LVDS ")+QString::number(n-8);
        return QString("line ")+QString::number(n);
    }
}

TriggersView_WaveformView::TriggersView_WaveformView(QWidget *p)
    : WaveformView(p), save_bin(0), save_max_t(0), max_t(0)
{
}

void TriggersView_WaveformView::write_file()
{
    QFile file("/data/triggers.csv");
    file.open(QIODevice::WriteOnly);
    QTextStream ts(&file);

    Q_FOREACH(int i, h.keys()) {
        const th1 &h1 = h[i];
        size_t nent = h1.get_nent();
        if (nent == 0) {
            continue;
        }
        ts << i << ",";
        ts << trigger_line_name(i) << ",";

        size_t nb = h1.get_nbins();
        size_t usenb = nb;
        for (size_t b=0; b < nb; b++) {
            double x = h1.bincenter(b);
            if (x > std::max(save_max_t, max_t)) {usenb = b+1; break;}
        }
        QStringList sl;
        for (size_t b=0; b < usenb; b++) {
            double x = h1.bincenter(b);
            double y = h1.get_bin(b);
            sl << QString("%1").arg(y);
        }
        ts << sl.join(",");
    }
}

void TriggersView_WaveformView::add_event(const event_t &ev)
{
    const trig_event_t &trig = ev.trig;
    const trig_cnts_t &cnts = trig.incnt;
    //  int en = ev->en;
    quint64 ts = trig.ts;
    Q_FOREACH(int ch, cnts.keys()) {
        if (h.find(ch) != h.end())
            continue;
        QString id = QString("triggersview_%1").arg(ch);
        QString name = id;
        h[ch].setup(id, name, triggersViewBins, 0, triggersViewRange);
    }
    /*
  if (n>0) {
        int cur_bin = h[0].findbin(ts);
        if (save_bin != cur_bin) {
          for (size_t i=0; i<new_hsz; i++) {
                for (int b=save_bin+1;b<=cur_bin; b++) {
                  h[i].set_bin_content(b, 0);
                }
          }
        }
        save_bin = cur_bin;
  }
*/
    Q_FOREACH(int ch, cnts.keys()) {
        const trig_cnt_t &count = cnts.value(ch);
        if (count == 0)
            continue;
        th1 &h1 = h[ch];
        double t = ts * TRIG_TS_STEP;
        int b = h1.findbin(t);
        double w = h1.bincenter(1) - h1.bincenter(0);
        double x = h1.bincenter(b);
        if (t==x) h1.filln(t, count);
        if (t<x) {
            double dx = x-t;
            h1.filln(x, count * (1-dx/w));
            h1.filln(x-w, count * dx/w);
        }
        if (t>x) {
            double dx = t-x;
            h1.filln(x, count * (1-dx/w));
            h1.filln(x+w, count * dx/w);
        }
        //	h1.filln(t, count);
        max_t = std::max(max_t, t);
    }

    /*
  wd.resize(trig.n);
  for (uint n = 0; n < trig.n; n++) {
    wd[n].c = n;

    wd[n].t.resize(en);
    for (uint t=0;t<en;t++) {
          wd[n].t.push_back(ts*TRIG_TS_STEP);
    }

        wd[n].p.resize(en);
    for (size_t i=0;i<en;i++) {
          wd[n].p[i].x = ts*TRIG_TS_STEP; // s
          wd[n].p[i].y = 1;
    }
//	wd[n].print();
  }
*/
    //  update();
}

void TriggersView_WaveformView::simulate()
{
    WaveformDataSimulator sim;
    WaveformDataAll wd(sim.run());
    setWaveformData(wd);
    //  update();
}

void TriggersView_WaveformView::update()
{
    // map histograms to plot and draw
    //  WaveformDataAll &wd(cache);
    WaveformDataAll wda;
    //  wda.resize(n);
    Q_FOREACH(int i, h.keys()) {
        const th1 &h1 = h[i];
        size_t nent = h1.get_nent();
        if (nent == 0) {
            //	  wda[i].p.clear();
            continue;
        }
        WaveformData wd;
        wd.c = i;
        size_t nb = h1.get_nbins();
        size_t usenb = nb;
        for (size_t b=0; b < nb; b++) {
            double x = h1.bincenter(b);
            if (x > std::max(save_max_t, max_t)) {usenb = b+1; break;}
        }
        wd.p.resize(usenb);
        for (size_t b=0; b < usenb; b++) {
            double x = h1.bincenter(b);
            double y = h1.get_bin(b);
            wd.p[b].x = x;
            wd.p[b].y = y / triggersViewStep;
        }
        wd.name = trigger_line_name(i).toStdString();
        wda.push_back(wd);
    }
    setWaveformData(wda);
    WaveformView::update();
}

void TriggersView_WaveformView::clear()
{
    cache.clear();
    save_bin = 0;
    save_max_t = max_t;
    max_t = 0;
    h.clear();
    WaveformView::clear();
}
