#ifndef WAVEFORMSMODEL_H
#define WAVEFORMSMODEL_H

#include <vector>

//typedef int adc_sample_t;
typedef std::vector<double> average_adc_wf_t;

class WaveformsModel {

public:

  WaveformsModel();
//  WaveformsModel(const int nch, const int nsamp);
  void resize(const unsigned int new_nch, const unsigned int new_nsamp);
  virtual ~WaveformsModel();
  int map_to_adc_ch (const int nplot) const;
  void remap_adc_to_plot ();
  void clear_average();
  void clear_all() {clear_average();}

  bool is_autoscale() const {return autoscale;}
  bool is_averaging() const {return averaging;}
  int get_scale_min() const {return scale_min;}
  int get_scale_max() const {return scale_max;}
  void set_autoscale(bool b) {autoscale = b;}
  void set_averaging(bool b) {averaging = b;}
  void set_scale_min(int d) {scale_min = d;}
  void set_scale_max(int d) {scale_max = d;}
  bool is_cur_plot_en() const {return en_plot[cur_ch];}
  bool clip_detected() const {return iclip_detected;}
  
  void set_cur_ch(int d) {cur_ch = d;}

//protected:
  unsigned int nch; // number of waveforms stored, 0..DataPlot_NCH
  unsigned int cur_ch;

  unsigned int len; // waveform length, samples. dynamic, changed by resize()
  std::vector<bool> en_plot;
  std::vector<int> map_to_plot;
  std::vector<average_adc_wf_t> average;
  std::vector<int> average_count;

//  std::vector<adc_sample_t> cache;

  std::vector<double> px;
  std::vector< std::vector<double> > py;
  std::vector<unsigned short int> pn; // number of samples in py[c] if less than 'len'

protected:
  bool iclip_detected;

private:
  bool autoscale;
  bool averaging;
  int scale_min;
  int scale_max;
};

#endif /* WAVEFORMSMODEL_H */
