//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "QtMessageUdpReceiver.h"

#include <QUdpSocket>

#include "QtMessageTypes.h"
#include "QtMessageHandler.h"

QtMessageUdpReceiver::QtMessageUdpReceiver(QObject *parent)
    : QObject(parent), socket(new QUdpSocket(this))
{
    if (!socket->bind(QHostAddress::LocalHost, QtMessageUdpPort)) {
        qWarning() << "Can't bind udp port" << QtMessageUdpPort << ":" << socket->errorString();
        qWarning() << "Log message receive disabled";
    }
    connect(socket, SIGNAL(readyRead()), this, SLOT(readPendingDatagrams()));
}

QtMessageUdpReceiver::~QtMessageUdpReceiver()
{
    delete socket;
}

void QtMessageUdpReceiver::readPendingDatagrams()
{
    while (socket->hasPendingDatagrams()) {
        QByteArray datagram;
        datagram.resize(socket->pendingDatagramSize());
        QHostAddress sender;
        quint16 senderPort;

        qint64 bytesRead = socket->readDatagram(datagram.data(), datagram.size(),
                                &sender, &senderPort);
        if (sender == QHostAddress::LocalHost && bytesRead>0)
            processTheDatagram(datagram);
    }
}

void QtMessageUdpReceiver::processTheDatagram(const QByteArray &data)
{
    QtMessage message;
    QDataStream stream(data);
    stream >> message;
    if (stream.status() == QDataStream::Ok) {
        QtMessageHandler::myMessageOutput(message);
    }
}
