//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "QtMessageUdpSender.h"

#include <QUdpSocket>
#include <QDateTime>
#include <iostream>

#include "QtMessageTypes.h"
#include "QtMessageHandler.h"
#include "daqcommon/ansi_col.h"

QtMessageUdpSender::QtMessageUdpSender(QObject *parent)
    : QObject(parent), socket(new QUdpSocket(this))
{
    QtMessageHandler *mh = GlobalQtMessageHandler::Instance();
    connect(mh, SIGNAL(qtMessage(QtMessage)), this, SLOT(qtMessage(QtMessage)));
#ifdef Q_WS_X11
    ansiCapable = false;
    QString term(getenv("TERM"));
    if (term == "ansi" || term == "linux" || term == "xterm") ansiCapable = true;
#endif
}

QtMessageUdpSender::~QtMessageUdpSender()
{
    delete socket;
}

void QtMessageUdpSender::qtMessage(const QtMessage &message)
{
    QString typeStr;
    if (!ansiCapable) {
        switch(message.type) {
        case QtDebugMsg :
            typeStr = "debug";
            break;
        case QtWarningMsg :
            typeStr = "warn ";
            break;
        case QtCriticalMsg :
            typeStr = "crit ";
            break;
        case QtFatalMsg :
            typeStr = "fatal";
            break;
        }
    } else {
        switch(message.type) {
        case QtDebugMsg :
            typeStr = "debug";
            break;
        case QtWarningMsg :
            typeStr += ANSI_YELLOW;
            typeStr += "warn ";
            typeStr += ANSI_NORM;
            break;
        case QtCriticalMsg :
            typeStr += ANSI_RED;
            typeStr += "crit ";
            typeStr += ANSI_NORM;
            break;
        case QtFatalMsg :
            typeStr += ANSI_MAGENTA;
            typeStr += "fatal";
            typeStr += ANSI_NORM;
            break;
        }
    }
    typeStr = "["+typeStr+"]";
    // log to console
    std::cerr << QString("%1 ").arg(typeStr).toStdString() << message.text.toStdString() << std::endl;

    // log to network
    QByteArray buf;
    QDataStream stream(&buf, QIODevice::WriteOnly);
    stream << message;
    socket->writeDatagram(buf, QHostAddress::LocalHost, QtMessageUdpPort);
}
