
#include "daqcommon/daq_ipc.h"

#include <QDebug>
#include <QDateTime>

#include <cstring>
#include <cerrno>

void shm_status_reset(shm_status_t * st)
{
    Q_ASSERT(st);
  memset(st, 0, sizeof(shm_status_t));
  st->ds = ST_OFF;
  st->rp.cyclic = 0;
  st->magic = SHM_STATUS_MAGIC;
}

const char *daq_state_str(daq_state_t ds)
{
  switch (ds) {
  case ST_OFF:
    return ("OFF");
  case ST_INIT:
    return ("INIT");
  case ST_RESET:
    return ("RESET");
  case ST_PREPARERUN:
    return ("PREP");
  case ST_RUN:
    return ("RUN");
  case ST_PAUSE:
    return ("PAUSE");
  case ST_FAIL:
    return ("FAILURE");
  default:
    return ("unknown");
  }
}

const char *burst_state_str(burst_state_t bs)
{
  switch (bs) {
  case BS_OFF:
    return ("off");
  case BS_ON:
    return ("ON");
  default:
    return ("unknown");
  }
}

int put_pio_rawdata_word(unsigned int d, shm_pio_rawdata_t *pio_rawdata)
{
  Q_ASSERT(pio_rawdata);
  if (pio_rawdata->count >= pio_rawdata->size) {
        return -ENOSPC;
  }
  pio_rawdata->data[pio_rawdata->count] = d;
  pio_rawdata->count++;
  return 0;
}

int put_pio_rawdata_buf(const unsigned int *buf, size_t len, shm_pio_rawdata_t *pio_rawdata)
{
    Q_ASSERT(pio_rawdata);
  if (pio_rawdata->count+len >= pio_rawdata->size) {
        return -ENOSPC;
  }
  memcpy(&pio_rawdata->data[pio_rawdata->count], buf, len*4);
  pio_rawdata->count+=len;
  return 0;
}

void runcounts_add(runcounts_t *dst, const runcounts_t &src)
{
  dst->burst += src.burst;
  dst->event += src.event;
  dst->byte += src.byte;
  dst->xoff += src.xoff;
  dst->daqtime += src.daqtime;
  dst->walltime += src.walltime;
}

QDebug operator<<(QDebug dbg, const runparam_t &rp)
{
    dbg << "rundesc: " << QString::fromLocal8Bit(rp.rundesc) << "\n";
    dbg << "fn: " << QString::fromLocal8Bit(rp.fn) << "\n";
    dbg << "store: " << rp.store << "\n";
    dbg << "cyclic: " << rp.cyclic << "\n";
//    double t = rp.tstart.tv_sec + 1e-6*rp.tstart.tv_usec;
//    dbg << "tstart: " << QDateTime::fromMSecsSinceEpoch((qint64)(t/1000)) << "\n"; // qt4.7
    return dbg;
}

QDebug operator<<(QDebug dbg, const runcounts_t &rc)
{
    dbg << "runcounts.burst = " << rc.burst << "\n";
    dbg << "runcounts.event = " << rc.event << "\n";
    dbg << "runcounts.byte = " << rc.byte << "\n";
    dbg << "runcounts.xoff = " << rc.xoff << "\n";
    dbg << "runcounts.daqtime = " << rc.daqtime << "\n";
    dbg << "runcounts.walltime = " << rc.walltime << "\n";
    return dbg;
}
