#ifndef _DAQ_IPC_H
#define _DAQ_IPC_H 1

#include <QtCore/QtGlobal>
#include "daqcommon/vmedaq_gettimeofday.h"
#if defined(Q_OS_UNIX)
#include <unistd.h> /* pid_t */
#else
typedef int pid_t;
#endif

#include "daq_types.h"

// SHM Control
const unsigned int SHM_CONTROL_KEY = 0x1204;
const unsigned int DAQCTL_HDR_MAGIC = 0xd28caab7;

enum ctl_magic_t {
    DAQCTL_CMD_INITIALIZE_DATABASE = 0x405b4e3d,
    DAQCTL_CMD_INIT = 0x8f78c435,
    DAQCTL_CMD_RESET = 0x4e3b56a9,
    DAQCTL_CMD_PREPARERUN = 0x2307e3c5,
    DAQCTL_CMD_RUN = 0x56239507,
    DAQCTL_CMD_PAUSE = 0x78433096,
    DAQCTL_CMD_ACK = 0x6e9754a3,
};

struct shm_ctl_t {
    unsigned int magic;
    pid_t daq_pid;
    ctl_magic_t reset_magic;
};

// Spill Roll Picture
const double DAQ_SPR_TIME = 10.0;       /* default spill time, seconds */

// Per-Spill Statistics: Event Size Histogram
typedef double daq_spses_pt;

enum { DAQ_SPSES_NBINS = 8192 };
const unsigned int DAQ_SPSES_BINWID = 1;

struct runparam_t {
    char rundesc[1024];
    char fn[1024];
    int store;                  // enable storage
    int cyclic;                 // enable cyclic storage
    struct timeval tstart;
};

class QDebug;
QDebug operator<<(QDebug dbg, const runparam_t &rp);

struct runcounts_t
{
    runcounts_t() { burst = 0; event = 0; byte = 0; xoff = 0; daqtime = 0; walltime = 0; }
    void clear() { *this = runcounts_t(); }
    // Run counters
    quint64 burst, event, byte, xoff;
    double daqtime, walltime;
};

enum { SHM_STATUS_MAGIC = 0x54617453 }; /* 'StaT' */

struct shm_status_t {
    int magic;
    daq_state_t ds;
    burst_state_t bs;
    unsigned int spill_count;        // current spill number
    size_t raw_size;
    struct timeval t_burstreset; // time of last burst reset
    struct timeval t_endspill;  // time of last end-of-spill clock signal
    unsigned int run;           // Run Control
    runparam_t rp;
    quint64 stor_nb;              // number of bytes

    // Spill Roll Picture
    daq_spr_t spr;

    // Run and Last Burst counters
    runcounts_t rc, bc;
};

enum { SHM_RAWDATA_MAGIC = 0x44776152 }; /* 'RawD' */
const size_t DEFAULT_PIO_RAWDATA_BYTES = 256UL * 1024 * 1024;
struct shm_pio_rawdata_t {
    int magic;
    size_t size;
    size_t count;
    unsigned int data[];
};

void shm_status_reset (shm_status_t * st);

const char *daq_state_str (daq_state_t ds);
const char *burst_state_str (burst_state_t bs);

int put_pio_rawdata_word(unsigned int d, shm_pio_rawdata_t *pio_rawdata);
int put_pio_rawdata_buf (const unsigned int *buf, size_t len, shm_pio_rawdata_t *pio_rawdata);

void runcounts_add(runcounts_t *dst, const runcounts_t &src);
QDebug operator<<(QDebug dbg, const runcounts_t &rc);

#endif                          /* _DAQ_IPC_H */
