
#include "dsplib.h"

#include <cmath>
#include <cstdlib>

//! An unnormalized gaussian function with mean=0 and sigma
//! Divide result by sqrt(2*Pi) = 2.50662827463100024 if normalized output needed
double gauss(double x, double sigma)
{
    double arg, res;
    // Calculate a gaussian function with mean and sigma.
    if (sigma == 0)
        return 1.e30;
    arg = x / sigma;
    res = exp(-0.5 * arg * arg);
    return res;
    //   return res/(2.50662827463100024*sigma); //sqrt(2*Pi)=2.50662827463100024
}

//! Simulate detector pulse
double detpulse(double a, double t, double rt, double ft)
{
    double v, r, f, e;

    //  if (rt > ft) m = ft; else m = rt;

    //  e = TMath::Log(ADC_RES*10); // set error to 0.1 resolution
    e = log(1e6);

    r = t / rt;
    f = t / ft;
    if (f > e)
        return 0;
    if (r < -4)
        return 0;

#ifndef _MSC_VER
    v = a * (erf (r) + 1) / 2 * exp (-f);
#else
    if (f>0)
        v = a * exp (-f);
    else
        v = 0;
#endif
    // if (v<0) v=0;
    return v;
}

namespace {
    //! System buil-in
    //! Random number in range [0,1) with period approx. 16*((2**31)-1)
    double rndm()
    {
        double value;
        value = static_cast<double>(rand()) / RAND_MAX;
        return (value);
    }

    static unsigned int f1Seed = 0;

    //!  Machine independent random number generator.
    //!  Produces uniformly-distributed floating points between 0 and 1.
    //!  Identical sequence on all machines of >= 32 bits.
    //!  Periodicity = 10**8
    //!  Universal version (Fred James 1985).
    //!  generates a number in ]0,1]
    double rndm1()
    {

        const double kCONS = 4.6566128730774E-10;
        const int kMASK24 = 2147483392;
        unsigned int jy;

        f1Seed *= 69069;
        jy = (f1Seed & kMASK24);      // Set lower 8 bits to zero to assure exact float
        if (jy)
            return kCONS * jy;
        return rndm();
    }

    static int fSeed1 = 9876;
    static int fSeed2 = 54321;

    //!  Machine independent random number generator.
    //!  Produces uniformly-distributed floating points in [0,1]
    //!  Identical sequence on all machines of >= 32 bits.
    //!  Periodicity > 10**14
    double rndm2()
    {

        double iz, r;
        int k = static_cast<int>(fSeed1 / 53668);
        fSeed1 = 40014 * (fSeed1 - k * 53668) - k * 12211;
        if (fSeed1 < 0)
            fSeed1 += 2147483563;
        k = static_cast<int>(fSeed2 / 52774);
        fSeed2 = 40692 * (fSeed2 - k * 52774) - k * 3791;
        if (fSeed2 < 0)
            fSeed2 += 2147483399;
        iz = fSeed1 - fSeed2;
        if (iz <= 0)
            iz += 2147483562;
        r = iz * 4.6566128e-10;
        return r;
    }
}

//! returns a uniform deviate on the interval ]x1, x2].
double ranuni(double x1, double x2)
{
    double ans = rndm();
    return x1 + (x2 - x1) * ans;
}

//! returns a number distributed following a gaussian with sigma
double rangaus(double sigma)
{
    double x, y, z, result;

    y = rndm();
    z = rndm();
    x = z * 6.28318530717958623;
    result = sigma * sin(x) * sqrt(-2 * log(y));
    return result;
}
