
#include "fitter.h"

#include <QtCore/QtGlobal>

#include <cstdio>
#define _USE_MATH_DEFINES
#include <cmath>

#include "dsplib.h"

int MAX_FIT_POINTS = 21;


void FindGausApproxD(double *x, int len, double *par)
{

    int i;
    double tt, val, mean, rms;
    double allcha = 0, sumx = 0, sumx2 = 0;

    if (!x || !par)
        return;

    par[0] = 0;
    par[1] = 0;
    par[2] = 0;

    for (i = 0; i < len; i++) {
        tt = i + .5;
        val = x[i];
        if (val < 0)
            val = 0;                  // FIXME
        sumx += val * tt;
        sumx2 += val * tt * tt;
        allcha += val;
        //    printf("[%d]: x %g, sumx %g, sumx2 %g, allcha %g\n", i, val, sumx, sumx2, allcha);
    }
    if (allcha > 0) {
        mean = sumx / allcha;
        rms = sqrt(sumx2 / allcha - mean * mean);
        //    printf("mean: %g, rms %g\n", mean, rms);
        //   par[0] = 0.5*allcha/(sqrt(2*M_PI)*rms);
        par[0] = allcha / (sqrt(2 * M_PI) * rms);
        par[1] = mean;
        par[2] = rms;
    }
}

void FindGausApprox(int *x, int len, double *par)
{
    int i;
    double tt, val, mean, rms;
    double allcha = 0, sumx = 0, sumx2 = 0;

    if (!x || !par)
        return;

    par[0] = 0;
    par[1] = 0;
    par[2] = 0;

    for (i = 0; i < len; i++) {
        tt = i + .5;
        val = x[i];
        if (val < 0)
            val = 0;                  // FIXME
        sumx += val * tt;
        sumx2 += val * tt * tt;
        allcha += val;
        //    printf("[%d]: x %g, sumx %g, sumx2 %g, allcha %g\n", i, val, sumx, sumx2, allcha);
    }
    if (allcha > 0) {
        mean = sumx / allcha;
        rms = sqrt(sumx2 / allcha - mean * mean);
        //    printf("mean: %g, rms %g\n", mean, rms);
        //   par[0] = 0.5*allcha/(sqrt(2*M_PI)*rms);
        par[0] = allcha / (sqrt(2 * M_PI) * rms);
        par[1] = mean;
        par[2] = rms;
    }
}
/*
double FindGausChisq (int *x, int len, double *par)
{
  int i;
  double chisq = 0;

  for (i = 0; i < len; i++)
    chisq += sqr (x[i] - par[0] * gauss (i - par[1], par[2]));
  return chisq;
}
*/

# include "lmmin.h"

namespace {
    typedef struct {
        double *y;
        double fp[3];
        int fix;
    } fitdata_t;

    lm_control_type control;
    fitdata_t fitdata;
    lm_ws_t *g_ws;

    void lm_evaluate(double *par, int m_dat, double *fvec, double *fjac, void *data, int *info)
            /*
 *	par is an input array. At the end of the minimization, it contains
 *        the approximate solution vector.
 *
 *	m_dat is a positive integer input variable set to the number
 *	  of functions.
 *
 *	fvec is an output array of length m_dat which contains the function
 *        values the square sum of which ought to be minimized.
 *
 *	data is a read-only pointer to fitdata_t
 *
 *      info is an integer output variable. If set to a negative value, the
 *        minimization procedure will stop.
 */
    {
        int i;
        fitdata_t *mydata = static_cast<fitdata_t *>(data);
        double m1, arg, ex, fac, p0, p1, p2, p0ex;

        if (mydata->fix) {
            int nmp = 0;
            if (!(mydata->fix&1)) {p0 = par[nmp]; nmp++;} else {p0 = mydata->fp[nmp];}
            if (!(mydata->fix&2)) {p1 = par[nmp]; nmp++;} else {p1 = mydata->fp[nmp];}
            if (!(mydata->fix&4)) {p2 = par[nmp]; nmp++;} else {p2 = mydata->fp[nmp];}
        } else {
            p0 = par[0];
            p1 = par[1];
            p2 = par[2];
        }

        m1 = p2 ? 1. / p2 : 1;

        for (i = 0; i < m_dat; i++) {
            arg = (i - p1) * m1;
            ex = exp(-0.5*arg*arg);
            p0ex = -p0*ex;
            fvec[i] = mydata->y[i] +p0ex;
            //	if (fjac) {
            fac = p0ex*arg;
            fjac[0*m_dat+i] = -ex;
            fjac[1*m_dat+i] = fac*m1;
            fjac[2*m_dat+i] = fac*m1*arg;
            //	}
        }
        /*
*** Original gaussian() code for reference (not optimized)
  double arg, ex, fac, y;
  arg = (x - a[1]) / a[2];
  ex = exp (-arg * arg);
  fac = 2.0 * a[0] * ex * arg;
  y = a[0] * ex;
  jac[0] = ex;
  jac[1] = fac / a[2];
  jac[2] = jac[1] * arg;
  return y;
*/
        *info = *info;                /* to prevent a 'unused variable' warning */
        /* if <parameters drifted away> { *info = -1; } */
    }

    void lm_print_default(int n_par, const double* par, int m_dat, const double* fvec,
                          const void *data, int iflag, int iter, int nfev)
    /*
 *       data  : for soft control of printout behaviour, add control
 *                 variables to the data struct
 *       iflag : 0 (init) 1 (outer loop) 2(inner loop) -1(terminated)
 *       iter  : outer loop counter
 *       nfev  : number of calls to *evaluate
 */
    {
        Q_UNUSED(data);
        //    double f, y, t;
        //    const fitdata_t *mydata = (fitdata_t*)data;

        if (iflag==2) {
            printf("trying step in gradient direction\n");
        } else if (iflag==1) {
            printf("determining gradient (iteration %d)\n", iter);
        } else if (iflag==0) {
            printf("starting minimization\n");
        } else if (iflag==-1) {
            printf("terminated after %d evaluations\n", nfev);
        }

        printf("  par: ");
        for (int i = 0; i<n_par; i++)
            printf(" %12g", par[i]);
        printf(" => norm: %12g\n", lm_enorm(m_dat, fvec));
        /*
    if ( iflag == -1 ) {
        printf("  fitting data as follows:\n" );
        for( i=0; i<m_dat; ++i ) {
//            t = (mydata->t)[i];
            y = (mydata->y)[i];
//            f = mydata->user_func( t, par );
            printf("    t[%2d]=%12g y=%12g fit=%12g residue=%12g\n",
                    i, t, y, f, y-f );
        }
    }
*/
    }
    void lm_print_none(int n_par, const double *par, int m_dat, const double *fvec, const void *data, int iflag, int iter, int nfev)
    {
        Q_UNUSED(n_par);
        Q_UNUSED(par);
        Q_UNUSED(m_dat);
        Q_UNUSED(fvec);
        Q_UNUSED(data);
        Q_UNUSED(iflag);
        Q_UNUSED(iter);
        Q_UNUSED(nfev);
    }
}

int InitFindGaus()
{
    g_ws = lm_alloc_ws(MAX_FIT_POINTS, 3);
    return 0;
}

int FindGausParam(int fixpar, double *x, int len, double *par, fitstatus_t *fs)   // approximation
{
    int i, nmp;
    double mp[3];

    if (!x || !par || (len<=0))
        return -1;

    if (len>MAX_FIT_POINTS)
        len = MAX_FIT_POINTS;

    if (!g_ws)
        return(-1);

    for (i = 0; i < 3; i++) {
        par[i] = 0;
        //    err[i] = 0;
    }

    lm_initialize_control(&control);
    fitdata.y = x;
    fitdata.fix = fixpar;
    if (fs) control.maxcall = fs->niter;
    FindGausApproxD(x, len, par);
    nmp = 0;
    fitdata.fix=fixpar;
    if (!(fixpar&1)) {mp[nmp] = par[0]; nmp++;} else {fitdata.fp[0] = par[0];}
    if (!(fixpar&2)) {mp[nmp] = par[1]; nmp++;} else {fitdata.fp[1] = par[1];}
    if (!(fixpar&4)) {mp[nmp] = par[2]; nmp++;} else {fitdata.fp[2] = par[2];}
    lm_minimize(g_ws, len, nmp, mp, lm_evaluate, lm_print_none, &fitdata, &control);
    //  lm_minimize (g_ws, len, nmp, mp, lm_evaluate, lm_print_default, &fitdata, &control);
    nmp = 0;
    if (!(fixpar&1)) {par[0] = mp[nmp]; nmp++;}
    if (!(fixpar&2)) {par[1] = mp[nmp]; nmp++;}
    if (!(fixpar&4)) {par[2] = mp[nmp]; nmp++;}
    if (fs) {
        fs->fnorm = control.fnorm;
        fs->niter= control.nfev;
    }
    //  printf("status: %s after %d evaluations, fnorm=%e\n", lm_shortmsg[control.info], fs->niter, fs->fnorm);
    if (control.info <= 0) return(2);
    //  lm_free_ws(g_ws);
    return 0;
}

int QuickFitGaus(double *x, int len, double *par, fitstatus_t *fs) {

    lm_control_type lmc;
    fitdata_t lmfd;
    lm_ws_t *lmws;

    if (len<=0) return(-1);

    lmws = lm_alloc_ws(len, 3);
    if (!lmws) return(-1);

    lm_initialize_control(&lmc);
    lmfd.y = x;
    lmfd.fix = 0;

    FindGausApproxD(x,len,par);
    lm_minimize(lmws, len, 3, par, lm_evaluate, lm_print_none, &lmfd, &control);
    lm_free_ws(lmws);
    if (fs) {
        fs->fnorm = control.fnorm;
        fs->niter= control.nfev;
    }

    return(0);
}
