//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <QDebug>
#include "hal_types.h"

common_setup_t::common_setup_t()
    :
      magic(0),
      size(0),
      ndev(1), // important: must be non-zero
      nch(1), // important: must be non-zero
      vendor_id(0),
      device_id(MODULE_ID_DUMMY),
      device_rev(0),
      device_serial(0),
      fw_ver(0),
      fw_rev(0),
      device_slot(0),
      flags(0),
      stats(),
      mon()
{
}

QDebug operator<<(QDebug dbg, const mod_stats_t &m)
{
    dbg.nospace() << " mod_stats:\n";
    dbg.nospace() << "  total " << m.total << "\n";
    dbg.nospace() << "  fatal " << m.fatal << "\n";
    dbg.nospace() << "  err " << m.err << "\n";
    dbg.nospace() << "  warn " << m.warn << "\n";
    dbg.nospace() << "  over " << m.over << "\n";
    return dbg.maybeSpace();
}

QDebug operator<<(QDebug dbg, const mod_monitoring_t &m)
{
    dbg.nospace() << " mod_monitoring:\n";
    dbg.nospace() << "  temperature " << m.temperature << "\n";
    return dbg.maybeSpace();
}

QDebug operator<<(QDebug dbg, const common_setup_t &s)
{
    dbg.nospace() << " common_setup:\n";
    dbg.nospace() << "  magic " <<  QString("%1").arg(s.magic, 8, 16, QChar('0'))
        << ((s.magic == BASE_SETUP_MAGIC)?" Ok":" BAD") << "\n";
    dbg.nospace() << "  size " << s.size << "\n";
    dbg.nospace() << "  ndev " << s.ndev << "\n";
    dbg.nospace() << "  nch " << s.nch << "\n";
    dbg.nospace() << "  vendor_id " << QString("%1").arg(s.vendor_id, 2, 16, QChar('0')) << ((s.vendor_id == VME_CR_VENDOR_ID)?" Ok":" UNKNOWN") << "\n";
    dbg.nospace() << "  device_id " << QString("%1").arg(s.device_id, 2, 16, QChar('0')) << "(" << module_id_str(s.device_id) << ")" << "\n";
    dbg.nospace() << "  device_rev " << s.device_rev << "\n";
    dbg.nospace() << "  device_serial " << QString("%1").arg(s.device_serial, 8, 16, QChar('0')) << "\n";
    dbg.nospace() << "  firmware " << (s.fw_ver >> 8) << "." << (s.fw_ver % 0xFF) << " rev. " << s.fw_rev << "\n";
    dbg.nospace() << "  slot " << s.device_slot << "\n";
    dbg.nospace() << "  flags " << QString("0x%1").arg(s.flags, 0, 16, QChar('0')) << "\n";
    dbg.nospace() << s.stats;
    dbg.nospace() << s.mon;
    return dbg.maybeSpace();
}
