//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef HAL_TYPES_H
#define HAL_TYPES_H

#include <QtCore/QtGlobal>

#include "daqcommon/device_ids.h"
#include "hal/base_mod_types.h"

const unsigned int BASE_SETUP_MAGIC = ((unsigned int)0x224ca034);
//#define BASE_SETUP_TRAILER ((unsigned int)0x3f64a7bf)

class QDebug;

// module burst statistics
class mod_stats_t {
  public:
	quint64 total;
	quint64 fatal;
	quint64 err;
	quint64 warn;
	quint64 over;
};

QDebug operator<<(QDebug dbg, const mod_stats_t &);

class mod_monitoring_t {
  public:
	double temperature;
};

QDebug operator<<(QDebug dbg, const mod_monitoring_t &);

#pragma pack(push,4)
// setup data, same for all module types
class common_setup_t {
public:
  common_setup_t();
  unsigned int magic;
  quint32 size;
  int ndev;                     // number of subdevices
  int nch;                      // number of channels

  int vendor_id;
  mtype_t device_id;
  int device_rev;
  int device_serial;
  int fw_ver;
  int fw_rev;
  int device_slot; // interface-independent
  int flags; // software flags
  mod_stats_t stats;
  mod_monitoring_t mon;
};
#pragma pack(pop)


QDebug operator<<(QDebug dbg, const common_setup_t &);

#endif /* HAL_TYPES_H */
