#ifndef _RAWDATA_H
#define _RAWDATA_H

#include <QtGlobal>

// Run Mode Data Word Type - ADLVME
const quint32 ADLVME_DTYPE_MASK_ALL = 0xF0000000u;  // mask all data types
const quint32 ADLVME_DTYPE_MASK_CTRL = 0x80000000u; // mask only adlvme-specific
const quint32 ADLVME_DTYPE_MHDR = 0x80000000u;
const quint32 ADLVME_DTYPE_MTRL = 0x90000000u;
const quint32 ADLVME_DTYPE_EHDR = 0xA0000000u;
const quint32 ADLVME_DTYPE_ETRL = 0xB0000000u;

const quint32 ADLVME_DTYPE_SHDR = 0xC0000000u;
const quint32 DATA_SHDR_SPILLNUM_MASK = 0x0FFFFFFFu;
inline unsigned int decode_strl_spill_number(quint32 d) { return d & DATA_SHDR_SPILLNUM_MASK; }

const quint32 ADLVME_DTYPE_STRL = 0xD0000000u;
const quint32 ADLVME_DTYPE_STAT = 0xE0000000u;

const quint32 DATA_EHDR_EN_MASK = 0x000FFFFFu;
inline int decode_ehdr_en(quint32 d) { return d & DATA_EHDR_EN_MASK; }

const quint32 DATA_ETRL_WC_MASK = 0x00FFFFFFu;
inline int decode_etrl_wc(quint32 d) { return d & DATA_ETRL_WC_MASK; }
//const quint32 DATA_ETRL_NXOFF_MASK = 0xFFu;
//inline int decode_etrl_nxoff(quint32 d) { return (d>>16) & DATA_ETRL_NXOFF_MASK; }

const quint32 ETRL_STATUS_BIT_TIMEOUT = 1u;
inline int decode_etrl_readout_status(quint32 d) { return (d>>24) & 0xF; }

const quint32 DATA_MHDR_EN_MASK = 0x0000FFFF;
inline int decode_mhdr_en(quint32 d) { return d & DATA_MHDR_EN_MASK; }

const quint32 DATA_MHDR_ID_MASK = 0x7Fu;
inline int decode_mhdr_id(quint32 d) { return (d>>16) & DATA_MHDR_ID_MASK; }

const quint32 DATA_MHDR_GA_MASK = 0x1Fu;
inline int decode_mhdr_ga(quint32 d) { return (d>>23) & DATA_MHDR_GA_MASK; }

const quint32 DATA_MTRL_FLAGS_MASK = 0x7u;
inline int decode_mtrl_flags(quint32 d) { return (d>>20) & DATA_MTRL_FLAGS_MASK; }

const quint32 DATA_MTRL_WC_MASK = 0x0000FFFFu;
inline int decode_mtrl_wc(quint32 d) { return d & DATA_MTRL_WC_MASK; }

// readout overflow bit, active low
inline int decode_mtrl_ro(quint32 d) { return (d & (1<<16))?0:1; }

// readout error bit, active low
inline int decode_mtrl_re(quint32 d) { return (d & (1<<17))?0:1; }

// checksum
inline unsigned int decode_mtrl_crc(quint32 d) { return (d>>20) & 0xFF; }

const quint32 DATA_STRL_WC_MASK = 0x0FFFFFFFu;
inline int decode_strl_wc(quint32 d) { return d & DATA_STRL_WC_MASK; }

const quint32 DATA_SHDR_ENDBURST = ADLVME_DTYPE_SHDR | 0x08A50000u;
const quint32 DATA_STRL_ENDBURST = ADLVME_DTYPE_STRL | 0x08A50000u;

const char *daq_str_dtype (int dt);

#endif /* _RAWDATA_H */
