
#include "daqcommon/sysutil.h"

#include <cstddef>
#include <ctime>
#include <cassert>

#include <QDir>
#include <QFileInfo>

#ifdef _WIN32
#include <windows.h>
#else // linux stuff
#include <sys/vfs.h>
#include <sys/stat.h>
#include <sys/statvfs.h>
#endif // _WIN32

bool get_disk_space(const QString &fn, double *bytes_total, double *bytes_avail)
{
    assert(bytes_total);
    assert(bytes_avail);
    QFileInfo fi(fn);
    QString dirPath = fi.path();
#ifdef _WIN32
    QString sCurDir = QDir::current().absolutePath();
    QDir::setCurrent( dirPath );

    ULARGE_INTEGER free,total;
    bool bRes = ::GetDiskFreeSpaceExA( 0 , &free , &total , NULL );
    if ( !bRes ) return false;

    QDir::setCurrent( sCurDir );

    *bytes_avail = static_cast<double>( static_cast<__int64>(free.QuadPart) );
    *bytes_total = static_cast<double>( static_cast<__int64>(total.QuadPart) );

#else // linux
    struct stat stst;
    struct statfs stfs;

    if ( ::stat(dirPath.toLocal8Bit(),&stst) == -1 ) return false;
    if ( ::statfs(dirPath.toLocal8Bit(),&stfs) == -1 ) return false;

    *bytes_avail = (double)stfs.f_bavail * stst.st_blksize;
    *bytes_total = (double)stfs.f_blocks * stst.st_blksize;
#endif
    return true;
}
