#ifndef _TH1_H
#define _TH1_H 1

#include <QtCore/QtGlobal>
#include <vector>
#include <map>
#include <QString>

typedef QString HistId;
class QDebug;

class HistDescr
{
public:
    HistDescr() : nbins(1000), min(0), max(1000) {}
    HistDescr(HistId p_id, QString p_title, int p_nbins, double p_min, double p_max)
        : id(p_id), title(p_title), nbins(p_nbins), min(p_min), max(p_max) {}
    HistId id;
    QString title;
    int nbins;
    double min;
    double max;
};

class th1 {

public:
  th1 ();
  th1 (const HistDescr &descr);
  th1 (const HistId &id, const QString &name, int fnbins, double fmin, double fmax);
  virtual ~ th1 ();
  void setup (const HistId &id, const QString &name, int fnbins, double fmin, double fmax);
  void setname (const QString &name);
  void setDescription (const QString &text) { description = text; }
  void setXLabel (const QString &text) { xlabel = text; }
  void setYLabel (const QString &text) { ylabel = text; }
  const QString getXLabel () const { return xlabel; }
  const QString getYLabel () const { return ylabel; }
  void setAccessLevel(int d) { accessLevel = d; }
  int getAccessLevel() const { return accessLevel; }
  QString getDescription () const { return description; }
//  void print() const;
  unsigned int get_nbins () const;
  void rebin(int nb);
  std::vector<double> get_py() const;
  double get_bin (unsigned int index) const;
  void fill (double x);
  void filln (double x, double count);
// optimized vector fill functions, 30% faster than fill()
  void fill(const std::vector<double> &x, double count = 1);
  void fill(const std::map<double, double> &m);
  void clear ();
  void reset ();
  void hold (bool state); //!< stop updating, use tmp_* for get()
  void save_to_file () const;
  void setrange (double inmin, double inmax);
  bool fitgaus (double *value, double *mean, double *sigma, int bin1, int bin2) const;
  QString getname () const;
  const HistId &getid () const;
  void shift_right ();

// statistical values
  quint64 get_nent () const;
  quint64 get_over () const;
  quint64 get_under () const;
//  bool is_modified () const;
  int modified_count() const;
  void clear_modified ();
  double get_xmin () const;
  double get_xmax () const;
  double bincenter (int n) const;
  bool isHold() const {return do_hold;}
  int findbin (double x) const;
  void set_bin_content(int bin, double count);


private:
  void init_pvt();
  void recalc_internals();
  void setid (const HistId &id);

  int mod, hold_mod;
  bool do_hold;
  std::vector<double> py, hold_py;
  mutable std::vector<double> fitpy;
  double under, over, nent;
  double hold_under, hold_over, hold_nent;
  double xmin, xmax;
  QString fname;
  HistId fid;
  QString description;
  QString xlabel;
  QString ylabel;
  unsigned int nbins;
// internal speed optimization constants
  double s_binwidth;
  int accessLevel;
};

QDebug operator<<(QDebug dbg, const th1 &h);

#endif /* _TH1_H */
