#include "daqcommon/vme_types.h"
#include "daqcommon/vme_util.h"

#include <string>
#include <sstream>
#include <iomanip>

using namespace std;

const char *vme_am_str(VME_AM am)
{
  switch (am) {
    case VME_AM_A24_PRIV_BLT: return "A24_PRIV_BLT";
    case VME_AM_A24_PRIV_PROG: return "A24_PRIV_PROG";
    case VME_AM_A24_PRIV: return "A24_PRIV";
    case VME_AM_A24_PRIV_MBLT: return "A24_PRIV_MBLT";
    case VME_AM_A24_BLT: return "A24_BLT";
    case VME_AM_A24_PROG: return "A24_PROG";
    case VME_AM_A24: return "A24";
    case VME_AM_A24_MBLT: return "A24_MBLT";
    case VME_AM_A40_BLT: return "A40_BLT";
    case VME_AM_A40_LCK: return "A40_LCK";
    case VME_AM_A40: return "A40";
    case VME_AM_A24_LCK: return "A24_LCK";
    case VME_AM_CRCSR: return "CR/CSR";
    case VME_AM_A16_PRIV: return "A16_PRIV";
    case VME_AM_A16_LCK: return "A16_LCK";
    case VME_AM_A16: return "A16";
    case VME_AM_3U_2eVME: return "3U_2eVME";
    case VME_AM_6U_2eVME: return "6U_2eVME";
    case VME_AM_A32_PRIV_BLT: return "A32_PRIV_BLT";
    case VME_AM_A32_PRIV_PROG: return "A32_PRIV_PROG";
    case VME_AM_A32_PRIV: return "A32_PRIV";
    case VME_AM_A32_PRIV_MBLT: return "A32_PRIV_MBLT";
    case VME_AM_A32_BLT: return "A32_BLT";
    case VME_AM_A32_PROG: return "A32_PROG";
    case VME_AM_A32: return "A32";
    case VME_AM_A32_MBLT: return "A32_MBLT";
    case VME_AM_A32_LCK: return "A32_LCK";
    case VME_AM_A64_LCK: return "A64_LCK";
    case VME_AM_A64_BLT: return "A64_BLT";
    case VME_AM_A64: return "A64";
    case VME_AM_A64_MBLT: return "A64_MBLT";
	default: return "unknown";
  }
}

std::string cr_str(const struct vme_cr_t &cr)
{
  std::ostringstream os;
  os << hex << setw(2) << setfill('0');
  os << "CR vendor 0x" << cr.vendor;
  os << ", type 0x" << cr.type;
  os << ", rev 0x" << cr.serial;
  os << ", reserved 0x" << cr.reserved;
  return os.str();
}

std::string csr_str(const struct vme_csr_t &csr)
{
  std::ostringstream os;
  os << "CSR base 0x" << hex << setw(8) << setfill('0') << csr.base;
  return os.str();
}
