//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "CalibrationData.h"

#include <QCoreApplication>
#include <QDir>
#include <QDateTime>
#include <QDebug>

namespace cal {
const int TQDC_NCH = 16;
double thr_offset_cal_0168E1AF[TQDC_NCH] = { // TQDC-16  Temp 26.88 C
                                             -19.5, -27.0, -17.9,   0.5,
                                             -9.2, -23.9, -26.4,  -7.9,
                                             -14.2, -32.3,   2.1,  -7.6,
                                             -1.1, -22.9,  -9.2, -21.1
                                           };
double adc_baseline_cal_0168E1AF[TQDC_NCH] = { // TQDC-16  Temp 26.88 C
                                               1.1,   1.5,  -2.6,   8.6,
                                               -4.3,  -0.5,  -8.1,  11.6,
                                               1.1,  -0.7,  -6.4,   5.0,
                                               -2.9,   4.6,  -0.9,  -1.0
                                             };
double thr_offset_cal_0168CAA1[TQDC_NCH] = { // TQDC-16  Temp 27.56 C
                                             1.7, -25.1, -19.8,  -6.7,
                                             -19.2, -18.6, -15.1, -31.7,
                                             -3.6,   1.7, -13.6, -22.6,
                                             -24.8,  -4.8,  -8.6, -17.6
                                           };
double adc_baseline_cal_0168CAA1[TQDC_NCH] = { // TQDC-16  Temp 27.56 C
                                               3.2,  -1.0,  -8.4,   8.0,
                                               -6.3,  13.5,  -3.0,  -0.8,
                                               -3.0,  -1.8,  -0.5,   0.7,
                                               1.1,   4.3,  -4.9,   4.7
                                             };
double thr_offset_cal_0168ED35[TQDC_NCH] = { // TQDC-16  Temp 36.00 C
                                             1.7, -17.6,   4.6,  -6.7,
                                             -8.6,   2.1, -22.6, -12.9,
                                             -11.4,  -7.3,   3.6, -28.3,
                                             -17.0, -23.3, -19.8, -29.8
                                           };
double adc_baseline_cal_0168ED35[TQDC_NCH] = { // TQDC-16  Temp 36.00 C
                                               -1.8,   2.6,  -0.5,  -7.0,
                                               1.1,   1.9,  -0.2,  -1.2,
                                               -0.9,  -3.1, -10.0,  -2.1,
                                               -4.3,  -0.4,  -0.8,   1.3
                                             };
double thr_offset_cal_0168D9BF[TQDC_NCH] = { // TQDC-16  Temp 34.50 C
                                             -12.6, -26.4, -14.2,  -8.6,
                                             -22.0,  -6.1, -30.1, -27.0,
                                             -32.9,  -6.1, -31.7, -16.1,
                                             -14.5, -22.0,  -7.6, -39.5
                                           };
double adc_baseline_cal_0168D9BF[TQDC_NCH] = { // TQDC-16  Temp 34.50 C
                                               -6.1,  -4.7,  -1.0,  -4.1,
                                               -8.2,  -2.8,  -3.6,  -6.0,
                                               2.0,  -2.9,  -9.1,  -3.0,
                                               -8.6,   1.6,  -5.0,  -1.7
                                             };
double thr_offset_cal_01692E27[TQDC_NCH] = { // TQDC-16  Temp 31.62 C
                                             21.7,  -0.4,  -0.1,   0.2,
                                             -28.9, -26.4, -11.1, -11.4,
                                             24.9,   5.2,  13.6,  10.2,
                                             -15.4,  21.7,  -4.8,   4.6
                                           };
double adc_baseline_cal_01692E27[TQDC_NCH] = { // TQDC-16  Temp 31.62 C
                                               1.8,   8.0,   4.5,  -5.0,
                                               -2.2,   5.0,  -6.2,   0.6,
                                               -7.1,   3.7,  -2.2,   1.9,
                                               -7.0,   5.4,  -8.9,   1.2
                                             };
double thr_offset_cal_01692EB5[TQDC_NCH] = { // TQDC-16  Temp 28.81 C
                                             -15.4,  -9.2, -11.1, -29.2,
                                             -12.9, -17.0, -28.9, -18.9,
                                             -23.9, -18.6, -11.7, -27.6,
                                             -34.5,  -8.3,  -7.0,  -6.4
                                           };
double adc_baseline_cal_01692EB5[TQDC_NCH] = { // TQDC-16  Temp 28.81 C
                                               -1.6,  -2.2,  -4.0,   5.6,
                                               0.1,  -1.2,  -9.9,   4.6,
                                               -10.2,  -0.5,  -1.6,   1.6,
                                               4.9,   2.2, -13.5,   1.3
                                             };
double thr_offset_cal_01694095[TQDC_NCH] = { // TQDC-16  Temp 31.50 C
                                             -34.5, -30.8, -25.4, -20.4,
                                             -27.9, -32.3, -22.3, -22.9,
                                             -19.8,  15.5, -16.7,  12.7,
                                             5.2, -28.9, -18.9, -33.3
                                           };
double adc_baseline_cal_01694095[TQDC_NCH] = { // TQDC-16  Temp 31.50 C
                                               -2.1,  -2.2,  -3.5,   6.0,
                                               5.0,   2.2,  -3.4,  -3.2,
                                               -3.3,   7.0,  -4.1,   0.0,
                                               4.1,   1.1,  -6.3,  11.3
                                             };
double thr_offset_cal_0168E22D[TQDC_NCH] = { // TQDC-16  Temp 26.94 C
                                             -0.8,  -8.3,  -7.3, -18.6,
                                             -5.8,   2.1,  23.9, -12.3,
                                             -9.8, -10.1, -25.1, -22.3,
                                             -23.3,  -6.7, -19.5,  -6.7
                                           };
double adc_baseline_cal_0168E22D[TQDC_NCH] = { // TQDC-16  Temp 26.94 C
                                               -7.2,   1.9,  -3.2,   0.0,
                                               2.5,  -3.7,  -6.9,  -0.5,
                                               -11.6,  -1.9,   2.2,   2.5,
                                               -1.1,   1.2,   1.4,  -3.6
                                             };
double thr_offset_cal_0169319D[TQDC_NCH] = { // TQDC-16  Temp 29.62 C
                                             0.0, -22.6, -14.5,  -2.6,
                                             -36.1, -16.1, -20.1, -27.3,
                                             -34.8, -10.4,  -3.3,  11.4,
                                             -13.6, -12.0,  -5.1, -10.8
                                           };
double adc_baseline_cal_0169319D[TQDC_NCH] = { // TQDC-16  Temp 29.62 C
                                               4.0,   0.7,  -7.7,   6.9,
                                               -0.0,   2.7,  -4.0,  -3.2,
                                               1.7,  -1.3,  -3.2,  -0.5,
                                               -1.9,  -0.9,  -8.9,   3.5
                                             };
double thr_offset_cal_0168F52E[TQDC_NCH] = { // TQDC-16  Temp 51.25 C
                                             -14.2, -25.1,   4.2, -11.7,
                                             -23.9, -15.1, -36.7,  -5.8,
                                             -9.2, -23.6, -16.4,  -6.4,
                                             -16.4,  -9.2, -32.0, -11.7
                                           };
double adc_baseline_cal_0168F52E[TQDC_NCH] = { // TQDC-16  Temp 51.25 C
                                               -5.8,  -9.8,  -0.2,  -5.2,
                                               4.3,   7.0, -12.9,   1.2,
                                               -4.1,   8.7,  -4.1,  -2.3,
                                               -9.1,  -5.5,  -7.2,  -2.3
                                             };
double thr_offset_cal_0169206B[TQDC_NCH] = { // TQDC-16  Temp 51.31 C
                                             -23.3, -31.1,  -4.2, -23.6,
                                             -32.3, -30.8, -30.8,   0.5,
                                             -38.6, -15.4, -32.3, -10.4,
                                             -43.3, -15.1, -36.1, -29.5
                                           };
double adc_baseline_cal_0169206B[TQDC_NCH] = { // TQDC-16  Temp 51.31 C
                                               1.5,  -2.1,  -6.9,  -4.1,
                                               -5.7,  -2.4,  -5.3,   8.6,
                                               -3.2,  -7.5, -11.0,  -5.0,
                                               -5.1,  -1.0,  -6.0,   8.0
                                             };
double thr_offset_cal_01693D94[TQDC_NCH] = { // TQDC-16  Temp 48.69 C
                                             -10.1,  12.7,   8.0,   1.1,
                                             3.3,   3.9,  11.4,   4.2,
                                             -6.4,   6.4, -19.5,  19.2,
                                             -1.4,  -7.3, -20.4,  -5.4
                                           };
double adc_baseline_cal_01693D94[TQDC_NCH] = { // TQDC-16  Temp 48.69 C
                                               1.1,   1.4,  -9.1,  -2.0,
                                               -6.3,   2.8,  -1.6,   3.7,
                                               -5.8,   4.2,  -1.6,   2.8,
                                               -4.8,  -5.3,  -5.0,   6.1
                                             };
double thr_offset_cal_01691778[TQDC_NCH] = { // TQDC-16  Temp 45.12 C
                                             10.8, -11.4,  -2.3, -24.2,
                                             -10.8,   7.7,  -3.3,  -6.7,
                                             13.0,  -6.1,  -1.1,  13.9,
                                             -10.1,  -5.1,   1.4, -24.5
                                           };
double adc_baseline_cal_01691778[TQDC_NCH] = { // TQDC-16  Temp 45.12 C
                                               -6.9,  -3.0,   1.2,  -5.9,
                                               -6.8,   3.9,  -6.4,  -1.1,
                                               -0.9,  -3.9, -11.0,  -5.2,
                                               -4.1,   2.1,  -9.1,   3.8
                                             };
double thr_offset_cal_01690370[TQDC_NCH] = { // TQDC-16  Temp 36.94 C
                                             7.4, -15.4, -34.2, -20.8,
                                             -1.7,   2.7, -13.3, -22.6,
                                             -30.4, -10.8, -29.8, -42.6,
                                             -12.6, -30.1, -28.3, -30.1
                                           };
double adc_baseline_cal_01690370[TQDC_NCH] = { // TQDC-16  Temp  36.9 C
                                               -0.9,   5.9,  -2.7,   0.1,
                                               -2.9,   1.1,  -3.9,  -1.8,
                                               -5.2,   0.0,  -1.8,   3.1,
                                               -4.2,   1.0,  -0.0,  -1.1
                                             };
double thr_offset_cal_016912fe[TQDC_NCH] = { // TQDC-16  Temp 33.25 C
                                             -18.9,   6.4, -19.5,   2.4,
                                             -24.5, -17.6,  -8.6, -35.4,
                                             -8.3, -27.3,  -3.3, -11.1,
                                             -17.6, -15.4,  -3.6, -14.2
                                           };
double adc_baseline_cal_016912fe[TQDC_NCH] = { // TQDC-16  Temp  33.2 C
                                               -5.9,   5.1,  -6.6,  -3.2,
                                               -3.4,   3.9,  -4.9,  -0.8,
                                               1.9,  -2.4,   1.9,   0.0,
                                               -4.9,  -1.8,  -7.6,  -1.0
                                             };
double thr_offset_cal_01693d9e[TQDC_NCH] = { // TQDC-16  Temp 27.88 C
                                             -13.9, -10.8,  -8.3, -11.4,
                                             -13.3, -23.3, -24.2, -19.2,
                                             -26.4, -25.1, -16.4, -21.7,
                                             -17.0, -16.4,  -6.7, -14.2
                                           };
double adc_baseline_cal_01693d9e[TQDC_NCH] = { // TQDC-16  Temp  27.9 C
                                               -3.8,  -1.0,  -2.8,   0.7,
                                               -3.8,   4.1,  -4.2,  -8.0,
                                               -10.3,  -2.0,   3.0,  -0.8,
                                               3.7,   6.9,  -6.4,   2.1
                                             };
double thr_offset_cal_016923f6[TQDC_NCH] = { // TQDC-16  Temp 28.56 C
                                             -16.4, -34.8, -36.7, -15.4,
                                             -26.7, -17.3, -29.8, -27.6,
                                             -12.9, -26.4, -19.5, -32.6,
                                             -12.6, -24.2, -32.6, -23.3
                                           };
double adc_baseline_cal_016923f6[TQDC_NCH] = { // TQDC-16  Temp  28.6 C
                                               -1.8,   0.9,  -0.4,   0.4,
                                               -2.0,   1.0,  -2.1,  -4.9,
                                               -2.8,   3.5,  -0.1,  -6.0,
                                               -4.0,  -0.7,  -0.4,  -1.6
                                             };
double thr_offset_cal_01692b55[TQDC_NCH] = { // TQDC-16  Temp 30.62 C
                                             -15.8, -29.2,  -8.3, -35.8,
                                             -11.4, -11.4, -14.8,  -9.8,
                                             -11.7, -10.8,   4.9, -10.1,
                                             -7.0,  10.2, -16.7, -18.6
                                           };
double adc_baseline_cal_01692b55[TQDC_NCH] = { // TQDC-16  Temp  30.6 C
                                               -7.3,  -3.3,  -2.9,   3.1,
                                               -5.6,   5.7, -10.6,  -4.9,
                                               -2.0,  -0.4,  -5.0,  -0.9,
                                               -8.7,  -2.0,  -3.3,   2.1
                                             };
double thr_offset_cal_016910d5[TQDC_NCH] = { // TQDC-16  Temp 30.81 C
                                             -10.8,   4.9, -17.9, -17.3,
                                             -20.1, -13.6, -22.0, -14.8,
                                             -4.2,  -6.1,  -8.9, -27.6,
                                             -19.5, -25.1, -14.2, -22.6
                                           };
double adc_baseline_cal_016910d5[TQDC_NCH] = { // TQDC-16  Temp  30.8 C
                                               3.3,   7.0,  -6.6,  -5.1,
                                               -3.4,  -0.0,  -9.7,  -2.0,
                                               -2.5,  -5.4,  -4.9, -10.0,
                                               -4.3,   6.3,   0.2,  -1.7
                                             };
double thr_offset_cal_0168dd27[TQDC_NCH] = { // TQDC-16  Temp 36.44 C
                                             -13.2, -47.6,   0.0, -21.4,
                                             -10.4, -32.6, -31.7,  -8.6,
                                             -30.8, -41.1, -19.8, -13.6,
                                             -13.6, -29.2,  -4.5, -28.2
                                           };
double adc_baseline_cal_0168dd27[TQDC_NCH] = { // TQDC-16  Temp  36.4 C
                                               -3.0,  -3.0,   1.0,  -2.0,
                                               3.0,   0.9,  -3.0,   2.1,
                                               -2.4,  -0.5,  -1.0,  -6.3,
                                               0.9,  -1.5,  -4.8,   1.1
                                             };
double thr_offset_cal_0168ca03[TQDC_NCH] = { // TQDC-16  Temp 46.06 C
                                             0.8, -17.6, -31.1, -27.6,
                                             -20.1,   0.0, -32.9, -37.6,
                                             -47.9, -34.8, -42.3, -42.6,
                                             -46.1,  -3.6, -44.2,   0.0
                                           };
double adc_baseline_cal_0168ca03[TQDC_NCH] = { // TQDC-16  Temp  46.1 C
                                               2.0,   3.0,  -0.1,   2.1,
                                               -2.0,  -0.9,   0.0,  -1.0,
                                               -512.0,   1.0,  -3.5,  -2.0,
                                               1.8,   0.9,  -2.0,  -4.0
                                             };
double thr_offset_cal_0168e76d[TQDC_NCH] = { // TQDC-16  Temp 46.62 C
                                             -35.1, -23.6, -19.8,  -5.4,
                                             -30.1,  -6.1, -32.6, -10.8,
                                             -30.4, -45.8, -26.4, -41.1,
                                             -28.2, -20.8, -28.9, -42.9
                                           };
double adc_baseline_cal_0168e76d[TQDC_NCH] = { // TQDC-16  Temp  46.6 C
                                               3.0,   5.2,   4.0,  -1.0,
                                               1.0,  -2.0,   3.0,   0.0,
                                               -2.0,   1.5,   0.9,  -1.0,
                                               -5.8,   4.0,   0.7,  -2.0
                                             };
double thr_offset_cal_0168fc83[TQDC_NCH] = { // TQDC-16  Temp 35.62 C
                                             -37.6, -27.0, -27.9, -27.9,
                                             -45.8, -49.8, -32.6, -21.4,
                                             -33.2, -27.0, -35.8, -37.0,
                                             -41.4, -19.8, -27.3, -12.3
                                           };
double adc_baseline_cal_0168fc83[TQDC_NCH] = { // TQDC-16  Temp  35.6 C
                                               -2.9,   3.6,  -2.0,  -2.0,
                                               -4.0,   6.7,   2.7,   3.0,
                                               -0.9,   3.8,   1.0,  -1.0,
                                               0.0,   3.9,   0.0,  -1.0
                                             };
double thr_offset_cal_0169265c[TQDC_NCH] = { // TQDC-16  Temp 38.12 C
                                             -0.8, -12.0, -17.6, -28.2,
                                             -26.7, -15.1, -29.2, -10.8,
                                             -37.6, -16.1, -10.4,  -3.9,
                                             -24.5, -12.0, -34.8, -23.6
                                           };
double adc_baseline_cal_0169265c[TQDC_NCH] = { // TQDC-16  Temp  33.8 C
                                               4.2,  -1.4,  -2.0,   3.0,
                                               -5.8,   6.0,  -6.0,  14.5,
                                               2.0,   0.9,  -2.3,   5.0,
                                               14.2,  23.1,  -0.0,   7.0
                                             };
double thr_offset_cal_0169131b[TQDC_NCH] = { // TQDC-16  Temp 38.38 C
                                             -22.0, -41.7,  -7.6, -16.1,
                                             -27.9,  -7.6, -12.6,  -7.0,
                                             8.6,  -2.0,  -5.1,   0.8,
                                             -15.1,  -7.6,  -8.6,  -9.5
                                           };
double adc_baseline_cal_0169131b[TQDC_NCH] = { // TQDC-16  Temp  38.4 C
                                               -4.7,   4.0,  -1.1,  13.0,
                                               -1.0,  -0.0,  24.6,   0.9,
                                               0.7,   2.2,   1.0,  -2.2,
                                               -8.0,  -1.6,  -3.4,   1.0
                                             };
double thr_offset_cal_0168ca7e[TQDC_NCH] = { // TQDC-16  Temp 35.81 C
                                             -6.7, -27.0,  -3.6,  -9.5,
                                             -21.4,  -9.8,   4.2,  -8.9,
                                             -23.2, -13.9, -39.8, -32.6,
                                             -0.8, -19.8, -18.6, -25.8
                                           };
double adc_baseline_cal_0168ca7e[TQDC_NCH] = { // TQDC-16  Temp  35.8 C
                                               -4.0,   1.9,   0.1,   3.5,
                                               -4.0,  -0.7,  -1.9,  -3.5,
                                               1.1,  -3.2,  -4.3,  -3.0,
                                               0.5,   1.0,   0.6,   2.2
                                             };
double thr_offset_cal_0168ebae[TQDC_NCH] = { // TQDC-16  Temp 36.25 C
                                             -17.0, -26.1,  -8.6, -25.1,
                                             -10.4,   1.4,  -3.6, -19.5,
                                             -30.8, -11.7, -24.8, -14.5,
                                             -36.1, -24.2,   5.2, -15.8
                                           };
double adc_baseline_cal_0168ebae[TQDC_NCH] = { // TQDC-16  Temp  36.2 C
                                               5.0,   3.0,   4.0,   3.9,
                                               1.6,   6.8,  -3.9,   2.0,
                                               -3.0,   1.4,   1.0,   1.0,
                                               -2.0,   1.8,   0.1,  -5.7
                                             };
double thr_offset_cal_01690410[TQDC_NCH] = { // TQDC-16  Temp 34.75 C
                                             -22.0, -26.4, -12.9, -27.3,
                                             -11.7,  -3.6,   5.8, -18.2,
                                             -24.8, -47.0, -29.8, -13.2,
                                             -39.2,  -7.9, -17.0,  -8.9
                                           };
double adc_baseline_cal_01690410[TQDC_NCH] = { // TQDC-16  Temp  34.8 C
                                               1.0,   1.0,  -4.8,   1.0,
                                               -0.6,   3.0,  -0.8,  -3.6,
                                               4.0,  12.0,   1.5,   5.5,
                                               -2.0,  -1.5,   1.0,  -5.5
                                             };
double thr_offset_cal_01690c44[TQDC_NCH] = { // TQDC-16  Temp 35.94 C
                                             -20.8,  -7.6,   2.4, -31.7,
                                             -40.1, -37.9, -42.6, -44.8,
                                             -35.8, -61.4, -60.8, -12.6,
                                             -29.8, -27.0, -42.3, -48.9
                                           };
double adc_baseline_cal_01690c44[TQDC_NCH] = { // TQDC-16  Temp  35.9 C
                                               1.0,  -1.0,  -6.0,  -3.0,
                                               1.4,   2.0,   1.0,   0.7,
                                               -1.0,   3.7,   0.3,   2.1,
                                               0.0,  -3.1,  10.0,  -3.0
                                             };


double get_thr_offset(quint64 serial, int ch) {
    //  printf("thr_offset: serial=%d\n", serial);
    if (ch<0 || ch > 15) return 0;

    if (serial == 0x0168CAA1) return thr_offset_cal_0168CAA1[ch];
    if (serial == 0x0168D9BF) return thr_offset_cal_0168D9BF[ch];
    if (serial == 0x0168E1AF) return thr_offset_cal_0168E1AF[ch];
    if (serial == 0x0168E22D) return thr_offset_cal_0168E22D[ch];
    if (serial == 0x0168ED35) return thr_offset_cal_0168ED35[ch];
    if (serial == 0x0168F52E) return thr_offset_cal_0168F52E[ch];
    if (serial == 0x01691778) return thr_offset_cal_01691778[ch];
    if (serial == 0x0169206B) return thr_offset_cal_0169206B[ch];
    if (serial == 0x01692E27) return thr_offset_cal_01692E27[ch];
    if (serial == 0x01692EB5) return thr_offset_cal_01692EB5[ch];
    if (serial == 0x0169319D) return thr_offset_cal_0169319D[ch];
    if (serial == 0x01693D94) return thr_offset_cal_01693D94[ch];
    if (serial == 0x01694095) return thr_offset_cal_01694095[ch];
    if (serial == 0x01690370) return thr_offset_cal_01690370[ch];
    if (serial == 0x016912fe) return thr_offset_cal_016912fe[ch];
    if (serial == 0x01693d9e) return thr_offset_cal_01693d9e[ch];
    if (serial == 0x016923f6) return thr_offset_cal_016923f6[ch];
    if (serial == 0x01692b55) return thr_offset_cal_01692b55[ch];
    if (serial == 0x016910d5) return thr_offset_cal_016910d5[ch];
    if (serial == 0x0168dd27) return thr_offset_cal_0168dd27[ch];
    if (serial == 0x0168ca03) return thr_offset_cal_0168ca03[ch];
    if (serial == 0x0168e76d) return thr_offset_cal_0168e76d[ch];
    if (serial == 0x0168fc83) return thr_offset_cal_0168fc83[ch];
    if (serial == 0x0169265c) return thr_offset_cal_0169265c[ch];
    if (serial == 0x0169131b) return thr_offset_cal_0169131b[ch];
    if (serial == 0x0168ca7e) return thr_offset_cal_0168ca7e[ch];
    if (serial == 0x0168ebae) return thr_offset_cal_0168ebae[ch];
    if (serial == 0x01690410) return thr_offset_cal_01690410[ch];
    if (serial == 0x01690c44) return thr_offset_cal_01690c44[ch];
    //  if (serial == 0x) return thr_offset_cal_[ch];

    return 0;
}

double get_adc_baseline(quint64 serial, int ch) {
    //  printf("adc_baseline: serial=%d\n", serial);
    if (ch<0 || ch > 15) return 0;

    if (serial == 0x0168CAA1) return adc_baseline_cal_0168CAA1[ch];
    if (serial == 0x0168D9BF) return adc_baseline_cal_0168D9BF[ch];
    if (serial == 0x0168E1AF) return adc_baseline_cal_0168E1AF[ch];
    if (serial == 0x0168E22D) return adc_baseline_cal_0168E22D[ch];
    if (serial == 0x0168ED35) return adc_baseline_cal_0168ED35[ch];
    if (serial == 0x0168F52E) return adc_baseline_cal_0168F52E[ch];
    if (serial == 0x01691778) return adc_baseline_cal_01691778[ch];
    if (serial == 0x0169206B) return adc_baseline_cal_0169206B[ch];
    if (serial == 0x01692E27) return adc_baseline_cal_01692E27[ch];
    if (serial == 0x01692EB5) return adc_baseline_cal_01692EB5[ch];
    if (serial == 0x0169319D) return adc_baseline_cal_0169319D[ch];
    if (serial == 0x01693D94) return adc_baseline_cal_01693D94[ch];
    if (serial == 0x01694095) return adc_baseline_cal_01694095[ch];
    if (serial == 0x01690370) return adc_baseline_cal_01690370[ch];
    if (serial == 0x016912fe) return adc_baseline_cal_016912fe[ch];
    if (serial == 0x01693d9e) return adc_baseline_cal_01693d9e[ch];
    if (serial == 0x016923f6) return adc_baseline_cal_016923f6[ch];
    if (serial == 0x01692b55) return adc_baseline_cal_01692b55[ch];
    if (serial == 0x016910d5) return adc_baseline_cal_016910d5[ch];
    if (serial == 0x0168dd27) return adc_baseline_cal_0168dd27[ch];
    if (serial == 0x0168ca03) return adc_baseline_cal_0168ca03[ch];
    if (serial == 0x0168e76d) return adc_baseline_cal_0168e76d[ch];
    if (serial == 0x0168fc83) return adc_baseline_cal_0168fc83[ch];
    if (serial == 0x0169265c) return adc_baseline_cal_0169265c[ch];
    if (serial == 0x0169131b) return adc_baseline_cal_0169131b[ch];
    if (serial == 0x0168ca7e) return adc_baseline_cal_0168ca7e[ch];
    if (serial == 0x0168ebae) return adc_baseline_cal_0168ebae[ch];
    if (serial == 0x01690410) return adc_baseline_cal_01690410[ch];
    if (serial == 0x01690c44) return adc_baseline_cal_01690c44[ch];
    //  if (serial == 0x) return adc_baseline_cal_[ch];

    return 0;
}
}

CalibrationData::CalibrationData(QObject *parent)
//    : QSettings(getCalibrationDataFileName(), QSettings::IniFormat, parent)
    : QSettings(QSettings::IniFormat, QSettings::UserScope,
          QCoreApplication::organizationName(),
          "vmedaq_cal",
          parent)
{
    qDebug() << fileName();
    /*
    QList<quint64> ids;
    ids += 0x0168ca03;
    ids += 0x0168ca7e;
    ids += 0x0168CAA1;
    ids += 0x0168D9BF;
    ids += 0x0168dd27;
    ids += 0x0168E1AF;
    ids += 0x0168E22D;
    ids += 0x0168e76d;
    ids += 0x0168ebae;
    ids += 0x0168ED35;
    ids += 0x0168F52E;
    ids += 0x0168fc83;
    ids += 0x01690370;
    ids += 0x01690410;
    ids += 0x01690c44;
    ids += 0x016910d5;
    ids += 0x016912fe;
    ids += 0x0169131b;
    ids += 0x01691778;
    ids += 0x0169206B;
    ids += 0x016923f6;
    ids += 0x0169265c;
    ids += 0x01692b55;
    ids += 0x01692E27;
    ids += 0x01692EB5;
    ids += 0x0169319D;
    ids += 0x01693D94;
    ids += 0x01693d9e;
    ids += 0x01694095;

    QMap<quint64, double> temp_bl, temp_thr;
    temp_bl[0x0168ca03] = 46.1;
    temp_bl[0x0168ca7e] = 35.8;
    temp_bl[0x0168caa1] = 27.56;
    temp_bl[0x0168d9bf] = 34.50;
    temp_bl[0x0168dd27] = 36.4;
    temp_bl[0x0168e1af] = 26.88;
    temp_bl[0x0168e22d] = 26.94;
    temp_bl[0x0168e76d] = 46.6;
    temp_bl[0x0168ebae] = 36.2;
    temp_bl[0x0168ed35] = 36.00;
    temp_bl[0x0168f52e] = 51.25;
    temp_bl[0x0168fc83] = 35.6;
    temp_bl[0x01690370] = 36.9;
    temp_bl[0x01690410] = 34.8;
    temp_bl[0x01690c44] = 35.9;
    temp_bl[0x016910d5] = 30.8;
    temp_bl[0x016912fe] = 33.2;
    temp_bl[0x0169131b] = 38.4;
    temp_bl[0x01691778] = 45.12;
    temp_bl[0x0169206b] = 51.31;
    temp_bl[0x016923f6] = 28.6;
    temp_bl[0x0169265c] = 33.8;
    temp_bl[0x01692b55] = 30.6;
    temp_bl[0x01692e27] = 31.62;
    temp_bl[0x01692eb5] = 28.81;
    temp_bl[0x0169319d] = 29.62;
    temp_bl[0x01693d94] = 48.69;
    temp_bl[0x01693d9e] = 27.9;
    temp_bl[0x01694095] = 31.50;

    temp_thr[0x0168ca03] = 46.06;
    temp_thr[0x0168ca7e] = 35.81;
    temp_thr[0x0168caa1] = 27.56;
    temp_thr[0x0168d9bf] = 34.50;
    temp_thr[0x0168dd27] = 36.44;
    temp_thr[0x0168e1af] = 26.88;
    temp_thr[0x0168e22d] = 26.94;
    temp_thr[0x0168e76d] = 46.62;
    temp_thr[0x0168ebae] = 36.25;
    temp_thr[0x0168ed35] = 36.00;
    temp_thr[0x0168f52e] = 51.25;
    temp_thr[0x0168fc83] = 35.62;
    temp_thr[0x01690370] = 36.94;
    temp_thr[0x01690410] = 34.75;
    temp_thr[0x01690c44] = 35.94;
    temp_thr[0x016910d5] = 30.81;
    temp_thr[0x016912fe] = 33.25;
    temp_thr[0x0169131b] = 38.38;
    temp_thr[0x01691778] = 45.12;
    temp_thr[0x0169206b] = 51.31;
    temp_thr[0x016923f6] = 28.56;
    temp_thr[0x0169265c] = 38.12;
    temp_thr[0x01692b55] = 30.62;
    temp_thr[0x01692e27] = 31.62;
    temp_thr[0x01692eb5] = 28.81;
    temp_thr[0x0169319d] = 29.62;
    temp_thr[0x01693d94] = 48.69;
    temp_thr[0x01693d9e] = 27.88;
    temp_thr[0x01694095] = 31.50;

//    beginWriteArray("TQDC");
//    int i=0;
//    Q_FOREACH(quint64 id, ids) {
//        setArrayIndex(i++);
//        QString idstr = QString("%1").arg(id, 8, 16, QChar('0'));
//        setValue("id", idstr);
//    }
//    endArray();
    Q_FOREACH(quint64 id, ids) {
        QString idstr = QString("%1").arg(id, 8, 16, QChar('0'));
        QString key = QString("TQDC-%1").arg(idstr);
        setValue(QString("/TQDC/%1").arg(idstr), "");
        //        setValue(QString("/keys/%1-adc_baseline").arg(key), "");
        //        setValue(QString("/keys/%1-thr_offset").arg(key), "");
        setValue(QString("%1-adc_baseline/date").arg(key), QDateTime::currentDateTime().toUTC().toString());
        setValue(QString("%1-thr_offset/date").arg(key), QDateTime::currentDateTime().toUTC().toString());
        setValue(QString("%1-adc_baseline/temp").arg(key), temp_bl[id]);
        setValue(QString("%1-thr_offset/temp").arg(key), temp_thr[id]);
        for(int i=0; i<cal::TQDC_NCH; i++) {
            double thr = cal::get_thr_offset(id, i);
            setValue(QString("%1-thr_offset/%2").arg(key).arg(i), thr);
            double bl = cal::get_thr_offset(id, i);
            setValue(QString("%1-adc_baseline/%2").arg(key).arg(i), bl);
        }
    }
*/

    sync();
}

QString CalibrationData::moduleKey(const QString &moduleName, quint64 serialId)
{
    return QString("%1-%2").arg(moduleName).arg(serialId, 8, 16, QChar('0'));
}

//QString CalibrationData::getCalibrationDataFileName()
//{
//    return QDesktopServices::storageLocation(QDesktopServices::DataLocation)
//            + QDir::separator() + "vmedaq_cal.ini";
//    return "vmedaq_cal.ini";
//}
