//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <QtGlobal>

#include "decoder/decode_trig.h"
#include "modules/trig_types.h"
#include "daqcommon/rawdata.h"
#include "DbufState.h"

DbufState::DbufState()
{
  clear();
}

unsigned int DbufState::getTrigTimestampRaw() const
{
    return trig_decode_make_ts(trig_ts_hi, trig_ts_lo);
}

double DbufState::getTrigTimestamp() const
{
    return getTrigTimestampRaw() * TRIG_TS_STEP;
}

void DbufState::clear()
{
  clearSpill();
}

void DbufState::clearSpill()
{
  clearEvent();
}
void DbufState::clearEvent()
{
  clearModule();
  en = 0;
  trig_ts_lo = 0;
  trig_ts_hi = 0;
}
void DbufState::clearModule()
{
  mid = 0;
  slot = 0;
  modflags = 0;
  q = 0;
  nq = 0;
  mod_word_count_by_datatype.clear();
}

void DbufState::handleSHDR(unsigned int data)
{
    Q_UNUSED(data);
    clearSpill();
}

void DbufState::handleSTRL(unsigned int data)
{
    Q_UNUSED(data);
//  clearSpill();
}

void DbufState::handleMHDR(unsigned int data)
{
  clearModule();
  mid = decode_mhdr_id(data);
  slot = decode_mhdr_ga(data);
}

void DbufState::handleMTRL(unsigned int data)
{
//  clearModule();
  modflags = decode_mtrl_flags(data);
}

void DbufState::handleEHDR(unsigned int data)
{
  clearEvent();
  en = decode_ehdr_en(data);
}

void DbufState::handleETRL(unsigned int data)
{
    Q_UNUSED(data);
//  clearEvent();
}

std::string DbufState::getModuleName() const
{
  return module_id_str((mtype_t)(getModuleId()));
}
