
#include "decoder/decode_base.h"

#include <QDebug>
#include <vector>

#include "daqcommon/daqexc.h"
#include "daqcommon/vme_types.h"
#include "hal/base_mod.h"
#include "hal/hw_manager.h"
#include "decoder/decoder.h"
#include "daqcommon/rawdata.h"
#include "decoder/event.h"

local_base_decoder::local_base_decoder(const hw_manager &p, event_t * const event)
    : hwf(p), ev(event)
{
    Q_ASSERT(ev);
    buf = 0;
    len = 0;
    first_ch = 0;
    error_flag = 0;
    hwmod = 0;
    updateDataFormat();
}

void local_base_decoder::clear()
{

}

void local_base_decoder::print() const
{
    Q_ASSERT(ev);
    qDebug() << "*** base decoder state ***";
    qDebug().nospace() << "ev=" << ev->en << ", first_ch=" << first_ch;
    qDebug() << "**************************";
}

void local_base_decoder::updateDataFormat()
{
    map_modules();
}

void local_base_decoder::map_modules()
{
    /*
  key_by_slot.clear();
  for (int slot_num=VME_MIN_GA; slot_num<=VME_MAX_GA; slot_num++) {
        const base_mod *hwfmod = hwf->find_mod_by_slot_const(slot_num);
        if (!hwfmod) continue;
        ddb_key_t key = hwfmod->get_ddb_key();
        key_by_slot[slot_num] = key;
  }
*/
}

void local_base_decoder::addStat(const std::string &name, qint64 count)
{
    Q_ASSERT(ev);
    Q_ASSERT(hwmod);
    ddb_key_t key = hwmod->get_ddb_key();
    ev->eventStats[name][key] += count;
}
void local_base_decoder::setStat(const std::string &name, qint64 count)
{
    Q_ASSERT(ev);
    Q_ASSERT(hwmod);
    ddb_key_t key = hwmod->get_ddb_key();
    ev->eventStats[name][key] = count;
}

//! input parameters: raw data (without module header/trailer)
//  target event
//  module number (of given module ID)
//  Return 0 on success, -1 on error
//  Derived decoders must call local_base_decoder::decode() for initialization
void local_base_decoder::prepare_decode(const unsigned int *f_buf, size_t f_len, int f_first_ch, const base_mod *f_hwmod)
{
    buf = f_buf;
    len = f_len;
    first_ch = f_first_ch;
    hwmod = f_hwmod;
}

void local_base_decoder::post_decode()
{
    setStat("events_total");
}

/*
int event_decode_base (const unsigned int *buf, size_t len, event_t * ev, int nm, const hw_manager &p)
{
//  printf("event_decode_base: %d\n", nm);
  local_base_decoder ldec(buf, len, ev, nm, p);
  int code = ldec.decode();
//  ldec.print(); // Uncomment to see verbose event printout
  return code;
}
*/

bool local_base_decoder::decode_module_header()
{
    if (len<1) return false;
    //  unsigned int mhdr = buf[0];
    return true;
}

bool local_base_decoder::decode_module_trailer()
{
    if (len<1) return false;
    unsigned int mtrl = buf[len];
    //  if ((mtrl&ADLVME_DTYPE_MASK_ALL) != ADLVME_DTYPE_MTRL) setStat("mtrl format error");

    // readout error
    bool re = decode_mtrl_re(mtrl);
    if (re) setStat("readout_error");
    bool ro = decode_mtrl_ro(mtrl);
    if (ro) setStat("readout_overflow");
    return true;
}

void local_base_decoder::handle_hptdc_error(unsigned int d_raw)
{
    int err = d_raw&0x7FFF;

    std::vector<unsigned int> sum(3, 0);
    for (int g = 0; g<4; g++)
        for (int i = 0; i<3; i++) {
        int b = (g<<3) + i;
        if (err & (1<<b)) sum[i]++;
    }
    // specific errors
    int tdc_hit_lost_readout_overflow = sum[0];
    int tdc_hit_lost_l1_overflow = sum[1];
    int tdc_hit_error = sum[2];
    int tdc_hit_lost_event_limit = (err & (1<<12))?1:0;
    int tdc_event_lost = (err & (1<<13))?1:0;
    int tdc_internal_error = (err & (1<<14))?1:0;
    int tdc_readout_overflow = (err & (1<<16))?1:0;
    int tdc_readout_error = (err & (1<<17))?1:0;

    if (tdc_hit_lost_readout_overflow)
        setStat("tdc_hit_lost_readout_overflow");
    if (tdc_hit_lost_l1_overflow)
        setStat("tdc_hit_lost_l1_overflow");
    if (tdc_hit_error)
        setStat("tdc_hit_error");
    if (tdc_hit_lost_event_limit)
        setStat("tdc_hit_lost_event_limit");
    if (tdc_event_lost)
        setStat("tdc_event_lost");
    if (tdc_internal_error)
        setStat("tdc_internal_error");
    if (tdc_readout_overflow)
        setStat("tdc_readout_overflow");
    if (tdc_readout_error)
        setStat("tdc_readout_error");
    // summary
    if (tdc_hit_lost_readout_overflow || tdc_hit_lost_l1_overflow ||
        tdc_hit_lost_event_limit || tdc_readout_overflow)
        setStat("events_with_overflow");
    if (tdc_hit_error || tdc_internal_error)
        setStat("events_with_warning");
    if (tdc_event_lost || tdc_readout_error)
        setStat("events_with_error");
}
