
#include <QDebug>

#include "daqcommon/rawdata.h"
#include "decoder/decoder.h"
#include "decode_msc.h"
#include "modules/msc_types.h"
#include "decoder/event.h"
#include "hal/hw_manager.h"

local_msc_decoder::local_msc_decoder(const hw_manager &p, event_t * const event)
    : local_base_decoder(p, event)
//    , prev_cnt(new msc_counts_t)
{
    clear();
}

void local_msc_decoder::print() const
{
    qDebug() << "*** MSC decoder state ***";
    Q_ASSERT(ev);
    if (!ev) return;
    qDebug().nospace() << "ev=" << ev->en << ", first_ch=" << first_ch;
    const msc_event_t &msc = ev->msc;
    // TODO
}

void local_msc_decoder::clear()
{
    prev_ts = 0;
//    prev_cnt->clear();
}

int local_msc_decoder::decode()
{
    Q_ASSERT(ev);
    msc_event_t &msc = ev->msc;
    QVector<int> cnt(MSC_NCH * MSC_NDEV, 0);

    for (size_t p = 1; p < len; p++) {
        unsigned int d = buf[p];
        unsigned int dt = (d & ADLVME_DTYPE_MASK_ALL) >> 28;
        switch (dt) {
        case 0:
            cnt[0] =  d      & 0x7F;
            cnt[1] = (d>>7)  & 0x7F;
            cnt[2] = (d>>14) & 0x7F;
            cnt[3] = (d>>21) & 0x7F;
            break;
        case 1:
            cnt[4] =  d      & 0x7F;
            cnt[5] = (d>>7)  & 0x7F;
            cnt[6] = (d>>14) & 0x7F;
            cnt[7] = (d>>21) & 0x7F;
            break;
        case 2:
            cnt[8]  =  d      & 0x7F;
            cnt[9]  = (d>>7)  & 0x7F;
            cnt[10] = (d>>14) & 0x7F;
            cnt[11] = (d>>21) & 0x7F;
            break;
        case 3:
            cnt[12] =  d      & 0x7F;
            cnt[13] = (d>>7)  & 0x7F;
            cnt[14] = (d>>14) & 0x7F;
            cnt[15] = (d>>21) & 0x7F;
            break;
        case 4: {
                quint32 ts = d & 0xFFFFFFF;
                if (ts < prev_ts)
                    qCritical() << "msc: timestamp wrapped " << prev_ts << " to " << ts;
                prev_ts = ts;
//                msc.data[ts] = cnt;
                for(int i=0; i<cnt.size(); i++)
                    msc.data[i].push_back(cnt[i]);
                break;
            }
            // TODO
        default: ; // unknown data type
        }
    }
    return 0;
}
