
#include <cstddef>
#include <QDebug>

#include "daqcommon/rawdata.h"
#include "decoder/decoder.h"
#include "decoder/decode_trig.h"
#include "modules/trig_types.h"
#include "decoder/event.h"

namespace {
    const int table_trig_state_by_input[TRIG_N_INPUTS] = {
        -1, //  0 - N/A
        3, 3, 3, 3, 3, 3, 3, // 1..7 - NIM
        0, // 8 - ST
        -1, -1, -1, -1, // 9..12 - N/A
        1, // 13 - TQDC1
        -1, -1, -1, // 14..16 - N/A
        1, // 17 - TQDC1
        -1, -1, -1, // 18..20 - N/A
        1, // 21 - TQDC1
        -1, -1, -1, // 22..24 - N/A
        2, // 25 - TQDC2
        -1, -1, -1, // 26..28 - N/A
        2, // 29 - TQDC2
        -1, -1, -1, // 30..32 - N/A
        2, // 33 - TQDC2
        -1, -1, -1, // 34..36 - N/A
        2, // 37 - TQDC2
        -1, -1 // 38..39 - N/A
    };

    const int table_trig_state_by_bit[TRIG_N_BITS] = {
        -1, //  0 - N/A
        3, 3, 3, 3, 3, 3, 3, // 1..7 - NIM
        0, // 8 - ST
        1, 1, 1, // 9..11 - TQDC1
        2, 2, 2, 2, // 12..15 - TQDC2
        -1, -1, -1, -1 // special
    };

}

int trig_get_state_by_bit(unsigned int bit)
{
    if (bit >= TRIG_N_BITS) return -1;
    return table_trig_state_by_bit[bit];
}

void local_trig_decoder::print() const
{
    Q_ASSERT(ev);
    qDebug() << "*** TRIG decoder state ***";
    if (!ev) return;
    qDebug().nospace() << "ev=" << ev->en << ", first_ch=" << first_ch;
    qDebug() << "**************************";
}

int local_trig_decoder::decode()
{
    Q_ASSERT(ev);
    if (!ev) return -1;
    //  unsigned int mhdr = buf[0];
    unsigned int ts_lo = 0, ts_hi = 0;
    unsigned int cnt_num = 0, logic_cnt_all_num = 0, logic_cnt_read_num = 0;
    trig_event_t &trig = ev->trig;
    trig_cnts_t &cnts = trig.incnt;
    trig_cnts_t &cnt_logic_all = trig.logic_all;
    trig_cnts_t &cnt_logic_read = trig.logic_read;

    //  printf("ev %d, event_decode_trig: len=%d\n", ev->en, len);
    for (size_t p = 1; p < len; p++) {
        unsigned int d = buf[p];
        unsigned int dt = d & ADLVME_DTYPE_MASK_ALL;
        if (dt == 0x00000000) { // logic state counters, matched (excluding dead time)
            int count = d & 0x0FFFFFFF;
            if (count>0) {
                cnt_logic_read[logic_cnt_read_num] = count;
            }
            logic_cnt_read_num++;
        }
        if (dt == 0x10000000) { // logic state counters, all (triggered and dead time)
            int count = d & 0x0FFFFFFF;
            if (count>0) {
                cnt_logic_all[logic_cnt_all_num] = count;
            }
            logic_cnt_all_num++;
        }
        if (dt == 0x40000000) {
            //		ts_lo = d & 0x0FFFFFFF;
            ts_lo = trig_d4_decode_ts_lo(d);
            //		printf("trig ts_lo=%08X\n", ts_lo);
        }
        if (dt == 0x50000000) {
            ts_hi = trig_d5_decode_ts_hi(d);
            trig.trigbits = trig_d5_decode_trigbits(d);
            //		printf("trig ts_hi=%08X, bits=%04X\n", ts_hi, ev->trig.trigbits);
        }
        if (dt == 0x70000000) {
            int count = d & 0x0FFFFFFF;
            if (count>0) {
                cnts[cnt_num] = count;
            }
            if (cnt_num < TRIG_N_INPUTS)
                cnt_num++;
        }
    }
    trig.ts = trig_decode_make_ts(ts_hi, ts_lo);
    //  printf("trig ts=%08X, bits=%04X\n", ev->trig.ts, ev->trig.trigbits);
    return 0;
}
