#ifndef _DECODER_H
#define _DECODER_H

#include <cstddef>
#include <string>
#include <vector>
#include <QVector>
#include <map>

#include "daqcommon/daq.h"
#include "daqcommon/device_ids.h"


class hw_manager;

class local_base_decoder;
class local_dch_decoder;
class local_msc_decoder;
class local_trig_decoder;
class local_tqdc_decoder;
class local_tdc32vl_decoder;
class local_tdc64v_decoder;
class local_tdc96_decoder;
class local_mtdc64_decoder;
class local_phtdc_decoder;
class base_mod;
class event_t;

class base_decoder {
public:
  base_decoder(const hw_manager &p);
  virtual ~base_decoder();
  virtual void reset();
  virtual void clear();
  virtual void print() const;
  virtual bool decode_buf(const unsigned int *p, size_t s, unsigned int rawdata_flags);
  virtual const event_t &get_event() const {return *ev;}
  virtual void dataFormatChanged();

protected:
  void register_decoder(local_base_decoder *);
  virtual void updateDataFormat();
  virtual void map_channels();
  virtual void map_modules();

private:
  virtual bool decode_module_data(mtype_t mid, size_t pmh, size_t pmt, const base_mod *hwmod);
  virtual void setbuf(const unsigned int *p, size_t s) {buf=p; len=s;}

  event_t * const ev;
  const hw_manager &hwf;
  const unsigned int *buf;
  size_t len;

  local_dch_decoder * const dbric;
  local_msc_decoder * const dmsc;
  local_trig_decoder * const dtrig;
  local_tqdc_decoder * const dtqdc;
  local_tdc32vl_decoder * const dtdc32vl;
  local_tdc64v_decoder * const dtdc64v;
  local_tdc96_decoder * const dtdc96;
  local_mtdc64_decoder * const dmtdc64;
  local_phtdc_decoder * const dphtdc;

  std::map<int, int> ch_offset;
//  std::map<int, ddb_key_t> key_by_slot;
  QVector<local_base_decoder *> decoders;
};

//#include "evlist.h"

#endif                          /* _DECODER_H */
