//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef EVENT_H
#define EVENT_H

#include <QtCore/QtGlobal>
#include <vector>
#include <map>

#include "decoder/eventstats.h"
#include "event_types.h"

class tdc_hit_t {
public:
    tdc_hit_t() : time(0), wid(0), trail_edge(false) {}
    tdc_hit_t(int set_time, int set_wid, bool set_trail_edge=false): time(set_time), wid(set_wid), trail_edge(set_trail_edge) {}
    void clear() { *this = tdc_hit_t(); }
    bool operator<(const tdc_hit_t &r) const { return time<r.time; }
    int time;
    int wid;
    bool trail_edge;
};

class tdc_event_t {
public:
    tdc_event_t() : error(0) {}
    //	tdc_event_t() : hits(DECODER_TDC_MAXHITS), cnt(DECODER_TDC_MAXCNT) { clear(); }
    void clear() { hits.clear(); cnts.clear(); error=0;}
    tdc_hits_t hits;
    tdc_cnts_t cnts;
    quint32 error;
};

class tqdc_hit_q_t {
public:
    tqdc_hit_q_t() : ts(0), charge(0) {}
    void clear() {*this = tqdc_hit_q_t();}
    int ts;
    double charge;
};

class tqdc_wf_t {
public:
    tqdc_wf_t() : ts(0) {}
    void clear() {*this = tqdc_wf_t();}
    double getCharge() const;
    int ts;
    tqdc_samples_t samp;
};

class tqdc_event_t {
public:
    tqdc_event_t() : ts(0), tdc_error(0) {}
    void clear() { *this = tqdc_event_t(); }
    int ts;
    quint32 tdc_error;
    tqdc_hits_t_t hits_t;
    tqdc_hits_q_t hits_q;
    tqdc_wfs_t wfs;
    tqdc_cnts_t cnts;
};

class trig_event_t {
public:
    trig_event_t() : ts(0), trigbits(0) {}
    void clear() { *this = trig_event_t(); }
    trig_cnts_t incnt, logic_all, logic_read;
    quint64 ts;
    unsigned int trigbits;
};

class dch_window_t {
public:
    void clear() { bits.clear();}
    std::vector<unsigned char> bits;
};

class dch_branch_t {
public:
    dch_branch_t() : timeout(false), disconnect(false), data_error(false), overflow(false) {}
    void clear() { *this = dch_branch_t(); }
    bool isOk() const { return !timeout && !disconnect && !data_error; }
    bool timeout;
    bool disconnect;
    bool data_error;
    bool overflow;
    dch_windows_t wins;
};

class dch_event_t {
public:
    dch_event_t() : bad_branch_id(false), bad_window_id(false) {}
    void clear() { *this = dch_event_t(); }
    bool bad_branch_id;
    bool bad_window_id;
    dch_branches_t branches;
};

class msc_event_t {
public:
    msc_event_t() {}
    void clear() { *this = msc_event_t(); }
    msc_data_t data;
};

class event_t {
public:
    event_t() : status(0), en(0), no_downscale(0), evtype(EVT_NORM), size(0), time_dist(0) {}
    void clear() { *this = event_t(); }
    int status;
    int en;
    int no_downscale; // set to1 to disable downscaling of this event. used for end-of-burst events.
    evtype_t evtype; // event type
    size_t size;
    double time_dist; // us
    tdc_event_t tdc;
    tqdc_event_t tqdc;
    trig_event_t trig;
    dch_event_t bric;
    msc_event_t msc;
    EventStats eventStats;
};

#endif // EVENT_H
