
#include "inlcorr.h"

#include <QDebug>
#include <cmath>

QVector<double> InlCorr::makeDNLCorrectionTable(const QVector<double> code_density)
{
    if (code_density.size() != 1024) {
        qWarning() << "makeDNLCorrectionTable: invalid input vector size";
        return QVector<double>();
    }
    double sumv = 0;
    for(int i=0; i<1024; i++) {
        double v = code_density[i];
        sumv += v;
    }
    QVector<double> inl(1024, 0.0);
    double avgv = sumv / 1024.;
    double sumrelv = 0;
    double inlsum = 0;
    for(int i=0; i<1024; i++) {
        double relv = (code_density[i] - avgv) / avgv;
        sumrelv += relv;
        inl[i] = sumrelv;
        inlsum += sumrelv;
    }
//                qDebug() << "avgv = " << avgv; // Ok
//                qDebug() << "sumrelv = " << sumrelv;
//                qDebug() << "inlsum = " << inlsum;
    QVector<double> corr(1024, 0.0);
    for(int i=0; i<1024; i++) {
        corr[i] = inl[i] - inlsum/1024.;
        corr[i] = round(corr[i]*100.0) / 100.0; // quantize
    }
//                qDebug() << "TQDC DNL for channel " << ch << corr;
    return corr;
}
