//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "tqdc_calibration_set.h"

#include <QDebug>

#include "decoder/event.h"
#include "modules/tqdc_types.h"

double tqdc_calibration_set::get_baseline(int ch) const
{
    if (baseline_num.find(ch) == baseline_num.end()) return 0; // not exists
    int n = baseline_num.find(ch)->second;
    if (n<1) return 0; // error?
    return baseline_val.find(ch)->second/n; // may throw exception
}

void tqdc_calibration_set::set_baseline(int ch, double value)
{
    baseline_val[ch] = value;
    baseline_num[ch] = 1;
}

void tqdc_calibration_set::add(int ch, double value)
{
    baseline_val[ch] += value;
    baseline_num[ch]++;
}

void tqdc_calibration_set::add(const tqdc_hits_q_t &hits_q)
{
    Q_FOREACH(int ch, hits_q.keys()) {
        const tqdc_hitv_q_t &hitv_q = hits_q.value(ch);
        for(tqdc_hitv_q_t::const_iterator jt = hitv_q.begin(); jt != hitv_q.end(); ++jt) {
            const tqdc_hit_q_t &hit_q = (*jt);
            add(ch, hit_q.charge);
        }
    }
}

void tqdc_calibration_set::print() const
{
    qDebug() << "# TQDC calibration set";
    unsigned int j = 0;
    for(baseline_val_t::const_iterator it = baseline_val.begin(); it != baseline_val.end(); ++it) {
        int i = it->first;
        j = i % TQDC_NCH;
        QString str;
        if (j==0)
            str += "tqdc_baseline[] = { ";
        str += QString("%1").arg(get_baseline(i)) + ", ";
        if (j==(TQDC_NCH-1))
            str += " };";
        qDebug() << str;
        i++; j++;
    }
}

