#ifndef FVME_REG_H
#define FVME_REG_H

typedef union {
  unsigned int all;
  struct {
	  unsigned int spill_src: 3;
	  unsigned int trig_src: 3;
          unsigned int softclear: 1;
          unsigned int stat_clr: 1;
          unsigned int reserved: 8;
  } bit;
} fvme_reg_control_0_t;

typedef union {
  unsigned int all;
  struct {
	  unsigned int reserved: 15;
	  unsigned int run_mode: 1;
  } bit;
} fvme_reg_control_1_t;

typedef union {
  unsigned int all;
  struct {
	  unsigned int am: 6;
	  unsigned int reserved: 7;
	  unsigned int read: 1;
	  unsigned int iack_n: 1;
	  unsigned int cblt: 1;
  } bit;
} fvme_reg_am_1_t;

typedef union {
  unsigned int all;
  struct {
	  unsigned int tmout: 1;
	  unsigned int dtack: 1;
	  unsigned int berr: 1;
	  unsigned int reserved: 5;
	  unsigned int irqs: 8;
  } bit;
} fvme_reg_status_0_t;

// FVME registers
enum FVME_REQ {
  FVME_REQ_REG_CTRL_0   = 0x0000, // vme general control
  FVME_REQ_REG_CTRL_1   = 0x0001, // vme run mode control
  FVME_REQ_REG_AM_0     = 0x0002, // vme am, flags in program mode
  FVME_REQ_REG_AM_1     = 0x0003, // vme am, flags in run mode
  FVME_REQ_REG_AD_0     = 0x0004, // vme address lo (bits 0-15)
  FVME_REQ_REG_AD_1     = 0x0005, // vme address hi (bits 16-31)
  FVME_REQ_REG_WR_0     = 0x0006, // vme write data lo (bits 0-15)
  FVME_REQ_REG_WR_1     = 0x0007, // vme write data hi (bits 16-31)
  FVME_REQ_REG_RD_0     = 0x0008, // vme read data lo (bits 0-15)
  FVME_REQ_REG_RD_1     = 0x0009, // vme read data hi (bits 16-31)
  FVME_REQ_REG_ST_0     = 0x000A, // status_0
  FVME_REQ_REG_ST_1     = 0x000B, // status_1
  FVME_REQ_REG_FR_0     = 0x000C, // hardware frame repetition interval, 25ns step, default 0x8000
  FVME_REQ_REG_TEST_0   = 0x000D, // test register, reserved for selftest
  FVME_REQ_REG_SPWC_0   = 0x000E, // spill wordcount lo (bits 0-15)
  FVME_REQ_REG_SPWC_1   = 0x000F, // spill wordcount hi (bits 16-31)
};

enum { FVME_REQ_RDWR_MASK = 0x80000000 };
enum { FVME_REQ_RD = 0x80000000 };
enum { FVME_REQ_WR = 0x00000000 };

// FVME register AM bits
enum {
  FVME_REQ_REG_AM_BIT_WRITE = 0x0000,
  FVME_REQ_REG_AM_BIT_READ  = 0x2000
};
enum { FVME_REQ_REG_AM_BIT_IACK_N  = 0x4000 };
enum { FVME_REQ_REG_AM_BIT_CBLT  = 0x8000 };

#endif /* FVME_REG_H */
