#ifndef HWIF_VME_MLINK_H
#define HWIF_VME_MLINK_H

#include <QtCore/QtGlobal>
#include <vector>
#include <QVector>

#include "hal/hwif_vme.h"
#include "fvme_reg.h"

struct vme_cr_t;
struct vme_csr_t;

typedef std::vector<quint32> fvme_req_buf_t;

class hwif_vme_mlink:public virtual hwif_vme {
public:
    hwif_vme_mlink ();
    virtual ~hwif_vme_mlink ();

    // single bus operations
    virtual void w_a16_d16 (unsigned int addr, int data);
    virtual void r_a16_d16 (unsigned int addr, int *data);
    virtual void w_a16_d32 (unsigned int addr, int data);
    virtual void r_a16_d32 (unsigned int addr, int *data);
    virtual void w_a16_d64 (unsigned int addr, qint64 data);
    virtual void r_a16_d64 (unsigned int addr, qint64 *data);
    virtual void w_r_a16_d32 (unsigned int addr, int wdata, int *rdata);
    virtual void w_a24_d32 (unsigned int addr, int data);
    virtual void r_a24_d32 (unsigned int addr, int *data);

    // block bus operation
    virtual int blk_io(vme_block_io_t &multi_io);

    // grouped operations (not BLT, really)
    void w_blk_a24_d32 (unsigned int addr, size_t n32, int *data);
    void r_blk_a24_d32 (unsigned int addr, size_t n32, int *data);

    virtual void w_a16_d32(const std::vector<int> &addr, const std::vector<int> &data);
    virtual std::vector<int> r_a16_d32(const std::vector<int> &addr);

    // test operations
    virtual bool testreg32 (int addr, unsigned int loops);
    //  virtual bool testreg16 (int addr, unsigned int loops);
    virtual bool testram32 (int addr, unsigned int radix, unsigned int count);
    //  virtual bool testram16 (int addr, unsigned int radix, unsigned int count);

    virtual bool check_readout_chain_status();

    virtual void start_communication();
    virtual void reset();
    virtual void post_start_run();
    virtual void pre_stop_run();
    virtual bool interspill_test();
    virtual void start_of_burst_signal();
    virtual void end_of_burst_signal();

    virtual void get_events(hwev_queue_t *q, int timeout_ms);
    virtual bool selftest();
    virtual unsigned int get_spwc() const {return fvme_spwc;}
    virtual size_t max_blk_io_size() const;

protected:
    // CR/CSR
    virtual void w_crcsr_d16 (unsigned int addr, int data);
    virtual void r_crcsr_d16 (unsigned int addr, int *data);
    void driverVersionCheck();

private:
    quint16 fvme_get_reg16(int reg);
    void fvme_set_reg16(int reg, quint16 data);
    quint32 fvme_get_reg32(int reg);
    void fvme_set_reg32(int reg, quint32 data);
    quint64 fvme_get_reg64(int reg);
    void fvme_set_reg64(int reg, quint64 data);
    bool fvme_get_device_info();
    QVector<quint32> fvme_get_stats();
    void set_ctrl_0();
    void set_ctrl_1();
    void sysreset();
    void softclear();
    void stat_clr();
    void clear_events();
    void set_run_mode (bool new_run_mode);
    fvme_req_buf_t fvme_request(const fvme_req_buf_t &req); // wrapper
    fvme_req_buf_t fvme_request_impl(const fvme_req_buf_t &req);
    bool check_link_state();

    /* VME bus operations - calls vme_blk_io */

    void vme_maw16 (VME_AM am, int addr, int data);
    void vme_maw32 (VME_AM am, int addr, int data);
    void vme_mar16 (VME_AM am, int addr, int *data);
    void vme_mar32 (VME_AM am, int addr, int *data);
    void vme_maw32_blk(size_t count, VME_AM am, const int *addr, const int *data);
    void vme_mar32_blk(size_t count, VME_AM am, const int *addr, int *data);

    virtual void set_cblt_address(unsigned int chain_address);

    unsigned int fvme_req_seq;
    bool fvme_run_mode;
    unsigned int fvme_spwc;
    fvme_reg_control_0_t fvme_ctrl_0;
    fvme_reg_control_1_t fvme_ctrl_1;
    bool spill_started_after_run_enable;
    bool isOpen;
};

#endif /* HWIF_VME_MLINK_H */
