//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <cassert>

#include "modlist_be.h"
#include "base_bmod_factory.h"

base_bmod *base_bmod_factory::create(mtype_t id, const bmod_props &props)
{
    //  ostringstream ost;
    base_bmod *p=0;
    switch (id) {
        // generic modules
    case MODULE_ID_SLVME:
    case MODULE_ID_ROC48:
    case MODULE_ID_ADLVME:
    case MODULE_ID_PHLOM:
        p = new base_bmod (props);
        break;

        // special modules
#ifdef PHTDC_BMOD_H
    case MODULE_ID_PHTDC:
        p = new phtdc_bmod (props);
        break;
#endif
#ifdef TDC32VL_BMOD_H
    case MODULE_ID_TDC32VL:
        p = new tdc32vl_bmod (props);
        break;
#endif
#ifdef TDC64V_BMOD_H
    case MODULE_ID_TDC64V:
        p = new tdc64v_bmod (props);
        break;
#endif
#ifdef TDC96_BMOD_H
    case MODULE_ID_TDC96:
        p = new tdc96_bmod (props);
        break;
#endif
#ifdef MSC_BMOD_H
    case MODULE_ID_MSC16V:
        p = new msc_bmod (props);
        break;
#endif
#ifdef MTDC64_BMOD_H
    case MODULE_ID_MTDC64:
        p = new mtdc64_bmod (props);
        break;
#endif
#ifdef TQDC_BMOD_H
    case MODULE_ID_TQDC:
        p = new tqdc_bmod (props);
        break;
#endif
#ifdef TRIG_BMOD_H
    case MODULE_ID_TRIG:
        p = new trig_bmod (props);
        break;
#endif
#ifdef BRIC_BMOD_H
    case MODULE_ID_BRIC:
        p = new bric_bmod (props);
        break;
#endif
        break;
    default: {
            id = MODULE_ID_DUMMY;
            p = new base_bmod (props);
        }
        break;
    }
    assert(p);
    p->set_id(id);
    return p;
}
