
#include "hal/base_mod.h"

#include <QDebug>
#include <QString>
#include <typeinfo>

#include "daqcommon/daqexc.h"
#include "hal_xml/base_mod_impl.h"
#include "daqcommon/hal_types.h"
#include "daqcommon/device_ids.h"
//#include "hal/calibrationdata.h"
#include "decoder/CalibrationData.h"

namespace {
    mfam_t mtype2mfam(mtype_t mt)
    {
        switch (mt) {
        case MODULE_ID_PHTDC:
            return MF_TDC;
        case MODULE_ID_TDC32VL:
            return MF_TDC;
        case MODULE_ID_TDC64V:
            return MF_TDC;
        case MODULE_ID_TDC96:
            return MF_TDC;
        case MODULE_ID_MSC16V:
            return MF_MSC;
        case MODULE_ID_MTDC64:
            return MF_TDC;
        case MODULE_ID_TRIG:
            return MF_TRIG;
        case MODULE_ID_PHLOM:
            return MF_PHLOM;
        case MODULE_ID_TQDC:
            return MF_TQDC;
        case MODULE_ID_BRIC:
            return MF_DCH;
        default:
            return MF_OTHER;
        }
    }
}

base_mod::base_mod()
    :
    bmi(new base_mod_impl(this)),
    setup(0),
    calibrationData(GlobalCalibrationData::Instance())
//    calibrationData(new CalibrationData)
{
}

base_mod::~base_mod()
{
//    delete calibrationData;
    delete bmi;
}

int base_mod::get_nch() const {if (setup) return setup->nch; else return 0;}
int base_mod::get_ndev() const {if (setup) return setup->ndev; else return 0;}
size_t base_mod::get_setup_size() const {return sizeof(common_setup_t);}
mod_stats_t base_mod::get_stats() const {return setup->stats;}

mfam_t base_mod::get_fam() const {
    if (setup_valid())
        return mtype2mfam(get_id());
    else
        return MF_OTHER;
}

const char *base_mod::get_fam_name() const
{
    switch (get_fam()) {
    case MF_OTHER: return "OTHER";
    case MF_TDC: return "TDC";
    case MF_TRIG: return "TRIG";
    case MF_TQDC: return "TQDC";
    case MF_PHLOM: return "PHLOM";
    case MF_DCH: return "DCH";
    case MF_MSC: return "MSC";
    default:
        return "UNKNOWN";
    }
}

//const string base_mod::get_name() {
//  return module_id_str(get_id());
//}

bool base_mod::copy_setup(const common_setup_t *f_setup, size_t f_size)
{
    Q_ASSERT(setup);
    Q_ASSERT(f_setup);
    Q_ASSERT(f_size >= sizeof(common_setup_t));
    if (!setup || f_size < sizeof(common_setup_t)) {
        throw exc_base_mod("base_mod::copy_setup: setup not set");
    }
    if (!f_setup) {
        throw exc_base_mod("base_mod::copy_setup: NULL data ptr");
    }
    if (f_size != get_setup_size()) {
        return false;
        //	throw exc("base_mod::copy_setup: incompatible size");
    }
    memcpy(setup, f_setup, f_size);
    return true;
}

void base_mod::set_setup(common_setup_t *f_setup, size_t f_size)
{
    //  if (!setup_struct_valid(typeid(*this).name(), (common_setup_t*)f_setup, f_size)) {
    //        throw exc("base_mod::set_setup: invalid setup supplied");
    //  }
    size_t my_size = get_setup_size(); //! virtual, depends on module type
    if (f_size != my_size) {
        QString str = QString("%1::set_setup: incompatible size %2, expected %3").arg(get_name()).arg(f_size).arg(my_size);
        throw exc(str.toStdString());
    }
    setup = static_cast<common_setup_t *>(f_setup);
    //  setup_size = f_size;
    //  flags &= ~STDMOD_FLAG_UNINITIALIZED;
}
/*
void base_mod::get_setup(void **f_setup, size_t *f_size) {
  if (f_setup) *f_setup = setup;
  if (f_size) *f_size = setup_size;
}
*/
void base_mod::print_info() const
{
    if (setup_valid()) {
        QString s;
        s = QString("%1 %2, nch %3, setup %4").arg(info_str())
                .arg(flags_str())
                //    <<(get_cblt_enable()?" CBLT":"")
                .arg(get_nch())
                .arg(setup_valid()?"valid":"bad");
        qWarning() << s;
    } else {
        qCritical() << "Module info: Invalid setup";
    }
}

QString base_mod::info_str() const
{
    return QString("ID %1:%2 (%3), serial %4, slot %5, fw %6.%7.%8")
            .arg(get_vend(), 2, 16, QChar('0'))
            .arg(get_id(), 2, 16, QChar('0'))
            .arg(module_id_str(get_id()))
            .arg(get_serial(), 8, 16, QChar('0'))
            .arg(get_slot())
            .arg(get_fw_ver() >> 8)
            .arg(get_fw_ver() - ((get_fw_ver() >> 8) << 8))
            .arg(get_fw_rev())
            ;
}

QString base_mod::flags_str() const
{
    const base_mod_flags flags = get_flags();
    return QString("flags=%1 %2%3%4%5%6%7%8%9%10%11").arg(flags, 8, 16, QChar('0'))
            .arg(flags&STDMOD_FLAG_CBLT?" CBLT":"")
            .arg(flags&STDMOD_FLAG_FIRST?" F":"")
            .arg(flags&STDMOD_FLAG_MID?" M":"")
            .arg(flags&STDMOD_FLAG_LAST?" L":"")
            .arg(flags&STDMOD_FLAG_FAIL?" FAIL":"")
            .arg(flags&STDMOD_FLAG_ERR?" ERR":"")
            .arg(flags&STDMOD_FLAG_WARN?" WARN":"")
            .arg(flags&STDMOD_FLAG_OVER?" OVER":"")
            .arg(flags&STDMOD_FLAG_TESTOK?" TESTOK":"")
            .arg(flags&STDMOD_FLAG_UNINITIALIZED?" NO_SETUP":"");
}

//void base_mod::print_flags() const
//{
//  int prio;
//  if (get_failure_flag()) prio = LOG_CRIT;
//  else if (get_error_flag()) prio = LOG_ERR;
//  else if (get_warning_flag()) prio = LOG_WARNING;
//  else if (get_overflow_flag()) prio = LOG_NOTICE;
//  else prio = LOG_INFO;
//  log_pri_mod(prio, 0, print_flags_str());
//}

void base_mod::print_setup() const
{
    qDebug() << "+---------- Module setup ----------";
    print_base_setup();
    qDebug() << "type: " << typeid(*this).name();
    print_private_setup();
    qDebug() << "+-------------- end ---------------";
}

void base_mod::print_base_setup() const
{
    print_info();
}

void base_mod::print_private_setup() const
{
    if (setup_valid()) {
        qDebug() << QString("%1: No private setup").arg(get_name());
    } else {
        qDebug() << QString("%1: invalid setup").arg(get_name());
    }
}

bool base_mod::setup_struct_valid(const QString &name, const common_setup_t *p, size_t sz)
{
    if (!p) return false;
    if (sz < sizeof(common_setup_t)) {
        qDebug() << name << " setup size " << sz << " < " << sizeof(common_setup_t);
        return false;
    }
    if (p->magic != BASE_SETUP_MAGIC) {
        qDebug() << name << QString(" setup magic %1 != %2").arg(p->magic, 8, 16, QChar('0')).arg(BASE_SETUP_MAGIC, 8, 16, QChar('0'));
        return false;
    }
    return true;
}

bool base_mod::setup_valid() const
{
    return setup_struct_valid(typeid(*this).name(), setup, get_setup_size());
}

/*
std::ostream& operator<< (std::ostream& o, const base_mod& base_mod)
{
  return o<<"print...";
}
*/

bool base_mod::validate_mch(int mch) const
{
    int nch = get_nch();
    if (nch==0 && mch==0) return true;
    return (mch>=0 && mch < get_nch());
}

QVariant base_mod::get_var(const std::string &tag, unsigned int mch) const
{
    QString str = QString("Not implemented %1::get_var, tag=%3, mch=%4").arg(get_name()).arg(QString::fromStdString(tag)).arg(mch);
    qWarning() << str;
    return QVariant();
}

void base_mod::set_var(const QVariant &value, const std::string &tag, int mch)
{
    QString str = QString("Not implemented %1::set_var, tag=%3, mch=%4, value=%5").arg(get_name()).arg(QString::fromStdString(tag)).arg(mch).arg(value.toString());
    qWarning() << str;
}

int base_mod::get_int(const std::string &tag, unsigned int mch) const
{
    QString str = QString("Not implemented %1::get_int, tag=%3, mch=%4").arg(get_name()).arg(QString::fromStdString(tag)).arg(mch);
    qWarning() << str;
    return 0;
}

void base_mod::set_int(int value, const std::string &tag, int mch)
{
    QString str = QString("Not implemented %1::set_int, tag=%3, mch=%4, value=%5").arg(get_name()).arg(QString::fromStdString(tag)).arg(mch).arg(value);
    qWarning() << str;
}

std::string base_mod::get_string(const std::string &tag, unsigned int mch) const
{
    QString str = QString("Not implemented %1::get_string, tag=%3, mch=%4").arg(get_name()).arg(QString::fromStdString(tag)).arg(mch);
    qWarning() << str;
    return std::string();
}

void base_mod::set_string(const std::string &text, const std::string &tag, int mch)
{
    QString str = QString("Not implemented %1::set_string, tag=%3, mch=%4, value=%5").arg(get_name()).arg(QString::fromStdString(tag)).arg(mch).arg(QString::fromStdString(text));
    qWarning() << str;
}

void base_mod::set_defaults()
{
    Q_ASSERT(setup);
    //  printf("base_mod::set_defaults: setup %p, size %d\n", setup, get_setup_size());
    //  size_t sz = get_setup_size();
    //  memset(setup, 0, sz);
    setup->magic = BASE_SETUP_MAGIC;
    //  setup->vendor_id = vendor_id;
    //  setup->device_id = device_id;
    //  setup->device_serial = serial;
    //  setup->flags = flags;

    //  mtype_t mt = get_id();
    set_cblt_enable(true);
}

bool base_mod::get_failure_flag() const
{
    return (get_flags() & (STDMOD_FLAG_FAIL));
}

bool base_mod::get_error_flag() const
{
    return (get_flags() & (STDMOD_FLAG_ERR|STDMOD_FLAG_FAIL));
}

bool base_mod::get_warning_flag() const
{
    return (get_flags() & (STDMOD_FLAG_ERR|STDMOD_FLAG_FAIL|STDMOD_FLAG_WARN));
}

bool base_mod::get_overflow_flag() const
{
    return (get_flags() & (STDMOD_FLAG_OVER));
}

base_mod_flags base_mod::get_flags() const
{
    //  return flags;
    if (setup_valid())
        return base_mod_flags(setup->flags);
    return STDMOD_FLAG_UNINITIALIZED;
}

double base_mod::get_temperature() const
{
    if (setup_valid())
        return setup->mon.temperature;
    return 0;
}

void base_mod::set_flags(base_mod_flags d)
{
    //  flags=d;
    if (setup_valid())
        setup->flags = d;
}
void base_mod::set_flags_bit(STDMOD_FLAG d)
{
    //  flags=d;
    if (setup_valid())
        setup->flags |= d;
}

void base_mod::clr_flags_bit(STDMOD_FLAG d)
{
    //  flags=d;
    if (setup_valid())
        setup->flags &= ~d;
}

bool base_mod::get_cblt_enable() const
{
    return (get_flags() & STDMOD_FLAG_CBLT) == STDMOD_FLAG_CBLT;
}

void base_mod::set_cblt_enable(bool d)
{
    if (d) set_flags_bit(STDMOD_FLAG_CBLT);
    else clr_flags_bit(STDMOD_FLAG_CBLT);
}

int base_mod::get_vend() const
{
    if (setup_valid())
        return setup->vendor_id;
    return 0;
}

void base_mod::set_vend(int d)
{
    if (setup_valid())
        setup->vendor_id = d;
}

mtype_t base_mod::get_id() const
{
    if (setup_valid())
        return setup->device_id;
    return MODULE_ID_DUMMY;
}

void base_mod::set_id(mtype_t d)
{
    if (setup_valid())
        setup->device_id = d;
}

int base_mod::get_rev() const
{
    if (setup_valid())
        return setup->device_rev;
    return 0;
}

void base_mod::set_rev(int d)
{
    if (setup_valid())
        setup->device_rev = d;
}

quint32 base_mod::get_serial() const
{
    if (setup_valid())
        return setup->device_serial;
    return 0;
}

void base_mod::set_serial(quint32 d)
{
    if (setup_valid())
        setup->device_serial = d;
}

quint32 base_mod::get_fw_ver() const
{
    if (setup_valid())
        return setup->fw_ver;
    return 0;
}

void base_mod::set_fw_ver(quint32 d)
{
    if (setup_valid())
        setup->fw_ver = d;
}

quint32 base_mod::get_fw_rev() const
{
    if (setup_valid())
        return setup->fw_rev;
    return 0;
}

void base_mod::set_fw_rev(quint32 d)
{
    if (setup_valid())
        setup->fw_rev = d;
}

QString base_mod::getFirmwareStr() const
{
    const quint32 ver = get_fw_ver();
    const quint32 rev = get_fw_rev();
    quint32 ver_major = (ver >> 8) & 0xFF;
    quint32 ver_minor = (ver) & 0xFF;
    bool ver_present = (ver != ~0u && ver != 0u);
    bool rev_present = (rev != ~0u && rev != 0u);
    if (ver_present || rev_present)
        return QString("%1.%2.%3").arg(ver_major).arg(ver_minor).arg(rev);
    else
        return QString();
}

int base_mod::get_slot() const
{
    if (setup_valid())
        return setup->device_slot;
    return 0;
}

void base_mod::set_slot(int d)
{
    if (setup_valid())
        setup->device_slot = d;
}

ddb_key_t base_mod::get_ddb_key() const
{
//    debug_mod(3, QString("get_ddb_key(): dev_id %1, dev_serial %2")
//              .arg(setup->device_id, 8, 16, QChar('0'))
//              .arg(setup->device_serial, 8, 16, QChar('0')));
    return static_cast<quint32>(setup->device_serial) ^
            (static_cast<quint32>(setup->device_id) << 24);
}

bool base_mod::readout_enabled() const
{
    int modflags = get_flags();
    return modflags & STDMOD_FLAG_CBLT;
}

//bool base_mod::is_enabled() const
//{
//  int modflags = get_flags();
//  return (modflags & STDMOD_FLAG_DISABLE) == 0;
//}

const char *base_mod::task_id_str(bmod_task_id_t task_id)
{
    switch (task_id) {
    case BMOD_TASK_READ_CONFIG: return("read_config");
    case BMOD_TASK_SELFTEST: return("selftest");
    case BMOD_TASK_RESET: return("reset");
    case BMOD_TASK_UPDATE_STATUS: return("update_status");
    case BMOD_TASK_PREPARERUN: return("preparerun");
    case BMOD_TASK_CALIBRATION: return("calibration");
    case BMOD_TASK_START_RUN: return("start_run");
    case BMOD_TASK_POST_START_RUN: return("post_start_run");
    case BMOD_TASK_PRE_STOP_RUN: return("pre_stop_run");
    case BMOD_TASK_STOP_RUN: return("stop_run");
    case BMOD_TASK_START_SPILL: return("start_spill");
    case BMOD_TASK_STOP_SPILL: return("start_spill");
    case BMOD_TASK_START_OF_BURST: return("start_of_burst");
    case BMOD_TASK_END_OF_BURST: return("end_of_burst");
    case BMOD_TASK_START_OF_BURST_SIGNAL: return("start_of_burst_signal");
    case BMOD_TASK_END_OF_BURST_SIGNAL: return("end_of_burst_signal");
    case BMOD_TASK_CLEAR_STATISTICS: return("clear_statistics");
    case BMOD_TASK_UPDATE_STATISTICS: return("update_statistics");
    case BMOD_TASK_INTERSPILL_TEST: return("interspill_test");
    default: return("unknown_task");
    }
}
/*
CalibrationData base_mod::getCalibrationData() const
{
    return *calibrationData;
}

void base_mod::setCalibrationCondition(const QString &name, const QVariant &val) { calibrationData->setCondition(name, val); }
QVariant base_mod::getCalibrationCondition(const QString &name) const { return calibrationData->getCondition(name); }
void base_mod::setCalibrationParam(const QString &name, int index, const QVariant &val) { calibrationData->setPar(name, index, val); }
QVariant base_mod::getCalibrationParam(const QString &name, int index) const { return calibrationData->getPar(name, index); }
*/

QString base_mod::calibrationKey() const
{
    return CalibrationData::moduleKey(QString::fromAscii(get_name()).trimmed(), get_serial());
}

void base_mod::setCalibrationValue(const QString &name, const QVariant &data)
{
    QString key = QString("%1-%2").arg(calibrationKey()).arg(name);
    calibrationData->setValue(key, data);
}

QVariant base_mod::getCalibrationValue(const QString &name)
{
    QString key = QString("%1-%2").arg(calibrationKey()).arg(name);
    return calibrationData->value(key);
}
