#ifndef BASE_MOD_H
#define BASE_MOD_H

#include <QString>
#include <QVariant>

#include "hal/device_db_types.h"
#include "daqcommon/device_ids.h"
#include "mf_types.h"
#include "base_mod_types.h"

class base_mod_impl;

class common_setup_t;
class mod_stats_t;
class CalibrationData;

class base_mod
{
public:
    enum { LOG_DEBUG, LOG_INFO, LOG_NOTICE, LOG_WARNING, LOG_ERR };
    base_mod();
    virtual ~base_mod();
    // Attributes
    virtual int get_vend() const;
    virtual void set_vend(int d);
    virtual mtype_t get_id() const;
    virtual void set_id(mtype_t d);
    virtual quint32 get_serial() const;
    virtual void set_serial(quint32 d);
    virtual quint32 get_fw_ver() const;
    virtual void set_fw_ver(quint32 d);
    virtual quint32 get_fw_rev() const;
    virtual void set_fw_rev(quint32 d);
    QString getFirmwareStr() const;
    virtual int get_rev() const;
    virtual void set_rev(int d);
    virtual bool get_cblt_enable() const;
    virtual void set_cblt_enable(bool d);
    virtual int get_nch() const;
    virtual int get_ndev() const;
    virtual ddb_key_t get_ddb_key() const;
    virtual mfam_t get_fam() const;
    virtual const char *get_fam_name() const;
    virtual const char* get_name() const {return module_id_str(get_id());}
    //  const char* get_class_name() const {return typeid (*this).name();}
    //  virtual const std::string get_name() const;
    virtual int get_slot() const;
    virtual void set_slot(int d);
    virtual base_mod_flags get_flags() const;
    QString info_str() const;
    QString flags_str() const;
    //  void print_flags() const;
    virtual double get_temperature() const;
    bool readout_enabled() const;
    //  bool is_enabled() const;

    virtual bool copy_setup(const common_setup_t *f_setup, size_t f_size);
    virtual void set_setup(common_setup_t *f_setup, size_t f_size);
    virtual common_setup_t *get_setup() const {return setup;}
    virtual size_t get_setup_size() const;
    virtual void print_info() const;
    virtual void print_setup() const;
    virtual void print_base_setup() const;
    //  virtual void print() const {print_info();}
    virtual bool setup_valid() const;

    virtual QVariant get_var(const std::string &tag, unsigned int mch=GET_FIRST_CH) const;
    virtual void set_var(const QVariant &value, const std::string &tag, int mch=SET_ALL_CH);
    virtual int get_int(const std::string &tag, unsigned int mch=GET_FIRST_CH) const;
    virtual void set_int(int value, const std::string &tag, int mch=SET_ALL_CH);
    virtual std::string get_string(const std::string &tag, unsigned int mch = GET_FIRST_CH) const;
    virtual void set_string(const std::string &text, const std::string &tag, int mch = SET_ALL_CH);

    virtual void set_defaults();

    static const char *task_id_str(bmod_task_id_t task_id);

/*
    //    CalibrationData getCalibrationData() const;
    void setCalibrationCondition(const QString &name, const QVariant &val);
    QVariant getCalibrationCondition(const QString &name) const;
    void setCalibrationParam(const QString &name, int index, const QVariant &val);
    QVariant getCalibrationParam(const QString &name, int index) const;
*/
    void setCalibrationValue(const QString &name, const QVariant &data);
    QVariant getCalibrationValue(const QString &name);
    //  friend std::ostream& operator<< (std::ostream& o, const base_mod& base_mod);

    mod_stats_t get_stats() const;

    virtual void set_flags(base_mod_flags d);

    base_mod_impl * const bmi;

protected:
    virtual void set_flags_bit(STDMOD_FLAG d);
    virtual void clr_flags_bit(STDMOD_FLAG d);
    virtual bool get_failure_flag() const;
    virtual bool get_error_flag() const;
    virtual bool get_warning_flag() const;
    virtual bool get_overflow_flag() const;
    virtual void print_private_setup() const;
    virtual bool validate_mch(int mch) const;

    //  virtual void write_calibration(QDomElement nodeModule) const;
    //  virtual void read_calibration(QDomElement nodeModule);
    QString calibrationKey() const;

    common_setup_t *setup;
    CalibrationData * const calibrationData;

private:
    static bool setup_struct_valid(const QString &name, const common_setup_t *p, size_t sz);

};

#endif /* BASE_MOD_H */
