
/*
 * BRIC module
 */

#include "bric_mod.h"

#include <QDebug>
#include <QString>

#include "modules/bric_types.h"
#include "daqcommon/daqexc.h"

size_t bric_mod::get_setup_size() const {return sizeof(bric_setup_t);}

void bric_mod::print_private_setup() const
{
    Q_ASSERT(setup_valid());
    const bric_setup_t *s = static_cast<const bric_setup_t *>(setup);
    Q_ASSERT(s);

    qDebug() << QString("Window enable mask: %1").arg(s->win_enable, 4, 16, QChar('0'));
}

void bric_mod::set_defaults()
{
    base_mod::set_defaults();
    bric_setup_t *s = static_cast<bric_setup_t *>(setup);
    Q_ASSERT(s);
    s->data_shift_period = bric_setup_t().data_shift_period;
    s->data_strobe_delay = bric_setup_t().data_strobe_delay;
    s->win_enable = bric_setup_t().win_enable;
    s->branch_size_limit_index = bric_setup_t().branch_size_limit_index;
}

int bric_mod::get_int(const std::string &tag, unsigned int mch) const
{
    const bric_setup_t *s = static_cast<const bric_setup_t *>(setup);
    Q_ASSERT(s);

    if (tag == "win_enable") {return s->win_enable;}
    if (tag == "branch_size_limit_index") {return s->branch_size_limit_index;}

    // unhandled
    QString str = QString("%1::get_int: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    qWarning() << str;
    throw exc_mod_unknown_tag(str.toStdString());
}

void bric_mod::set_int(int value, const std::string &tag, int mch)
{
    bric_setup_t *s = static_cast<bric_setup_t *>(setup);
    Q_ASSERT(s);

    if (tag == "win_enable") {
        s->win_enable = value;
        return;
    }
    if (tag == "branch_size_limit_index") {
        s->branch_size_limit_index = value;
        return;
    }

    // unhandled
    QString str = QString("%1::set_int: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    qWarning() << str;
    throw exc_mod_unknown_tag(str.toStdString());
}
