//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DEVICE_DB_H
#define DEVICE_DB_H

#include <QString>
#include <QByteArray>
#include <QMap>

#include "hal/device_db_types.h"

class ddb_ent: public QByteArray {
public:
    ddb_ent() : QByteArray() {}
    explicit ddb_ent(const void *data, size_t size);
    explicit ddb_ent(const QByteArray &p);
};

typedef QMap<ddb_key_t, ddb_ent> ddbmap_t;

class device_db {
public:
  device_db(const QString &filename);
  ddb_ent find(ddb_key_t key) const;
  void put(ddb_key_t key, const void *data, size_t size);
  void clear(); // purge database
  bool load();
  bool save() const;
  bool saveAs(const QString &fileName) const;
  static QString defaultFileName();
  QList<ddb_key_t> keys() const;

protected:
    void put(ddb_key_t key, const ddb_ent &data);

private:
  const QString fn;
  ddbmap_t devs;
};

#endif /* DEVICE_DB_H */
