/*
 * Hardware backend: user-interface functions
 */

#ifndef HW_BACKEND_H
#define HW_BACKEND_H

#include <list>
#include <map>

#include "hal/hw_manager.h"
#include "hal/base_hwif.h"

class QString;
class device_db;
class base_hwif;
class base_bmod;
class SharedSetupBackend;
struct shm_status_t;
struct shm_pio_rawdata_t;

class hw_backend: public hw_manager {
public:
    hw_backend(shm_status_t *st, shm_pio_rawdata_t *rd);
    virtual ~hw_backend();
  virtual bool isReadonly() const {return false;}
//    virtual void init ();
//  virtual void close();
//  virtual void open();
  virtual void print() const;
  virtual void update_status();

  virtual void start_communication();
  virtual void autodetect();
  virtual void endburst();
  virtual void interspill_test();
  virtual void preparerun();
  virtual void calibration();
  virtual void reset();
  virtual void selftest();
  virtual void start_run();
  virtual void stop_run();
  virtual void start_spill();
  virtual void stop_spill();
  virtual void start_of_burst_signal();
  virtual void end_of_burst_signal();

  virtual void get_events(hwev_queue_t *q, int timeout_ms);
  virtual unsigned int get_spwc() const;
  virtual bool ddb_save_as(const QString &fn) const;
  void initialize_database();
  static QString defaultDbFile();

protected:
  virtual void add_module (const int key, base_mod *);
  virtual void build_cblt_chain ();
//  virtual void set_run_mode(bool mode);

  void bmod_task(bmod_task_id_t task_id);

private:
  virtual void autodetect_modules(base_hwif *);
  virtual void open_ddb();
  virtual void sync_to_db();

  device_db *const ddb;
  SharedSetupBackend * const sharedSetup;
  base_hwif * const hwif;
  shm_status_t * const shm_status;
  shm_pio_rawdata_t * const shm_pio_rawdata;
};

#endif /* HW_BACKEND_H */

