/*
 * Hardware manager: module management functions
 */

#ifndef HW_MANAGER_H
#define HW_MANAGER_H

#include <cstddef>
#include <list>
#include <QMap>
#include <QVariant>
#include <string>

#include "daqcommon/device_ids.h"
#include "mf_types.h"
#include "hal/device_db_types.h"
#include "base_mod_types.h"

class base_mod;
class common_setup_t;

typedef std::list <base_mod * > modlist_t;
typedef QMap < int, base_mod * > modmap_t;

class hw_manager {
public:
    virtual ~hw_manager () {}
  virtual bool isReadonly() const {return true;}
//  virtual void init_hws ();
//  virtual void create_modules_from_hws();
  virtual void delete_modules();
  virtual void print() const {print_summary();}
  virtual void print_modules() const;

  virtual const modmap_t get_modmap() const {return mods;}
//  int get_ch_info (mfam_t mf, int ch, mtype_t * mt, int *key) const;
  virtual bool mf_present(mfam_t mf) const;
  virtual bool mt_present(mtype_t mt) const;
  virtual int get_nmod () const {return mods.size();}
  virtual int get_nch_byfam (mfam_t mf) const;
  virtual int get_nmod_byfam (mfam_t mf) const;

  virtual bool is_all_channels_same(const std::string &tag, mfam_t mf) const;

  virtual QVariant    get_var   (const std::string &tag, mfam_t mf, int ch = GET_FIRST_CH) const;
  virtual int         get_int   (const std::string &tag, mfam_t mf, int ch = GET_FIRST_CH) const;
  virtual std::string get_string(const std::string &tag, mfam_t mf, int ch);
  virtual void set_var   (const QVariant &value,  const std::string &tag, mfam_t mf, int ch = SET_ALL_CH);
  virtual void set_int   (int value,              const std::string &tag, mfam_t mf, int ch = SET_ALL_CH);
  virtual void set_string(const std::string &str, const std::string &tag, mfam_t mf, int ch);

  const base_mod *find_mod_by_slot_const(int slot) const;
  base_mod *find_mod_by_slot(int slot);
//  std::list <base_mod *> find_mod_by_type(mtype_t mt) const;
  const base_mod *find_mod_by_fam_ch_const (mfam_t mf, int ch, int *mch) const;
  base_mod *find_mod_by_fam_ch (mfam_t mf, int ch, int *mch);

  const base_mod *find_mod_by_key_const (ddb_key_t key) const;

  virtual void insert_module(int key, common_setup_t *setup, size_t size);

protected:
  hw_manager ();
  virtual void print_summary() const;

  modmap_t mods;
  bool daq_hal_initialized;

private:
  virtual int hal_initialized ();
  int get_mod_nch_by_fam_nmod (mfam_t mf, int find_nmod) const;
  bool match_mfam(mfam_t, mfam_t) const;
  virtual int get_nch(mtype_t) const;
//  int get_mid_by_mfch(mfam_t, int) const;
  modlist_t find_mod_by_fam(mfam_t mf);
//  modlist_t find_mod_by_fam_const(mfam_t mf) const;

};

#endif /* HW_MANAGER_H */
