
#include <QtCore/QtGlobal>
#include <QtDebug>

#include "daqcommon/hal_types.h"
#include "hal/hws.h"

namespace {
    const unsigned int HWS_HDR_MAGIC = ((unsigned int)0xc32528c3);
    const unsigned int HWS_ENT_MAGIC = ((unsigned int)0x5ee5631b);
}

bool hws_t::hdr_valid(const hws_hdr_t *hdr)
{
    if (!hdr) return false;
    if (hdr->magic != HWS_HDR_MAGIC) return false;
    if (hdr->nent < 0 || hdr->nent >= MAX_HW_MODULES) return false;
    return true;
}

bool hws_ent_valid(const hws_ent_t *ent)
{
    if (!ent) return false;
    if (ent->magic != HWS_ENT_MAGIC) return false;
    if (ent->sz < sizeof(common_setup_t)) return false;
    return true;
}

void hws_initialize_hdr(hws_hdr_t *hdr)
{
    Q_ASSERT(hdr);
    hdr->nent = 0;
    hdr->magic = HWS_HDR_MAGIC;
}

//void hws_print_hdr(const hws_hdr_t &hdr)
//{
//    printf("hws hdr: magic %08X (%s), %d entries\n", hdr.magic, (hdr.magic == HWS_HDR_MAGIC)?"Ok":"BAD", hdr.nent);
//}

//void hws_print_ent(const hws_ent_t *ent, int n)
//{
//  if (ent)
//  printf("hws entry #%d: magic %08X (%s), size %zu\n", n, ent->magic, (ent->magic==HWS_ENT_MAGIC)?"Ok":"BAD",
//  ent->sz);
//  else
//        printf("hws entry: <null>\n");
//}

//void hws_print_data(const void *data, int n)
//{
//  const common_setup_t *s = static_cast<const common_setup_t *>(data);
//  if (data)
//  printf("hws data #%d: @%p magic %08X (%s), ID %02X:%02X, serial %d\n", n, s,
//  s->magic, (s->magic == BASE_SETUP_MAGIC)?"Ok":"BAD",
//  s->vendor_id, s->device_id, s->device_serial);
//  else
//        printf("hws entry: <null>\n");
//}

hws_t::~hws_t()
{
}

common_setup_t *hws_t::get_data_ptr(int n)
{
    if (!hws_t::hdr_valid(hdr)) return 0;
    if (n<0 || n>=count()) return 0; // range check
    return data[n];
}

size_t hws_t::get_data_size(int n) const
{
    if (!hws_t::hdr_valid(hdr)) return 0;
    if (n<0 || n>=count()) return 0; // range check
    const hws_ent_t *ent = &hdr->ent[n];
    if (!hws_ent_valid(ent)) return 0;
    return ent->sz;
}



//void hws_t::print() const
//{
//  hws_print_hdr(*hdr);
//  for (int i=0; i<count();i++) {
//      hws_print_ent(&hdr->ent[i], i);
//      hws_print_data(data[i], i);
//  }
//}

bool hws_t::hasSpace() const
{
    return count() < MAX_HW_MODULES;
}

int hws_t::count() const
{
    Q_ASSERT(hdr);
    return hdr->nent;
}

int hws_t::addEntry(void *ptr, size_t size)
{
    Q_ASSERT(ptr);
    if (!hasSpace())
        throw std::bad_alloc();
    const int index = hdr->nent;
    data[index] = static_cast<common_setup_t *>(ptr);
    hws_ent_t *ent = &hdr->ent[index];
    ent->magic = HWS_ENT_MAGIC;
    ent->sz = size;
    hdr->nent++;
    return index;
}

void hws_t::attachData(int index, void *ptr)
{
    Q_ASSERT(ptr);
    Q_ASSERT(index >= 0);
    Q_ASSERT(index < hdr->nent);
    data[index] = static_cast<common_setup_t *>(ptr);
}
