#ifndef MOD_REGS_H
#define MOD_REGS_H

enum {
  MODULE_REG_CTRL = 0, // Control register
  MODULE_REG_JTAG  = 3, // JTAG register
  MODULE_REG_TEST = 7,
  MODULE_REG_JTAG32 = 8, // JTAG 32-bit data register
  MODULE_REG_STATUS = 9, // Status register
  MODULE_REG_SERIAL = 0x10,
  MODULE_REG_SERIAL_lo = 0x10,
  MODULE_REG_SERIAL_hi = 0x11,
  MODULE_REG_TEMPERATURE_1 = 0x12,
  MODULE_REG_TEMPERATURE_2 = 0x13,
  MODULE_REG_FW_VER = 0x14,
  MODULE_REG_FW_REV = 0x15
};

const unsigned int MODULE_CTRL_DEFAULT = 0x00000000;
const unsigned int MODULE_STATUS_DEFAULT = 0xFFFFFFFF;

#pragma pack(push, 1)
typedef union {
  unsigned int all;
  struct {
	unsigned int run:1;
	unsigned int reset:1;
	unsigned int cblt:1;
	unsigned int dummy_blt_data:1;
	unsigned int reserved2:12;
	unsigned int reserved3:4;
	unsigned int mid:1;
	unsigned int last:1;
	unsigned int first:1;
	unsigned int reserved4:1;
	unsigned int reserved5:8;
  } bit;
} module_ctrl_reg_t;

//! Status register has inverse logic.
//! 0xFFFFFFFF = Ok
//! Error is 0.
typedef union {
  unsigned int all;
  struct {
	unsigned int ttc_lock:1;    // no clock on TTC bus
	unsigned int device_ok:1;   // device ok
	unsigned int subdev_0_ok:1; // subdevice 0 ok
	unsigned int subdev_1_ok:1; // subdevice 1 ok
	unsigned int subdev_2_ok:1; // subdevice 2 ok
	unsigned int subdev_3_ok:1; // subdevice 3 ok
	unsigned int subdev_4_ok:1; // subdevice 4 ok
	unsigned int subdev_5_ok:1; // subdevice 5 ok

	unsigned int reserved_5:5;
	unsigned int readerror:1; // readout error
	unsigned int overflow:1; // readout buffer overflow
	unsigned int reserved:1;

	unsigned int reserved_16:16;
  } bit;
} module_status_reg_t;
#pragma pack(pop)

#endif /* MOD_REGS_H */

