
/*
 * MSC module
 */


#include <QDebug>
#include <QString>

#include "modules/msc_types.h"
#include "msc_mod.h"
#include "daqcommon/daqexc.h"

namespace {
    double msc_zero_dac_offset = -1000.; // mV
}

size_t msc_mod::get_setup_size() const {return sizeof(msc_setup_t);}

void msc_mod::set_defaults()
{
    msc_setup_t *s = static_cast<msc_setup_t *>(setup);
    for (int i = 0; i < MSC_NCH; i++) {
        s->thr[i] = 50;
    }
}

void msc_mod::set_setup(common_setup_t *f_setup, size_t f_size)
{
    base_mod::set_setup(f_setup, f_size);
    msc_setup_t *s = static_cast<msc_setup_t *>(setup);
}

void msc_mod::print_private_setup() const
{
    Q_ASSERT(setup_valid());
    const msc_setup_t *s = static_cast<const msc_setup_t *>(setup);
    Q_ASSERT(s);

    qDebug()  << "*** MSC private setup";
//TODO: print thresholds
}

double msc_mod::dac_mv_min () const
{
    return (dac_mv (dac_min()));
}

double msc_mod::dac_mv_max () const
{
    return (dac_mv (dac_max()));
}

double msc_mod::dac_mv (int value)
{
    return (1. * value / 2.048 + msc_zero_dac_offset);
}

int msc_mod::dac_value (double mv)
{
    int v = static_cast<int>((mv - msc_zero_dac_offset) * 2.048);
    if (v < dac_min()) v = dac_min();
    if (v > dac_max()) v = dac_max();
    return v;
}


int msc_mod::get_int(const std::string &tag, unsigned int mch) const
{
    const msc_setup_t *s = static_cast<const msc_setup_t *>(setup);
    Q_ASSERT(s);

    if (!validate_mch(mch)) {
        QString str = QString("%1::get_int: bad channel number %1").arg(get_name()).arg(mch);
        throw exc(str.toStdString());
    }

    if (tag == "thr") {return s->thr[mch];}
    if (tag == "min_thr") {return static_cast<int>(dac_mv_min());}
    if (tag == "max_thr") {return static_cast<int>(dac_mv_max());}

    // unhandled
    QString str = QString("%1::get_int: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    throw exc_mod_unknown_tag(str.toStdString());
}

void msc_mod::set_int(int value, const std::string &tag, int mch)
{
    msc_setup_t *s = static_cast<msc_setup_t *>(setup);
    Q_ASSERT(s);

    if (tag == "thr") {
        if (mch == SET_ALL_CH)
            for (int c=0;c<get_nch();c++)
                s->thr[c]=value;
        else
            s->thr[mch]=value;
        return;
    }

    // unhandled
    QString str = QString("%1::set_int: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    throw exc_mod_unknown_tag(str.toStdString());
}

int msc_mod::dac_bits()
{
    return 12;
}

int msc_mod::dac_min()
{
    return 0;
}

int msc_mod::dac_max()
{
    return (1<<dac_bits())-1;
}
