
/*
 * MTDC64 backend (hardware) implementation
 */


#include "mtdc64_bmod.h"
#include "modules/mtdc64_types.h"

#include "daqcommon/daq_ipc.h"
#include "daqcommon/daqexc.h"

//! input: tdc_dev (0..N), tdc_ch(0..31)
//! output: module channel number
int mtdc64_bmod::map_chtdc2inp(int tdc_dev, int ch)
{
  if (tdc_dev < 0 || tdc_dev >= 2) return 0;
  if (ch < 0 || ch >= 32) return 0;
//  return tdc_dev*32+ch;
  return (1-tdc_dev)*32 + mtdc64_tdcch2ch[ch%32];
};

void mtdc64_bmod::end_of_burst_readout ()
{
  const mtdc64_counters_t &cnt = get_counters();

// encode
  unsigned int wc = 0;

  append_rawdata_mhdr();

  for (size_t n=0; n<cnt.size(); n++) {
        unsigned int cnthi, cntlo;
        cnthi = cnt[n] >> 16;
        cntlo = cnt[n] & 0xFFFF;
        append_rawdata( (0<<28)|(n<<16)|cnthi); wc++;
        append_rawdata( (1<<28)|(n<<16)|cntlo); wc++;
  }

  append_rawdata_status(); wc++;
  append_rawdata_mtrl(wc);
}

mtdc64_counters_t mtdc64_bmod::get_counters ()
{
    mtdc64_counters_t cnt(MTDC64_N_COUNTERS);
    r_blk_a24_d32(0x0, cnt.size(), static_cast<int *>(static_cast<void *>(&cnt[0])));
    bool countersAvailable = false;

    for (size_t n=0; n<cnt.size(); n++) {
        if (cnt[n] != 0xFFFFFFFFu)
            countersAvailable = true;
    }
    if (countersAvailable)
        return cnt;
    else
        return mtdc64_counters_t();
}

// end
