
/*
 * PHTDC module
 */


#include "phtdc_mod.h"
#include "modules/phtdc_types.h"
#include "hptdc_settings.h"

class hptdc_settings_phtdc : public hptdc_settings_tdc
{
public:
    hptdc_settings_phtdc(hptdc_setup_t *ps, int *pmask) : hptdc_settings_tdc(ps, pmask) {}
    virtual void set_defaults();
};

void hptdc_settings_phtdc::set_defaults()
{
    hptdc_settings_tdc::set_defaults();
    Q_ASSERT(s);
    set_lvds_clk();
    set_ttl_hit();
}

size_t phtdc_mod::get_setup_size() const {return sizeof(phtdc_setup_t);}
unsigned int phtdc_mod::tdc_nch() const {return PHTDC_NCH;} // number of tdc channels altogether
unsigned int phtdc_mod::tdc_ndev() const {return PHTDC_NDEV;} // number of hptdc chips
int phtdc_mod::hptdc_dev_num(int mch) const {if (mch<0 || mch>=PHTDC_NCH) return 0; return mch/16;}

void phtdc_mod::set_defaults()
{
    tdc_mod::set_defaults();

    int n=0;
    for(int i=0; i<hptdc.size(); i++) {
        QSharedPointer<hptdc_settings_tdc> x = hptdc[i];
        QSharedPointer<hptdc_settings_phtdc> p = qSharedPointerDynamicCast<hptdc_settings_phtdc>(x);
        Q_ASSERT(p);
        p->set_defaults(); // different defaults for phtdc!
        p->set_id (1<<n);
        n++;
    }

    phtdc_setup_t *s = static_cast<phtdc_setup_t *>(setup);
    Q_ASSERT(s);

    for (int i = 0; i < PHTDC_NCH; i++) {
        s->ch_enable[i] = 1;
    }
}

void phtdc_mod::set_setup(common_setup_t *f_setup, size_t f_size)
{
    base_mod::set_setup(f_setup, f_size);
    phtdc_setup_t *s = static_cast<phtdc_setup_t *>(setup);
    Q_ASSERT(s);
    hptdc.clear();
    for (unsigned int i=0;i<tdc_ndev();i++) {
        QSharedPointer<hptdc_settings_phtdc> p(new hptdc_settings_phtdc(&s->tdcsetup[i], &s->ch_enable[i]));
        hptdc.push_back(p);
    }
}

int phtdc_norm_map_ch_code[4][16] = {   // map ntdc, tdcch/2 to fp-num
    {
        0x17, 0x16, 0x07, 0x06,
        0x15, 0x14, 0x05, 0x04,
        0x13, 0x12, 0x03, 0x02,
        0x11, 0x10, 0x01, 0x00},
{
        0x1F, 0x1E, 0x0F, 0x0E,
        0x1D, 0x1C, 0x0D, 0x0C,
        0x1B, 0x1A, 0x0B, 0x0A,
        0x19, 0x18, 0x09, 0x08},
{
        0x37, 0x36, 0x27, 0x26,
        0x35, 0x34, 0x25, 0x24,
        0x33, 0x32, 0x23, 0x22,
        0x31, 0x30, 0x21, 0x20},
{
        0x3F, 0x3E, 0x2F, 0x2E,
        0x3D, 0x3C, 0x2D, 0x2C,
        0x3B, 0x3A, 0x2B, 0x2A,
        0x39, 0x38, 0x29, 0x28}
};

int phtdc_norm_map_ch_ntdc[64] = {  // fp-num to ntdc
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3
};

int phtdc_norm_map_ch_tdcch[64] = { // fp-num to tdcch
    30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0,
    30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0,
    30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0,
    30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0
};
