
/*
 * TDC32VL module
 */


#include "hal/tdc32vl_mod.h"
#include "modules/tdc32vl_types.h"
#include "hptdc_settings.h"

class hptdc_settings_tdc32vl : public hptdc_settings_tdc
{
public:
    hptdc_settings_tdc32vl(hptdc_setup_t *ps, int *pmask) : hptdc_settings_tdc(ps, pmask) {}
    virtual void set_defaults();
};

void hptdc_settings_tdc32vl::set_defaults()
{
    if (!s) return;
    hptdc_settings_tdc::set_defaults();
    set_lvds_clk();
    set_lvds_hit();
    set_resolution (TDC_RES_INDEX_25);
}

size_t tdc32vl_mod::get_setup_size() const {return sizeof(tdc32vl_setup_t);}
unsigned int tdc32vl_mod::tdc_nch() const {return TDC32VL_NCH;} // number of tdc channels altogether
unsigned int tdc32vl_mod::tdc_ndev() const {return TDC32VL_NDEV;} // number of hptdc chips
int tdc32vl_mod::hptdc_dev_num(int mch) const {if (mch<0 || mch>=TDC32VL_NCH) return 0; return mch/32;}

void tdc32vl_mod::set_defaults()
{
    tdc_mod::set_defaults();

    int n=0;
    for(int i=0; i<hptdc.size(); i++) {
        QSharedPointer<hptdc_settings_tdc> x = hptdc[i];
        QSharedPointer<hptdc_settings_tdc32vl> p = qSharedPointerDynamicCast<hptdc_settings_tdc32vl>(x);
        Q_ASSERT(p);
        p->set_defaults(); // different defaults for tdc32vl!
        p->set_id (1<<n);
        n++;
    }

    tdc32vl_setup_t *s = static_cast<tdc32vl_setup_t *>(setup);

    for (int i = 0; i < TDC32VL_NCH; i++) {
        s->ch_enable[i] = 1;
    }
}

void tdc32vl_mod::set_setup(common_setup_t *f_setup, size_t f_size)
{
    base_mod::set_setup(f_setup, f_size);
    tdc32vl_setup_t *s = static_cast<tdc32vl_setup_t *>(setup);
    hptdc.clear();
    for (unsigned int i=0;i<tdc_ndev();i++) {
        QSharedPointer<hptdc_settings_tdc32vl> p(new hptdc_settings_tdc32vl(&s->tdcsetup[i], &s->ch_enable[i]));
        hptdc.push_back(p); // ?????
    }
}

void tdc32vl_mod::set_int(int value, const std::string &tag, int mch)
{
    tdc_setup_t *s = static_cast<tdc_setup_t *>(setup);

    // BUG: emulate read-only behaviour
    if (tag == "res") return;
    if (tag == "pres") return;

    tdc_mod::set_int(value, tag, mch);
}
