
/*
 * TDC64V module
 */


#include "hal/tdc64v_mod.h"
#include "modules/tdc64v_types.h"
#include "hptdc_settings.h"

class hptdc_settings_tdc64v : public hptdc_settings_tdc
{
public:
    hptdc_settings_tdc64v(hptdc_setup_t *ps, int *pmask) : hptdc_settings_tdc(ps, pmask) {}
    virtual void set_defaults();
};

void hptdc_settings_tdc64v::set_defaults()
{
    if (!s) return;
    hptdc_settings_tdc::set_defaults();
    set_lvds_clk();
    set_ttl_hit();
}

size_t tdc64v_mod::get_setup_size() const {return sizeof(tdc64v_setup_t);}
unsigned int tdc64v_mod::tdc_nch() const {return TDC64V_NCH;} // number of tdc channels altogether
unsigned int tdc64v_mod::tdc_ndev() const {return TDC64V_NDEV;} // number of hptdc chips
int tdc64v_mod::hptdc_dev_num(int mch) const {if (mch<0 || mch>=TDC64V_NCH) return 0; return mch/32;}

void tdc64v_mod::set_defaults()
{
    tdc_mod::set_defaults();

    int n=0;
    for(int i=0; i<hptdc.size(); i++) {
        QSharedPointer<hptdc_settings_tdc> x = hptdc[i];
        QSharedPointer<hptdc_settings_tdc64v> p = qSharedPointerDynamicCast<hptdc_settings_tdc64v>(x);
        Q_ASSERT(p);
        p->set_defaults(); // different defaults for tdc64v!
        p->set_id (1<<n);
        n++;
    }

    tdc64v_setup_t *s = static_cast<tdc64v_setup_t *>(setup);

    for (int i = 0; i < TDC64V_NCH; i++) {
        s->ch_enable[i] = 1;
    }
}

void tdc64v_mod::set_setup(common_setup_t *f_setup, size_t f_size)
{
    base_mod::set_setup(f_setup, f_size);
    tdc64v_setup_t *s = static_cast<tdc64v_setup_t *>(setup);
    hptdc.clear();
    for (unsigned int i=0;i<tdc_ndev();i++) {
        QSharedPointer<hptdc_settings_tdc64v> p(new hptdc_settings_tdc64v(&s->tdcsetup[i], &s->ch_enable[i]));
        hptdc.push_back(p); // ?????
    }
}
