
/*
 * TDC96 module
 */


#include "hal/tdc96_mod.h"
#include "modules/tdc96_types.h"
#include "hptdc_settings.h"

class hptdc_settings_tdc96 : public hptdc_settings_tdc
{
public:
    hptdc_settings_tdc96(hptdc_setup_t *ps, int *pmask) : hptdc_settings_tdc(ps, pmask) {}
    virtual void set_defaults();
};

void hptdc_settings_tdc96::set_defaults()
{
    if (!s) return;
    hptdc_settings_tdc::set_defaults();
    set_lvds_clk();
    set_ttl_hit();
}

size_t tdc96_mod::get_setup_size() const {return sizeof(tdc96_setup_t);}
unsigned int tdc96_mod::tdc_nch() const {return TDC96_NCH;} // number of tdc channels altogether
unsigned int tdc96_mod::tdc_ndev() const {return TDC96_NDEV;} // number of hptdc chips
int tdc96_mod::hptdc_dev_num(int mch) const {if (mch<0 || mch>=TDC96_NCH) return 0; return mch/32;}

void tdc96_mod::set_defaults()
{
    tdc_mod::set_defaults();

    int n=0;
    for(int i=0; i<hptdc.size(); i++) {
        QSharedPointer<hptdc_settings_tdc> x = hptdc[i];
        QSharedPointer<hptdc_settings_tdc96> p = qSharedPointerDynamicCast<hptdc_settings_tdc96>(x);
        Q_ASSERT(p);
        p->set_defaults(); // different defaults for tdc96!
        p->set_id (1<<n);
        n++;
    }

    tdc96_setup_t *s = static_cast<tdc96_setup_t *>(setup);

    for (int i = 0; i < TDC96_NCH; i++) {
        s->ch_enable[i] = 1;
    }
}

void tdc96_mod::set_setup(common_setup_t *f_setup, size_t f_size)
{
    base_mod::set_setup(f_setup, f_size);
    tdc96_setup_t *s = static_cast<tdc96_setup_t *>(setup);
    hptdc.clear();
    for (unsigned int i=0;i<tdc_ndev();i++) {
        QSharedPointer<hptdc_settings_tdc96> p(new hptdc_settings_tdc96(&s->tdcsetup[i], &s->ch_enable[i]));
        hptdc.push_back(p); // ?????
    }
}
/*
int tdc96_map_ch_code[3][32] = {   // map ntdc, tdcch to fp-num
    {
      0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
      0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
      0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
      0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F
    },
    {
      0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,
      0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
      0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
      0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F
    },
    {
      0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,
      0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F,
      0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,
      0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F
    }
};
*/
