
/*
 * TDC backend (hardware) declaration
 */

#ifndef TDC_BMOD_H
#define TDC_BMOD_H

#include "hal/tdc_mod.h"
#include "base_bmod.h"

class Jtag_Impl_Fvme;
class JTAG_Engine_HPTDC;

class tdc_bmod: public virtual tdc_mod, public base_bmod
{
public:
    tdc_bmod(const bmod_props &props);
    virtual ~tdc_bmod();
    // module actions
    virtual bool detect ();
    virtual bool selftest ();
    //  virtual void reset (bool messages = true);
    virtual void reset ();
    virtual void preparerun ();
    //  virtual void start_run ();
    //  virtual void stop_run ();
    virtual void end_of_burst ();
    virtual bool interspill_test ();

    virtual double read_temp() { return 0;} // not implemented

protected:
    //  virtual void update_jtag_address ();


    virtual void clear_error_flag();

    //! \param tdc_dev (0..N)
    //! \param tdc_ch (0..31)
    //! \return module channel number
    virtual int map_chtdc2inp(int tdc_dev, int ch) = 0;

    //! \param ch module channel number
    //! \return tdc_ch
//    virtual int map_inp2chtdc(int ch) = 0;

    //! \param ch module channel number
    //! \return tdc number
//    virtual int map_inp2ntdc(int ch) = 0;

    virtual module_status_reg_t get_status_reg();
    virtual int get_regnumber_test() const;
    virtual bool status_ok() const;

private:
    // hptdc_chain
    bool check_dll(int *status);
    bool reset_hptdc_chips_pll();
    void reset_hptdc_chips();
    std::vector<quint32> jtag_readout();
    //    void tdc_print_status (int d) const;
    //    int tdc_get_status (const tdc_setup_t * s, int *st);
    //    int tdc_check_runstatus (const tdc_setup_t * s);
    bool test_idcode(int loops);
    bool test_bypass(int loops);
    bool test_bist(int loops);

    bool check_status();

    //! load input channel mask, if present
    void write_channel_mask();

    //! write hptdc setup, retry on errors
    bool setup_with_retries(size_t num_tries);

    bool hptdc_lock;
    bool hptdc_idcode_test;
    bool hptdc_bypass_test;
    bool hptdc_err;
    bool hptdc_warn;
    bool hptdc_initialized;
    Jtag_Impl_Fvme * const jtagImpl;
    JTAG_Engine_HPTDC * const jtag;
};

#endif /* TDC_BMOD_H */
