
/*
 * TDC generic module with HPTDC-1.3 devices
 */

#ifndef TDC_MOD_H
#define TDC_MOD_H

#include <QVector>
#include <QSharedPointer>

#include "hal/base_mod.h"
#include "hptdc_settings.h"

class hptdc_settings_tdc: public hptdc_settings {
public:
    hptdc_settings_tdc(hptdc_setup_t *ps, int *pmask) : hptdc_settings(ps, pmask) {}
    virtual ~hptdc_settings_tdc() {}
    virtual void set_defaults();
};

class tdc_mod : public virtual base_mod {
public:
    virtual void set_defaults();
    virtual void set_setup(common_setup_t *f_setup, size_t f_size);
    virtual int get_int(const std::string &tag, unsigned int mch=GET_FIRST_CH) const;
    virtual void set_int(int value, const std::string &tag, int mch=SET_ALL_CH);
    virtual size_t get_setup_size() const;

protected:
    tdc_mod() {}
    virtual void print_private_setup() const;
    virtual int hptdc_dev_num(int mch) const {if (mch<0 || mch>=16) return 0; return mch/8;}
    virtual unsigned int tdc_nch() const = 0; // number of tdc channels altogether
    virtual unsigned int tdc_ndev() const = 0; // number of hptdc chips

    //private:
    QVector<QSharedPointer<hptdc_settings_tdc> > hptdc;
};

#endif /* TDC_MOD_H */
