
/*
 * TQDC backend (hardware) declaration
 */

#ifndef TQDC_BMOD_H
#define TQDC_BMOD_H

#include "tqdc_mod.h"
#include "tdc_bmod.h"
#include "modules/tqdc_types.h"

typedef std::vector<quint32> tqdc_counters_t;

class tqdc_bmod: public tdc_bmod, public tqdc_mod {
 public:
    tqdc_bmod(const bmod_props &props) : tdc_bmod(props) {}

  // module actions
  virtual bool detect ();
  virtual bool selftest ();
  virtual void reset ();
  virtual void preparerun ();
  virtual void calibration();
//  virtual void start_run ();
//  virtual void stop_run ();
  virtual void end_of_burst_readout ();
//  virtual bool interspill_test ();
  virtual double read_temp() { return base_bmod::read_temp();}

 protected:
  enum {NO_FORCE_ZERO = 0, FORCE_ZERO = 1};
//! reimplemented tdc_bmod::map_chtdc2inp()
//! input: tdc_dev (0..N), tdc_ch(0..31)
//! output: module channel number
  virtual int map_chtdc2inp(int tdc_dev, int ch);
//! input: module channel number
//! output: tdc_ch
//  virtual int map_inp2chtdc(int ch);
//! input: module channel number
//! output: tdc number
//  virtual int map_inp2ntdc(int ch);

// tqdc
  void dac_exec(const std::vector<int> &cmd);
  static std::vector<int> dac_set_cmd (int ch, int val);
  static std::vector<int> dac_set_all_cmd(int dac);
  static std::vector<int> dac_reset_cmd ();
  static std::vector<int> dac_load_cmd ();
//  void dac_shift (int n, int d);
  void dac_reset ();
  void dac_load ();
  void dac_set (int ch, int val);
  void dac_set_all(int dac);
  void dac_set_load(int ch, int dac);
  void dac_set_all_load(int dac);
  void set_adc (const tqdc_setup_t &s);
  void normalize_trigset();
  void set_trigset (int force_zero = NO_FORCE_ZERO);
  void set_thr (int force_zero);
  void set_trlat ();
  void load_registers(int force_zero = NO_FORCE_ZERO);
  tqdc_counters_t get_counters();
  virtual int get_regnumber_test() const;

private:
  double thr_offset(int ch);
  double adc_baseline(int ch);
};

#endif /* TQDC_BMOD_H */
