
/*
 * TRIG module
 */


#include <QDebug>
#include <QString>

#include "modules/trig_types.h"
#include "trig_mod.h"
#include "daqcommon/daqexc.h"

size_t trig_mod::get_setup_size() const
{
    return sizeof(trig_setup_t);
}

void trig_mod::set_defaults ()
{
    base_mod::set_defaults();
    trig_setup_t *s = static_cast<trig_setup_t *>(setup);
    Q_ASSERT(s);

    s->spill_select = TRIG_SPILL_TIMER; // 1=timer, 0=ext_start, 2=ext_gate
    //  s->spill_ext_enable = true;
    s->spill_delay = 6; // ms
    s->spill_duration = 1000;
    s->spill_repeat = 4000;
    for (int i=0;i<TRIG_NCH;i++) {
        s->ch_enable[i] = true;
        s->ch_delay[i] = 0;
    }
    s->in_shape = 0x3;
    for (int i=0;i<TRIG_N_INPUTS;i++) {
        s->input_enable[i] = true;
    }
    s->nim_maj_min = 1;
    s->nim_maj_max = 7;
    //  s->lvds_enable[0]=1;
    //  s->lvds_enable[1]=1;
    //  s->lvds_enable[2]=0;
    //  s->lvds_enable[3]=0;
    //  s->lvds_enable[4]=1;
    //  s->lvds_enable[5]=1;
    //  s->lvds_enable[6]=0;
    //  s->lvds_enable[7]=0;
    s->noise_trigger = 0;
    s->pulser_trigger = 1;
    s->pulser_freq = 100;
    s->timers.all = 0;
    s->timers.bit.ttc_delay = 0x7;
    s->timers.bit.read_delay = 0x000F;
    s->timers.bit.xoff_wait = 0xF;
    s->en_xoff_in = 1;
    s->en_spill_end_wait = 1;
    s->logic = 0xFFFE;

    s->dch_clock_adj = 0;
    /*
// default - anytrigger
  s->bitmap[0] = 0;
  for (i = 1; i < TRIG_BITMAP_SIZE; i++)
    s->bitmap[i] = 1;
*/
    s->out_sel_1 = 3;
    s->out_sel_2 = 8;
}

QVariant trig_mod::get_var(const std::string &tag, unsigned int mch) const
{
    const trig_setup_t *s = static_cast<const trig_setup_t *>(setup);
    Q_ASSERT(s);
    if (tag == "in_en") {
        QMap<QString, QVariant> m;
        for(int i=0; i<TRIG_N_INPUTS; i++)
            m[QString::number(i)] = s->input_enable[i];
        return m;
    }

    // unhandled
    QString str = QString("%1::get_var: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    qWarning() << str;
    throw exc_mod_unknown_tag(str.toStdString());
}

int trig_mod::get_int(const std::string &tag, unsigned int mch) const
{
    const trig_setup_t *s = static_cast<const trig_setup_t *>(setup);
    Q_ASSERT(s);

    //  if (tag == "spill_ext_enable") {return s->spill_ext_enable;}
    if (tag == "spill_delay") {return s->spill_delay;}
    if (tag == "min_spill_delay") {return static_cast<int>(1. * 0x2 * TRIG_SPILLTIMER_STEP * 1e3);}
    if (tag == "max_spill_delay") {return static_cast<int>(1. * 0xFFFF * TRIG_SPILLTIMER_STEP * 1e3);}
    if (tag == "spill_duration") {return s->spill_duration;}
    if (tag == "min_spill_duration") {return static_cast<int>(1. * 0x2 * TRIG_SPILLTIMER_STEP * 1e3);}
    if (tag == "max_spill_duration") {return static_cast<int>(1. * ((get_serial() >= 16)?0x7FFFFFFF:0xFFFF) * TRIG_SPILLTIMER_STEP * 1e3);}
    if (tag == "spill_repeat") {return s->spill_repeat;}
    if (tag == "min_spill_repeat") {return static_cast<int>(1. * 0x32 * TRIG_SPILLTIMER_STEP * 1e3);}
    if (tag == "max_spill_repeat") {return static_cast<int>(1. * 0xFFFF * TRIG_SPILLTIMER_STEP * 1e3);}
    if (tag == "spill_select") {return s->spill_select;}
    if (tag == "en") {return s->ch_enable[mch];}
//    if (tag == "in_en") {return s->input_enable[mch];}
    if (tag == "in_shape") {return s->in_shape;}
    if (tag == "nim_maj_min") {return s->nim_maj_min;}
    if (tag == "nim_maj_max") {return s->nim_maj_max;}
    if (tag == "delay") {return s->ch_delay[mch];}
    if (tag == "max_delay") {return 0xF;}
    //  if (tag == "lvds_en") {return s->lvds_enable[mch];}
    if (tag == "pulser_trigger") {return s->pulser_trigger;}
    if (tag == "noise_trigger") {return s->noise_trigger;}
    if (tag == "pulser_freq") {return s->pulser_freq;}
    if (tag == "ttc_delay") {return s->timers.bit.ttc_delay;}
    if (tag == "min_ttc_delay") {return (1)-1;}
    if (tag == "max_ttc_delay") {return (1<<TRIG_TTC_DELAY_BITS)-1;}
    if (tag == "read_delay") {return s->timers.bit.read_delay;}
    if (tag == "min_read_delay") {return (1);}
    if (tag == "max_read_delay") {return (1<<TRIG_READ_DELAY_BITS)-1;}
    if (tag == "dch_clock_adj") {return s->dch_clock_adj;}
    if (tag == "xoff_wait") {return s->timers.bit.xoff_wait;}
    if (tag == "min_xoff_wait") {return (1);}
    if (tag == "max_xoff_wait") {return (1<<TRIG_XOFF_WAIT_BITS)-1;}
    if (tag == "en_xoff_in") {return s->en_xoff_in;}
    //  if (tag == "en_spill_end_wait") {return s->en_spill_end_wait;}
    if (tag == "logic") {return s->logic;}
    if (tag == "out_sel_1") {return s->out_sel_1;}
    if (tag == "out_sel_2") {return s->out_sel_2;}

    // unhandled
    QString str = QString("%1::get_int: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    qWarning() << str;
    throw exc_mod_unknown_tag(str.toStdString());
}

void trig_mod::set_var(const QVariant &value, const std::string &tag, int mch)
{
    trig_setup_t *s = static_cast<trig_setup_t *>(setup);
    Q_ASSERT(s);

    QMap<QString, QVariant> m = value.toMap();
    if (tag == "in_en") {
        for(int i=0; i<TRIG_N_INPUTS; i++)
        s->input_enable[i]=m.value(QString::number(i)).toBool();
        return;
    }

    // unhandled
    QString str = QString("%1::set_var: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    qWarning() << str;
    throw exc_mod_unknown_tag(str.toStdString());
}

void trig_mod::set_int(int value, const std::string &tag, int mch)
{
    trig_setup_t *s = static_cast<trig_setup_t *>(setup);
    Q_ASSERT(s);

    //  if (tag == "spill_ext_enable") {s->spill_ext_enable=value;return;}
    if (tag == "spill_delay") {s->spill_delay=value;return;}
    if (tag == "spill_duration") {s->spill_duration=value;return;}
    if (tag == "spill_repeat") {s->spill_repeat=value;return;}
    if (tag == "spill_select") {s->spill_select=value;return;}
    if (tag == "in_shape") {s->in_shape = value; return; }
    if (tag == "nim_maj_min") {s->nim_maj_min = value; return; }
    if (tag == "nim_maj_max") {s->nim_maj_max = value; return; }

    if (tag == "en") {
        if (mch == SET_ALL_CH)
            for (int c=0;c<get_nch();c++)
                s->ch_enable[c]=value;
        else
            s->ch_enable[mch]=value;
        return;
    }


    if (tag == "delay") {
        if (mch == SET_ALL_CH)
            for (int c=0;c<get_nch();c++)
                s->ch_delay[c]=value;
        else
            s->ch_delay[mch]=value;
        return;
    }
    /*
  if (tag == "lvds_en") {
        if (mch == SET_ALL_CH)
          for (int c=0;c<get_nch();c++)
                s->lvds_enable[c]=value;
        else
        s->lvds_enable[mch]=value;
        return;
  }
*/
    if (tag == "pulser_trigger") {s->pulser_trigger=value; return;}
    if (tag == "noise_trigger") {s->noise_trigger=value; return;}
    if (tag == "pulser_freq") {s->pulser_freq=value; return;}
    if (tag == "ttc_delay") {s->timers.bit.ttc_delay=value; return;}
    if (tag == "read_delay") {s->timers.bit.read_delay=value; return;}
    if (tag == "dch_clock_adj") {s->dch_clock_adj=value; return;}
    if (tag == "xoff_wait") {s->timers.bit.xoff_wait=value; return;}
    if (tag == "en_xoff_in") {s->en_xoff_in=value; return;}
    //  if (tag == "en_spill_end_wait") {s->en_spill_end_wait=value; return;}
    if (tag == "logic") {s->logic=value; return;}
    if (tag == "out_sel_1") {s->out_sel_1=value; return;}
    if (tag == "out_sel_2") {s->out_sel_2=value; return;}

    // unhandled
    QString str = QString("%1::set_int: Unknown tag '%2'").arg(get_name()).arg(QString::fromStdString(tag));
    qWarning() << str;
    throw exc_mod_unknown_tag(str.toStdString());
}
