#ifndef VMEDAQ_JTAG_ENGINE_H
#define VMEDAQ_JTAG_ENGINE_H

#include <cstddef>
#include <deque>

//  JTAG state machine states

typedef enum {
  JS_TEST_LOGIC_RESET = 0,      // RESET -- stable state
  JS_RUN_TEST_IDLE,             // IDLE -- stable state
  JS_SELECT_DR_SCAN,
  JS_CAPTURE_DR,
  JS_SHIFT_DR,
  JS_EXIT1_DR,
  JS_PAUSE_DR,                  // DRPAUSE -- stable state
  JS_EXIT2_DR,
  JS_UPDATE_DR,
  JS_SELECT_IR_SCAN,
  JS_CAPTURE_IR,
  JS_SHIFT_IR,
  JS_EXIT1_IR,
  JS_PAUSE_IR,                  // IRPAUSE -- stable state
  JS_EXIT2_IR,
  JS_UPDATE_IR,
} JTAG_STATE;

class Jtag_Impl;

class JTAG_Engine {
public:
  JTAG_Engine(Jtag_Impl &p);
//  void setImpl(Jtag_Impl *p) { impl = p; }
//  Jtag_Impl *getImpl() { return impl; }

// JTAG operations
  void jtag_reset ();
  void jtag_pause(unsigned int count); // do 'count' times with TMS=0 TDI=0
  void jtag_drscan (unsigned int count, const void *tdiData, void *tdoData);
//  bool jtag_drscan_chk (unsigned int count, const void *tdiData, void *tdoData);
  void jtag_irscan (unsigned int count, const void *tdiData, void *tdoData);

protected:
  bool _perform_scan(size_t count, const char *tdiData, char *tdoData);
  void _internal_jtag_reset ();
  void _internal_jtag_drscan (const unsigned int count, const char *tdiData, char *tdoData);
  void _internal_jtag_irscan (unsigned int count, const char *tdiData, char *tdoData);

  void debug(int level, const char *fmt, ...) const;

private:
  Jtag_Impl &impl;
  JTAG_STATE jtagState;
  JTAG_STATE irStopState;  // end state after instruction register scan (one of the 4 idle states)
  // JTAG_STATE drStopState;  // end state after data register scan
  std::deque<bool> btms;
  std::deque<bool> btdi;
  std::deque<bool> btdo;
};

#endif /* VMEDAQ_JTAG_ENGINE_H */
