#ifndef VMEDAQ_JTAG_ENGINE_HPTDC_H
#define VMEDAQ_JTAG_ENGINE_HPTDC_H

#include "vmedaq_jtag_engine.h"

#include <QtCore/QtGlobal>
#include <vector>

#include "hptdc/hptdc_types.h"

class Jtag_Impl;

class JTAG_Engine_HPTDC: public JTAG_Engine
{
public:
    JTAG_Engine_HPTDC(Jtag_Impl &p) : JTAG_Engine(p) { }
    std::vector<quint32> jchain_detect_ndev();
    int jchain_test_idcode (unsigned int load_ir, unsigned int ndev);
    int jchain_test_bypass (unsigned int ndev);   /* test JTAG chain with BYPASS opcode */

    /* JTAG register operations for the chain of <ndev> HPTDC devices */
    std::vector<hptdc_status_t> hptdc_get_status(size_t ndev);
    std::vector<hptdc_bsr_t> hptdc_get_bsr(size_t ndev);
    std::vector<hptdc_bist_t> hptdc_get_bist(size_t ndev);
    std::vector<hptdc_coretest_t> hptdc_get_coretest(size_t ndev);
    std::vector<hptdc_readout_t> hptdc_get_jtag_readout(size_t ndev);

    void hptdc_write_setup (const std::vector<hptdc_setup_t> &setup);
    void hptdc_write_control(const std::vector<hptdc_control_t> &control);

protected:
    void jchain_irscan (const std::vector<hptdc_ir_t> &ir);
    void jchain_irscan_all (unsigned int ir_in, unsigned int ndev);
};

#endif /* VMEDAQ_JTAG_ENGINE_HPTDC_H */
