
#include "hal_xml/base_mod_impl.h"

#include <QDomElement>
#include <QDebug>

#include <sstream>
#include <cstdio>

#include "hal/base_mod.h"
#include "daqcommon/daqexc.h"
#include "hal/oldcalibrationdata.h"

namespace {
    QDomElement xml_getCalibrationElement(QDomElement *nodeModule)
    {
        QDomNode n = nodeModule->firstChild();
        while (!n.isNull()) {
            if (n.isElement()) {
                QDomElement e = n.toElement();
                if (e.tagName() == "calibration")
                    return e;
            }
            n = n.nextSibling();
        }
        return QDomElement();
    }

    double xml_getCalibrationTemperature(const QDomElement &nodeCal)
    {
        QDomNode n = nodeCal.firstChild();
        while (!n.isNull()) {
            if (n.isElement()) {
                QDomElement e = n.toElement();
                if (e.tagName() == "temperature") {
                    bool ok;
                    double temp = e.nodeValue().toDouble(&ok);
                    if (ok)
                        return temp;
                    break;
                }
            }
            n = n.nextSibling();
        }
        return 0;
    }
}

void base_mod_impl::write_xml_attr(QDomElement *nodeModule) const
{
    nodeModule->setAttribute("vendor", QString("%1").arg(m->get_vend(), 2, 16, QChar('0')));

    nodeModule->setAttribute("id", QString("%1").arg(m->get_id(), 2, 16, QChar('0')));

    nodeModule->setAttribute("name", QString("%1").arg(module_id_str(m->get_id())));

    nodeModule->setAttribute("serial", QString("%1").arg(m->get_serial(), 8, 16, QChar('0')));

    nodeModule->setAttribute("slot", QString("%1").arg(m->get_slot()));

    nodeModule->setAttribute("readout", QString("%1").arg(m->get_cblt_enable()?1:0));

    nodeModule->setAttribute("channels", QString("%1").arg(m->get_nch()));

    nodeModule->setAttribute("ddb_key", QString("%1").arg(m->get_ddb_key(), 8, 16, QChar('0')));

    const base_mod_flags flags = m->get_flags();
    bool warn = flags & STDMOD_FLAG_WARN;
    bool err = flags & STDMOD_FLAG_ERR;
    bool fail = flags & STDMOD_FLAG_FAIL;
    //    bool testok = flags & STDMOD_FLAG_TESTOK;
    //    bool ok = testok & !err & !fail;

    QString ststr("ok");
    if (warn) ststr = "warning";
    if (err) ststr = "error";
    if (fail) ststr = "failure";
    nodeModule->setAttribute("status", ststr);
}

void base_mod_impl::write_xml_setup(QDomElement *nodeModule) const
{
    nodeModule->appendChild(nodeModule->ownerDocument().createElement("setup"));
}
/*
void base_mod_impl::write_xml_calibration(QDomElement *nodeModule) const
{
    // find
    QDomElement nodeCal = xml_getCalibrationElement(nodeModule);
    if (nodeCal.isNull())
        nodeCal = nodeModule->appendChild(nodeModule->ownerDocument().createElement("calibration")).toElement();

    const CalibrationData &cal = m->getCalibrationData();
    cal.print();
    const CalibrationData::cond_map_t &cond = cal.getConditions();
    Q_FOREACH(const QString &cname, cond.keys()) {
//        const std::string &cname = (*it).first;
        const QVariant &val = cond[cname];
        QDomElement nodeC = nodeCal.appendChild(nodeCal.ownerDocument().createElement("condition")).toElement();
        nodeC.setAttribute("name", cname);
        nodeC.setAttribute("val", val.toString());
    }
    const CalibrationData::par_map_t &par = cal.getParams();
    Q_FOREACH(const QString &pname, par.keys()) {
        const CalibrationData::par_t &p = par[pname];
        Q_FOREACH(int index, p.keys()) {
            const QVariant &val = p[index];
            QDomElement nodeP = nodeCal.appendChild(nodeCal.ownerDocument().createElement("p")).toElement();
            nodeP.setAttribute("name", pname);
            nodeP.setAttribute("index", QString("%1").arg(index));
            nodeP.setAttribute("val", val.toString());
        }
    }
}

void base_mod_impl::read_xml_calibration(const QDomElement &nodeModule)
{
    double temp = m->getCalibrationCondition("current_temperature").toDouble();
    m->log_pri_mod(base_mod::LOG_INFO, 0, QString("base_mod_impl::read_xml_calibration, cur_temp = %1").arg(temp, 0, 'f', 1));

    QDomElement nodeCal;
    double min_dist = 1e38; // big number
    QDomNode n = nodeModule.firstChild();
    while (!n.isNull()) {
        if (n.isElement()) {
            QDomElement e = n.toElement();
            if (e.tagName() == "calibration") {
                double this_temp = xml_getCalibrationTemperature(e);
                double dist = this_temp - temp;
                if (dist < min_dist) {
                    min_dist = dist;
                    nodeCal = e;
                }
            }
        }
        n = n.nextSibling();
    }

    if (!nodeCal.isNull()) {
        qDebug() << QString("Found nearest match: dist = %1").arg(temp - xml_getCalibrationTemperature(nodeCal));
    }
}
*/
