/*
 *
 *  hptdc.h -- definitions for the HPTDC chip JTAG operations
 *
 *  High Performance Time to Digital Converter (HPTDC)
 *  http://micdigital.web.cern.ch/micdigital/hptdc.htm
 *
 *  Copyright (C) 2004   Ilja.Slepnev@jinr.ru
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
*/

#ifndef _HPTDC_H
#define _HPTDC_H

#include <cstddef>

#include "hptdc/hptdc_types.h"

extern int HPTDC_STATUS_ERROR_MASK;

//extern const double tdc_tbin[6];
//extern const int tdc_fbins[6];

//unsigned int check_parity_buf (const void *buf, unsigned int bits);

class QString;

void hptdc_set_setup_defaults (hptdc_setup_t * setup);
int hptdc_set_control_defaults (hptdc_control_t * control);
void hptdc_print_status_bit (const hptdc_status_t &status);
void hptdc_print_status (const hptdc_status_t &status);
QString hptdc_status_error_toString(int err);
//void snprintf_hptdc_status_error (char *str, size_t size, int err);
void hptdc_print_control (const hptdc_control_t &ctrl);
void hptdc_print_setup (const hptdc_setup_t &setup);
void hptdc_print_brief_setup (const hptdc_setup_t &setup);
void hptdc_print_bsr (const hptdc_bsr_t &bsr);
void hptdc_print_bist (const hptdc_bist_t &bist);
int hptdc_bist_finished (const hptdc_bist_t &bist);
int hptdc_bist_failed (const hptdc_bist_t &bist);
int hptdc_bist_ok (const hptdc_bist_t &bist);
void hptdc_print_coretest (const hptdc_coretest_t &coretest);
void hptdc_print_readout (const hptdc_readout_t &data);
void hptdc_print_err (int err);
QString hptdc_err_str(int err);
void hptdc_print_dataword (int d);
QString hptdc_dataword_str(int d);

void hptdc_control_set_parity (hptdc_control_t * control);

void hptdc_setup_set_parity (hptdc_setup_t * setup);
void hptdc_setup_set_id (hptdc_setup_t * setup, int tdcid);
void hptdc_setup_set_readout (hptdc_setup_t * setup, int readout);
void hptdc_setup_set_edge (hptdc_setup_t * setup, int edge);
void hptdc_setup_set_pattern (hptdc_setup_t * setup, int pattern);
void hptdc_setup_set_trigmode (hptdc_setup_t * setup);
void hptdc_setup_get_resindex (const hptdc_setup_t &setup, TdcResIndex *resindex);
void hptdc_setup_set_resindex (hptdc_setup_t * setup, TdcResIndex resindex);

double tdc_res_s (TdcResIndex res_index);
double tdc_res_ns (TdcResIndex res_index);
bool tdc_res_vh (TdcResIndex res_index);
int tdc_rawfbins (TdcResIndex res_index);

int hptdc_pack_rcadj(const int rcadj[4]);
void hptdc_unpack_rcadj(int packed, int rcadj[4]);
int decode_tdc_rawtime (int d, TdcResIndex resindex);
int decode_tdc_rawwid (int d, TdcResIndex presindex);

#endif                          /* _HPTDC_H */
