/*
 *
 *  hptdc_types.h -- definitions for the HPTDC chip JTAG registers
 *
 *  High Performance Time to Digital Converter (HPTDC)
 *  http://micdigital.web.cern.ch/micdigital/hptdc.htm
 *
 *  Copyright (C) 2004-2008   Ilja.Slepnev@jinr.ru
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
*/

#ifndef _HPTDC_TYPES_H
#define _HPTDC_TYPES_H

extern int HPTDC_STATUS_ERROR_MASK;

const unsigned int HPTDC_IDCODE_1_2 = 0x8470DACDu;
const unsigned int HPTDC_IDCODE_1_3 = 0x8470DACEu;

const unsigned int HPTDC_HIRES_CHMASK = 0x88888888u;

enum { TDC_SETUP_BITS = 647 };
enum { TDC_SETUP_BYTES = ((TDC_SETUP_BITS+7)/8) };
enum { TDC_CONTROL_BITS = 40 };
enum { TDC_CONTROL_BYTES = ((TDC_CONTROL_BITS+7)/8) };
enum { TDC_STATUS_BITS = 62 };
enum { TDC_STATUS_BYTES = ((TDC_STATUS_BITS+7)/8) };
enum { TDC_READOUT_BITS = 33 };
enum { TDC_READOUT_BYTES = ((TDC_READOUT_BITS+7)/8) };
enum { TDC_BIST_BITS = 10 };
enum { TDC_BIST_BYTES = ((TDC_BIST_BITS+7)/8) };
enum { TDC_BSR_BITS = 83 };
enum { TDC_BSR_BYTES = ((TDC_BSR_BITS+7)/8) };
enum { TDC_CORETEST_BITS = 749 };
enum { TDC_CORETEST_BYTES = ((TDC_CORETEST_BITS+7)/8) };

const unsigned int HPTDC_PATTERN_MASK = 0xF0FFFFFFu;

// Data Word Type
const unsigned int HPTDC_DMASK_CTRL = 0x80000000u; // mask
const unsigned int HPTDC_DTYPE_THDR = 0x20000000u;
const unsigned int HPTDC_DTYPE_TTRL = 0x30000000u;
const unsigned int HPTDC_DTYPE_TLDM = 0x40000000u;
const unsigned int HPTDC_DTYPE_TTRM = 0x50000000u;
const unsigned int HPTDC_DTYPE_TERR = 0x60000000u;

enum TdcResIndex {
    TDC_RES_INDEX_25_UNPACKED = -2,
    TDC_RES_INDEX_25 = -1,
    TDC_RES_INDEX_100 = 0,
    TDC_RES_INDEX_200 = 1,
    TDC_RES_INDEX_400 = 2,
    TDC_RES_INDEX_800 = 3,
    TDC_RES_INDEX_1600 = 4
};

enum HPTDC_EDGE {
    HPTDC_EDGE_LEAD = 1,
    HPTDC_EDGE_TRAIL = 2,
    HPTDC_EDGE_BOTH = 3,
};

// setup values
typedef enum {
    HPTDC_CORE_CLOCK_40 = 0,
    HPTDC_CORE_CLOCK_80 = 1,
    HPTDC_CORE_CLOCK_160 = 2,
} hptdc_core_clock_t;

typedef char hptdc_ir_t;

#pragma pack(push, 1)
typedef union {
  unsigned int all;
  struct {
    unsigned int vernier:1; // 0
    unsigned int coarse:1; // 1
    unsigned int synch:1; // 2
    unsigned int l1_parity:1; // 3
    unsigned int trig_parity:1; // 4
    unsigned int trigger_state_err:1; // 5
    unsigned int read_fifo_parity:1; // 6
    unsigned int readout_err:1; // 7
    unsigned int setup_parity:1; // 8
    unsigned int control_parity:1; // 9
    unsigned int jtag_ir_parity:1; // 10
  } bit;
} status_error_t;


  typedef union {
    unsigned char all[TDC_BSR_BYTES];
    struct {
      unsigned int token_out:1;
      unsigned int strobe_out:1;
      unsigned int serial_out:1;
      unsigned int test:1;
      unsigned int error:1;
      unsigned int data_ready:1;
      unsigned int parallel_enable:1;
      unsigned int parallel_data_out:32;
      unsigned int encoded_control:1;
      unsigned int trigger:1;
      unsigned int event_reset:1;
      unsigned int bunch_reset:1;
      unsigned int get_data:1;
      unsigned int serial_bypass_in:1;
      unsigned int serial_in:1;
      unsigned int token_bypass_in:1;
      unsigned int token_in:1;
      unsigned int reset:1;
      unsigned int aux_clock:1;
      unsigned int clk:1;
      unsigned int hit:32;
    } bit;
  } hptdc_bsr_t;

  typedef union {
    unsigned char all[TDC_CORETEST_BYTES];
    struct {

/* Common matching state */
      unsigned int cms_idle:1;
      unsigned int cms_header:1;
      unsigned int cms_lost_header:1;
      unsigned int cms_error:1;
      unsigned int cms_trailer:1;
      unsigned int cms_lost_trailer:1;
      unsigned int cms_separator:1;
      unsigned int cms_occupancy:1;
      unsigned int cms_matching:1;

/* trigger data */
      unsigned int bunch_id:12;
      unsigned int event_id:12;
      unsigned int separator:1;
      unsigned int trigger_lost:1;
      unsigned int parity:1;
      unsigned int trigger_ready:1;

/* GROUP 3 */
      /* matching state */
      unsigned int g3_idle:1;
      unsigned int g3_write_occupancy:1;
      unsigned int g3_active:1;
      unsigned int g3_waiting_for_separator:1;
      unsigned int g3_wait_end:1;

      /* measurement */
      unsigned int g3_finetime:8;
      unsigned int g3_coarsetime:12;
      unsigned int g3_type_width:7;
      unsigned int g3_channel:3;
      unsigned int g3_error:1;
      unsigned int g3_overflow_start:1;
      unsigned int g3_overflow_stop:1;
      unsigned int g3_separator:1;
      unsigned int g3_parity:1;
      unsigned int g3_l1_empty:1;
      unsigned int g3_l1_ready:1;

/* GROUP 2 */
      /* matching state */
      unsigned int g2_idle:1;
      unsigned int g2_write_occupancy:1;
      unsigned int g2_active:1;
      unsigned int g2_waiting_for_separator:1;
      unsigned int g2_wait_end:1;

      /* measurement */
      unsigned int g2_finetime:8;
      unsigned int g2_coarsetime:12;
      unsigned int g2_type_width:7;
      unsigned int g2_channel:3;
      unsigned int g2_error:1;
      unsigned int g2_overflow_start:1;
      unsigned int g2_overflow_stop:1;
      unsigned int g2_separator:1;
      unsigned int g2_parity:1;
      unsigned int g2_l1_empty:1;
      unsigned int g2_l1_ready:1;

/* GROUP 1 */
      /* matching state */
      unsigned int g1_idle:1;
      unsigned int g1_write_occupancy:1;
      unsigned int g1_active:1;
      unsigned int g1_waiting_for_separator:1;
      unsigned int g1_wait_end:1;

      /* measurement */
      unsigned int g1_finetime:8;
      unsigned int g1_coarsetime:12;
      unsigned int g1_type_width:7;
      unsigned int g1_channel:3;
      unsigned int g1_error:1;
      unsigned int g1_overflow_start:1;
      unsigned int g1_overflow_stop:1;
      unsigned int g1_separator:1;
      unsigned int g1_parity:1;
      unsigned int g1_l1_empty:1;
      unsigned int g1_l1_ready:1;

/* GROUP 0 */
      /* matching state */
      unsigned int g0_idle:1;
      unsigned int g0_write_occupancy:1;
      unsigned int g0_active:1;
      unsigned int g0_waiting_for_separator:1;
      unsigned int g0_wait_end:1;

      /* measurement */
      unsigned int g0_finetime:8;
      unsigned int g0_coarsetime:12;
      unsigned int g0_type_width:7;
      unsigned int g0_channel:3;
      unsigned int g0_error:1;
      unsigned int g0_overflow_start:1;
      unsigned int g0_overflow_stop:1;
      unsigned int g0_separator:1;
      unsigned int g0_parity:1;
      unsigned int g0_l1_empty:1;
      unsigned int g0_l1_ready:1;

/* GROUP 3 */
      /* hit data */
      unsigned int g3_hd_1vernier:32;
      unsigned int g3_hd_1coarse_1:15;
      unsigned int g3_hd_1coarse_1_parity:1;
      unsigned int g3_hd_1coarse_2:15;
      unsigned int g3_hd_1coarse_2_parity:1;
      unsigned int g3_hd_1edgetype:1;
      unsigned int g3_hd_2vernier:32;
      unsigned int g3_hd_2coarse_1:15;
      unsigned int g3_hd_2coarse_1_parity:1;
      unsigned int g3_hd_2coarse_2:15;
      unsigned int g3_hd_2coarse_2_parity:1;
      unsigned int g3_hd_2edgetype:1;
      unsigned int g3_hd_channel:3;
      unsigned int g3_hd_select_error:1;
      unsigned int g3_hd_load:1;

/* GROUP 2 */
      /* hit data */
      unsigned int g2_hd_1vernier:32;
      unsigned int g2_hd_1coarse_1:15;
      unsigned int g2_hd_1coarse_1_parity:1;
      unsigned int g2_hd_1coarse_2:15;
      unsigned int g2_hd_1coarse_2_parity:1;
      unsigned int g2_hd_1edgetype:1;
      unsigned int g2_hd_2vernier:32;
      unsigned int g2_hd_2coarse_1:15;
      unsigned int g2_hd_2coarse_1_parity:1;
      unsigned int g2_hd_2coarse_2:15;
      unsigned int g2_hd_2coarse_2_parity:1;
      unsigned int g2_hd_2edgetype:1;
      unsigned int g2_hd_channel:3;
      unsigned int g2_hd_select_error:1;
      unsigned int g2_hd_load:1;

/* GROUP 1 */
      /* hit data */
      unsigned int g1_hd_1vernier:32;
      unsigned int g1_hd_1coarse_1:15;
      unsigned int g1_hd_1coarse_1_parity:1;
      unsigned int g1_hd_1coarse_2:15;
      unsigned int g1_hd_1coarse_2_parity:1;
      unsigned int g1_hd_1edgetype:1;
      unsigned int g1_hd_2vernier:32;
      unsigned int g1_hd_2coarse_1:15;
      unsigned int g1_hd_2coarse_1_parity:1;
      unsigned int g1_hd_2coarse_2:15;
      unsigned int g1_hd_2coarse_2_parity:1;
      unsigned int g1_hd_2edgetype:1;
      unsigned int g1_hd_channel:3;
      unsigned int g1_hd_select_error:1;
      unsigned int g1_hd_load:1;

/* GROUP 0 */
      /* hit data */
      unsigned int g0_hd_1vernier:32;
      unsigned int g0_hd_1coarse_1:15;
      unsigned int g0_hd_1coarse_1_parity:1;
      unsigned int g0_hd_1coarse_2:15;
      unsigned int g0_hd_1coarse_2_parity:1;
      unsigned int g0_hd_1edgetype:1;
      unsigned int g0_hd_2vernier:32;
      unsigned int g0_hd_2coarse_1:15;
      unsigned int g0_hd_2coarse_1_parity:1;
      unsigned int g0_hd_2coarse_2:15;
      unsigned int g0_hd_2coarse_2_parity:1;
      unsigned int g0_hd_2edgetype:1;
      unsigned int g0_hd_channel:3;
      unsigned int g0_hd_select_error:1;
      unsigned int g0_hd_load:1;
      unsigned int idle:1;
    } bit;
  } hptdc_coretest_t;

  typedef union {
    unsigned short int all;     // 10 bits
    struct {
      unsigned int readout_end:1;
      unsigned int readout_fail:1;
      unsigned int group3_end:1;
      unsigned int group3_fail:1;
      unsigned int group2_end:1;
      unsigned int group2_fail:1;
      unsigned int group1_end:1;
      unsigned int group1_fail:1;
      unsigned int group0_end:1;
      unsigned int group0_fail:1;
    } bit;
  } hptdc_bist_t;

  typedef union {
    unsigned char all[8];       // 62 bits
    struct {
      unsigned int error:11;
      unsigned int have_token:1;
      unsigned int readout_fifo_occupancy:8;
      unsigned int readout_fifo_full:1;
      unsigned int readout_fifo_empty:1;
      unsigned int group3_l1_occupancy:8;
      unsigned int group2_l1_occupancy:8;
      unsigned int group1_l1_occupancy:8;
      unsigned int group0_l1_occupancy:8;
      unsigned int trigger_fifo_occupancy:4;
      unsigned int trigger_fifo_full:1;
      unsigned int trigger_fifo_empty:1;
      unsigned int dll_lock:1;
      unsigned int setup_601:1;
    } bit;
  } hptdc_status_t;

  typedef union {
    unsigned char all[5];       // 33 bits
    struct {
      unsigned int ready:1;
      unsigned int data;
    } bit;
  } hptdc_readout_t;

  typedef union {
    unsigned char all[81];
    struct {
      unsigned int test_select:4;
      unsigned int enable_error_mark:1;
      unsigned int enable_error_bypass:1;
      unsigned int enable_error:11;
      unsigned int readout_single_cycle_speed:3;
      unsigned int serial_delay:4;
      unsigned int strobe_select:2;
      unsigned int readout_speed_select:1;
      unsigned int token_delay:4;
      unsigned int enable_local_trailer:1;
      unsigned int enable_local_header:1;
      unsigned int enable_global_trailer:1;
      unsigned int enable_global_header:1;
      unsigned int keep_token:1;
      unsigned int master:1;
      unsigned int enable_bytewise:1;
      unsigned int enable_serial:1;
      unsigned int enable_jtag_readout:1;
      unsigned int tdc_id:4;
      unsigned int select_bypass_inputs:1;
      unsigned int readout_fifo_size:3;
      unsigned int reject_count_offset:12;
      unsigned int search_window:12;
      unsigned int match_window:12;
      unsigned int leading_resolution:3;
      unsigned int fixed_pattern:28;
      unsigned int enable_fixed_pattern:1;
      unsigned int max_event_size:4;
      unsigned int reject_readout_fifo_full:1;
      unsigned int enable_readout_occupancy:1;
      unsigned int enable_readout_separator:1;
      unsigned int enable_overflow_detect:1;
      unsigned int enable_relative:1;
      unsigned int enable_automatic_reject:1;
      unsigned int event_count_offset:12;
      unsigned int trigger_count_offset:12;
      unsigned int enable_set_counters_on_bunch_reset:1;
      unsigned int enable_master_reset_code:1;
      unsigned int enable_master_reset_on_event_reset:1;
      unsigned int enable_reset_channel_buffer_when_separator:1;
      unsigned int enable_separator_on_event_reset:1;
      unsigned int enable_separator_on_bunch_reset:1;
      unsigned int enable_direct_event_reset:1;
      unsigned int enable_direct_bunch_reset:1;
      unsigned int enable_direct_trigger:1;
      unsigned int offset_31:9;
      unsigned int offset_30:9;
      unsigned int offset_29:9;
      unsigned int offset_28:9;
      unsigned int offset_27:9;
      unsigned int offset_26:9;
      unsigned int offset_25:9;
      unsigned int offset_24:9;
      unsigned int offset_23:9;
      unsigned int offset_22:9;
      unsigned int offset_21:9;
      unsigned int offset_20:9;
      unsigned int offset_19:9;
      unsigned int offset_18:9;
      unsigned int offset_17:9;
      unsigned int offset_16:9;
      unsigned int offset_15:9;
      unsigned int offset_14:9;
      unsigned int offset_13:9;
      unsigned int offset_12:9;
      unsigned int offset_11:9;
      unsigned int offset_10:9;
      unsigned int offset_09:9;
      unsigned int offset_08:9;
      unsigned int offset_07:9;
      unsigned int offset_06:9;
      unsigned int offset_05:9;
      unsigned int offset_04:9;
      unsigned int offset_03:9;
      unsigned int offset_02:9;
      unsigned int offset_01:9;
      unsigned int offset_00:9;
      unsigned int coarse_count_offset:12;
      unsigned int dll_tap_adjust_00:3;
      unsigned int dll_tap_adjust_01:3;
      unsigned int dll_tap_adjust_02:3;
      unsigned int dll_tap_adjust_03:3;
      unsigned int dll_tap_adjust_04:3;
      unsigned int dll_tap_adjust_05:3;
      unsigned int dll_tap_adjust_06:3;
      unsigned int dll_tap_adjust_07:3;
      unsigned int dll_tap_adjust_08:3;
      unsigned int dll_tap_adjust_09:3;
      unsigned int dll_tap_adjust_10:3;
      unsigned int dll_tap_adjust_11:3;
      unsigned int dll_tap_adjust_12:3;
      unsigned int dll_tap_adjust_13:3;
      unsigned int dll_tap_adjust_14:3;
      unsigned int dll_tap_adjust_15:3;
      unsigned int dll_tap_adjust_16:3;
      unsigned int dll_tap_adjust_17:3;
      unsigned int dll_tap_adjust_18:3;
      unsigned int dll_tap_adjust_19:3;
      unsigned int dll_tap_adjust_20:3;
      unsigned int dll_tap_adjust_21:3;
      unsigned int dll_tap_adjust_22:3;
      unsigned int dll_tap_adjust_23:3;
      unsigned int dll_tap_adjust_24:3;
      unsigned int dll_tap_adjust_25:3;
      unsigned int dll_tap_adjust_26:3;
      unsigned int dll_tap_adjust_27:3;
      unsigned int dll_tap_adjust_28:3;
      unsigned int dll_tap_adjust_29:3;
      unsigned int dll_tap_adjust_30:3;
      unsigned int dll_tap_adjust_31:3;
      unsigned int rc_adjust_0:3;
      unsigned int rc_adjust_1:3;
      unsigned int rc_adjust_2:3;
      unsigned int rc_adjust_3:3;
      unsigned int rc_adjust_4:3;
      unsigned int low_power_mode:1;
      unsigned int width_select:4;
      unsigned int vernier_offset:5;
      unsigned int dll_control:4;
      unsigned int dead_time:2;
      unsigned int test_invert:1;
      unsigned int test_mode:1;
      unsigned int enable_trailing:1;
      unsigned int enable_leading:1;
      unsigned int mode_rc_compression:1;
      unsigned int mode_rc:1;
      unsigned int dll_mode:2;
      unsigned int pll_control:8;
      unsigned int serial_clock_delay:4;
      unsigned int io_clock_delay:4;
      unsigned int core_clock_delay:4;
      unsigned int dll_clock_delay:4;
      unsigned int serial_clock_source:2;
      unsigned int io_clock_source:2;
      unsigned int core_clock_source:2;
      unsigned int dll_clock_source:3;
      unsigned int roll_over:12;
      unsigned int enable_matching:1;
      unsigned int enable_pair:1;
      unsigned int enable_ttl_serial:1;
      unsigned int enable_ttl_control:1;
      unsigned int enable_ttl_reset:1;
      unsigned int enable_ttl_clock:1;
      unsigned int enable_ttl_hit:1;
      unsigned int parity:1;
    } bit;
  } hptdc_setup_t;

  typedef union {
    unsigned char all[5];
    struct {
      unsigned int enable_pattern:4;
      unsigned int Global_reset:1;
      unsigned int enable_channel:32;
      unsigned int dll_reset:1;
      unsigned int pll_reset:1;
      unsigned int parity:1;
    } bit;
  } hptdc_control_t;
#pragma pack(pop)

#endif /* _HPTDC_TYPES_H */
