//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <QDebug>
#include "tqdc_types.h"

QDebug operator<<(QDebug dbg, const tqdc_setup_t &s)
{
    dbg.nospace() << static_cast<const tdc_setup_t &>(s);
    dbg.nospace() << " tqdc_setup:\n";
    dbg.nospace() << "  adc_mode " << s.adc_mode << "\n";
    dbg.nospace() << "  adc_search " << s.adc_search << "\n";
    dbg.nospace() << "  adc_match " << s.adc_match << "\n";
    dbg.nospace() << "  trlat " << s.trlat << "\n";
    dbg.nospace() << "  thr {";
    for (int i=0; i<s.nch; i++)
        dbg.nospace() << s.thr[i] << " ";
    dbg.nospace() << "}\n";
    dbg.nospace() << "  maj " << s.maj << "\n";
    dbg.nospace() << "  min " << s.min << "\n";
    dbg.nospace() << "  trlat " << s.trlat << "\n";
    dbg.nospace() << "  neg_a " << s.neg_a << "\n";
    dbg.nospace() << "  neg_b " << s.neg_b << "\n";
//    dbg.nospace() << "  trigset " << s.trigset << "\n";
    return dbg.maybeSpace();
}

tqdc_setup_t::tqdc_setup_t()
{
    set_defaults();
}

void tqdc_setup_t::set_defaults()
{
    // search, match, trlat must be multiples of 5 !!!
    trlat = 49;
    adc_mode = 2;
    adc_search = 5;
    adc_match = 8;
    //  ch_enable.resize(TQDC_NCH);
    for (int i = 0; i < TQDC_NCH; i++) {
        thr[i] = 50;
        ch_enable[i] = 1;
    }
    // setup trigger submodule
    trigset.bit.shape = 1; // 20ns
    trigset.bit.alt_ch_1 = 0; // alt1 -> ch 0
    trigset.bit.alt_ch_2 = 0; // alt2 -> ch0
    trigset.bit.alt_en_1 = 0; // alt1 disabled
    trigset.bit.alt_en_2 = 0; // alt2 disabled
    trigset.bit.reserved = 0; // just set 0
    trigset.bit.match8 = 0; // don't use 8&8 matcher
    trigset.bit.enable = 0xFFFF; // enable all
    maj = 8; // maximum multiplicity for trigger
    min = 1; // minimum ...
    neg_a = 0;
    neg_b = 0;
    twoedge = 0;
}
