//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef _TQDC_TYPES_H
#define _TQDC_TYPES_H

#include <vector>

#include "daqcommon/hal_types.h"
#include "modules/tdc_types.h"
#include "hptdc/hptdc_types.h"


enum { TQDC_NCH = 16 };
enum { TQDC_NDEV = 2 };

#if (TQDC_NCH > TDC_MAXNCH)
# error
#endif
#if (TQDC_NDEV > TDC_MAXDEV)
# error
#endif

const unsigned int TQDC_ADC_BITS = 10;
const unsigned int TQDC_ADC_MAXRAW = (1<<10)-1;

const unsigned int TQDC_DTYPE_ADC_TS   = 0x40000000;
const unsigned int TQDC_DTYPE_ADC_DATA = 0x50000000;

#pragma pack(push, 1)
typedef union {
  unsigned int all;
  struct {
        unsigned int shape:4;    // 3:0  10ns step
        unsigned int alt_ch_1:4; // 7:4
        unsigned int alt_ch_2:4; // 11:8
        unsigned int alt_en_1:1; // 12
        unsigned int alt_en_2:1; // 13
        unsigned int reserved:1; // 14
        unsigned int match8:1;   // 15
        unsigned int enable:16;  // 31:16
  } bit;
} tqdc_trigset_reg_t;
#pragma pack(pop)

#pragma pack(push,4)
class tqdc_setup_t : public tdc_setup_t {
public:
    tqdc_setup_t();
    void set_defaults();

//  int ch_enable[TQDC_NCH];
//  hptdc_setup_t tdcsetup[TQDC_MAXDEV];
//  int matchwin;
//  int latency;
  int adc_mode;
  int adc_search;
  int adc_match;
  int trlat;
  int thr[TQDC_NCH];
  int maj;
  int min;
  int neg_a, neg_b, twoedge;
  tqdc_trigset_reg_t trigset;
};
#pragma pack(pop)

QDebug operator<<(QDebug dbg, const tqdc_setup_t &);

enum { TQDC_N_COUNTERS = 16 };

#endif /* _TQDC_TYPES_H */
