//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef _TRIG_TYPES_H
#define _TRIG_TYPES_H

#include "daqcommon/hal_types.h"

enum {TRIG_NCH = 4};
enum {TRIG_N_INPUTS = 40};
enum {TRIG_N_BITS = 20};
enum {TRIG_LOGIC_WIRES = TRIG_NCH};

enum {TRIG_SPILL_EXT=0, TRIG_SPILL_TIMER=1, TRIG_SPILL_GATE=2};

const double TRIG_TS_STEP = 1./(80*1e6); // timestamp frequency

const double TRIG_SPILLTIMER_STEP = 1./2.e7*65536.;
const double TRIG_SPILLTIMER_STEP_MS = 1000./2.e7*65536.;

const double TRIG_v1_1_SPILLTIMER_STEP = 1e-3;
const double TRIG_v1_1_SPILLTIMER_STEP_MS = 1.0;

const unsigned int TRIG_TTC_DELAY_BITS = 4;
const double TRIG_TTC_DELAY_STEP_US = 0.05;

const unsigned int TRIG_READ_DELAY_BITS = 16;
const int TRIG_READ_DELAY_OFFSET = 2;
const double TRIG_READ_DELAY_STEP_US = 0.05;

const unsigned int TRIG_XOFF_WAIT_BITS = 4;
const int TRIG_XOFF_WAIT_OFFSET = -1;
const double TRIG_XOFF_WAIT_STEP_US = 0.05;

#pragma pack(push, 1)
typedef union {
  unsigned int all;
  struct {
    unsigned int spill_ext_enable:1; // 0
    unsigned int nim1_enable:1; // 1
    unsigned int spill_gate:1; // 2
    unsigned int soft_spill:1; // 3
    unsigned int spill_select:1; // 4
    unsigned int noise_enable:1; // 5
    unsigned int en_xoff_in:1; // 6, 0=norm, 1=en_xoff_in
    unsigned int pulser_enable:1; // 7
    unsigned int lvds_enable:8; // 8-15
    unsigned int nim_enable:7; // 22-16
    unsigned int reserved4:9; // 31-23
  } bit;
} trig_trigset_t;

typedef union {
  unsigned int all;
  struct {
    unsigned int ttc_delay:TRIG_TTC_DELAY_BITS; // 0-3
    unsigned int xoff_wait:TRIG_XOFF_WAIT_BITS; // 4-7
    unsigned int reserved1:8; // 8-15
    unsigned int read_delay:TRIG_READ_DELAY_BITS; // 16-31
  } bit;
} trig_timers_t;

typedef union {
  unsigned int all;
  struct {
    unsigned int spillwidth:31;
    unsigned int en_spill_end_wait:1;
  } bit;
} spilwid_reg_t;
#pragma pack(pop)

#pragma pack(push,4)
class trig_setup_t : public common_setup_t {
public:
  int spill_select; // 1=internal timer, 0=external pulse + internal timer, 2=external gate (no timer)
//  bool spill_ext_enable;
  unsigned int spill_delay; // 1ms step
  unsigned int spill_duration;
  unsigned int spill_repeat;
  bool ch_enable[TRIG_NCH];
  bool input_enable[TRIG_N_INPUTS];
//  bool lvds_enable[8];
  bool pulser_trigger;
  bool noise_trigger;
  double pulser_freq;
  bool en_xoff_in;
  bool en_spill_end_wait; // forced to 1
  unsigned int ch_delay[TRIG_NCH];
  unsigned int in_shape;
  int nim_maj_min;
  int nim_maj_max;
//  unsigned int ttc_delay;
//  unsigned int read_delay;
//  unsigned int xoff_wait;
  trig_timers_t timers;
  unsigned int logic;
  unsigned int dch_clock_adj;
//  int bitmap[TRIG_BITMAP_SIZE];
  unsigned int out_sel_1;
  unsigned int out_sel_2;
};
#pragma pack(pop)

class QDebug;
QDebug operator<<(QDebug dbg, const trig_setup_t &);

#endif /* _TRIG_TYPES_H */
