
#include "dataplot.h"

#include <QVector>

#include <qwt/qwt_math.h>
#include <qwt/qwt_plot.h>
#include <qwt/qwt_symbol.h>
#include <qwt/qwt_plot_curve.h>
#include <qwt/qwt_plot_marker.h>

#include <cstdlib>

DataPlot::DataPlot (QWidget * p)
    : QwtPlot (p),
    mark_y0(new QwtPlotMarker())
{
    uint i;
    QVector<QColor> cmap(DataPlot_NCH);
    for (i = 0; i < DataPlot_NCH; i++)
        cmap[i] = QColor (rgbcmap[i % 8][0], rgbcmap[i % 8][1], rgbcmap[i % 8][2]);

    // Assign a title
    //    setTitle("ADC waveform");
    //  setAutoLegend (FALSE);

    for (uint c = 0; c < DataPlot_NCH; c++) {
        // Insert new curves
        QString str = QString("ch %1").arg(c);
        //    curv[c] = insertCurve (str);
        //    QwtPlotCurve *crv = curve (curv[c]);
        my_curve_t crv(new QwtPlotCurve(str));
        curves[c] = crv;
        crv->attach(this);
        crv->setPen (QPen (cmap[c], 2));
        //    crv->setStyle(QwtCurve::Steps);
        crv->setSymbol (QwtSymbol (QwtSymbol::XCross, QBrush(Qt::black), QPen(Qt::black), QSize (2, 2)));
    }

    //  Insert zero line at y = 0
    //  long mY = insertLineMarker ("", QwtPlot::yLeft);
    //  setMarkerYPos (mY, 0.0);
    // marker
    mark_y0->setYValue(0.0);
    mark_y0->setLineStyle(QwtPlotMarker::HLine);
    mark_y0->setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
    mark_y0->setLinePen(QPen(Qt::green, 0, Qt::DashDotLine));
    mark_y0->attach(this);

    // Set axis titles
    //    setAxisTitle(QwtPlot::xBottom, "Time/seconds");
    //    setAxisTitle(QwtPlot::yLeft, "Values");

    //    setAxisOptions(QwtPlot::yLeft, QwtAutoScale::Floating | QwtAutoScale::Logarithmic);
}

DataPlot::~DataPlot()
{
    // in case auto deletion disabled
    mark_y0->detach();
    delete mark_y0;
}

//void DataPlot::setpoints (int ch, const double *inx, const double *iny, int len)
//{
///*
//  if (!curves[ch]) return;
//  if (!inx||!iny||len<=0) {
//	curves[ch]->detach();
//  } else {
//	curves[ch]->setData (inx, iny, len);
//	curves[ch]->attach(this);
//  }
//*/
//}
void DataPlot::clearpoints (int ch)
{
    if (!curves[ch]) return;
    curves[ch]->setData(0, 0, 0);
}

void DataPlot::copypoints (int ch, const double *inx, const double *iny, int len)
{
    if (!curves[ch]) return;
    curves[ch]->setData (inx, iny, len);
}

void DataPlot::setstyle (int ch, int style, int options = 0)
{
    if ((ch < 0) || (ch >= static_cast<int>(DataPlot_NCH)))
        return;

    int ndiv, ncol, nmul;
    QColor cmap[DataPlot_NCH];
    /*
        ndiv = (options & 0xFF)+1;
        nmul = ((options>>8) & 0xFF)+1;
//	ncol = ((options>>8) & 0x7)+1;
*/
    ncol = 8;
    nmul = 1;
    ndiv = 1;
    if (options) {
        nmul = 2;
        ndiv = 8;
    }
    for (unsigned int i = 0; i < DataPlot_NCH; i++)
        cmap[i] = QColor (rgbcmap[(i / ndiv * nmul) % ncol][0], rgbcmap[(i / ndiv * nmul) % ncol][1], rgbcmap[(i / ndiv * nmul) % ncol][2]);

    my_curve_t crv(curves[ch]);
    if (!crv)
        return;
    int size, psize;;
    if (style == 0) {
        size = 1;
        psize = 2;
    } else {
        size = 3;
        psize = 4;
    }
    crv->setPen (QPen (cmap[ch], size));
    crv->setSymbol (QwtSymbol (QwtSymbol::XCross, QBrush(Qt::black), QPen(Qt::black), QSize (psize, psize)));
    // if (crv) crv->setStyle(style, options);
}

void DataPlot::setname (int ch, const char *name)
{
    if (!curves[ch]) return;
    curves[ch]->setTitle (name);
}

void DataPlot::setSymbol (int ch, const QwtSymbol &s)
{
    if (!curves[ch]) return;
    //  curves[ch]->setPen(QPen(cmap[ch], size));
    curves[ch]->setSymbol (s);
}

void DataPlot::setPen (int ch, const QPen &p)
{
    if (!curves[ch]) return;
    curves[ch]->setPen (p);
}

void DataPlot::setStyle (int ch, QwtPlotCurve::CurveStyle s)
{
    if (!curves[ch]) return;
    curves[ch]->setStyle (s);
}

DataPlot::my_curve_t DataPlot::getCurve (int ch)
{
    return (curves[ch]);
}
