
#include "histplot.h"

#include <QDebug>

#include <qwt/qwt_scale_engine.h>
#include <qwt/qwt_plot_curve.h>
#include <qwt/qwt_plot_grid.h>

#include <cstdlib>
#include <cfloat>
#include <iostream>

HistPlot::HistPlot (QWidget * parent = 0)
    : QwtPlot (parent),
    curv(new QwtPlotCurve("hist")),
    curv_f(new QwtPlotCurve ("fit")),
    grid(new QwtPlotGrid)
{
    range_imin = 0;
    range_imax = 1;
    log_y = false;
    y_max = 1;

    vbmin = 0; vbmax = 0; // disable autoset
    ar_threshold = 0.025;
    //  QwtPlotCurve *crv;
    //  printf("HistPlot constructor\n");
    //  setAutoLegend (FALSE);

    curv->attach(this);
    curv->setPen (QPen (Qt::black, 2));

    curv->setStyle (QwtPlotCurve::Steps);
    //  curv->setCurveAttribute (QwtPlotCurve::Inverted);
    curv->setBrush (Qt::blue);
    //  curv->setSymbol(QwtSymbol(QwtSymbol::XCross, black, black, QSize(2,2)));


    // fit function
    curv_f->setPen (QPen (Qt::black, 1));
    curv_f->setStyle (QwtPlotCurve::Lines);

    //  Insert zero line at y = 0
    //  long mY = insertLineMarker ("", QwtPlot::yLeft);
    //  setMarkerYPos (mY, 0.0);

    // Set axis titles
    //    setAxisTitle(QwtPlot::xBottom, "Time/seconds");
    //    setAxisTitle(QwtPlot::yLeft, "Values");

    //    setAxisOptions(QwtPlot::yLeft, QwtAutoScale::Floating | QwtAutoScale::Logarithmic);
    //  setAxisOptions(QwtPlot::yLeft, QwtAutoScale::Floating);
    //  setAxisRange(QwtPlot::yLeft, 0.1, LOG_MAX, 0);

    //  px=0;py=0;len=0;

    grid->enableXMin(true);
    grid->enableYMin(true);
    grid->setMajPen(QPen(Qt::black, 0, Qt::DotLine));
    grid->setMinPen(QPen(Qt::gray, 0 , Qt::DotLine));
    grid->attach(this);
}

HistPlot::~HistPlot()
{
    // manually delete detached objects since QwtPlot will not delete them
    if (! curv->plot()) delete curv;
    if (! curv_f->plot()) delete curv_f;
}

void HistPlot::setpoints (const hist_pt &inx, const hist_pt &iny)
{
    px = inx;
    py = iny;

    if (!px.size() || (px.size() < py.size())) {
        curv->detach();
    } else {
        curv->setData (&px[0], &py[0], py.size()); // copy, but may optimize
        if (!curv->plot())
            curv->attach(this);
    }
    update_yscale();
}

void HistPlot::update_yscale()
{
    double m = -DBL_MAX;
    for (uint i=0;i<py.size();i++)
        if (py[i] > m) m = py[i];
    for (uint i=0;i<pfy.size();i++)
        if (pfy[i] > m) m = pfy[i];
    y_max = m;
    if (log_y)
        setAxisScale(QwtPlot::yLeft, 0.5, y_max, 0);
    else
        setAxisAutoScale(QwtPlot::yLeft);
}

void HistPlot::set_fit_points (const hist_pt &inx, const hist_pt &iny)
{
    pfx = inx;
    pfy = iny;

    if (!pfx.size() || (pfx.size() < pfy.size())) {
        curv_f->detach();
    } else {
        curv_f->setData (&pfx[0], &pfy[0], pfy.size()); // copy, but may optimize
        if (!curv_f->plot())
            curv_f->attach(this);
    }
}
void HistPlot::hide_fit_points ()
{
    if (curv_f->plot())
        curv_f->detach();
}

//! find peak in bin range [bmin,bmax)
double HistPlot::get_max_value(int bmin, int bmax) const
{
    double maxvalue = 1;
    int len = py.size();
    if (bmin<0) bmin=0;
    if (bmax > len) bmax = len;
    if (bmax == 0) bmax = len;
    if (bmax == bmin) maxvalue = py[bmin];
    for (int i = bmin; i < bmax; i++) {
        if (py[i] > maxvalue) maxvalue = py[i];
    }

    //  printf("maxvalue=%f\n", maxvalue);

    const int nb = 1000;
    std::vector<double> hh(nb, 0.);
    //  double hsum = 0;
    for (int i = bmin; i < bmax; i++) {
        if (py[i]<=0) continue;
        int hbin = py[i]/maxvalue*(nb-1);
        if (hbin >= 0 && hbin < nb) hh[hbin] ++;
        //        hsum += py[i];
    }
    double hmax = 1;
    for (int i=0; i<nb; i++) {
        if (hh[i] > hmax)
            hmax = hh[i];
    }
    //  printf("hmax=%f\n", hmax);
    for (int i=0; i<nb; i++) {
        hh[i] /= hmax;
        //	printf("hh[%2d] = %.9f\n", i, hh[i]);
    }
    //  double topsum = 0;
    for (int i=nb-1; i>0; i--) {
        //	topsum += hh[i];
        //	if (topsum > 20 || hh[i] > 1) return (double)maxvalue*(i+1)/nb;
        if (hh[i] > 0.001) return 1. * maxvalue * (i+1) / nb;
    }

    return maxvalue;
}

void HistPlot::set_autorange_threshold(double thr)
{
    ar_threshold = thr;
}

void HistPlot::autorange() {
    //return;
    int len = py.size();
    if (len<2) return;
    bool autoset = (vbmin==0 && vbmax == 0);

    if (vbmin > vbmax) {int tmp = vbmax; vbmax=vbmin; vbmin=tmp;} // swap
    if (vbmin < 0) vbmin = 0;
    if (vbmax >= len) vbmax = len-1;

    double maxvalue = get_max_value(vbmin, vbmax);

    if (autoset) {
        double threshold = maxvalue * ar_threshold;
        if (threshold < 2) threshold = 2;

        int imin=0, imax=len-1;
        int found=0, norescale=0;
        while (imin+1 < len) {
            if (py[imin] >= threshold) {
                found=1;
                break;
            }
            imin++;
        }
        if (found) {
            while ((imax > imin) && (py[imax] < threshold))
                imax--;
            imin--; imax++; imax++;
            if (imin<0) imin=0;
            if (imax+1 > len) imax = len-1;
            if (imin < imax) {
                range_imin = imin;
                range_imax = imax;
            } else {
                norescale=1;
            }
        } else {
            norescale=1;
        }
        if (norescale) {
            range_imin = 0;
            range_imax = len-1;
        }
    } else { // not autoset
        range_imin = vbmin;
        range_imax = vbmax;
    }
    setAxisScale (QwtPlot::xBottom, px[range_imin], px[range_imax], 0);
    setAxisScale (QwtPlot::yLeft, log_y?0.5:0.0, maxvalue, 0);
}

void HistPlot::set_log_y(bool state) {
    QwtScaleEngine *sa;
    log_y = state;
    if (log_y)
        sa = new QwtLog10ScaleEngine;
    else
        sa = new QwtLinearScaleEngine;
    setAxisScaleEngine(QwtPlot::yLeft, sa);
    update_yscale();
    /*
  int opt = axisOptions(QwtPlot::yLeft);
  if (log_y) {
        opt |= (QwtAutoScale::Logarithmic);
  } else {
        opt &= ~(QwtAutoScale::Logarithmic);
//    setAxisScale(QwtPlot::yLeft, 0, ymax, 0);
  }
*/
}

void HistPlot::set_x_scale(double xmin, double xmax)
{
    setAxisScale(QwtPlot::xBottom, xmin, xmax);
}

void HistPlot::set_y_scale(double ymin, double ymax)
{
    if (log_y && ymin < 0.5) { ymin = 0.5; }
    setAxisScale(QwtPlot::yLeft, ymin, ymax);
}
/*
void HistPlot::get_x_scale(double *xmin, double *xmax) const
{
  *xmin = px[range_imin];
  *xmax = px[range_imax];
}
void HistPlot::get_y_scale(double *ymin, double *ymax) const
{
  *ymin = 0; *ymax = get_max_value();
}
*/

void HistPlot::set_visible_bin_range(int bmin, int bmax)
{
    vbmin = bmin;
    vbmax = bmax;
}

void HistPlot::setCurveStyle(QwtPlotCurve::CurveStyle style)
{
    if (curv)
        curv->setStyle (style);
}

void HistPlot::setCurvePen(const QPen &pen)
{
    if (curv)
        curv->setPen(pen);
}

void HistPlot::setCurveBrush(const QBrush &brush)
{
    if (curv)
        curv->setBrush(brush);
}
