#ifndef _HISTPLOT_H
#define _HISTPLOT_H

#include <vector>
#include <string>
#include <qwt/qwt_plot.h>
#include <qwt/qwt_plot_curve.h>

typedef std::vector<double> hist_pt;

class QwtScaleEngine;
class QwtPlotCurve;
class QwtPlotGrid;

class HistPlot:public QwtPlot {
public:
  HistPlot (QWidget *parent);
  virtual ~HistPlot();
  void setpoints (const hist_pt &inx, const hist_pt &iny);
  void set_fit_points (const hist_pt &inx, const hist_pt &iny);
  void hide_fit_points ();
  const hist_pt getpointsx() {return px;}
  const hist_pt getpointsy() {return py;}
  const hist_pt getfitpointsx() {return pfx;}
  const hist_pt getfitpointsy() {return pfy;}
  void autorange();
  void set_log_y(bool);
  int get_ar_imin() const {return range_imin;}
  int get_ar_imax() const {return range_imax;}
  void set_visible_bin_range(int bmin, int bmax);
  void set_x_scale(double xmin, double xmax);
  void set_y_scale(double ymin, double ymax);
//  void get_x_scale(double *xmin, double *xmax) const;
//  void get_y_scale(double *ymin, double *ymax) const;
  double get_max_value(int bmin, int bmax) const;
  void set_autorange_threshold(double thr);
  void setCurveStyle(QwtPlotCurve::CurveStyle);
  void setCurvePen(const QPen &);
  void setCurveBrush(const QBrush &);

protected:
  void update_yscale();

private:
  QwtPlotCurve * const curv, * const curv_f;
  QwtPlotGrid * const grid;
  hist_pt px, py, pfx, pfy;
  int range_imin, range_imax, vbmin, vbmax;
  bool log_y;
  double y_max;
  double ar_threshold;
};

#endif /* _HISTPLOT_H */
