
#include "wfdata.h"

#include <QDebug>
#include <vector>
#include <cmath>
#include <cstdlib>

#include "daqcommon/dsplib.h"
#include "daqcommon/hptimer.h"

namespace {
    //! eliminate edge effects
    //! t - fine offset (0..1)
    /*
double find_fine_integral(std::vector<double> v, double t)
{
//  t-=.5;
  unsigned int n = v.size();
  if (n<2) return v[0]; // one point - no 1st derivative possible
  double i1 = (v[1]+v[0])/2*t; // subtract this
//  double vn = 2*v[n-1] - v[n-2]; // linear extrapolation
//  double i2 = (vn+v[n-1])/2*t; // add this
  double i2 = (2*v[n-1] + t*(v[n-1]-v[n-2]))*t;
  double i = 0;
  for (uint k=0;k<n;k++)
        i += v[k];
  i -= i1*.5;
  i += i2*.5;
  return i;
}
*/
    double find_fine_integral(std::vector<double> v, double t)
    {
        size_t n = v.size();
        if (n<2) return v[0]; // one point - no 1st derivative possible
        double dvb = v[1]-v[0];
        double vb = v[0]+dvb*t;
        double dve = v[n-2]-v[n-1];
        double ve = v[n-2]+dve*t;
        double i1 = (v[0]+vb)/2*t; // subtract this
        double i2 = (v[n-1]+ve)/2*(1-t); // subtract this
        double i = 0;
        for (size_t k=0; k<n; k++)
            i += v[k];
        i -= i1;
        i += i2;
        return i;
    }
}

void WaveformData::simulate()
{
    int ch = rand() % 1000;
    int t_start = rand() % 30; // waveform start offset
    int len = 10;
    double dt = 1.;
    // pulse amplitude parameters
    double a = 1.0/* + rangaus(0.001)*/;
    double rt = 1.5;
    double ft = 5.;
    // TDC hits
    bool has_t0 = 1;//(rand()%100 < 95); // 5% tdc hit is lost
    // 10% - one pileup
    // 2% - two pileup
    unsigned int num_tx = 0;
    int r = rand()%100;
    if (r >= 90) num_tx++;
    if (r >= 98) num_tx++;
    double pulse_t = 3.+ranuni(0.0, 1.0); // pulse time offset inside waveform slide
    t.clear();
    if (has_t0) t.push_back(t_start + pulse_t);
    for (unsigned int i=0;i<num_tx;i++)
        t.push_back(t_start + ranuni(0., len));

    //  if (a<0) a=-a;

    p.resize(len);

    std::vector<double> py(len);
    for (int i=0;i<len;i++) {
        py[i] = detpulse (a, i*dt-pulse_t, rt, ft) + rangaus(0.001);
    }

    //  detpulse_v (a, t, rt, ft, &py[0], len, dt);

    c = ch;
    for (int i=0;i<len;i++) {
        p[i].x = t_start + i*dt;
        p[i].y = py[i];
    }
    //  double m_t = has_t0?t[0]-t_start-0.5:0;
    //  double m_t = has_t0?(t[0]-floor(t[0])):0;
    //  double m_a = find_fine_integral(py, m_t);
}

void WaveformData::print() const
{
    qDebug().nospace() << "WaveformData, channel=" << c << ", len=" << p.size();
}
/*
void WaveformModel::simulate() {
  hpTimer hpt;
  hpt.start();
  for (uint i=0;i<wd.size();i++)
        wd[i].simulate();
  hpt.stop();
  double dt = hpt.elapsed();
  stats->add("simulation time", dt);
//  stats->print();
}
*/

WaveformDataAll WaveformDataSimulator::run()
{
    WaveformDataAll wd;
    wd.resize(10);
    hpTimer hpt;
    hpt.start();
    for (size_t i=0;i<wd.size();i++)
        wd[i].simulate();
    hpt.stop();
    //  double dt = hpt.elapsed();
    //  stats->add("simulation time", dt);
    return wd;
}
