
#include "wfplot.h"

#include <QWidget>
#include <QColor>
#include <qwt/qwt_symbol.h>
#include <qwt/qwt_plot.h>
#include <qwt/qwt_plot_grid.h>
#include <qwt/qwt_plot_curve.h>
#include <qwt/qwt_plot_marker.h>
#include <qwt/qwt_legend.h>

//const int min_n_curves_threshold = 20; // keep at least this much curves

WFPlot::WFPlot(QWidget *parent): QwtPlot(parent)
{
    colormap.push_back(QColor(255, 150, 0));
    colormap.push_back(QColor(255, 0, 0));
    colormap.push_back(QColor(255, 0, 190));
    colormap.push_back(QColor(110, 0, 190));
    colormap.push_back(QColor(0, 100, 255));
    colormap.push_back(QColor(137, 137, 255));
    colormap.push_back(QColor(0, 200, 200));
    colormap.push_back(QColor(0, 190, 0));

    // grid
    QwtPlotGrid *grid = new QwtPlotGrid;
    //    grid->enableXMin(true);
    grid->setMajPen(QPen(Qt::white, 0, Qt::DotLine));
    //    grid->setMinPen(QPen(Qt::gray, 0 , Qt::DotLine));
    grid->attach(this);
    insertLegend(new QwtLegend(this), QwtPlot::RightLegend);
}

void WFPlot::setAllData(const WaveformDataAll &data)
{
    // curve[n] may have 3 states: NULL, hidden, shown.
    // All existing curves, shown and hidden

    // make ids: count waveforms for same channel
    std::map<int, int> ids;
    //  for (unsigned int i=0;i<data.size();i++) {
    //	const WaveformData &wd = data[i];
    //  }

    // make map of new shown curves
    std::map<int, bool> shown;
    for (unsigned int i=0; i<data.size() ;i++) {
        const WaveformData &wd = data[i];
        //    wd.print();
        //	int id = hw_to_plot(wd.c);
        //	int id = wd.c;
        //	int id = i;
        int id = wd.c + ids[wd.c]*colormap.size();
        ids[wd.c]++;
        shown[id] = true;
        //	WFCurve c(curves[id]);
        QPen curvePen = QPen (colormap[id%colormap.size()], 2);
        QString curveName = QString::fromStdString(wd.name);
        if (curveName.isEmpty()) {
            curveName = QString("%1").arg(wd.c);
        }
        if (curves[id] == NULL) { // create new curve and attach it
            // insert new curve
            WFCurve crv(new QwtPlotCurve(curveName));
            crv->setSymbol (QwtSymbol (QwtSymbol::XCross, QBrush(Qt::black), QPen(Qt::black), QSize (2, 2)));
            crv->setPen (curvePen);
            curves[id] = crv;
        }
        if (curves[id] && !curves[id]->plot()) { // not attached, do it
            curves[id]->attach(this);
        }
        // set data
        if (curves[id]) {
            std::vector<double> x, y;
            uint len = wd.p.size();
            x.resize(len);
            y.resize(len);
            for (uint k=0;k<len;k++) {
                x[k] = wd.p[k].x;
                y[k] = wd.p[k].y;
            }
            curves[id]->setData(&x[0], &y[0], len);
            curves[id]->setTitle(curveName);
        }
        // ref mark
#ifdef ENABLE_REF_MARKERS
        if (markref[id] == NULL) {
            WFMarker mark(new QwtPlotMarker());
            mark->setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
            mark->setLineStyle(QwtPlotMarker::HLine);
            //	  mark->setLineStyle(QwtPlotMarker::NoLine);
            mark->setLinePen(curvePen);
            mark->setLabel(curveName);
            markref[id] = mark;
        }
        if (markref[id] && !markref[id]->plot()) { // not attached, do it
            markref[id]->attach(this);
        }
        size_t len = wd.p.size();
        double last_y = 0;
        if (len>0)
            last_y = wd.p[len-1].y;
        markref[id]->setYValue(last_y);
#endif
    }
    // now find curves, markers to hide
    Q_FOREACH(int id, curves.keys()) {
        if ( !shown[id] ) {
            if (curves[id] && curves[id]->plot()) {
                curves[id]->detach();
            }
            if (markref[id] && markref[id]->plot()) {
                markref[id]->detach();
            }
        }
    }
}

void WFPlot::clear()
{
    // now find curves,markers to hide
    Q_FOREACH(int id, curves.keys()) {
        if (curves[id] && curves[id]->plot()) {
            curves[id]->detach();
        }
        if (markref[id] && markref[id]->plot()) {
            markref[id]->detach();
        }
    }
}
